@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: ExampleStamp.java 627367 2008-02-13 12:03:30Z maxberger $ */
- 
+/* $Id: ExampleStamp.java 747752 2009-02-25 11:31:16Z jeremias $ */
+
 package embedding.intermediate;
 
 import java.io.File;
@@ -28,16 +28,18 @@ import javax.xml.transform.Transformer;
 import javax.xml.transform.TransformerException;
 import javax.xml.transform.TransformerFactory;
 import javax.xml.transform.sax.SAXResult;
+import javax.xml.transform.stream.StreamResult;
 import javax.xml.transform.stream.StreamSource;
 
+import org.xml.sax.SAXException;
+
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.apps.FopFactory;
 import org.apache.fop.apps.MimeConstants;
-import org.apache.fop.area.AreaTreeModel;
-import org.apache.fop.area.AreaTreeParser;
-import org.apache.fop.area.RenderPagesModel;
-import org.apache.fop.fonts.FontInfo;
-import org.xml.sax.SAXException;
+import org.apache.fop.render.intermediate.IFDocumentHandler;
+import org.apache.fop.render.intermediate.IFException;
+import org.apache.fop.render.intermediate.IFParser;
+import org.apache.fop.render.intermediate.IFUtil;
 
 import embedding.ExampleObj2XML;
 import embedding.model.ProjectTeam;
@@ -50,89 +52,91 @@ public class ExampleStamp {
 
     // configure fopFactory as desired
     private FopFactory fopFactory = FopFactory.newInstance();
-    
+
     /**
      * Stamps an intermediate file and renders it to a PDF file.
-     * @param atfile the intermediate file (area tree XML)
+     * @param iffile the intermediate file (area tree XML)
      * @param stampSheet the stylesheet that does the stamping
      * @param pdffile the target PDF file
      * @throws IOException In case of an I/O problem
      * @throws TransformerException In case of a XSL transformation problem
      * @throws SAXException In case of an XML-related problem
+     * @throws IFException if there was an IF-related error while creating the output file
      */
-    public void stampToPDF(File atfile, File stampSheet, File pdffile) 
-            throws IOException, TransformerException, SAXException {
+    public void stampToPDF(File iffile, File stampSheet, File pdffile)
+            throws IOException, TransformerException, SAXException, IFException {
         // Setup output
         OutputStream out = new java.io.FileOutputStream(pdffile);
         out = new java.io.BufferedOutputStream(out);
         try {
-            //Setup fonts and user agent
-            FontInfo fontInfo = new FontInfo();
+            //user agent
             FOUserAgent userAgent = fopFactory.newFOUserAgent();
 
-            //Construct the AreaTreeModel that will received the individual pages
-            AreaTreeModel treeModel = new RenderPagesModel(userAgent, 
-                    MimeConstants.MIME_PDF, fontInfo, out);
-            
-            //Iterate over all intermediate files
-            AreaTreeParser parser = new AreaTreeParser();
-            Source src = new StreamSource(atfile);
+            //Setup target handler
+            String mime = MimeConstants.MIME_PDF;
+            IFDocumentHandler targetHandler = fopFactory.getRendererFactory().createDocumentHandler(
+                    userAgent, mime);
+
+            //Setup fonts
+            IFUtil.setupFonts(targetHandler);
+            targetHandler.setResult(new StreamResult(pdffile));
+
+            IFParser parser = new IFParser();
+
+            Source src = new StreamSource(iffile);
             Source xslt = new StreamSource(stampSheet);
-            
+
             //Setup Transformer for XSLT processing
             TransformerFactory tFactory = TransformerFactory.newInstance();
             Transformer transformer = tFactory.newTransformer(xslt);
-            
+
             //Send XSLT result to AreaTreeParser
-            SAXResult res = new SAXResult(parser.getContentHandler(treeModel, userAgent));
-            
+            SAXResult res = new SAXResult(parser.getContentHandler(targetHandler, userAgent));
+
             //Start XSLT transformation and area tree parsing
             transformer.transform(src, res);
-            
-            //Signal the end of the processing. The renderer can finalize the target document.
-            treeModel.endDocument();
         } finally {
             out.close();
         }
     }
-    
+
     /**
      * Main method.
      * @param args command-line arguments
      */
     public static void main(String[] args) {
         try {
-            System.out.println("FOP ExampleConcat\n");
-            
+            System.out.println("FOP ExampleConcat (for the Intermediate Format)\n");
+
             //Setup directories
             File baseDir = new File(".");
             File outDir = new File(baseDir, "out");
             outDir.mkdirs();
-            
+
             //Setup output file
             File xsltfile = new File(baseDir, "xml/xslt/projectteam2fo.xsl");
-            File atfile = new File(outDir, "team.at.xml");
-            File stampxsltfile = new File(baseDir, "xml/xslt/atstamp.xsl");
-            File pdffile = new File(outDir, "ResultStamped.pdf");
-            System.out.println("Intermediate file : " + atfile.getCanonicalPath());
+            File iffile = new File(outDir, "team.if.xml");
+            File stampxsltfile = new File(baseDir, "xml/xslt/ifstamp.xsl");
+            File pdffile = new File(outDir, "ResultIFStamped.pdf");
+            System.out.println("Intermediate file : " + iffile.getCanonicalPath());
             System.out.println("Stamp XSLT: " + stampxsltfile.getCanonicalPath());
             System.out.println("PDF Output File: " + pdffile.getCanonicalPath());
             System.out.println();
 
             ProjectTeam team1 = ExampleObj2XML.createSampleProjectTeam();
-            
+
             //Create intermediate file
             ExampleConcat concatapp = new ExampleConcat();
             concatapp.convertToIntermediate(
-                    team1.getSourceForProjectTeam(), 
-                    new StreamSource(xsltfile), atfile);
-            
+                    team1.getSourceForProjectTeam(),
+                    new StreamSource(xsltfile), iffile);
+
             //Stamp document and produce a PDF from the intermediate format
             ExampleStamp app = new ExampleStamp();
-            app.stampToPDF(atfile, stampxsltfile, pdffile);
-            
+            app.stampToPDF(iffile, stampxsltfile, pdffile);
+
             System.out.println("Success!");
-            
+
         } catch (Exception e) {
             e.printStackTrace(System.err);
             System.exit(-1);
