@@ -1,13 +1,13 @@
-/* 
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: GenerateLineBreakUtils.java 489620 2006-12-22 11:13:46Z manuel $ */
+/* $Id: GenerateLineBreakUtils.java 810896 2009-09-03 11:36:02Z spepping $ */
 
 package org.apache.fop.text.linebreak;
 
@@ -30,7 +30,8 @@ import java.util.Arrays;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-import java.util.StringTokenizer;
+
+import org.apache.fop.util.License;
 
 /**
  * <p>Utility for generating a Java class representing line break properties
@@ -55,6 +56,7 @@ public class GenerateLineBreakUtils {
     private static final byte COMBINING_PROHIBITED_BREAK = 3;   // @ in table
     private static final byte PROHIBITED_BREAK = 4;             // ^ in table
     private static final byte EXPLICIT_BREAK = 5;               // ! in rules
+    private static final String BREAK_CLASS_TOKENS = "_%#@^!";
     private static final String notInPairTable[] = { "AI", "BK", "CB", "CR", "LF", "NL", "SA", "SG", "SP", "XX" };
 
     private static final byte lineBreakProperties[] = new byte[0x10000];
@@ -67,7 +69,7 @@ public class GenerateLineBreakUtils {
      * table for the table driven line breaking algorithm described in
      * <a href="http://unicode.org/reports/tr14/#PairBasedImplementation">UTR #14</a>.
      * TODO: Code points above the base plane are simply ignored.
-     * 
+     *
      * @param lineBreakFileName Name of line break property file (part of Unicode files).
      * @param propertyValueFileName Name of property values alias file (part of Unicode files).
      * @param breakPairFileName Name of pair table file (<i>not</i> part of the unicode files).
@@ -103,27 +105,32 @@ public class GenerateLineBreakUtils {
         BufferedReader b = new BufferedReader(new FileReader(breakPairFileName));
         String line = b.readLine();
         int lineNumber = 1;
+        String[] lineTokens;
+        String name;
         // read header
         if (line != null) {
-            StringTokenizer tok = new StringTokenizer(line);
+            lineTokens = line.split("\\s+");
             byte columnNumber = 0;
-            while (tok.hasMoreTokens()) {
-                String name = tok.nextToken();
-                if (columnNumber >= columnHeader.length) {
-                    throw new Exception(breakPairFileName + ':' + lineNumber + ": unexpected column header " + name);
+
+            for (int i = 0; i < lineTokens.length; ++i) {
+                name = lineTokens[i];
+                if (name.length() > 0) {
+                    if (columnNumber >= columnHeader.length) {
+                        throw new Exception(breakPairFileName + ':' + lineNumber + ": unexpected column header " + name);
+                    }
+                    if (notInPairTableMap.get(name) != null) {
+                        throw new Exception(breakPairFileName + ':' + lineNumber + ": invalid column header " + name);
+                    }
+                    Byte v = (Byte)lineBreakPropertyValues.get(name);
+                    if (v != null) {
+                        byte vv = v.byteValue();
+                        columnHeader[columnNumber] = vv;
+                        columnMap[vv] = columnNumber;
+                    } else {
+                        throw new Exception(breakPairFileName + ':' + lineNumber + ": unknown column header " + name);
+                    }
+                    columnNumber++;
                 }
-                if (notInPairTableMap.get(name) != null) {
-                    throw new Exception(breakPairFileName + ':' + lineNumber + ": invalid column header " + name);
-                }
-                Byte v = (Byte)lineBreakPropertyValues.get(name);
-                if (v != null) {
-                    byte vv = v.byteValue();
-                    columnHeader[columnNumber] = vv;
-                    columnMap[vv] = columnNumber;
-                } else {
-                    throw new Exception(breakPairFileName + ':' + lineNumber + ": unknown column header " + name);
-                }
-                columnNumber++;
             }
             if (columnNumber < columnHeader.length) {
                 StringBuffer missing = new StringBuffer();
@@ -156,9 +163,9 @@ public class GenerateLineBreakUtils {
                 throw new Exception(breakPairFileName + ':' + lineNumber + ": unexpected row " + line);
             }
             pairTable[rowNumber] = new byte[tableSize];
-            StringTokenizer tok = new StringTokenizer(line);
-            if (tok.hasMoreTokens()) {
-                String name = tok.nextToken();
+            lineTokens = line.split("\\s+");
+            if (lineTokens.length > 0) {
+                name = lineTokens[0];
                 if (notInPairTableMap.get(name) != null) {
                     throw new Exception(breakPairFileName + ':' + lineNumber + ": invalid row header " + name);
                 }
@@ -174,27 +181,15 @@ public class GenerateLineBreakUtils {
                 throw new Exception(breakPairFileName + ':' + lineNumber + ": can't read row header");
             }
             int columnNumber = 0;
-            while (tok.hasMoreTokens()) {
-                String token = tok.nextToken();
+            String token;
+            for (int i = 1; i < lineTokens.length; ++i) {
+                token = lineTokens[i];
                 if (token.length() == 1) {
-                    switch (token.charAt(0)) {
-                        case '^' :
-                            pairTable[rowNumber][columnNumber] = PROHIBITED_BREAK;
-                            break;
-                        case '%' :
-                            pairTable[rowNumber][columnNumber] = INDIRECT_BREAK;
-                            break;
-                        case '_' :
-                            pairTable[rowNumber][columnNumber] = DIRECT_BREAK;
-                            break;
-                        case '#' :
-                            pairTable[rowNumber][columnNumber] = COMBINING_INDIRECT_BREAK;
-                            break;
-                        case '@' :
-                            pairTable[rowNumber][columnNumber] = COMBINING_PROHIBITED_BREAK;
-                            break;
-                        default :
-                            throw new Exception(breakPairFileName + ':' + lineNumber + ": unexpected token: " + token);
+                    byte tokenBreakClass = (byte)BREAK_CLASS_TOKENS.indexOf(token.charAt(0));
+                    if (tokenBreakClass >= 0) {
+                        pairTable[rowNumber][columnNumber] = tokenBreakClass;
+                    } else {
+                        throw new Exception(breakPairFileName + ':' + lineNumber + ": unexpected token: " + token);
                     }
                 } else {
                     throw new Exception(breakPairFileName + ':' + lineNumber + ": token too long: " + token);
@@ -233,45 +228,40 @@ public class GenerateLineBreakUtils {
         int idx = 0;
         StringBuffer doStaticLinkCode = new StringBuffer();
         PrintWriter out = new PrintWriter(new FileWriter(outFileName));
-        out.println("/*");
-        out.println(" * Licensed to the Apache Software Foundation (ASF) under one or more");
-        out.println(" * contributor license agreements.  See the NOTICE file distributed with");
-        out.println(" * this work for additional information regarding copyright ownership.");
-        out.println(" * The ASF licenses this file to You under the Apache License, Version 2.0");
-        out.println(" * (the \"License\"); you may not use this file except in compliance with");
-        out.println(" * the License.  You may obtain a copy of the License at");
-        out.println(" * ");
-        out.println(" *      http://www.apache.org/licenses/LICENSE-2.0");
-        out.println(" * ");
-        out.println(" * Unless required by applicable law or agreed to in writing, software");
-        out.println(" * distributed under the License is distributed on an \"AS IS\" BASIS,");
-        out.println(" * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.");
-        out.println(" * See the License for the specific language governing permissions and");
-        out.println(" * limitations under the License.");
-        out.println(" */");
-        out.println();
-        out.println("/* $Id: GenerateLineBreakUtils.java 489620 2006-12-22 11:13:46Z manuel $ */");
+        License.writeJavaLicenseId(out);
         out.println();
         out.println("package org.apache.fop.text.linebreak;");
         out.println();
-        out.println("/* ");
-        out.println(" * This is a generated file, DO NOT CHANGE!");
+        out.println("/*");
+        out.println(" * !!! THIS IS A GENERATED FILE !!!");
+        out.println(" * If updates to the source are needed, then:");
+        out.println(" * - apply the necessary modifications to");
+        out.println(" *   'src/codegen/unicode/java/org/apache/fop/text/linebreak/GenerateLineBreakUtils.java'");
+        out.println(" * - run 'ant codegen-unicode', which will generate a new LineBreakUtils.java");
+        out.println(" *   in 'src/java/org/apache/fop/text/linebreak'");
+        out.println(" * - commit BOTH changed files");
         out.println(" */");
         out.println();
-        out.println("class LineBreakUtils {");
+        out.println("public final class LineBreakUtils {");
         out.println();
+        out.println("    /** Break class constant */");
         out.println("    public static final byte DIRECT_BREAK = " + DIRECT_BREAK + ';');
+        out.println("    /** Break class constant */");
         out.println("    public static final byte INDIRECT_BREAK = " + INDIRECT_BREAK + ';');
+        out.println("    /** Break class constant */");
         out.println("    public static final byte COMBINING_INDIRECT_BREAK = " + COMBINING_INDIRECT_BREAK + ';');
+        out.println("    /** Break class constant */");
         out.println("    public static final byte COMBINING_PROHIBITED_BREAK = " + COMBINING_PROHIBITED_BREAK + ';');
+        out.println("    /** Break class constant */");
         out.println("    public static final byte PROHIBITED_BREAK = " + PROHIBITED_BREAK + ';');
+        out.println("    /** Break class constant */");
         out.println("    public static final byte EXPLICIT_BREAK = " + EXPLICIT_BREAK + ';');
         out.println();
         out.println("    private static final byte PAIR_TABLE[][] = {");
         boolean printComma = false;
         for (int i = 1; i <= lineBreakPropertyValueCount; i++) {
             if (printComma) {
-                out.println(',');
+                out.println(",");
             } else {
                 printComma = true;
             }
@@ -279,7 +269,7 @@ public class GenerateLineBreakUtils {
             boolean localPrintComma = false;
             for (int j = 1; j <= lineBreakPropertyValueCount; j++) {
                 if (localPrintComma) {
-                    out.print(',');
+                    out.print(", ");
                 } else {
                     localPrintComma = true;
                 }
@@ -313,7 +303,7 @@ public class GenerateLineBreakUtils {
                         found = true;
                         doStaticLinkCode.append("        lineBreakProperties[");
                         doStaticLinkCode.append(i);
-                        doStaticLinkCode.append("]=lineBreakProperties[");
+                        doStaticLinkCode.append("] = lineBreakProperties[");
                         doStaticLinkCode.append(j);
                         doStaticLinkCode.append("];\n");
                         break;
@@ -322,7 +312,7 @@ public class GenerateLineBreakUtils {
             }
             if (!found) {
                 if (rowsPrinted >= 64) {
-                    out.println("    };");
+                    out.println("    }");
                     out.println();
                     initSections++;
                     out.println("    private static void init_" + initSections + "() {");
@@ -334,7 +324,7 @@ public class GenerateLineBreakUtils {
                 for (int k = 0; k < blocksize; k++) {
                     row[i][k] = lineBreakProperties[idx + k];
                     if (printLocalComma) {
-                        out.print(',');
+                        out.print(", ");
                     } else {
                         printLocalComma = true;
                     }
@@ -345,20 +335,21 @@ public class GenerateLineBreakUtils {
             }
             idx += blocksize;
         }
-        out.println("    };");
+        out.println("    }");
         out.println();
         out.println("    static {");
         for (int i = 0; i <= initSections; i++) {
             out.println("        init_" + i + "();");
         }
         out.print(doStaticLinkCode);
-        out.println("    };");
+        out.println("    }");
         out.println();
         for (int i = 0; i < lineBreakPropertyShortNames.size(); i++) {
             String shortName = (String)lineBreakPropertyShortNames.get(i);
+            out.println("    /** Linebreak property constant */");
             out.print("    public static final byte LINE_BREAK_PROPERTY_");
             out.print(shortName);
-            out.print('=');
+            out.print(" = ");
             out.print(i + 1);
             out.println(';');
         }
@@ -368,10 +359,15 @@ public class GenerateLineBreakUtils {
         int lineLength = shortNamePrefix.length();
         printComma = false;
         for (int i = 0; i < lineBreakPropertyShortNames.size(); i++) {
-            String name = (String)lineBreakPropertyShortNames.get(i);
+            name = (String)lineBreakPropertyShortNames.get(i);
             if (printComma) {
-                out.print(',');
-                lineLength++;
+                if (lineLength <= MAX_LINE_LENGTH - 2) {
+                    out.print(", ");
+                } else {
+                    out.print(",");
+                }
+                // count the space anyway to force a linebreak if the comma causes lineLength == MAX_LINE_LENGTH
+                lineLength += 2;
             } else {
                 printComma = true;
             }
@@ -392,7 +388,7 @@ public class GenerateLineBreakUtils {
         lineLength = longNamePrefix.length();
         printComma = false;
         for (int i = 0; i < lineBreakPropertyLongNames.size(); i++) {
-            String name = (String)lineBreakPropertyLongNames.get(i);
+            name = (String)lineBreakPropertyLongNames.get(i);
             if (printComma) {
                 out.print(',');
                 lineLength++;
@@ -411,32 +407,62 @@ public class GenerateLineBreakUtils {
         }
         out.println("};");
         out.println();
+        out.println("    /**");
+        out.println("     * Return the short name for the linebreak property corresponding");
+        out.println("     * to the given symbolic constant.");
+        out.println("     *");
+        out.println("     * @param i the numeric value of the linebreak property");
+        out.println("     * @return the short name of the linebreak property");
+        out.println("     */");
         out.println("    public static String getLineBreakPropertyShortName(byte i) {");
-        out.println("        if (i>0 && i<=lineBreakPropertyShortNames.length) {");
-        out.println("            return lineBreakPropertyShortNames[i-1];");
+        out.println("        if (i > 0 && i <= lineBreakPropertyShortNames.length) {");
+        out.println("            return lineBreakPropertyShortNames[i - 1];");
         out.println("        } else {");
         out.println("            return null;");
         out.println("        }");
         out.println("    }");
         out.println();
+        out.println("    /**");
+        out.println("     * Return the long name for the linebreak property corresponding");
+        out.println("     * to the given symbolic constant.");
+        out.println("     *");
+        out.println("     * @param i the numeric value of the linebreak property");
+        out.println("     * @return the long name of the linebreak property");
+        out.println("     */");
         out.println("    public static String getLineBreakPropertyLongName(byte i) {");
-        out.println("        if (i>0 && i<=lineBreakPropertyLongNames.length) {");
-        out.println("            return lineBreakPropertyLongNames[i-1];");
+        out.println("        if (i > 0 && i <= lineBreakPropertyLongNames.length) {");
+        out.println("            return lineBreakPropertyLongNames[i - 1];");
         out.println("        } else {");
         out.println("            return null;");
         out.println("        }");
         out.println("    }");
         out.println();
+        out.println("    /**");
+        out.println("     * Return the linebreak property constant for the given <code>char</code>");
+        out.println("     *");
+        out.println("     * @param c the <code>char</code> whose linebreak property to return");
+        out.println("     * @return the constant representing the linebreak property");
+        out.println("     */");
         out.println("    public static byte getLineBreakProperty(char c) {");
-        out.println("        return lineBreakProperties[c/" + blocksize + "][c%" + blocksize + "];");
+        out.println("        return lineBreakProperties[c / " + blocksize + "][c % " + blocksize + "];");
         out.println("    }");
         out.println();
+        out.println("    /**");
+        out.println("     * Return the break class constant for the given pair of linebreak");
+        out.println("     * property constants.");
+        out.println("     *");
+        out.println("     * @param lineBreakPropertyBefore the linebreak property for the first character");
+        out.println("     *        in a two-character sequence");
+        out.println("     * @param lineBreakPropertyAfter the linebreak property for the second character");
+        out.println("     *        in a two-character sequence");
+        out.println("     * @return the constant representing the break class");
+        out.println("     */");
         out.println(
-            "    public static byte getLineBreakPairProperty(int lineBreakPropertyBefore,int lineBreakPropertyAfter) {");
-        out.println("        return PAIR_TABLE[lineBreakPropertyBefore-1][lineBreakPropertyAfter-1];");
+            "    public static byte getLineBreakPairProperty(int lineBreakPropertyBefore, int lineBreakPropertyAfter) {");
+        out.println("        return PAIR_TABLE[lineBreakPropertyBefore - 1][lineBreakPropertyAfter - 1];");
         out.println("    }");
         out.println();
-        out.println("};");
+        out.println("}");
         out.flush();
         out.close();
     }
@@ -445,7 +471,7 @@ public class GenerateLineBreakUtils {
      * Read line break property value names and the actual properties for the Unicode
      * characters from the respective Unicode files.
      * TODO: Code points above the base plane are simply ignored.
-     * 
+     *
      * @param lineBreakFileName Name of line break property file.
      * @param propertyValueFileName Name of property values alias file.
      * @throws Exception in case anything goes wrong.
@@ -564,7 +590,7 @@ public class GenerateLineBreakUtils {
      * Determine a good block size for the two stage optimized storage of the
      * line breaking properties. Note: the memory utilization calculation is a rule of thumb,
      * don't take it too serious.
-     * 
+     *
      * @param lineBreakFileName Name of line break property file.
      * @param propertyValueFileName Name of property values alias file.
      * @throws Exception in case anything goes wrong.
