@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FOURIResolver.java 611278 2008-01-11 19:50:53Z jeremias $ */
+/* $Id: FOURIResolver.java 772672 2009-05-07 15:08:30Z cbowditch $ */
 
 package org.apache.fop.apps;
 
@@ -24,6 +24,8 @@ import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.net.MalformedURLException;
+import java.net.URI;
+import java.net.URISyntaxException;
 import java.net.URL;
 import java.net.URLConnection;
 
@@ -32,17 +34,17 @@ import javax.xml.transform.TransformerException;
 import javax.xml.transform.URIResolver;
 import javax.xml.transform.stream.StreamSource;
 
+import org.apache.commons.io.FileUtils;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
 import org.apache.xmlgraphics.util.io.Base64EncodeStream;
-
-import org.apache.fop.util.DataURIResolver;
+import org.apache.xmlgraphics.util.uri.CommonURIResolver;
 
 /**
  * Provides FOP specific URI resolution. This is the default URIResolver
- * {@link FOUserAgent} will use unless overidden.
- * 
+ * {@link FOUserAgent} will use unless overridden.
+ *
  * @see javax.xml.transform.URIResolver
  */
 public class FOURIResolver implements javax.xml.transform.URIResolver {
@@ -50,15 +52,61 @@ public class FOURIResolver implements javax.xml.transform.URIResolver {
     // log
     private Log log = LogFactory.getLog("FOP");
 
-    /** URIResolver for RFC 2397 data URLs */
-    private URIResolver dataURIResolver = new DataURIResolver();
-    
+    /** Common URIResolver */
+    private CommonURIResolver commonURIResolver = new CommonURIResolver();
+
     /** A user settable URI Resolver */
     private URIResolver uriResolver = null;
 
     /** true if exceptions are to be thrown if the URIs cannot be resolved. */
     private boolean throwExceptions = false;
 
+    /**
+     * Checks if the given base URL is acceptable. It also normalizes the URL.
+     * @param base the base URL to check
+     * @return the normalized URL
+     * @throws MalformedURLException if there's a problem with a file URL
+     */
+    public String checkBaseURL(String base) throws MalformedURLException {
+        // replace back slash with forward slash to ensure windows file:/// URLS are supported
+        base = base.replace('\\', '/');
+        if (!base.endsWith("/")) {
+            // The behavior described by RFC 3986 regarding resolution of relative
+            // references may be misleading for normal users:
+            // file://path/to/resources + myResource.res -> file://path/to/myResource.res
+            // file://path/to/resources/ + myResource.res -> file://path/to/resources/myResource.res
+            // We assume that even when the ending slash is missing, users have the second
+            // example in mind
+            base += "/";
+        }
+        File dir = new File(base);
+        if (dir.isDirectory()) {
+            return dir.toURI().toASCIIString();
+        } else {
+            URI baseURI;
+            try {
+                baseURI = new URI(base);
+                String scheme = baseURI.getScheme();
+                boolean directoryExists = true;
+                if ("file".equals(scheme)) {
+                    dir = FileUtils.toFile(baseURI.toURL());
+                    directoryExists = dir.isDirectory();
+                }
+                if (scheme == null || !directoryExists) {
+                    String message = "base " + base + " is not a valid directory";
+                    if (throwExceptions) {
+                        throw new MalformedURLException(message);
+                    }
+                    log.error(message);
+                }
+                return baseURI.toASCIIString();
+            } catch (URISyntaxException e) {
+                //TODO not ideal: our base URLs are actually base URIs.
+                throw new MalformedURLException(e.getMessage());
+            }
+        }
+    }
+
     /**
      * Default constructor
      */
@@ -68,7 +116,7 @@ public class FOURIResolver implements javax.xml.transform.URIResolver {
 
     /**
      * Additional constructor
-     * 
+     *
      * @param throwExceptions
      *            true if exceptions are to be thrown if the URIs cannot be
      *            resolved.
@@ -79,7 +127,9 @@ public class FOURIResolver implements javax.xml.transform.URIResolver {
 
     /**
      * Handles resolve exceptions appropriately.
-     * 
+     *
+     * @param e
+     *            the exception
      * @param errorStr
      *            error string
      * @param strict
@@ -105,7 +155,7 @@ public class FOURIResolver implements javax.xml.transform.URIResolver {
      * If the method is successful it will return a Source of type
      * {@link javax.xml.transform.stream.StreamSource} with its SystemID set to
      * the resolved URL used to open the underlying InputStream.
-     * 
+     *
      * @param href
      *            An href attribute, which may be relative or absolute.
      * @param base
@@ -119,11 +169,11 @@ public class FOURIResolver implements javax.xml.transform.URIResolver {
      */
     public Source resolve(String href, String base) throws TransformerException {
         Source source = null;
-        
+
         // data URLs can be quite long so evaluate early and don't try to build a File
         // (can lead to problems)
-        source = dataURIResolver.resolve(href, base);
-        
+        source = commonURIResolver.resolve(href, base);
+
         // Custom uri resolution
         if (source == null && uriResolver != null) {
             source = uriResolver.resolve(href, base);
@@ -145,9 +195,9 @@ public class FOURIResolver implements javax.xml.transform.URIResolver {
             if (file.canRead() && file.isFile()) {
                 try {
                     if (fragment != null) {
-                        absoluteURL = new URL(file.toURL().toExternalForm() + fragment);
+                        absoluteURL = new URL(file.toURI().toURL().toExternalForm() + fragment);
                     } else {
-                        absoluteURL = file.toURL();
+                        absoluteURL = file.toURI().toURL();
                     }
                 } catch (MalformedURLException mfue) {
                     handleException(mfue, "Could not convert filename '" + href
@@ -184,13 +234,13 @@ public class FOURIResolver implements javax.xml.transform.URIResolver {
                     /*
                      * This piece of code is based on the following statement in
                      * RFC2396 section 5.2:
-                     * 
+                     *
                      * 3) If the scheme component is defined, indicating that
                      * the reference starts with a scheme name, then the
                      * reference is interpreted as an absolute URI and we are
                      * done. Otherwise, the reference URI's scheme is inherited
                      * from the base URI's scheme component.
-                     * 
+                     *
                      * Due to a loophole in prior specifications [RFC1630], some
                      * parsers allow the scheme name to be present in a relative
                      * URI if it is the same as the base URI scheme.
@@ -200,21 +250,20 @@ public class FOURIResolver implements javax.xml.transform.URIResolver {
                      * removing the scheme if it matches that of the base URI
                      * and the scheme is known to always use the <hier_part>
                      * syntax.
-                     * 
+                     *
                      * The URL class does not implement this work around, so we
                      * do.
                      */
+                    assert (baseURL != null);
                     String scheme = baseURL.getProtocol() + ":";
-                    if (href.startsWith(scheme)) {
+                    if (href.startsWith(scheme) && "file:".equals(scheme)) {
                         href = href.substring(scheme.length());
-                        if ("file:".equals(scheme)) {
-                            int colonPos = href.indexOf(':');
-                            int slashPos = href.indexOf('/');
-                            if (slashPos >= 0 && colonPos >= 0
-                                    && colonPos < slashPos) {
-                                href = "/" + href; // Absolute file URL doesn't
-                                // have a leading slash
-                            }
+                        int colonPos = href.indexOf(':');
+                        int slashPos = href.indexOf('/');
+                        if (slashPos >= 0 && colonPos >= 0
+                                && colonPos < slashPos) {
+                            href = "/" + href; // Absolute file URL doesn't
+                            // have a leading slash
                         }
                     }
                     try {
@@ -253,7 +302,7 @@ public class FOURIResolver implements javax.xml.transform.URIResolver {
      * before the connect() method is called. Subclass FOURIResolver and
      * override this method to do things like adding the user name and password
      * for HTTP basic authentication.
-     * 
+     *
      * @param connection
      *            the URLConnection instance
      * @param href
@@ -267,7 +316,7 @@ public class FOURIResolver implements javax.xml.transform.URIResolver {
      * This is a convenience method for users who want to override
      * updateURLConnection for HTTP basic authentication. Simply call it using
      * the right username and password.
-     * 
+     *
      * @param connection
      *            the URLConnection to set up for HTTP basic authentication
      * @param username
@@ -299,7 +348,7 @@ public class FOURIResolver implements javax.xml.transform.URIResolver {
      * Sets the custom URI Resolver. It is used for resolving factory-level URIs like
      * hyphenation patterns and as backup for URI resolution performed during a
      * rendering run.
-     * 
+     *
      * @param resolver
      *            the new URI resolver
      */
@@ -309,7 +358,7 @@ public class FOURIResolver implements javax.xml.transform.URIResolver {
 
     /**
      * Returns the custom URI Resolver.
-     * 
+     *
      * @return the URI Resolver or null, if none is set
      */
     public URIResolver getCustomURIResolver() {
