@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,12 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: FOUserAgent.java 628826 2008-02-18 17:24:23Z jeremias $ */
+/* $Id: FOUserAgent.java 830293 2009-10-27 19:07:52Z vhennebert $ */
 
 package org.apache.fop.apps;
 
 // Java
 import java.io.File;
+import java.net.MalformedURLException;
 import java.util.Date;
 import java.util.Map;
 
@@ -36,10 +37,20 @@ import org.apache.xmlgraphics.image.loader.ImageSessionContext;
 import org.apache.xmlgraphics.image.loader.impl.AbstractImageSessionContext;
 
 import org.apache.fop.Version;
+import org.apache.fop.accessibility.Accessibility;
+import org.apache.fop.accessibility.StructureTree;
+import org.apache.fop.events.DefaultEventBroadcaster;
+import org.apache.fop.events.Event;
+import org.apache.fop.events.EventBroadcaster;
+import org.apache.fop.events.EventListener;
+import org.apache.fop.events.FOPEventListenerProxy;
+import org.apache.fop.events.LoggingEventListener;
 import org.apache.fop.fo.FOEventHandler;
+import org.apache.fop.fonts.FontManager;
 import org.apache.fop.render.Renderer;
 import org.apache.fop.render.RendererFactory;
 import org.apache.fop.render.XMLHandlerRegistry;
+import org.apache.fop.render.intermediate.IFDocumentHandler;
 
 /**
  * This is the user agent for FOP.
@@ -70,26 +81,28 @@ public class FOUserAgent {
     private static Log log = LogFactory.getLog("FOP");
 
     private FopFactory factory;
-    
+
     /**
      *  The base URL for all URL resolutions, especially for
      *  external-graphics.
      */
     private String base = null;
 
-    /** The base URL for all font URL resolutions. */
-    private String fontBase = null;
-
     /** A user settable URI Resolver */
     private URIResolver uriResolver = null;
-    
+
     private float targetResolution = FopFactoryConfigurator.DEFAULT_TARGET_RESOLUTION;
     private Map rendererOptions = new java.util.HashMap();
     private File outputFile = null;
+    private IFDocumentHandler documentHandlerOverride = null;
     private Renderer rendererOverride = null;
     private FOEventHandler foEventHandlerOverride = null;
     private boolean locatorEnabled = true; // true by default (for error messages).
-    
+    private boolean conserveMemoryPolicy = false;
+    private EventBroadcaster eventBroadcaster = new FOPEventBroadcaster();
+
+    private StructureTree structureTree;
+
     /** Producer:  Metadata element for the system/software that produces
      * the document. (Some renderers can store this in the document.)
      */
@@ -100,18 +113,20 @@ public class FOUserAgent {
      */
     protected String creator = null;
 
-    /** Creation Date:  Override of the date the document was created. 
+    /** Creation Date:  Override of the date the document was created.
      * (Some renderers can store this in the document.)
      */
     protected Date creationDate = null;
-    
+
     /** Author of the content of the document. */
     protected String author = null;
     /** Title of the document. */
     protected String title = null;
+    /** Subject of the document. */
+    protected String subject = null;
     /** Set of keywords applicable to this document. */
     protected String keywords = null;
-    
+
     private ImageSessionContext imageSessionContext = new AbstractImageSessionContext() {
 
         public ImageContext getParentContext() {
@@ -125,11 +140,11 @@ public class FOUserAgent {
         public Source resolveURI(String uri) {
             return FOUserAgent.this.resolveURI(uri);
         }
-        
+
     };
-    
+
     /**
-     * Main constructor. <b>This constructor should not be called directly. Please use the 
+     * Main constructor. <b>This constructor should not be called directly. Please use the
      * methods from FopFactory to construct FOUserAgent instances!</b>
      * @param factory the factory that provides environment-level information
      * @see org.apache.fop.apps.FopFactory
@@ -140,20 +155,38 @@ public class FOUserAgent {
         }
         this.factory = factory;
         setBaseURL(factory.getBaseURL());
-        setFontBaseURL(factory.getFontBaseURL());
         setTargetResolution(factory.getTargetResolution());
+        setAccessibility(factory.isAccessibilityEnabled());
     }
-    
+
     /** @return the associated FopFactory instance */
     public FopFactory getFactory() {
         return this.factory;
     }
-    
+
     // ---------------------------------------------- rendering-run dependent stuff
-    
+
     /**
-     * Sets an explicit renderer to use which overrides the one defined by the 
-     * render type setting.  
+     * Sets an explicit document handler to use which overrides the one that would be
+     * selected by default.
+     * @param documentHandler the document handler instance to use
+     */
+    public void setDocumentHandlerOverride(IFDocumentHandler documentHandler) {
+        this.documentHandlerOverride = documentHandler;
+
+    }
+
+    /**
+     * Returns the overriding {@link IFDocumentHandler} instance, if any.
+     * @return the overriding document handler or null
+     */
+    public IFDocumentHandler getDocumentHandlerOverride() {
+        return this.documentHandlerOverride;
+    }
+
+    /**
+     * Sets an explicit renderer to use which overrides the one defined by the
+     * render type setting.
      * @param renderer the Renderer instance to use
      */
     public void setRendererOverride(Renderer renderer) {
@@ -168,9 +201,10 @@ public class FOUserAgent {
         return rendererOverride;
     }
 
+
     /**
      * Sets an explicit FOEventHandler instance which overrides the one
-     * defined by the render type setting.  
+     * defined by the render type setting.
      * @param handler the FOEventHandler instance
      */
     public void setFOEventHandlerOverride(FOEventHandler handler) {
@@ -186,7 +220,7 @@ public class FOUserAgent {
     }
 
     /**
-     * Sets the producer of the document.  
+     * Sets the producer of the document.
      * @param producer source of document
      */
     public void setProducer(String producer) {
@@ -202,7 +236,7 @@ public class FOUserAgent {
     }
 
     /**
-     * Sets the creator of the document.  
+     * Sets the creator of the document.
      * @param creator of document
      */
     public void setCreator(String creator) {
@@ -218,7 +252,7 @@ public class FOUserAgent {
     }
 
     /**
-     * Sets the creation date of the document.  
+     * Sets the creation date of the document.
      * @param creationDate date of document
      */
     public void setCreationDate(Date creationDate) {
@@ -234,7 +268,7 @@ public class FOUserAgent {
     }
 
     /**
-     * Sets the author of the document.  
+     * Sets the author of the document.
      * @param author of document
      */
     public void setAuthor(String author) {
@@ -251,7 +285,7 @@ public class FOUserAgent {
 
     /**
      * Sets the title of the document. This will override any title coming from
-     * an fo:title element.  
+     * an fo:title element.
      * @param title of document
      */
     public void setTitle(String title) {
@@ -267,7 +301,23 @@ public class FOUserAgent {
     }
 
     /**
-     * Sets the keywords for the document.  
+     * Sets the subject of the document.
+     * @param subject of document
+     */
+    public void setSubject(String subject) {
+        this.subject = subject;
+    }
+
+    /**
+     * Returns the subject of the document
+     * @return the subject
+     */
+    public String getSubject() {
+        return subject;
+    }
+
+    /**
+     * Sets the keywords for the document.
      * @param keywords for the document
      */
     public void setKeywords(String keywords) {
@@ -289,7 +339,7 @@ public class FOUserAgent {
     public Map getRendererOptions() {
         return rendererOptions;
     }
-    
+
     /**
      * Sets the base URL.
      * @param baseUrl base URL
@@ -299,11 +349,16 @@ public class FOUserAgent {
     }
 
     /**
-     * sets font base URL
+     * Sets font base URL.
      * @param fontBaseUrl font base URL
+     * @deprecated Use {@link FontManager#setFontBaseURL(String)} instead.
      */
     public void setFontBaseURL(String fontBaseUrl) {
-        this.fontBase = fontBaseUrl;
+        try {
+            getFactory().getFontManager().setFontBaseURL(fontBaseUrl);
+        } catch (MalformedURLException e) {
+            throw new IllegalArgumentException(e.getMessage());
+        }
     }
 
     /**
@@ -336,7 +391,7 @@ public class FOUserAgent {
      * to the default resolver.
      * @param uri URI to access
      * @return A {@link javax.xml.transform.Source} object, or null if the URI
-     * cannot be resolved. 
+     * cannot be resolved.
      * @see org.apache.fop.apps.FOURIResolver
      */
     public Source resolveURI(String uri) {
@@ -350,7 +405,7 @@ public class FOUserAgent {
      * @param href URI to access
      * @param base the base URI to resolve against
      * @return A {@link javax.xml.transform.Source} object, or null if the URI
-     * cannot be resolved. 
+     * cannot be resolved.
      * @see org.apache.fop.apps.FOURIResolver
      */
     public Source resolveURI(String href, String base) {
@@ -395,9 +450,9 @@ public class FOUserAgent {
      * @see #getTargetResolution()
      */
     public float getTargetPixelUnitToMillimeter() {
-        return 25.4f / this.targetResolution; 
+        return 25.4f / this.targetResolution;
     }
-    
+
     /** @return the resolution for resolution-dependant output */
     public float getTargetResolution() {
         return this.targetResolution;
@@ -412,7 +467,7 @@ public class FOUserAgent {
     public void setTargetResolution(float dpi) {
         this.targetResolution = dpi;
         if (log.isDebugEnabled()) {
-            log.debug("target-resolution set to: " + targetResolution 
+            log.debug("target-resolution set to: " + targetResolution
                     + "dpi (px2mm=" + getTargetPixelUnitToMillimeter() + ")");
         }
     }
@@ -426,7 +481,7 @@ public class FOUserAgent {
     public void setTargetResolution(int dpi) {
         setTargetResolution((float)dpi);
     }
-    
+
     /**
      * Returns the image session context for the image package.
      * @return the ImageSessionContext instance for this rendering run
@@ -434,12 +489,17 @@ public class FOUserAgent {
     public ImageSessionContext getImageSessionContext() {
         return this.imageSessionContext;
     }
-    
+
     // ---------------------------------------------- environment-level stuff
     //                                                (convenience access to FopFactory methods)
 
-    /** @return the font base URL */
+    /**
+     * Returns the font base URL.
+     * @return the font base URL
+     * @deprecated Use {@link FontManager#getFontBaseURL()} instead. This method is not used by FOP.
+     */
     public String getFontBaseURL() {
+        String fontBase = getFactory().getFontManager().getFontBaseURL();
         return fontBase != null ? fontBase : getBaseURL();
     }
 
@@ -450,9 +510,9 @@ public class FOUserAgent {
      * @see #getSourceResolution()
      */
     public float getSourcePixelUnitToMillimeter() {
-        return getFactory().getSourcePixelUnitToMillimeter(); 
+        return getFactory().getSourcePixelUnitToMillimeter();
     }
-    
+
     /** @return the resolution for resolution-dependant input */
     public float getSourceResolution() {
         return getFactory().getSourceResolution();
@@ -461,25 +521,25 @@ public class FOUserAgent {
     /**
      * Gets the default page-height to use as fallback,
      * in case page-height="auto"
-     * 
+     *
      * @return the page-height, as a String
      * @see FopFactory#getPageHeight()
      */
     public String getPageHeight() {
         return getFactory().getPageHeight();
     }
-    
+
     /**
      * Gets the default page-width to use as fallback,
      * in case page-width="auto"
-     * 
+     *
      * @return the page-width, as a String
      * @see FopFactory#getPageWidth()
      */
     public String getPageWidth() {
         return getFactory().getPageWidth();
     }
-    
+
     /**
      * Returns whether FOP is strictly validating input XSL
      * @return true of strict validation turned on, false otherwise
@@ -490,7 +550,7 @@ public class FOUserAgent {
     }
 
     /**
-     * @return true if the indent inheritance should be broken when crossing reference area 
+     * @return true if the indent inheritance should be broken when crossing reference area
      *         boundaries (for more info, see the javadoc for the relative member variable)
      * @see FopFactory#isBreakIndentInheritanceOnReferenceAreaBoundary()
      */
@@ -511,11 +571,11 @@ public class FOUserAgent {
     public XMLHandlerRegistry getXMLHandlerRegistry() {
         return getFactory().getXMLHandlerRegistry();
     }
-    
+
     /**
      * Controls the use of SAXLocators to provide location information in error
      * messages.
-     * 
+     *
      * @param enableLocator <code>false</code> if SAX Locators should be disabled
      */
     public void setLocatorEnabled(boolean enableLocator) {
@@ -530,5 +590,107 @@ public class FOUserAgent {
         return locatorEnabled;
     }
 
+    /**
+     * Returns the event broadcaster that control events sent inside a processing run. Clients
+     * can register event listeners with the event broadcaster to listen for events that occur
+     * while a document is being processed.
+     * @return the event broadcaster.
+     */
+    public EventBroadcaster getEventBroadcaster() {
+        return this.eventBroadcaster;
+    }
+
+    private class FOPEventBroadcaster extends DefaultEventBroadcaster {
+
+        private EventListener rootListener;
+
+        public FOPEventBroadcaster() {
+            //Install a temporary event listener that catches the first event to
+            //do some initialization.
+            this.rootListener = new EventListener() {
+                public void processEvent(Event event) {
+                    if (!listeners.hasEventListeners()) {
+                        //Backwards-compatibility: Make sure at least the LoggingEventListener is
+                        //plugged in so no events are just silently swallowed.
+                        addEventListener(
+                                new LoggingEventListener(LogFactory.getLog(FOUserAgent.class)));
+                    }
+                    //Replace with final event listener
+                    rootListener = new FOPEventListenerProxy(
+                            listeners, FOUserAgent.this);
+                    rootListener.processEvent(event);
+                }
+            };
+        }
+
+        /** {@inheritDoc} */
+        public void broadcastEvent(Event event) {
+            rootListener.processEvent(event);
+        }
+
+    }
+
+    /**
+     * Check whether memory-conservation is enabled.
+     *
+     * @return true if FOP is to conserve as much as possible
+     */
+    public boolean isConserveMemoryPolicyEnabled() {
+        return this.conserveMemoryPolicy;
+    }
+
+    /**
+     * Control whether memory-conservation should be enabled
+     *
+     * @param conserveMemoryPolicy the cachingEnabled to set
+     */
+    public void setConserveMemoryPolicy(boolean conserveMemoryPolicy) {
+        this.conserveMemoryPolicy = conserveMemoryPolicy;
+    }
+
+    /**
+     * Activates accessibility (for output formats that support it).
+     *
+     * @param accessibility <code>true</code> to enable accessibility support
+     */
+    public void setAccessibility(boolean accessibility) {
+        if (accessibility) {
+            getRendererOptions().put(Accessibility.ACCESSIBILITY, Boolean.TRUE);
+        }
+    }
+
+    /**
+     * Check if accessibility is enabled.
+     * @return true if accessibility is enabled
+     */
+    public boolean isAccessibilityEnabled() {
+        Boolean enabled = (Boolean)this.getRendererOptions().get(Accessibility.ACCESSIBILITY);
+        if (enabled != null) {
+            return enabled.booleanValue();
+        } else {
+            return false;
+        }
+    }
+
+    /**
+     * Sets the document's structure tree, for use by accessible output formats.
+     *
+     * @param structureTree a simplified version of the FO tree, retaining only
+     * its logical structure
+     */
+    public void setStructureTree(StructureTree structureTree) {
+        this.structureTree = structureTree;
+    }
+
+    /**
+     * Returns the document's structure tree, for use by accessible output
+     * formats.
+     *
+     * @return a simplified version of the FO tree, retaining only its logical
+     * structure
+     */
+    public StructureTree getStructureTree() {
+        return this.structureTree;
+    }
 }
 
