@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Area.java 557337 2007-07-18 17:37:14Z adelmelle $ */
+/* $Id: Area.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.area;
 
@@ -119,8 +119,10 @@ public class Area extends AreaTreeObject implements Serializable {
     public static final int CLASS_MAX = CLASS_SIDE_FLOAT + 1;
 
     private int areaClass = CLASS_NORMAL;
+
     /** the area's inline-progression-dimension */
     protected int ipd;
+
     /** the area's block-progression-dimension */
     protected int bpd;
 
@@ -141,7 +143,7 @@ public class Area extends AreaTreeObject implements Serializable {
      * @return the area class
      */
     public int getAreaClass() {
-        return areaClass;
+        return this.areaClass;
     }
 
     /**
@@ -157,19 +159,19 @@ public class Area extends AreaTreeObject implements Serializable {
      * Set the inline progression dimension of content rectangle
      * for this area.
      *
-     * @param i the new inline progression dimension
-     * @see <a href="http://www.w3.org/TR/xsl/slice4.html#area-common">ipd</a>
+     * @param ipd the new inline progression dimension
+     * @see <a href="http://www.w3.org/TR/xsl/#inline-progression-dimension">ipd</a>
      */
-    public void setIPD(int i) {
-        ipd = i;
+    public void setIPD(int ipd) {
+        this.ipd = ipd;
     }
 
     /**
-     * Get the inline progression dimension of the content rectangle 
+     * Get the inline progression dimension of the content rectangle
      * for this area.
-     * 
+     *
      * @return the inline progression dimension
-     * @see <a href="http://www.w3.org/TR/xsl/slice4.html#area-common">ipd</a>
+     * @see <a href="http://www.w3.org/TR/xsl/#inline-progression-dimension">ipd</a>
      */
     public int getIPD() {
         return ipd;
@@ -179,11 +181,11 @@ public class Area extends AreaTreeObject implements Serializable {
      * Set the block progression dimension of the content rectangle
      * for this area.
      *
-     * @param b the new block progression dimension
-     * @see <a href="http://www.w3.org/TR/xsl/slice4.html#area-common">bpd</a>
+     * @param bpd the new block progression dimension
+     * @see <a href="http://www.w3.org/TR/xsl/#block-progression-dimension">bpd</a>
      */
-    public void setBPD(int b) {
-        bpd = b;
+    public void setBPD(int bpd) {
+        this.bpd = bpd;
     }
 
     /**
@@ -191,7 +193,7 @@ public class Area extends AreaTreeObject implements Serializable {
      * for this area.
      *
      * @return the block progression dimension
-     * @see <a href="http://www.w3.org/TR/xsl/slice4.html#area-common">bpd</a>
+     * @see <a href="http://www.w3.org/TR/xsl/#block-progression-dimension">bpd</a>
      */
     public int getBPD() {
         return bpd;
@@ -216,7 +218,7 @@ public class Area extends AreaTreeObject implements Serializable {
      * @return the total BPD allocation for this area
      */
     public int getAllocBPD() {
-        return getSpaceBefore() + getBorderAndPaddingWidthBefore() + getBPD() 
+        return getSpaceBefore() + getBorderAndPaddingWidthBefore() + getBPD()
                 + getBorderAndPaddingWidthAfter() + getSpaceAfter();
     }
 
@@ -231,7 +233,7 @@ public class Area extends AreaTreeObject implements Serializable {
         if (bps != null) {
             margin = bps.width;
         }
-        
+
         Integer padWidth = (Integer) getTrait(Trait.PADDING_BEFORE);
         if (padWidth != null) {
             margin += padWidth.intValue();
@@ -239,7 +241,7 @@ public class Area extends AreaTreeObject implements Serializable {
 
         return margin;
     }
-    
+
     /**
      * Return the sum of region border- and padding-after
      *
@@ -247,12 +249,12 @@ public class Area extends AreaTreeObject implements Serializable {
      */
     public int getBorderAndPaddingWidthAfter() {
         int margin = 0;
-        
+
         BorderProps bps = (BorderProps) getTrait(Trait.BORDER_AFTER);
         if (bps != null) {
             margin = bps.width;
         }
-        
+
         Integer padWidth = (Integer) getTrait(Trait.PADDING_AFTER);
         if (padWidth != null) {
             margin += padWidth.intValue();
@@ -272,7 +274,7 @@ public class Area extends AreaTreeObject implements Serializable {
         if (bps != null) {
             margin = bps.width;
         }
-        
+
         Integer padWidth = (Integer) getTrait(Trait.PADDING_START);
         if (padWidth != null) {
             margin += padWidth.intValue();
@@ -292,7 +294,7 @@ public class Area extends AreaTreeObject implements Serializable {
         if (bps != null) {
             margin = bps.width;
         }
-        
+
         Integer padWidth = (Integer) getTrait(Trait.PADDING_END);
         if (padWidth != null) {
             margin += padWidth.intValue();
@@ -314,7 +316,7 @@ public class Area extends AreaTreeObject implements Serializable {
         }
         return margin;
     }
-    
+
     /**
      * Returns the space after
      *
@@ -328,7 +330,7 @@ public class Area extends AreaTreeObject implements Serializable {
         }
         return margin;
     }
-    
+
     /**
      * Returns the space start
      *
@@ -342,7 +344,7 @@ public class Area extends AreaTreeObject implements Serializable {
         }
         return margin;
     }
-    
+
     /**
      * Returns the space end
      *
@@ -356,7 +358,7 @@ public class Area extends AreaTreeObject implements Serializable {
         }
         return margin;
     }
-    
+
     /**
      * Add a child to this area.
      * The default is to do nothing. Subclasses must override
@@ -367,18 +369,6 @@ public class Area extends AreaTreeObject implements Serializable {
     public void addChildArea(Area child) {
     }
 
-    /**
-     * Add a trait property to this area.
-     *
-     * @param prop the Trait to add
-     */
-    public void addTrait(Trait prop) {
-        if (props == null) {
-            props = new java.util.HashMap(20);
-        }
-        props.put(prop.getPropType(), prop.getData());
-    }
-
     /**
      * Add a trait to this area.
      *
@@ -405,7 +395,7 @@ public class Area extends AreaTreeObject implements Serializable {
     public boolean hasTraits() {
         return (this.props != null);
     }
-    
+
     /**
      * Get a trait from this area.
      *
@@ -415,7 +405,7 @@ public class Area extends AreaTreeObject implements Serializable {
     public Object getTrait(Object oTraitCode) {
         return (props != null ? props.get(oTraitCode) : null);
     }
-    
+
     /**
      * Checks whether a certain trait is set on this area.
      * @param oTraitCode the trait key
@@ -424,19 +414,14 @@ public class Area extends AreaTreeObject implements Serializable {
     public boolean hasTrait(Object oTraitCode) {
         return (getTrait(oTraitCode) != null);
     }
-    
+
     /**
      * Get a boolean trait from this area.
      * @param oTraitCode the trait key
      * @return the trait value
      */
-    public boolean getBooleanTrait(Object oTraitCode) {
-        final Object obj = getTrait(oTraitCode);
-        if (obj instanceof Boolean) {
-            return ((Boolean)obj).booleanValue();
-        } else {
-            return false;
-        }
+    public boolean getTraitAsBoolean(Object oTraitCode) {
+        return Boolean.TRUE.equals(getTrait(oTraitCode));
     }
 
     /**
@@ -455,10 +440,10 @@ public class Area extends AreaTreeObject implements Serializable {
                     + " could not be converted to an integer");
         }
     }
-    
+
     /**
      * {@inheritDoc}
-     * @return ipd and bpd of area 
+     * @return ipd and bpd of area
      * */
     public String toString() {
         StringBuffer sb = new StringBuffer(super.toString());
