@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: AreaTreeModel.java 615906 2008-01-28 15:04:39Z jeremias $ */
- 
+/* $Id: AreaTreeModel.java 721430 2008-11-28 11:13:12Z acumiskey $ */
+
 package org.apache.fop.area;
 
 // Java
@@ -36,11 +36,11 @@ import org.apache.commons.logging.LogFactory;
  * the life of the area tree model.
  */
 public class AreaTreeModel {
-    private List pageSequenceList = null;
+    private List/*<PageSequence>*/ pageSequenceList = null;
     private int currentPageSequenceIndex = -1;
     /** the current page sequence */
     protected PageSequence currentPageSequence;
-    private List offDocumentItems = new java.util.ArrayList();
+//    private List offDocumentItems = new java.util.ArrayList();
     /** logger instance */
     protected static Log log = LogFactory.getLog(AreaTreeModel.class);
 
@@ -48,7 +48,7 @@ public class AreaTreeModel {
      * Create a new store pages model
      */
     public AreaTreeModel() {
-        pageSequenceList = new java.util.ArrayList();
+        pageSequenceList = new java.util.ArrayList/*<PageSequence>*/();
     }
 
     /**
@@ -80,16 +80,16 @@ public class AreaTreeModel {
     }
 
     /**
-     * Handle an OffDocumentItem 
+     * Handle an OffDocumentItem
      * @param ext the extension to handle
      */
-    public void handleOffDocumentItem(OffDocumentItem ext) {};
+    public void handleOffDocumentItem(OffDocumentItem ext) { };
 
     /**
      * Signal the end of the document for any processing.
      * @throws SAXException if a problem was encountered.
      */
-    public void endDocument() throws SAXException {};
+    public void endDocument() throws SAXException { };
 
     /**
      * Returns the currently active page-sequence.
@@ -98,7 +98,7 @@ public class AreaTreeModel {
     public PageSequence getCurrentPageSequence() {
         return this.currentPageSequence;
     }
-    
+
     /**
      * Get the page sequence count.
      * @return the number of page sequences in the document.
