@@ -15,15 +15,18 @@
  * limitations under the License.
  */
 
-/* $Id: AreaTreeObject.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: AreaTreeObject.java 735026 2009-01-16 14:29:39Z jeremias $ */
 
 package org.apache.fop.area;
 
 import java.util.Collections;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Map;
 
-import org.apache.fop.util.QName;
+import org.apache.xmlgraphics.util.QName;
+
+import org.apache.fop.fo.extensions.ExtensionAttachment;
 
 /**
  * Abstract base class for all area tree objects.
@@ -32,7 +35,10 @@ public abstract class AreaTreeObject {
 
     /** Foreign attributes */
     protected Map foreignAttributes = null;
-    
+
+    /** Extension attachments */
+    protected List/*<ExtensionAttachment>*/ extensionAttachments = null;
+
     /**
      * Sets a foreign attribute.
      * @param name the qualified name of the attribute
@@ -44,7 +50,7 @@ public abstract class AreaTreeObject {
         }
         this.foreignAttributes.put(name, value);
     }
-    
+
     /**
      * Set foreign attributes from a Map.
      * @param atts a Map with attributes (keys: QName, values: String)
@@ -53,15 +59,15 @@ public abstract class AreaTreeObject {
         if (atts.size() == 0) {
             return;
         }
-        Iterator iter = atts.keySet().iterator();
+        Iterator iter = atts.entrySet().iterator();
         while (iter.hasNext()) {
-            QName qName = (QName)iter.next();
-            String value = (String)atts.get(qName);
-            //The casting is only to ensure type safety (too bad we can't use generics, yet) 
-            setForeignAttribute(qName, value);
+            Map.Entry entry = (Map.Entry)iter.next();
+            String value = (String)entry.getValue();
+            //The casting is only to ensure type safety (too bad we can't use generics, yet)
+            setForeignAttribute((QName)entry.getKey(), value);
         }
     }
-    
+
     /**
      * Returns the value of a foreign attribute on the area.
      * @param name the qualified name of the attribute
@@ -74,7 +80,7 @@ public abstract class AreaTreeObject {
             return null;
         }
     }
-    
+
     /** @return the foreign attributes associated with this area */
     public Map getForeignAttributes() {
         if (this.foreignAttributes != null) {
@@ -83,6 +89,46 @@ public abstract class AreaTreeObject {
             return Collections.EMPTY_MAP;
         }
     }
-    
-    
+
+    private void prepareExtensionAttachmentContainer() {
+        if (this.extensionAttachments == null) {
+            this.extensionAttachments = new java.util.ArrayList/*<ExtensionAttachment>*/();
+        }
+    }
+
+    /**
+     * Adds a new ExtensionAttachment instance to this page.
+     * @param attachment the ExtensionAttachment
+     */
+    public void addExtensionAttachment(ExtensionAttachment attachment) {
+        prepareExtensionAttachmentContainer();
+        extensionAttachments.add(attachment);
+    }
+
+    /**
+     * Set extension attachments from a List
+     * @param extensionAttachments a List with extension attachments
+     */
+    public void setExtensionAttachments(List extensionAttachments) {
+        prepareExtensionAttachmentContainer();
+        this.extensionAttachments.addAll(extensionAttachments);
+    }
+
+    /** @return the extension attachments associated with this area */
+    public List getExtensionAttachments() {
+        if (this.extensionAttachments != null) {
+            return Collections.unmodifiableList(this.extensionAttachments);
+        } else {
+            return Collections.EMPTY_LIST;
+        }
+    }
+
+    /**
+     * Indicates whether this area tree object has any extension attachments.
+     * @return true if there are extension attachments
+     */
+    public boolean hasExtensionAttachments() {
+        return this.extensionAttachments != null && !this.extensionAttachments.isEmpty();
+    }
+
 }
