@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: Block.java 618992 2008-02-06 14:07:03Z jeremias $ */
- 
+/* $Id: Block.java 707631 2008-10-24 13:42:59Z jeremias $ */
+
 package org.apache.fop.area;
 
 
@@ -59,7 +59,7 @@ public class Block extends BlockParent {
     private int positioning = STACK;
 
     protected transient boolean allowBPDUpdate = true;
-    
+
     // a block with may contain the dominant styling info in
     // terms of most lines or blocks with info
 
@@ -79,7 +79,7 @@ public class Block extends BlockParent {
      * @param autoHeight increase the height of the block.
      */
     public void addBlock(Block block, boolean autoHeight) {
-        if (autoHeight && allowBPDUpdate) {
+        if (autoHeight && allowBPDUpdate && block.isStacked()) {
             bpd += block.getAllocBPD();
         }
         addChildArea(block);
@@ -120,7 +120,7 @@ public class Block extends BlockParent {
     public boolean isStacked() {
         return (getPositioning() == Block.STACK || getPositioning() == Block.RELATIVE);
     }
-    
+
     /**
      * @return the start-indent trait
      */
