@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: BlockParent.java 557337 2007-07-18 17:37:14Z adelmelle $ */
- 
+/* $Id: BlockParent.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+
 package org.apache.fop.area;
 
 import java.util.ArrayList;
@@ -51,7 +51,7 @@ public class BlockParent extends Area {
     protected List children = null;
 
     // orientation if reference area
-    private int orientation = ORIENT_0;
+//    private int orientation = ORIENT_0;
 
     /** {@inheritDoc} */
     public void addChildArea(Area childArea) {
@@ -60,7 +60,7 @@ public class BlockParent extends Area {
         }
         children.add(childArea);
     }
-    
+
     /**
      * Add the block area to this block parent.
      *
