@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,27 +15,25 @@
  * limitations under the License.
  */
 
-/* $Id: PageViewport.java 581906 2007-10-04 13:51:25Z cbowditch $ */
+/* $Id: PageViewport.java 798511 2009-07-28 12:53:58Z jeremias $ */
 
 package org.apache.fop.area;
 
 import java.awt.Rectangle;
-import java.awt.geom.Rectangle2D;
-import java.io.ObjectOutputStream;
+import java.io.IOException;
 import java.io.ObjectInputStream;
+import java.io.ObjectOutputStream;
 import java.util.ArrayList;
-import java.util.Collections;
-import java.util.List;
-import java.util.Map;
 import java.util.HashMap;
 import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
 import java.util.Set;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
 import org.apache.fop.fo.Constants;
-import org.apache.fop.fo.extensions.ExtensionAttachment;
 import org.apache.fop.fo.pagination.SimplePageMaster;
 
 /**
@@ -49,20 +47,20 @@ import org.apache.fop.fo.pagination.SimplePageMaster;
 public class PageViewport extends AreaTreeObject implements Resolvable, Cloneable {
 
     private Page page;
-    private Rectangle2D viewArea;
+    private Rectangle viewArea;
     private String simplePageMasterName;
-    
+
     /**
-     * Unique key to identify the page. pageNumberString and pageIndex are both no option 
+     * Unique key to identify the page. pageNumberString and pageIndex are both no option
      * for this.
      */
     private String pageKey;
-    
+
     private int pageNumber = -1;
     private String pageNumberString = null;
     private int pageIndex = -1; //-1 = undetermined
     private boolean blank;
-    
+
     private transient PageSequence pageSequence;
 
     // list of id references and the rectangle on the page
@@ -85,10 +83,6 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
     private Map markerFirstAny = null;
     private Map markerLastEnd = null;
     private Map markerLastAny = null;
-    
-    //Arbitrary attachments to the page from extensions that need to pass information
-    //down to the renderers.
-    private List extensionAttachments = null;
 
     /**
      * logging instance
@@ -104,7 +98,8 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
      */
     public PageViewport(SimplePageMaster spm, int pageNumber, String pageStr, boolean blank) {
         this.simplePageMasterName = spm.getMasterName();
-        this.extensionAttachments = spm.getExtensionAttachments();
+        setExtensionAttachments(spm.getExtensionAttachments());
+        setForeignAttributes(spm.getForeignAttributes());
         this.blank = blank;
         int pageWidth = spm.getPageWidth().getValue();
         int pageHeight = spm.getPageHeight().getValue();
@@ -116,17 +111,21 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
     }
 
     /**
-     * Copy constructor. 
+     * Copy constructor.
      * @param original the original PageViewport to copy from
      */
     public PageViewport(PageViewport original) {
         if (original.extensionAttachments != null) {
-            this.extensionAttachments = new java.util.ArrayList(original.extensionAttachments);
+            setExtensionAttachments(original.extensionAttachments);
         }
+        if (original.foreignAttributes != null) {
+            setForeignAttributes(original.foreignAttributes);
+        }
+        this.pageIndex = original.pageIndex;
         this.pageNumber = original.pageNumber;
         this.pageNumberString = original.pageNumberString;
         this.page = (Page)original.page.clone();
-        this.viewArea = (Rectangle2D)original.viewArea.clone();
+        this.viewArea = new Rectangle(original.viewArea);
         this.simplePageMasterName = original.simplePageMasterName;
         this.blank = original.blank;
     }
@@ -139,7 +138,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
      * @param simplePageMasterName name of the original simple-page-master that generated this page
      * @param blank true if this is a blank page
      */
-    public PageViewport(Rectangle2D viewArea, int pageNumber, String pageStr, 
+    public PageViewport(Rectangle viewArea, int pageNumber, String pageStr,
             String simplePageMasterName, boolean blank) {
         this.viewArea = viewArea;
         this.pageNumber = pageNumber;
@@ -147,7 +146,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
         this.simplePageMasterName = simplePageMasterName;
         this.blank = blank;
     }
-    
+
     /**
      * Sets the page sequence this page belongs to
      * @param seq the page sequence
@@ -155,17 +154,17 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
     public void setPageSequence(PageSequence seq) {
         this.pageSequence = seq;
     }
-    
+
     /** @return the page sequence this page belongs to */
     public PageSequence getPageSequence() {
         return this.pageSequence;
     }
-    
+
     /**
      * Get the view area rectangle of this viewport.
      * @return the rectangle for this viewport
      */
-    public Rectangle2D getViewArea() {
+    public Rectangle getViewArea() {
         return viewArea;
     }
 
@@ -176,7 +175,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
     public Page getPage() {
         return page;
     }
-    
+
     /**
      * Sets the page object for this PageViewport.
      * @param page the page
@@ -209,15 +208,15 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
     public void setPageIndex(int index) {
         this.pageIndex = index;
     }
-    
+
     /**
-     * @return the overall page index of the page in this rendering run (zero-based, 
+     * @return the overall page index of the page in this rendering run (zero-based,
      *         -1 if it is undetermined).
      */
     public int getPageIndex() {
         return this.pageIndex;
     }
-    
+
     /**
      * Sets the unique key for this PageViewport that will be used to reference this page.
      * @param key the unique key.
@@ -225,7 +224,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
     public void setKey(String key) {
         this.pageKey = key;
     }
-    
+
     /**
      * Get the key for this page viewport.
      * This is used so that a serializable key can be used to
@@ -290,7 +289,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
      * @return true if the page is resolved and can be rendered
      */
     public boolean isResolved() {
-        return unresolvedIDRefs == null 
+        return unresolvedIDRefs == null
             || unresolvedIDRefs.size() == 0;
     }
 
@@ -304,7 +303,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
     }
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
     public void resolveIDRef(String id, List pages) {
         if (page == null) {
@@ -343,7 +342,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
      * are starting, replacing earlier markers.
      * For "last-ending-within-page" it adds all markers that
      * are ending, replacing earlier markers.
-     * 
+     *
      * Should this logic be placed in the Page layout manager.
      *
      * @param marks the map of markers to add
@@ -358,12 +357,12 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
             return;
         }
         if (log.isDebugEnabled()) {
-            log.debug("--" + marks.keySet() + ": " 
-                    + (starting ? "starting" : "ending") 
-                    + (isfirst ? ", first" : "") 
+            log.debug("--" + marks.keySet() + ": "
+                    + (starting ? "starting" : "ending")
+                    + (isfirst ? ", first" : "")
                     + (islast ? ", last" : ""));
         }
-        
+
         // at the start of the area, register is-first and any areas
         if (starting) {
             if (isfirst) {
@@ -379,14 +378,14 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
                     if (!markerFirstStart.containsKey(key)) {
                         markerFirstStart.put(key, marks.get(key));
                         if (log.isTraceEnabled()) {
-                            log.trace("page " + pageNumberString + ": " 
+                            log.trace("page " + pageNumberString + ": "
                                     + "Adding marker " + key + " to FirstStart");
                         }
                     }
                     if (!markerFirstAny.containsKey(key)) {
                         markerFirstAny.put(key, marks.get(key));
                         if (log.isTraceEnabled()) {
-                            log.trace("page " + pageNumberString + ": " 
+                            log.trace("page " + pageNumberString + ": "
                                     + "Adding marker " + key + " to FirstAny");
                         }
                     }
@@ -397,7 +396,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
                 // last on page: replace all
                 markerLastStart.putAll(marks);
                 if (log.isTraceEnabled()) {
-                    log.trace("page " + pageNumberString + ": " 
+                    log.trace("page " + pageNumberString + ": "
                             + "Adding all markers to LastStart");
                 }
             } else {
@@ -410,7 +409,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
                     if (!markerFirstAny.containsKey(key)) {
                         markerFirstAny.put(key, marks.get(key));
                         if (log.isTraceEnabled()) {
-                            log.trace("page " + pageNumberString + ": " 
+                            log.trace("page " + pageNumberString + ": "
                                     + "Adding marker " + key + " to FirstAny");
                         }
                     }
@@ -425,7 +424,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
                 // last on page: replace all
                 markerLastEnd.putAll(marks);
                 if (log.isTraceEnabled()) {
-                    log.trace("page " + pageNumberString + ": " 
+                    log.trace("page " + pageNumberString + ": "
                             + "Adding all markers to LastEnd");
                 }
             }
@@ -435,7 +434,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
             // last on page: replace all
             markerLastAny.putAll(marks);
             if (log.isTraceEnabled()) {
-                log.trace("page " + pageNumberString + ": " 
+                log.trace("page " + pageNumberString + ": "
                         + "Adding all markers to LastAny");
             }
         }
@@ -446,7 +445,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
      * This will retrieve a marker with the class name
      * and position.
      *
-     * @param name The class name of the marker to retrieve 
+     * @param name The class name of the marker to retrieve
      * @param pos the position to retrieve
      * @return Object the marker found or null
      */
@@ -494,10 +493,10 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
                 throw new RuntimeException();
         }
         if (log.isTraceEnabled()) {
-            log.trace("page " + pageNumberString + ": " + "Retrieving marker " + name 
-                    + " at position " + posName); 
+            log.trace("page " + pageNumberString + ": " + "Retrieving marker " + name
+                    + " at position " + posName);
         }
-        return mark;    
+        return mark;
     }
 
     /** Dumps the current marker data to the logger. */
@@ -510,15 +509,15 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
             log.trace("LastStart: " + this.markerLastStart);
         }
     }
-    
+
     /**
      * Save the page contents to an object stream.
      * The map of unresolved references are set on the page so that
      * the resolvers can be properly serialized and reloaded.
      * @param out the object output stream to write the contents
-     * @throws Exception if there is a problem saving the page
+     * @throws IOException in case of an I/O error while serializing the page
      */
-    public void savePage(ObjectOutputStream out) throws Exception {
+    public void savePage(ObjectOutputStream out) throws IOException {
         // set the unresolved references so they are serialized
         page.setUnresolvedReferences(unresolvedIDRefs);
         out.writeObject(page);
@@ -531,9 +530,10 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
      * if there are any unresolved references that were resolved
      * while saved they will be resolved on the page contents.
      * @param in the object input stream to read the page from
-     * @throws Exception if there is an error loading the page
+     * @throws ClassNotFoundException if a class was not found while loading the page
+     * @throws IOException if an I/O error occurred while loading the page
      */
-    public void loadPage(ObjectInputStream in) throws Exception {
+    public void loadPage(ObjectInputStream in) throws IOException, ClassNotFoundException {
         page = (Page) in.readObject();
         unresolvedIDRefs = page.getUnresolvedReferences();
         if (unresolvedIDRefs != null && pendingResolved != null) {
@@ -573,37 +573,17 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
         sb.append(getPageNumberString());
         return sb.toString();
     }
-    
+
     /** @return the name of the simple-page-master that created this page */
     public String getSimplePageMasterName() {
         return this.simplePageMasterName;
     }
-    
-    /**
-     * Adds a new ExtensionAttachment instance to this page.
-     * @param attachment the ExtensionAttachment
-     */
-    public void addExtensionAttachment(ExtensionAttachment attachment) {
-        if (this.extensionAttachments == null) {
-            this.extensionAttachments = new java.util.ArrayList();
-        }
-        extensionAttachments.add(attachment);
-    }
-    
-    /** @return the list of extension attachments for this page */
-    public List getExtensionAttachments() {
-        if (this.extensionAttachments == null) {
-            return Collections.EMPTY_LIST;
-        } else {
-            return this.extensionAttachments;
-        }
-    }
-    
+
     /** @return True if this is a blank page. */
     public boolean isBlank() {
         return this.blank;
     }
-    
+
     /**
      * Convenience method to get BodyRegion of this PageViewport
      * @return BodyRegion object
@@ -611,64 +591,60 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
     public BodyRegion getBodyRegion() {
         return (BodyRegion) getPage().getRegionViewport(
                 Constants.FO_REGION_BODY).getRegionReference();
-    }    
+    }
 
     /**
      * Convenience method to create a new Span for this
      * this PageViewport.
-     * 
+     *
      * @param spanAll whether this is a single-column span
      * @return Span object created
      */
     public Span createSpan(boolean spanAll) {
         return getBodyRegion().getMainReference().createSpan(spanAll);
-    }    
+    }
 
     /**
      * Convenience method to get the span-reference-area currently
      * being processed
-     * 
+     *
      * @return span currently being processed.
      */
     public Span getCurrentSpan() {
         return getBodyRegion().getMainReference().getCurrentSpan();
     }
-    
+
     /**
-     * Convenience method to get the normal-flow-reference-area 
+     * Convenience method to get the normal-flow-reference-area
      * currently being processed
-     * 
+     *
      * @return span currently being processed.
      */
     public NormalFlow getCurrentFlow() {
         return getCurrentSpan().getCurrentFlow();
     }
-    
+
     /**
-     * Convenience method to increment the Span to the 
+     * Convenience method to increment the Span to the
      * next NormalFlow to be processed, and to return that flow.
-     * 
+     *
      * @return the next NormalFlow in the Span.
      */
     public NormalFlow moveToNextFlow() {
         return getCurrentSpan().moveToNextFlow();
     }
-    
+
     /**
-     * Convenience method to return a given region-reference-area, 
+     * Convenience method to return a given region-reference-area,
      * keyed by the Constants class identifier for the corresponding
-     * formatting object (ie. Constants.FO_REGION_BODY, FO_REGION_START, 
+     * formatting object (ie. Constants.FO_REGION_BODY, FO_REGION_START,
      * etc.)
-     * 
+     *
      * @param id the Constants class identifier for the region.
      * @return the corresponding region-reference-area for this page.
      */
     public RegionReference getRegionReference(int id) {
         return getPage().getRegionViewport(id).getRegionReference();
     }
-    
-    /** @return whether this page viewport has any extension attachments */
-    public boolean hasExtensionAttachments() {
-        return this.extensionAttachments != null && !this.extensionAttachments.isEmpty();
-    }
+
 }
