@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RegionReference.java 557337 2007-07-18 17:37:14Z adelmelle $ */
+/* $Id: RegionReference.java 746664 2009-02-22 12:40:44Z jeremias $ */
 
 package org.apache.fop.area;
 
@@ -30,17 +30,17 @@ import org.apache.fop.fo.pagination.Region;
  * so the page master can make copies from the original page and regions.
  */
 public class RegionReference extends Area implements Cloneable {
-    
+
     /** Reference to the region FO. */
     //protected Region regionFO;
     private int regionClass;
     private String regionName;
     private CTM ctm;
-    
+
 
     // the list of block areas from the static flow
     private ArrayList blocks = new ArrayList();
-    
+
     /** the parent RegionViewport for this object */
     protected RegionViewport regionViewport;
 
@@ -85,7 +85,7 @@ public class RegionReference extends Area implements Cloneable {
     public void setCTM(CTM ctm) {
         this.ctm = ctm;
     }
-    
+
     /**
      * @return Returns the parent RegionViewport.
      */
@@ -124,7 +124,7 @@ public class RegionReference extends Area implements Cloneable {
     public String getRegionName() {
         return this.regionName;
     }
-    
+
     /**
      * Add a block area to this region reference area.
      *
@@ -148,4 +148,13 @@ public class RegionReference extends Area implements Cloneable {
         return rr;
     }
 
+    /** {@inheritDoc} */
+    public String toString() {
+        StringBuffer sb = new StringBuffer(super.toString());
+        sb.append(" {regionName=").append(regionName);
+        sb.append(", regionClass=").append(regionClass);
+        sb.append(", ctm=").append(ctm);
+        sb.append("}");
+        return sb.toString();
+    }
 }
