@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: RenderPagesModel.java 615906 2008-01-28 15:04:39Z jeremias $ */
- 
+/* $Id: RenderPagesModel.java 721430 2008-11-28 11:13:12Z acumiskey $ */
+
 package org.apache.fop.area;
 
 // Java
@@ -31,6 +31,7 @@ import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.fonts.FontInfo;
 import org.apache.fop.render.Renderer;
+import org.apache.fop.render.RendererEventProducer;
 
 /**
  * This uses the AreaTreeModel to store the pages
@@ -45,13 +46,13 @@ public class RenderPagesModel extends AreaTreeModel {
      * The renderer that will render the pages.
      */
     protected Renderer renderer;
-    
+
     /**
      * Pages that have been prepared but not rendered yet.
      */
-    protected List prepared = new java.util.ArrayList();
-    private List pendingODI = new java.util.ArrayList();
-    private List endDocODI = new java.util.ArrayList();
+    protected List/*<PageViewport>*/ prepared = new java.util.ArrayList/*<PageViewport>*/();
+    private List/*<OffDocumentItem>*/ pendingODI = new java.util.ArrayList/*<OffDocumentItem>*/();
+    private List/*<OffDocumentItem>*/ endDocODI = new java.util.ArrayList/*<OffDocumentItem>*/();
 
     /**
      * Create a new render pages model with the given renderer.
@@ -61,11 +62,11 @@ public class RenderPagesModel extends AreaTreeModel {
      * @param stream OutputStream
      * @throws FOPException if the renderer cannot be properly initialized
      */
-    public RenderPagesModel (FOUserAgent userAgent, String outputFormat, 
+    public RenderPagesModel (FOUserAgent userAgent, String outputFormat,
         FontInfo fontInfo, OutputStream stream) throws FOPException {
 
         super();
-        renderer = userAgent.getRendererFactory().createRenderer(
+        this.renderer = userAgent.getRendererFactory().createRenderer(
                 userAgent, outputFormat);
 
         try {
@@ -112,14 +113,18 @@ public class RenderPagesModel extends AreaTreeModel {
             try {
                 renderer.renderPage(page);
             } catch (RuntimeException re) {
-                String err = "Error while rendering page " + page.getPageNumberString(); 
+                String err = "Error while rendering page " + page.getPageNumberString();
                 log.error(err, re);
                 throw re;
-            } catch (Exception e) {
-                //TODO use error handler to handle this FOP or IO Exception or propagate exception
-                String err = "Error while rendering page " + page.getPageNumberString(); 
+            } catch (IOException ioe) {
+                RendererEventProducer eventProducer = RendererEventProducer.Provider.get(
+                        renderer.getUserAgent().getEventBroadcaster());
+                eventProducer.ioError(this, ioe);
+            } catch (FOPException e) {
+                //TODO use error handler to handle this FOPException or propagate exception
+                String err = "Error while rendering page " + page.getPageNumberString();
                 log.error(err, e);
-                throw new IllegalStateException("Fatal error occurred. Cannot continue. " 
+                throw new IllegalStateException("Fatal error occurred. Cannot continue. "
                         + e.getClass().getName() + ": " + err);
             }
             page.clear();
@@ -156,23 +161,7 @@ public class RenderPagesModel extends AreaTreeModel {
                         && pageViewport.getPageSequence().isFirstPage(pageViewport)) {
                     renderer.startPageSequence(getCurrentPageSequence());
                 }
-                try {
-                    renderer.renderPage(pageViewport);
-                    if (!pageViewport.isResolved()) {
-                        String[] idrefs = pageViewport.getIDRefs();
-                        for (int count = 0; count < idrefs.length; count++) {
-                            log.warn("Page " + pageViewport.getPageNumberString()
-                                + ": Unresolved id reference \"" + idrefs[count] 
-                                + "\" found.");
-                        }
-                    }
-                } catch (Exception e) {
-                    // use error handler to handle this FOP or IO Exception
-                    log.error("Error while rendering page " + pageViewport.getPageIndex(), e);
-                    if (e instanceof RuntimeException) {
-                        throw (RuntimeException)e;
-                    }
-                }
+                renderPage(pageViewport);
                 pageViewport.clear();
                 iter.remove();
             } else {
@@ -185,6 +174,33 @@ public class RenderPagesModel extends AreaTreeModel {
         return renderer.supportsOutOfOrder() || prepared.isEmpty();
     }
 
+    /**
+     * Renders the given page and notified about unresolved IDs if any.
+     * @param pageViewport the page to be rendered.
+     */
+    protected void renderPage(PageViewport pageViewport) {
+        try {
+            renderer.renderPage(pageViewport);
+            if (!pageViewport.isResolved()) {
+                String[] idrefs = pageViewport.getIDRefs();
+                for (int count = 0; count < idrefs.length; count++) {
+                    AreaEventProducer eventProducer = AreaEventProducer.Provider.get(
+                            renderer.getUserAgent().getEventBroadcaster());
+                    eventProducer.unresolvedIDReferenceOnPage(this,
+                            pageViewport.getPageNumberString(), idrefs[count]);
+                }
+            }
+        } catch (Exception e) {
+            AreaEventProducer eventProducer = AreaEventProducer.Provider.get(
+                    renderer.getUserAgent().getEventBroadcaster());
+            eventProducer.pageRenderingError(this,
+                    pageViewport.getPageNumberString(), e);
+            if (e instanceof RuntimeException) {
+                throw (RuntimeException)e;
+            }
+        }
+    }
+
     /**
      * Prepare a page.
      * An unresolved page can be prepared if the renderer supports
