@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Trait.java 611278 2008-01-11 19:50:53Z jeremias $ */
+/* $Id: Trait.java 830293 2009-10-27 19:07:52Z vhennebert $ */
 
 package org.apache.fop.area;
 
@@ -175,19 +175,19 @@ public class Trait implements Serializable {
 
     /** The space-before trait. */
     public static final Integer SPACE_BEFORE  = new Integer(29);
-    
+
     /** The space-after trait. */
     public static final Integer SPACE_AFTER  = new Integer(30);
-    
+
     /** The is-reference-area trait. */
     public static final Integer IS_REFERENCE_AREA = new Integer(31);
-    
+
     /** The is-viewport-area trait. */
     public static final Integer IS_VIEWPORT_AREA = new Integer(32);
-    
+
     /** Blinking trait used when rendering inline parent. */
     public static final Integer BLINK = new Integer(33);
-    
+
     /** Trait for color of underline decorations when rendering inline parent. */
     public static final Integer UNDERLINE_COLOR = new Integer(34);
     /** Trait for color of overline decorations when rendering inline parent. */
@@ -195,11 +195,14 @@ public class Trait implements Serializable {
     /** Trait for color of linethrough decorations when rendering inline parent. */
     public static final Integer LINETHROUGH_COLOR = new Integer(36);
     
+    /** The ptr trait. Used for accessibility   */
+    public static final Integer PTR = new Integer(37);
+
     /** Maximum value used by trait keys */
-    public static final int MAX_TRAIT_KEY = 36;
-    
+    public static final int MAX_TRAIT_KEY = 37;
+
     private static final TraitInfo[] TRAIT_INFO = new TraitInfo[MAX_TRAIT_KEY + 1];
-    
+
     private static class TraitInfo {
         private String name;
         private Class clazz; // Class of trait data
@@ -221,12 +224,13 @@ public class Trait implements Serializable {
     private static void put(Integer key, TraitInfo info) {
         TRAIT_INFO[key.intValue()] = info;
     }
-    
+
     static {
         // Create a hashmap mapping trait code to name for external representation
         //put(ID_LINK, new TraitInfo("id-link", String.class));
+        put(PTR, new TraitInfo("ptr", String.class));
         put(INTERNAL_LINK, new TraitInfo("internal-link", InternalLink.class));
-        put(EXTERNAL_LINK, new TraitInfo("external-link", String.class));
+        put(EXTERNAL_LINK, new TraitInfo("external-link", ExternalLink.class));
         put(FONT,         new TraitInfo("font", FontTriplet.class));
         put(FONT_SIZE,    new TraitInfo("font-size", Integer.class));
         put(COLOR, new TraitInfo("color", Color.class));
@@ -277,7 +281,7 @@ public class Trait implements Serializable {
                 new TraitInfo("is-reference-area", Boolean.class));
         put(IS_VIEWPORT_AREA,
                 new TraitInfo("is-viewport-area", Boolean.class));
-        
+
     }
 
     /**
@@ -290,25 +294,6 @@ public class Trait implements Serializable {
         return TRAIT_INFO[((Integer)traitCode).intValue()].getName();
     }
 
-    /**
-     * Get the trait code for a trait name.
-     *
-     * @param sTraitName the name of the trait to find
-     * @return the trait code object
-     */
-    /*
-    public static Object getTraitCode(String sTraitName) {
-        Iterator iter = TRAIT_INFO.entrySet().iterator();
-        while (iter.hasNext()) {
-            Map.Entry entry = (Map.Entry) iter.next();
-            TraitInfo ti = (TraitInfo) entry.getValue();
-            if (ti != null && ti.getName().equals(sTraitName)) {
-                return entry.getKey();
-            }
-        }
-        return null;
-    }*/
-
     /**
      * Get the data storage class for the trait.
      *
@@ -319,99 +304,6 @@ public class Trait implements Serializable {
         return TRAIT_INFO[((Integer)traitCode).intValue()].getClazz();
     }
 
-    /**
-     * The type of trait for an area.
-     */
-    private Object propType;
-
-    /**
-     * The data value of the trait.
-     */
-    private Object data;
-
-    /**
-     * Create a new empty trait.
-     */
-    public Trait() {
-        this.propType = null;
-        this.data = null;
-    }
-
-    /**
-     * Create a trait with the value and type.
-     *
-     * @param propType the type of trait
-     * @param data the data value
-     */
-    public Trait(Object propType, Object data) {
-        this.propType = propType;
-        this.data = data;
-    }
-
-    /**
-     * Returns the trait data value.
-     * @return the trait data value
-     */
-    public Object getData() {
-        return this.data;
-    }
-
-    /**
-     * Returns the property type.
-     * @return the property type
-     */
-    public Object getPropType() {
-        return this.propType;
-    }
-
-    /**
-     * Return the string for debugging.
-     * {@inheritDoc}
-     */
-    public String toString() {
-        return data.toString();
-    }
-
-    /**
-     * Make a trait value.
-     *
-     * @param oCode trait code
-     * @param sTraitValue trait value as String
-     * @return the trait value as object
-     */
-    /*
-    public static Object makeTraitValue(Object oCode, String sTraitValue) {
-        // Get the code from the name
-        // See what type of object it is
-        // Convert string value to an object of that type
-        Class tclass = getTraitClass(oCode);
-        if (tclass == null) {
-            return null;
-        }
-        if (tclass.equals(String.class)) {
-            return sTraitValue;
-        }
-        if (tclass.equals(Integer.class)) {
-            return new Integer(sTraitValue);
-        }
-        // See if the class has a constructor from string or can read from a string
-        try {
-            Object o = tclass.newInstance();
-            //return o.fromString(sTraitValue);
-        } catch (IllegalAccessException e1) {
-            log.error("Can't create instance of "
-                               + tclass.getName());
-            return null;
-        } catch (InstantiationException e2) {
-            log.error("Can't create instance of "
-                               + tclass.getName());
-            return null;
-        }
-
-
-        return null;
-    }*/
-
     /**
      * Class for internal link traits.
      * Stores PageViewport key and producer ID
@@ -547,6 +439,78 @@ public class Trait implements Serializable {
         }
     }
 
+    /**
+     * External Link trait structure
+     */
+    public static class ExternalLink implements Serializable {
+
+        private String destination;
+        private boolean newWindow;
+
+        /**
+         * Constructs an ExternalLink object with the given destination
+         *
+         * @param destination   target of the link
+         * @param newWindow     true if the target should be opened in a new window
+         */
+        public ExternalLink(String destination, boolean newWindow) {
+            this.destination = destination;
+            this.newWindow = newWindow;
+        }
+
+        /**
+         * Create an <code>ExternalLink</code> from a trait value/attribute value in the
+         * area tree
+         * @param traitValue    the value to use (should match the result of {@link #toString()}
+         * @return an <code>ExternalLink</code> instance corresponding to the given value
+         */
+        protected static ExternalLink makeFromTraitValue(String traitValue) {
+            String dest = null;
+            boolean newWindow = false;
+            String[] values = traitValue.split(",");
+            for (int i = 0, c = values.length; i < c; i++) {
+                String v = values[i];
+                if (v.startsWith("dest=")) {
+                    dest = v.substring(5);
+                } else if (v.startsWith("newWindow=")) {
+                    newWindow = Boolean.valueOf(v.substring(10)).booleanValue();
+                } else {
+                    throw new IllegalArgumentException(
+                            "Malformed trait value for Trait.ExternalLink: " + traitValue);
+                }
+            }
+            return new ExternalLink(dest, newWindow);
+        }
+
+        /**
+         * Get the target/destination of the link
+         * @return  the destination of the link
+         */
+        public String getDestination() {
+            return this.destination;
+        }
+
+        /**
+         * Check if the target has to be displayed in a new window
+         * @return  <code>true</code> if the target has to be displayed in a new window
+         */
+        public boolean newWindow() {
+            return this.newWindow;
+        }
+
+        /**
+         * Return a String representation of the object.
+         * @return  a <code>String</code> of the form
+         *          "org.apache.fop.area.Trait.ExternalLink[dest=someURL,newWindow=false]"
+         */
+        public String toString() {
+            StringBuffer sb = new StringBuffer(64);
+            sb.append("newWindow=").append(newWindow);
+            sb.append(",dest=").append(this.destination);
+            return sb.toString();
+        }
+    }
+
     /**
      * Background trait structure.
      * Used for storing back trait information which are related.
@@ -558,7 +522,7 @@ public class Trait implements Serializable {
 
         /** The background image url if any. */
         private String url = null;
-        
+
         /** The background image if any. */
         private ImageInfo imageInfo = null;
 
@@ -698,7 +662,7 @@ public class Trait implements Serializable {
                 throw new IllegalStateException("Illegal repeat style: " + repeat);
             }
         }
-        
+
         /**
          * Return the string for debugging.
          * {@inheritDoc}
