@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: FilledArea.java 557337 2007-07-18 17:37:14Z adelmelle $ */
- 
+/* $Id: FilledArea.java 721430 2008-11-28 11:13:12Z acumiskey $ */
+
 package org.apache.fop.area.inline;
 
 import java.util.List;
@@ -35,7 +35,7 @@ import java.util.Iterator;
  */
 public class FilledArea extends InlineParent {
     private int unitWidth;
-    
+
     /**
      * Create a new filled area.
      */
@@ -70,10 +70,10 @@ public class FilledArea extends InlineParent {
     /**
      * Set the unit width for the areas to fill the full width.
      *
-     * @param w the unit width
+     * @param width the unit width
      */
-    public void setUnitWidth(int w) {
-        unitWidth = w;
+    public void setUnitWidth(int width) {
+        this.unitWidth = width;
     }
 
     /**
@@ -82,7 +82,7 @@ public class FilledArea extends InlineParent {
      * @return the unit width
      */
     public int getUnitWidth() {
-        return unitWidth;
+        return this.unitWidth;
     }
 
     /**
@@ -98,7 +98,7 @@ public class FilledArea extends InlineParent {
         }
         return bpd;
     }
-    
+
     /**
      * Get the child areas for this filled area.
      * This copies the references of the inline areas so that
@@ -115,9 +115,9 @@ public class FilledArea extends InlineParent {
         }
         return newList;
     }
-    
+
     /**
-     * recursively apply the variation factor to all descendant areas
+     * Recursively apply the variation factor to all descendant areas
      * @param variationFactor the variation factor that must be applied to adjustments
      * @param lineStretch     the total stretch of the line
      * @param lineShrink      the total shrink of the line
