@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,13 +15,10 @@
  * limitations under the License.
  */
 
-/* $Id: FOEventHandler.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: FOEventHandler.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fo;
 
-import java.util.HashSet;
-import java.util.Set;
-
 import org.xml.sax.SAXException;
 
 import org.apache.fop.apps.FOUserAgent;
@@ -43,11 +40,15 @@ import org.apache.fop.fo.flow.PageNumberCitation;
 import org.apache.fop.fo.flow.PageNumberCitationLast;
 import org.apache.fop.fo.flow.table.Table;
 import org.apache.fop.fo.flow.table.TableBody;
+import org.apache.fop.fo.flow.table.TableFooter;
+import org.apache.fop.fo.flow.table.TableHeader;
+import org.apache.fop.fo.flow.table.TablePart;
 import org.apache.fop.fo.flow.table.TableCell;
 import org.apache.fop.fo.flow.table.TableColumn;
 import org.apache.fop.fo.flow.table.TableRow;
 import org.apache.fop.fo.pagination.Flow;
 import org.apache.fop.fo.pagination.PageSequence;
+import org.apache.fop.fonts.FontEventAdapter;
 import org.apache.fop.fonts.FontInfo;
 
 /**
@@ -62,38 +63,17 @@ import org.apache.fop.fonts.FontInfo;
  * as our MIF and RTF output targets.
  */
 public abstract class FOEventHandler {
-    
-    /** 
+
+    /**
      * The FOUserAgent for this process
      */
     protected FOUserAgent foUserAgent;
 
-    /** 
+    /**
      * The Font information relevant for this document
      */
     protected FontInfo fontInfo;
 
-    /**
-     * The current set of id's in the FO tree.
-     * This is used so we know if the FO tree contains duplicates.
-     */
-    private Set idReferences = new HashSet();
-
-    /**
-     * The property list maker.
-     */
-    protected PropertyListMaker propertyListMaker;
-
-    /**
-     * The XMLWhitespaceHandler for this tree
-     */
-    protected XMLWhiteSpaceHandler whiteSpaceHandler = new XMLWhiteSpaceHandler();
-    
-    /**
-     * Indicates whether processing descendants of a marker
-     */
-    private boolean inMarker = false;
-    
     /**
      * Main constructor
      * @param foUserAgent the apps.FOUserAgent instance for this process
@@ -101,14 +81,7 @@ public abstract class FOEventHandler {
     public FOEventHandler(FOUserAgent foUserAgent) {
         this.foUserAgent = foUserAgent;
         this.fontInfo = new FontInfo();
-    }
-
-    /**
-     * Retuns the set of ID references.
-     * @return the ID references
-     */
-    public Set getIDReferences() {
-        return idReferences;
+        this.fontInfo.setEventListener(new FontEventAdapter(foUserAgent.getEventBroadcaster()));
     }
 
     /**
@@ -127,45 +100,6 @@ public abstract class FOEventHandler {
         return this.fontInfo;
     }
 
-    /**
-     * Return the propertyListMaker.
-    */
-    public PropertyListMaker getPropertyListMaker() {
-        return propertyListMaker;
-    }
-     
-    /**
-     * Set a new propertyListMaker.
-     */
-    public void setPropertyListMaker(PropertyListMaker propertyListMaker) {
-        this.propertyListMaker = propertyListMaker;
-    }
-    
-    /**
-     * Return the XMLWhiteSpaceHandler
-     * @return the whiteSpaceHandler
-     */
-    public XMLWhiteSpaceHandler getXMLWhiteSpaceHandler() {
-        return whiteSpaceHandler;
-    }
-
-    /**
-     * Switch to or from marker context
-     * (used by FOTreeBuilder when processing
-     *  a marker)
-     *
-     */
-    protected void switchMarkerContext(boolean inMarker) {
-        this.inMarker = inMarker;
-    }
-    
-    /**
-     * Check whether in marker context
-     */
-    protected boolean inMarker() {
-        return this.inMarker;
-    }
-    
     /**
      * This method is called to indicate the start of a new document run.
      * @throws SAXException In case of a problem
@@ -208,37 +142,37 @@ public abstract class FOEventHandler {
     }
 
     /**
-     * 
+     *
      * @param pageCite PageNumberCitation that is starting.
      */
     public void startPageNumberCitation(PageNumberCitation pageCite) {
     }
 
     /**
-     * 
+     *
      * @param pageCite PageNumberCitation that is ending.
      */
     public void endPageNumberCitation(PageNumberCitation pageCite) {
     }
 
     /**
-     * 
+     *
      * @param pageLast PageNumberCitationLast that is starting.
      */
     public void startPageNumberCitationLast(PageNumberCitationLast pageLast) {
     }
 
     /**
-     * 
+     *
      * @param pageLast PageNumberCitationLast that is ending.
      */
     public void endPageNumberCitationLast(PageNumberCitationLast pageLast) {
     }
 
     /**
-     * This method is called to indicate the start of a new fo:flow 
+     * This method is called to indicate the start of a new fo:flow
      * or fo:static-content.
-     * This method also handles fo:static-content tags, because the 
+     * This method also handles fo:static-content tags, because the
      * StaticContent class is derived from the Flow class.
      *
      * @param fl Flow that is starting.
@@ -326,44 +260,44 @@ public abstract class FOEventHandler {
 
     /**
      *
-     * @param th TableBody that is starting;
+     * @param header TableHeader that is starting;
      */
-    public void startHeader(TableBody th) {
+    public void startHeader(TableHeader header) {
     }
 
     /**
      *
-     * @param th TableBody that is ending.
+     * @param header TableHeader that is ending.
      */
-    public void endHeader(TableBody th) {
+    public void endHeader(TableHeader header) {
     }
 
     /**
      *
-     * @param tf TableFooter that is starting.
+     * @param footer TableFooter that is starting.
      */
-    public void startFooter(TableBody tf) {
+    public void startFooter(TableFooter footer) {
     }
 
     /**
      *
-     * @param tf TableFooter that is ending.
+     * @param footer TableFooter that is ending.
      */
-    public void endFooter(TableBody tf) {
+    public void endFooter(TableFooter footer) {
     }
 
     /**
      *
-     * @param tb TableBody that is starting.
+     * @param body TableBody that is starting.
      */
-    public void startBody(TableBody tb) {
+    public void startBody(TableBody body) {
     }
 
     /**
      *
-     * @param tb TableBody that is ending.
+     * @param body TableBody that is ending.
      */
-    public void endBody(TableBody tb) {
+    public void endBody(TableBody body) {
     }
 
     /**
@@ -513,7 +447,7 @@ public abstract class FOEventHandler {
      */
     public void startFootnote(Footnote footnote) {
     }
-    
+
     /**
      * Process the ending of a footnote.
      * @param footnote Footnote that is ending
@@ -527,7 +461,7 @@ public abstract class FOEventHandler {
      */
     public void startFootnoteBody(FootnoteBody body) {
     }
-    
+
     /**
      * Process the ending of a footnote body.
      * @param body FootnoteBody that is ending
@@ -555,7 +489,7 @@ public abstract class FOEventHandler {
      * @param start Offset for characters to process.
      * @param length Portion of array to process.
      */
-    public void characters(char data[], int start, int length) {
+    public void characters(char[] data, int start, int length) {
     }
 
     /**
