@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FOTreeBuilder.java 637791 2008-03-17 08:59:51Z jeremias $ */
+/* $Id: FOTreeBuilder.java 806365 2009-08-20 21:21:43Z adelmelle $ */
 
 package org.apache.fop.fo;
 
@@ -31,6 +31,8 @@ import org.xml.sax.helpers.DefaultHandler;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.xmlgraphics.util.QName;
+
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.apps.FormattingResults;
@@ -61,45 +63,49 @@ public class FOTreeBuilder extends DefaultHandler {
 
     /** Main DefaultHandler that handles the FO namespace. */
     protected MainFOHandler mainFOHandler;
-    
+
     /** Current delegate ContentHandler to receive the SAX events */
     protected ContentHandler delegate;
-    
+
+    /** Provides information used during tree building stage. */
+    private FOTreeBuilderContext builderContext;
+
     /** The object that handles formatting and rendering to a stream */
     private FOEventHandler foEventHandler;
 
     /** The SAX locator object managing the line and column counters */
-    private Locator locator; 
-    
+    private Locator locator;
+
     /** The user agent for this processing run. */
     private FOUserAgent userAgent;
-    
+
     private boolean used = false;
     private boolean empty = true;
-    
+
     private int depth;
-    
+
     /**
      * <code>FOTreeBuilder</code> constructor
-     * 
+     *
      * @param outputFormat the MIME type of the output format to use (ex. "application/pdf").
      * @param foUserAgent   the {@link FOUserAgent} in effect for this process
      * @param stream    the <code>OutputStream</code> to direct the results to
      * @throws FOPException if the <code>FOTreeBuilder</code> cannot be properly created
      */
     public FOTreeBuilder(
-                String outputFormat, 
+                String outputFormat,
                 FOUserAgent foUserAgent,
-                OutputStream stream) 
+                OutputStream stream)
             throws FOPException {
 
         this.userAgent = foUserAgent;
-        this.elementMappingRegistry = userAgent.getFactory().getElementMappingRegistry();        
+        this.elementMappingRegistry = userAgent.getFactory().getElementMappingRegistry();
         //This creates either an AreaTreeHandler and ultimately a Renderer, or
         //one of the RTF-, MIF- etc. Handlers.
         foEventHandler = foUserAgent.getRendererFactory().createFOEventHandler(
                 foUserAgent, outputFormat, stream);
-        foEventHandler.setPropertyListMaker(new PropertyListMaker() {
+        builderContext = new FOTreeBuilderContext();
+        builderContext.setPropertyListMaker(new PropertyListMaker() {
             public PropertyList make(FObj fobj, PropertyList parentPropertyList) {
                 return new StaticPropertyList(fobj, parentPropertyList);
             }
@@ -110,16 +116,16 @@ public class FOTreeBuilder extends DefaultHandler {
     public void setDocumentLocator(Locator locator) {
         this.locator = locator;
     }
-    
-    /** 
+
+    /**
      * @return a {@link Locator} instance if it is available and not disabled
      */
     protected Locator getEffectiveLocator() {
         return (userAgent.isLocatorEnabled() ? this.locator : null);
     }
-    
+
     /** {@inheritDoc} */
-    public void characters(char[] data, int start, int length) 
+    public void characters(char[] data, int start, int length)
                 throws SAXException {
         delegate.characters(data, start, length);
     }
@@ -130,6 +136,7 @@ public class FOTreeBuilder extends DefaultHandler {
             throw new IllegalStateException("FOTreeBuilder (and the Fop class) cannot be reused."
                     + " Please instantiate a new instance.");
         }
+
         used = true;
         empty = true;
         rootFObj = null;    // allows FOTreeBuilder to be reused
@@ -137,7 +144,7 @@ public class FOTreeBuilder extends DefaultHandler {
             log.debug("Building formatting object tree");
         }
         foEventHandler.startDocument();
-        this.mainFOHandler = new MainFOHandler(); 
+        this.mainFOHandler = new MainFOHandler();
         this.mainFOHandler.startDocument();
         this.delegate = this.mainFOHandler;
     }
@@ -146,8 +153,9 @@ public class FOTreeBuilder extends DefaultHandler {
     public void endDocument() throws SAXException {
         this.delegate.endDocument();
         if (this.rootFObj == null && empty) {
-            throw new ValidationException(
-                    "Document is empty (something might be wrong with your XSLT stylesheet).");
+            FOValidationEventProducer eventProducer
+                = FOValidationEventProducer.Provider.get(userAgent.getEventBroadcaster());
+            eventProducer.emptyDocument(this);
         }
         rootFObj = null;
         if (log.isDebugEnabled()) {
@@ -178,18 +186,6 @@ public class FOTreeBuilder extends DefaultHandler {
         }
     }
 
-    /**
-     * Finds the {@link Maker} used to create {@link FONode} objects of a particular type
-     * 
-     * @param namespaceURI URI for the namespace of the element
-     * @param localName name of the Element
-     * @return the ElementMapping.Maker that can create an FO object for this element
-     * @throws FOPException if a Maker could not be found for a bound namespace.
-     */
-    private Maker findFOMaker(String namespaceURI, String localName) throws FOPException {
-        return elementMappingRegistry.findFOMaker(namespaceURI, localName, locator);
-    }
-
     /** {@inheritDoc} */
     public void warning(SAXParseException e) {
         log.warn(e.getLocalizedMessage());
@@ -208,7 +204,7 @@ public class FOTreeBuilder extends DefaultHandler {
 
     /**
      * Provides access to the underlying {@link FOEventHandler} object.
-     * 
+     *
      * @return the FOEventHandler object
      */
     public FOEventHandler getEventHandler() {
@@ -219,30 +215,30 @@ public class FOTreeBuilder extends DefaultHandler {
      * Returns the results of the rendering process. Information includes
      * the total number of pages generated and the number of pages per
      * page-sequence.
-     * 
+     *
      * @return the results of the rendering process.
      */
     public FormattingResults getResults() {
         if (getEventHandler() instanceof AreaTreeHandler) {
             return ((AreaTreeHandler) getEventHandler()).getResults();
         } else {
-            //No formatting results available for output formats no 
+            //No formatting results available for output formats no
             //involving the layout engine.
-            return null;   
+            return null;
         }
     }
-    
+
     /**
      * Main <code>DefaultHandler</code> implementation which builds the FO tree.
      */
     private class MainFOHandler extends DefaultHandler {
-        
+
         /** Current formatting object being handled */
         protected FONode currentFObj = null;
 
         /** Current propertyList for the node being handled */
         protected PropertyList currentPropertyList;
-        
+
         /** Current marker nesting-depth */
         private int nestedMarkerDepth = 0;
 
@@ -257,42 +253,44 @@ public class FOTreeBuilder extends DefaultHandler {
             // Check to ensure first node encountered is an fo:root
             if (rootFObj == null) {
                 empty = false;
-                if (!namespaceURI.equals(FOElementMapping.URI) 
-                    || !localName.equals("root")) {
-                    throw new ValidationException(
-                        "Error: First element must be the fo:root formatting object. "
-                        + "Found " + FONode.getNodeString(namespaceURI, localName) 
-                        + " instead."
-                        + " Please make sure you're producing a valid XSL-FO document.");
+                if (!namespaceURI.equals(FOElementMapping.URI)
+                        || !localName.equals("root")) {
+                    FOValidationEventProducer eventProducer
+                        = FOValidationEventProducer.Provider.get(
+                                userAgent.getEventBroadcaster());
+                    eventProducer.invalidFORoot(this, FONode.getNodeString(namespaceURI, localName),
+                            getEffectiveLocator());
                 }
             } else { // check that incoming node is valid for currentFObj
-                if (namespaceURI.equals(FOElementMapping.URI)
-                    || namespaceURI.equals(ExtensionElementMapping.URI)) {
+                if (currentFObj.getNamespaceURI().equals(FOElementMapping.URI)
+                    || currentFObj.getNamespaceURI().equals(ExtensionElementMapping.URI)) {
                     currentFObj.validateChildNode(locator, namespaceURI, localName);
                 }
             }
-            
-            ElementMapping.Maker fobjMaker = 
-                findFOMaker(namespaceURI, localName);
+
+            ElementMapping.Maker fobjMaker = findFOMaker(namespaceURI, localName);
 
             try {
                 foNode = fobjMaker.make(currentFObj);
                 if (rootFObj == null) {
                     rootFObj = (Root) foNode;
+                    rootFObj.setBuilderContext(builderContext);
                     rootFObj.setFOEventHandler(foEventHandler);
                 }
                 propertyList = foNode.createPropertyList(
                                     currentPropertyList, foEventHandler);
-                foNode.processNode(localName, getEffectiveLocator(), 
+                foNode.processNode(localName, getEffectiveLocator(),
                                     attlist, propertyList);
                 if (foNode.getNameId() == Constants.FO_MARKER) {
-                    if (foEventHandler.inMarker()) {
+                    if (builderContext.inMarker()) {
                         nestedMarkerDepth++;
                     } else {
-                        foEventHandler.switchMarkerContext(true);
+                        builderContext.switchMarkerContext(true);
                     }
                 }
-                foNode.startOfNode();
+                if (foNode.getNameId() == Constants.FO_PAGE_SEQUENCE) {
+                    builderContext.getXMLWhiteSpaceHandler().reset();
+                }
             } catch (IllegalArgumentException e) {
                 throw new SAXException(e);
             }
@@ -300,27 +298,34 @@ public class FOTreeBuilder extends DefaultHandler {
             ContentHandlerFactory chFactory = foNode.getContentHandlerFactory();
             if (chFactory != null) {
                 ContentHandler subHandler = chFactory.createContentHandler();
-                if (subHandler instanceof ObjectSource 
+                if (subHandler instanceof ObjectSource
                         && foNode instanceof ObjectBuiltListener) {
                     ((ObjectSource) subHandler).setObjectBuiltListener(
                             (ObjectBuiltListener) foNode);
                 }
-                
+
                 subHandler.startDocument();
-                subHandler.startElement(namespaceURI, localName, 
+                subHandler.startElement(namespaceURI, localName,
                         rawName, attlist);
                 depth = 1;
                 delegate = subHandler;
             }
-            
+
             if (currentFObj != null) {
                 currentFObj.addChildNode(foNode);
             }
 
             currentFObj = foNode;
-            if (propertyList != null && !foEventHandler.inMarker()) {
+            if (propertyList != null && !builderContext.inMarker()) {
                 currentPropertyList = propertyList;
             }
+
+            // fo:characters can potentially be removed during
+            // white-space handling.
+            // Do not notify the FOEventHandler.
+            if (currentFObj.getNameId() != Constants.FO_CHARACTER) {
+                currentFObj.startOfNode();
+            }
         }
 
         /** {@inheritDoc} */
@@ -328,44 +333,48 @@ public class FOTreeBuilder extends DefaultHandler {
                     throws SAXException {
             if (currentFObj == null) {
                 throw new SAXException(
-                        "endElement() called for " + rawName 
+                        "endElement() called for " + rawName
                             + " where there is no current element.");
-            } else if (!currentFObj.getLocalName().equals(localName) 
+            } else if (!currentFObj.getLocalName().equals(localName)
                     || !currentFObj.getNamespaceURI().equals(uri)) {
-                throw new SAXException("Mismatch: " + currentFObj.getLocalName() 
-                        + " (" + currentFObj.getNamespaceURI() 
+                throw new SAXException("Mismatch: " + currentFObj.getLocalName()
+                        + " (" + currentFObj.getNamespaceURI()
                         + ") vs. " + localName + " (" + uri + ")");
             }
-            
-            currentFObj.endOfNode();
-            
+
+            // fo:characters can potentially be removed during
+            // white-space handling.
+            // Do not notify the FOEventHandler.
+            if (currentFObj.getNameId() != Constants.FO_CHARACTER) {
+                currentFObj.endOfNode();
+            }
+
             if (currentPropertyList != null
                     && currentPropertyList.getFObj() == currentFObj
-                    && !foEventHandler.inMarker()) {
-                currentPropertyList = 
-                    currentPropertyList.getParentPropertyList();
+                    && !builderContext.inMarker()) {
+                currentPropertyList = currentPropertyList.getParentPropertyList();
             }
-            
+
             if (currentFObj.getNameId() == Constants.FO_MARKER) {
                 if (nestedMarkerDepth == 0) {
-                    foEventHandler.switchMarkerContext(false);
+                    builderContext.switchMarkerContext(false);
                 } else {
                     nestedMarkerDepth--;
                 }
             }
-            
+
             if (currentFObj.getParent() == null) {
                 log.debug("endElement for top-level " + currentFObj.getName());
             }
-            
+
             currentFObj = currentFObj.getParent();
         }
 
         /** {@inheritDoc} */
-        public void characters(char[] data, int start, int length) 
+        public void characters(char[] data, int start, int length)
             throws FOPException {
             if (currentFObj != null) {
-                currentFObj.addCharacters(data, start, start + length, 
+                currentFObj.characters(data, start, length,
                         currentPropertyList, getEffectiveLocator());
             }
         }
@@ -373,7 +382,29 @@ public class FOTreeBuilder extends DefaultHandler {
         /** {@inheritDoc} */
         public void endDocument() throws SAXException {
             currentFObj = null;
-        }        
+        }
+
+        /**
+         * Finds the {@link Maker} used to create {@link FONode} objects of a particular type
+         *
+         * @param namespaceURI URI for the namespace of the element
+         * @param localName name of the Element
+         * @return the ElementMapping.Maker that can create an FO object for this element
+         * @throws FOPException if a Maker could not be found for a bound namespace.
+         */
+        private Maker findFOMaker(String namespaceURI, String localName) throws FOPException {
+            Maker maker = elementMappingRegistry.findFOMaker(namespaceURI, localName, locator);
+            if (maker instanceof UnknownXMLObj.Maker) {
+                FOValidationEventProducer eventProducer
+                    = FOValidationEventProducer.Provider.get(
+                        userAgent.getEventBroadcaster());
+                eventProducer.unknownFormattingObject(this, currentFObj.getName(),
+                        new QName(namespaceURI, localName),
+                        getEffectiveLocator());
+            }
+            return maker;
+        }
+
     }
 }
 
