@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FObj.java 668641 2008-06-17 12:59:25Z jeremias $ */
+/* $Id: FObj.java 825566 2009-10-15 17:07:08Z acumiskey $ */
 
 package org.apache.fop.fo;
 
@@ -30,32 +30,34 @@ import java.util.Set;
 import org.xml.sax.Attributes;
 import org.xml.sax.Locator;
 
+import org.apache.xmlgraphics.util.QName;
+
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.fo.extensions.ExtensionAttachment;
 import org.apache.fop.fo.flow.Marker;
 import org.apache.fop.fo.properties.PropertyMaker;
-import org.apache.fop.util.QName;
 
 /**
  * Base class for representation of formatting objects and their processing.
+ * All standard formatting object classes extend this class.
  */
 public abstract class FObj extends FONode implements Constants {
-    
+
     /** the list of property makers */
-    private static PropertyMaker[] propertyListTable
+    private static final PropertyMaker[] propertyListTable
                             = FOPropertyMapping.getGenericMappings();
-    
-    /** 
+
+    /**
      * pointer to the descendant subtree
      */
     protected FONode firstChild;
-    
+
     /** The list of extension attachments, null if none */
-    private List extensionAttachments = null;
-    
+    private List/*<ExtensionAttachment>*/ extensionAttachments = null;
+
     /** The map of foreign attributes, null if none */
-    private Map foreignAttributes = null;
-    
+    private Map/*<QName,String>*/ foreignAttributes = null;
+
     /** Used to indicate if this FO is either an Out Of Line FO (see rec)
      *  or a descendant of one. Used during FO validation.
      */
@@ -63,20 +65,19 @@ public abstract class FObj extends FONode implements Constants {
 
     /** Markers added to this element. */
     private Map markers = null;
-    
+
     // The value of properties relevant for all fo objects
     private String id = null;
     // End of property values
 
     /**
      * Create a new formatting object.
-     * All formatting object classes extend this class.
      *
      * @param parent the parent node
      */
     public FObj(FONode parent) {
         super(parent);
-        
+
         // determine if isOutOfLineFODescendant should be set
         if (parent != null && parent instanceof FObj) {
             if (((FObj) parent).getIsOutOfLineFODescendant()) {
@@ -91,9 +92,7 @@ public abstract class FObj extends FONode implements Constants {
         }
     }
 
-    /**
-     * {@inheritDoc} 
-     */
+    /** {@inheritDoc} */
     public FONode clone(FONode parent, boolean removeChildren)
         throws FOPException {
         FObj fobj = (FObj) super.clone(parent, removeChildren);
@@ -102,7 +101,7 @@ public abstract class FObj extends FONode implements Constants {
         }
         return fobj;
     }
-    
+
     /**
      * Returns the PropertyMaker for a given property ID.
      * @param propId the property ID
@@ -112,11 +111,9 @@ public abstract class FObj extends FONode implements Constants {
         return propertyListTable[propId];
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public void processNode(String elementName, Locator locator, 
-                            Attributes attlist, PropertyList pList) 
+    /** {@inheritDoc} */
+    public void processNode(String elementName, Locator locator,
+                            Attributes attlist, PropertyList pList)
                     throws FOPException {
         setLocator(locator);
         pList.addAttributesToList(attlist);
@@ -128,12 +125,12 @@ public abstract class FObj extends FONode implements Constants {
     }
 
     /**
-     * Create a default property list for this element. 
+     * Create a default property list for this element.
      * {@inheritDoc}
      */
-    protected PropertyList createPropertyList(PropertyList parent, 
+    protected PropertyList createPropertyList(PropertyList parent,
                     FOEventHandler foEventHandler) throws FOPException {
-        return foEventHandler.getPropertyListMaker().make(this, parent);
+        return getBuilderContext().getPropertyListMaker().make(this, parent);
     }
 
     /**
@@ -161,36 +158,18 @@ public abstract class FObj extends FONode implements Constants {
      * Setup the id for this formatting object.
      * Most formatting objects can have an id that can be referenced.
      * This methods checks that the id isn't already used by another FO
-     * 
+     *
      * @param id    the id to check
      * @throws ValidationException if the ID is already defined elsewhere
      *                              (strict validation only)
      */
     private void checkId(String id) throws ValidationException {
         if (!inMarker() && !id.equals("")) {
-            Set idrefs = getFOEventHandler().getIDReferences();
+            Set idrefs = getBuilderContext().getIDReferences();
             if (!idrefs.contains(id)) {
                 idrefs.add(id);
             } else {
-                if (getUserAgent().validateStrictly()) {
-                    throw new ValidationException("Property id \"" + id 
-                            + "\" previously used; id values must be unique"
-                            + " in document.", locator);
-                } else {
-                    if (log.isWarnEnabled()) {
-                        StringBuffer msg = new StringBuffer();
-                        msg.append("Found non-unique id on ").append(getName());
-                        if (locator != null && locator.getLineNumber() != -1) {
-                            msg.append(" (at ").append(locator.getLineNumber())
-                                .append("/").append(locator.getColumnNumber())
-                                .append(")");
-                        }
-                        msg.append("\nAny reference to it will be considered "
-                                + "a reference to the first occurrence "
-                                + "in the document.");
-                        log.warn(msg);
-                    }
-                }
+                getFOValidationEventProducer().idNotUnique(this, getName(), id, true, locator);
             }
         }
     }
@@ -199,21 +178,19 @@ public abstract class FObj extends FONode implements Constants {
      * Returns Out Of Line FO Descendant indicator.
      * @return true if Out of Line FO or Out Of Line descendant, false otherwise
      */
-    public boolean getIsOutOfLineFODescendant() {
+    boolean getIsOutOfLineFODescendant() {
         return isOutOfLineFODescendant;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc}*/
     protected void addChildNode(FONode child) throws FOPException {
-        if (canHaveMarkers() && child.getNameId() == FO_MARKER) {
+        if (child.getNameId() == FO_MARKER) {
             addMarker((Marker) child);
-        } else { 
+        } else {
             ExtensionAttachment attachment = child.getExtensionAttachment();
             if (attachment != null) {
-                /* This removes the element from the normal children, 
-                 * so no layout manager is being created for them 
+                /* This removes the element from the normal children,
+                 * so no layout manager is being created for them
                  * as they are only additional information.
                  */
                 addExtensionAttachment(attachment);
@@ -238,11 +215,11 @@ public abstract class FObj extends FONode implements Constants {
      * @param parent    the (cloned) parent node
      * @throws FOPException when the child could not be added to the parent
      */
-    protected static void addChildTo(FONode child, FObj parent) 
+    protected static void addChildTo(FONode child, FONode parent)
                             throws FOPException {
         parent.addChildNode(child);
     }
-    
+
     /** {@inheritDoc} */
     public void removeChild(FONode child) {
         FONode nextChild = null;
@@ -262,7 +239,7 @@ public abstract class FObj extends FONode implements Constants {
             }
         }
     }
-    
+
     /**
      * Find the nearest parent, grandparent, etc. FONode that is also an FObj
      * @return FObj the nearest ancestor FONode that is an FObj
@@ -284,16 +261,22 @@ public abstract class FObj extends FONode implements Constants {
         return false;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public FONodeIterator getChildNodes() {
-        if (firstChild != null) {
+        if (hasChildren()) {
             return new FObjIterator(this);
         }
         return null;
     }
 
+    /**
+     * Indicates whether this formatting object has children.
+     * @return true if there are children
+     */
+    public boolean hasChildren() {
+        return this.firstChild != null;
+    }
+
     /**
      * Return an iterator over the object's childNodes starting
      * at the passed-in node (= first call to iterator.next() will
@@ -324,15 +307,15 @@ public abstract class FObj extends FONode implements Constants {
 
     /**
      * Notifies a FObj that one of it's children is removed.
-     * This method is subclassed by Block to clear the 
+     * This method is subclassed by Block to clear the
      * firstInlineChild variable in case it doesn't generate
      * any areas (see addMarker()).
      * @param node the node that was removed
      */
-    protected void notifyChildRemoval(FONode node) {
+    void notifyChildRemoval(FONode node) {
         //nop
     }
-    
+
     /**
      * Add the marker to this formatting object.
      * If this object can contain markers it checks that the marker
@@ -349,8 +332,8 @@ public abstract class FObj extends FONode implements Constants {
                 if (node instanceof FObj
                         || (node instanceof FOText
                                 && ((FOText) node).willCreateArea())) {
-                    log.error(
-                            "fo:marker must be an initial child: " + mcname);
+                    getFOValidationEventProducer().markerNotInitialChild(this, getName(),
+                            mcname, locator);
                     return;
                 } else if (node instanceof FOText) {
                     iter.remove();
@@ -364,8 +347,8 @@ public abstract class FObj extends FONode implements Constants {
         if (!markers.containsKey(mcname)) {
             markers.put(mcname, marker);
         } else {
-            log.error("fo:marker 'marker-class-name' "
-                    + "must be unique for same parent: " + mcname);
+            getFOValidationEventProducer().markerNotUniqueForSameParent(this, getName(),
+                    mcname, locator);
         }
     }
 
@@ -383,6 +366,33 @@ public abstract class FObj extends FONode implements Constants {
         return markers;
     }
 
+    /** {@inheritDoc} */
+    protected String getContextInfoAlt() {
+        StringBuffer sb = new StringBuffer();
+        if (getLocalName() != null) {
+            sb.append(getName());
+            sb.append(", ");
+        }
+        if (hasId()) {
+            sb.append("id=").append(getId());
+            return sb.toString();
+        }
+        String s = gatherContextInfo();
+        if (s != null) {
+            sb.append("\"");
+            if (s.length() < 32) {
+                sb.append(s);
+            } else {
+                sb.append(s.substring(0, 32));
+                sb.append("...");
+            }
+            sb.append("\"");
+            return sb.toString();
+        } else {
+            return null;
+        }
+    }
+
     /** {@inheritDoc} */
     protected String gatherContextInfo() {
         if (getLocator() != null) {
@@ -412,17 +422,17 @@ public abstract class FObj extends FONode implements Constants {
      * incoming node is a member of the "%block;" parameter entity
      * as defined in Sect. 6.2 of the XSL 1.0 & 1.1 Recommendations
      * @param nsURI namespace URI of incoming node
-     * @param lName local name (i.e., no prefix) of incoming node 
+     * @param lName local name (i.e., no prefix) of incoming node
      * @return true if a member, false if not
      */
     protected boolean isBlockItem(String nsURI, String lName) {
-        return (FO_URI.equals(nsURI) 
-                && (lName.equals("block") 
-                        || lName.equals("table") 
-                        || lName.equals("table-and-caption") 
-                        || lName.equals("block-container")
-                        || lName.equals("list-block") 
-                        || lName.equals("float")
+        return (FO_URI.equals(nsURI)
+                && ("block".equals(lName)
+                        || "table".equals(lName)
+                        || "table-and-caption".equals(lName)
+                        || "block-container".equals(lName)
+                        || "list-block".equals(lName)
+                        || "float".equals(lName)
                         || isNeutralItem(nsURI, lName)));
     }
 
@@ -431,26 +441,26 @@ public abstract class FObj extends FONode implements Constants {
      * incoming node is a member of the "%inline;" parameter entity
      * as defined in Sect. 6.2 of the XSL 1.0 & 1.1 Recommendations
      * @param nsURI namespace URI of incoming node
-     * @param lName local name (i.e., no prefix) of incoming node 
+     * @param lName local name (i.e., no prefix) of incoming node
      * @return true if a member, false if not
      */
     protected boolean isInlineItem(String nsURI, String lName) {
-        return (FO_URI.equals(nsURI) 
-                && (lName.equals("bidi-override") 
-                        || lName.equals("character") 
-                        || lName.equals("external-graphic") 
-                        || lName.equals("instream-foreign-object")
-                        || lName.equals("inline") 
-                        || lName.equals("inline-container")
-                        || lName.equals("leader") 
-                        || lName.equals("page-number") 
-                        || lName.equals("page-number-citation")
-                        || lName.equals("page-number-citation-last")
-                        || lName.equals("basic-link")
-                        || (lName.equals("multi-toggle")
-                                && (getNameId() == FO_MULTI_CASE 
+        return (FO_URI.equals(nsURI)
+                && ("bidi-override".equals(lName)
+                        || "character".equals(lName)
+                        || "external-graphic".equals(lName)
+                        || "instream-foreign-object".equals(lName)
+                        || "inline".equals(lName)
+                        || "inline-container".equals(lName)
+                        || "leader".equals(lName)
+                        || "page-number".equals(lName)
+                        || "page-number-citation".equals(lName)
+                        || "page-number-citation-last".equals(lName)
+                        || "basic-link".equals(lName)
+                        || ("multi-toggle".equals(lName)
+                                && (getNameId() == FO_MULTI_CASE
                                         || findAncestor(FO_MULTI_CASE) > 0))
-                        || (lName.equals("footnote") 
+                        || ("footnote".equals(lName)
                                 && !isOutOfLineFODescendant)
                         || isNeutralItem(nsURI, lName)));
     }
@@ -460,7 +470,7 @@ public abstract class FObj extends FONode implements Constants {
      * incoming node is a member of the "%block;" parameter entity
      * or "%inline;" parameter entity
      * @param nsURI namespace URI of incoming node
-     * @param lName local name (i.e., no prefix) of incoming node 
+     * @param lName local name (i.e., no prefix) of incoming node
      * @return true if a member, false if not
      */
     protected boolean isBlockOrInlineItem(String nsURI, String lName) {
@@ -472,23 +482,24 @@ public abstract class FObj extends FONode implements Constants {
      * incoming node is a member of the neutral item list
      * as defined in Sect. 6.2 of the XSL 1.0 & 1.1 Recommendations
      * @param nsURI namespace URI of incoming node
-     * @param lName local name (i.e., no prefix) of incoming node 
+     * @param lName local name (i.e., no prefix) of incoming node
      * @return true if a member, false if not
      */
-    protected boolean isNeutralItem(String nsURI, String lName) {
-        return (FO_URI.equals(nsURI) 
-                && (lName.equals("multi-switch") 
-                        || lName.equals("multi-properties")
-                        || lName.equals("wrapper") 
-                        || (!isOutOfLineFODescendant && lName.equals("float"))
-                        || lName.equals("retrieve-marker")));
+    boolean isNeutralItem(String nsURI, String lName) {
+        return (FO_URI.equals(nsURI)
+                && ("multi-switch".equals(lName)
+                        || "multi-properties".equals(lName)
+                        || "wrapper".equals(lName)
+                        || (!isOutOfLineFODescendant && "float".equals(lName))
+                        || "retrieve-marker".equals(lName)
+                        || "retrieve-table-marker".equals(lName)));
     }
-    
+
     /**
      * Convenience method for validity checking.  Checks if the
      * current node has an ancestor of a given name.
      * @param ancestorID    ID of node name to check for (e.g., FO_ROOT)
-     * @return number of levels above FO where ancestor exists, 
+     * @return number of levels above FO where ancestor exists,
      *         -1 if not found
      */
     protected int findAncestor(int ancestorID) {
@@ -503,12 +514,19 @@ public abstract class FObj extends FONode implements Constants {
         }
         return -1;
     }
-    
+
+    /**
+     * Clears the list of child nodes.
+     */
+    public void clearChildNodes() {
+        this.firstChild = null;
+    }
+
     /** @return the "id" property. */
     public String getId() {
         return id;
     }
-    
+
     /** @return whether this object has an id set */
     public boolean hasId() {
         return id != null && id.length() > 0;
@@ -525,29 +543,29 @@ public abstract class FObj extends FONode implements Constants {
     }
 
     /**
-     * Add a new extension attachment to this FObj. 
+     * Add a new extension attachment to this FObj.
      * (see org.apache.fop.fo.FONode for details)
-     * 
+     *
      * @param attachment the attachment to add.
      */
-    public void addExtensionAttachment(ExtensionAttachment attachment) {
+    void addExtensionAttachment(ExtensionAttachment attachment) {
         if (attachment == null) {
             throw new NullPointerException(
                     "Parameter attachment must not be null");
         }
         if (extensionAttachments == null) {
-            extensionAttachments = new java.util.ArrayList();
+            extensionAttachments = new java.util.ArrayList/*<ExtensionAttachment>*/();
         }
         if (log.isDebugEnabled()) {
-            log.debug("ExtensionAttachment of category " 
-                    + attachment.getCategory() + " added to " 
+            log.debug("ExtensionAttachment of category "
+                    + attachment.getCategory() + " added to "
                     + getName() + ": " + attachment);
         }
         extensionAttachments.add(attachment);
     }
-    
+
     /** @return the extension attachments of this FObj. */
-    public List getExtensionAttachments() {
+    public List/*<ExtensionAttachment>*/ getExtensionAttachments() {
         if (extensionAttachments == null) {
             return Collections.EMPTY_LIST;
         } else {
@@ -555,24 +573,29 @@ public abstract class FObj extends FONode implements Constants {
         }
     }
 
+    /** @return true if this FObj has extension attachments */
+    public boolean hasExtensionAttachments() {
+        return extensionAttachments != null;
+    }
+
     /**
      * Adds a foreign attribute to this FObj.
      * @param attributeName the attribute name as a QName instance
      * @param value the attribute value
      */
     public void addForeignAttribute(QName attributeName, String value) {
-        /* TODO: Handle this over FOP's property mechanism so we can use 
+        /* TODO: Handle this over FOP's property mechanism so we can use
          *       inheritance.
          */
         if (attributeName == null) {
             throw new NullPointerException("Parameter attributeName must not be null");
         }
         if (foreignAttributes == null) {
-            foreignAttributes = new java.util.HashMap();
+            foreignAttributes = new java.util.HashMap/*<QName,String>*/();
         }
         foreignAttributes.put(attributeName, value);
     }
-    
+
     /** @return the map of foreign attributes */
     public Map getForeignAttributes() {
         if (foreignAttributes == null) {
@@ -581,41 +604,37 @@ public abstract class FObj extends FONode implements Constants {
             return foreignAttributes;
         }
     }
-    
+
     /** {@inheritDoc} */
     public String toString() {
         return (super.toString() + "[@id=" + this.id + "]");
     }
 
-
+    /** Basic {@link FONodeIterator} implementation */
     public class FObjIterator implements FONodeIterator {
-        
+
         private static final int F_NONE_ALLOWED = 0;
         private static final int F_SET_ALLOWED = 1;
         private static final int F_REMOVE_ALLOWED = 2;
-        
+
         private FONode currentNode;
-        private FObj parentNode;
+        private final FObj parentNode;
         private int currentIndex;
         private int flags = F_NONE_ALLOWED;
-        
-        protected FObjIterator(FObj parent) {
+
+        FObjIterator(FObj parent) {
             this.parentNode = parent;
             this.currentNode = parent.firstChild;
             this.currentIndex = 0;
             this.flags = F_NONE_ALLOWED;
         }
-        
-        /**
-         * {@inheritDoc}
-         */
+
+        /** {@inheritDoc} */
         public FObj parentNode() {
             return parentNode;
         }
-        
-        /**
-         * {@inheritDoc}
-         */
+
+        /** {@inheritDoc} */
         public Object next() {
             if (currentNode != null) {
                 if (currentIndex != 0) {
@@ -634,9 +653,7 @@ public abstract class FObj extends FONode implements Constants {
             }
         }
 
-        /**
-         * {@inheritDoc}
-         */
+        /** {@inheritDoc} */
         public Object previous() {
             if (currentNode.siblings != null
                     && currentNode.siblings[0] != null) {
@@ -648,10 +665,8 @@ public abstract class FObj extends FONode implements Constants {
                 throw new NoSuchElementException();
             }
         }
-        
-        /**
-         * {@inheritDoc}
-         */
+
+        /** {@inheritDoc} */
         public void set(Object o) {
             if ((flags & F_SET_ALLOWED) == F_SET_ALLOWED) {
                 FONode newNode = (FONode) o;
@@ -668,10 +683,8 @@ public abstract class FObj extends FONode implements Constants {
                 throw new IllegalStateException();
             }
         }
-        
-        /**
-         * {@inheritDoc}
-         */
+
+        /** {@inheritDoc} */
         public void add(Object o) {
             FONode newNode = (FONode) o;
             if (currentIndex == -1) {
@@ -691,9 +704,7 @@ public abstract class FObj extends FONode implements Constants {
             flags &= F_NONE_ALLOWED;
         }
 
-        /**
-         * {@inheritDoc}
-         */
+        /** {@inheritDoc} */
         public boolean hasNext() {
             return (currentNode != null)
                 && ((currentIndex == 0)
@@ -701,32 +712,24 @@ public abstract class FObj extends FONode implements Constants {
                             && currentNode.siblings[1] != null));
         }
 
-        /**
-         * {@inheritDoc}
-         */
+        /** {@inheritDoc} */
         public boolean hasPrevious() {
             return (currentIndex != 0)
                 || (currentNode.siblings != null
                     && currentNode.siblings[0] != null);
         }
-        
-        /**
-         * {@inheritDoc}
-         */
+
+        /** {@inheritDoc} */
         public int nextIndex() {
             return currentIndex + 1;
         }
 
-        /**
-         * {@inheritDoc}
-         */
+        /** {@inheritDoc} */
         public int previousIndex() {
             return currentIndex - 1;
         }
 
-        /**
-         * {@inheritDoc}
-         */
+        /** {@inheritDoc} */
         public void remove() {
             if ((flags & F_REMOVE_ALLOWED) == F_REMOVE_ALLOWED) {
                 parentNode.removeChild(currentNode);
@@ -746,9 +749,7 @@ public abstract class FObj extends FONode implements Constants {
             }
         }
 
-        /**
-         * {@inheritDoc}
-         */
+        /** {@inheritDoc} */
         public FONode lastNode() {
             while (currentNode != null
                     && currentNode.siblings != null
@@ -758,29 +759,22 @@ public abstract class FObj extends FONode implements Constants {
             }
             return currentNode;
         }
-        
-        /**
-         * {@inheritDoc}
-         */
+
+        /** {@inheritDoc} */
         public FONode firstNode() {
             currentNode = parentNode.firstChild;
             currentIndex = 0;
             return currentNode;
         }
-        
-        /**
-         * {@inheritDoc}
-         */
+
+        /** {@inheritDoc} */
         public FONode nextNode() {
             return (FONode) next();
         }
-        
-        /**
-         * {@inheritDoc}
-         */
+
+        /** {@inheritDoc} */
         public FONode previousNode() {
             return (FONode) previous();
         }
     }
-
 }
