@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,10 +15,16 @@
  * limitations under the License.
  */
 
-/* $Id: PropertyParser.java 620272 2008-02-10 12:31:46Z adelmelle $ */
+/* $Id: PropertyParser.java 766492 2009-04-19 17:54:06Z adelmelle $ */
 
 package org.apache.fop.fo.expr;
 
+import java.util.HashMap;
+import java.util.LinkedList;
+import java.util.List;
+
+import org.apache.xmlgraphics.util.UnitConv;
+
 import org.apache.fop.datatypes.Length;
 import org.apache.fop.datatypes.LengthBase;
 import org.apache.fop.datatypes.Numeric;
@@ -31,10 +37,6 @@ import org.apache.fop.fo.properties.PercentLength;
 import org.apache.fop.fo.properties.Property;
 import org.apache.fop.fo.properties.StringProperty;
 
-import java.util.HashMap;
-import java.util.LinkedList;
-import java.util.List;
-
 /**
  * Class to parse XSL-FO property expressions.
  * This class is heavily based on the epxression parser in James Clark's
@@ -45,7 +47,7 @@ public final class PropertyParser extends PropertyTokenizer {
 
     private static final String RELUNIT = "em";
     private static final HashMap FUNCTION_TABLE = new HashMap();
-    
+
     static {
         // Initialize the HashMap of XSL-defined functions
         FUNCTION_TABLE.put("ceiling", new CeilingFunction());
@@ -99,7 +101,7 @@ public final class PropertyParser extends PropertyTokenizer {
     /**
      * Private constructor. Called by the static parse() method.
      * @param propExpr The specified value (attribute on the xml element).
-     * @param propInfo A PropertyInfo object representing the context in
+     * @param pInfo A PropertyInfo object representing the context in
      * which the property expression is to be evaluated.
      */
     private PropertyParser(String propExpr, PropertyInfo pInfo) {
@@ -310,12 +312,13 @@ public final class PropertyParser extends PropertyTokenizer {
                                     propInfo.currentFontSize());
             } else {
                 if ("px".equals(unitPart)) {
-                    //pass the ratio between source-resolution and 
+                    //pass the ratio between target-resolution and
                     //the default resolution of 72dpi
+                    float resolution = propInfo.getPropertyList().getFObj()
+                            .getUserAgent().getSourceResolution();
                     prop = FixedLength.getInstance(
-                            numPart, unitPart, 
-                            propInfo.getPropertyList().getFObj()
-                                    .getUserAgent().getSourceResolution() / 72.0f);
+                            numPart, unitPart,
+                             UnitConv.IN2PT / resolution);
                 } else {
                     //use default resolution of 72dpi
                     prop = FixedLength.getInstance(numPart, unitPart);
@@ -344,7 +347,7 @@ public final class PropertyParser extends PropertyTokenizer {
             }
             propInfo.popFunction();
             return prop;
-        
+
         default:
             // TODO: add the token or the expr to the error message.
             throw new PropertyException("syntax error");
@@ -357,7 +360,7 @@ public final class PropertyParser extends PropertyTokenizer {
      * Parse a comma separated list of function arguments. Each argument
      * may itself be an expression. This method consumes the closing right
      * parenthesis of the argument list.
-     * @param function The function object for which the arguments are 
+     * @param function The function object for which the arguments are
      * collected.
      * @return An array of Property objects representing the arguments
      * found.
@@ -397,27 +400,27 @@ public final class PropertyParser extends PropertyTokenizer {
         }
         return args;
     }
-    
+
     /**
-     * 
+     *
      * Parse a comma separated list of function arguments. Each argument
      * may itself be an expression. This method consumes the closing right
      * parenthesis of the argument list.
-     * 
-     * The method differs from parseArgs in that it accepts a variable 
+     *
+     * The method differs from parseArgs in that it accepts a variable
      * number of arguments.
-     * 
-     * @param function The function object for which the arguments are 
+     *
+     * @param function The function object for which the arguments are
      * collected.
      * @return An array of Property objects representing the arguments
      * found.
      * @throws PropertyException If the number of arguments found isn't equal
      * to the number expected.
-     * 
-     * TODO Merge this with parseArgs? 
+     *
+     * TODO Merge this with parseArgs?
      */
     Property[] parseVarArgs(Function function) throws PropertyException {
-        // For variable argument functions the minimum number of arguments is returned as a 
+        // For variable argument functions the minimum number of arguments is returned as a
         // negative integer from the nbArgs method
         int nbArgs = -function.nbArgs();
         List args = new LinkedList();
