@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Block.java 669118 2008-06-18 09:02:45Z jeremias $ */
+/* $Id: Block.java 830293 2009-10-27 19:07:52Z vhennebert $ */
 
 package org.apache.fop.fo.flow;
 
@@ -40,11 +40,13 @@ import org.apache.fop.fo.properties.CommonMarginBlock;
 import org.apache.fop.fo.properties.CommonRelativePosition;
 import org.apache.fop.fo.properties.KeepProperty;
 import org.apache.fop.fo.properties.SpaceProperty;
+import org.apache.fop.fo.properties.StructurePointerPropertySet;
 
  /**
-  * Class modelling the fo:block object.
+  * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_block">
+  * <code>fo:block object</code></a>.
   */
-public class Block extends FObjMixed implements BreakPropertySet {
+public class Block extends FObjMixed implements BreakPropertySet, StructurePointerPropertySet {
 
     // used for FO validation
     private boolean blockOrInlineItemFound = false;
@@ -71,6 +73,7 @@ public class Block extends FObjMixed implements BreakPropertySet {
     private int lineHeightShiftAdjustment;
     private int lineStackingStrategy;
     private Numeric orphans;
+    private String ptr;  //used for accessibility
     private int whiteSpaceTreatment;
     private int span;
     private int textAlign;
@@ -79,6 +82,7 @@ public class Block extends FObjMixed implements BreakPropertySet {
     private int whiteSpaceCollapse;
     private Numeric widows;
     private int wrapOption;
+    private int disableColumnBalancing;
     // Unused but valid items, commented out for performance:
     //     private CommonAccessibility commonAccessibility;
     //     private CommonAural commonAural;
@@ -86,8 +90,10 @@ public class Block extends FObjMixed implements BreakPropertySet {
     //     private Length textAltitude;
     //     private int visibility;
     // End of property values
-    
+
     /**
+     * Base constructor
+     *
      * @param parent FONode that is the parent of this object
      *
      */
@@ -95,9 +101,7 @@ public class Block extends FObjMixed implements BreakPropertySet {
         super(parent);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void bind(PropertyList pList) throws FOPException {
         super.bind(pList);
         commonBorderPaddingBackground = pList.getBorderPaddingBackgroundProps();
@@ -121,6 +125,7 @@ public class Block extends FObjMixed implements BreakPropertySet {
         lineHeightShiftAdjustment = pList.get(PR_LINE_HEIGHT_SHIFT_ADJUSTMENT).getEnum();
         lineStackingStrategy = pList.get(PR_LINE_STACKING_STRATEGY).getEnum();
         orphans = pList.get(PR_ORPHANS).getNumeric();
+        ptr = pList.get(PR_X_PTR).getString();  //used for accessibility
         whiteSpaceTreatment = pList.get(PR_WHITE_SPACE_TREATMENT).getEnum();
         span = pList.get(PR_SPAN).getEnum();
         textAlign = pList.get(PR_TEXT_ALIGN).getEnum();
@@ -129,48 +134,38 @@ public class Block extends FObjMixed implements BreakPropertySet {
         whiteSpaceCollapse = pList.get(PR_WHITE_SPACE_COLLAPSE).getEnum();
         widows = pList.get(PR_WIDOWS).getNumeric();
         wrapOption = pList.get(PR_WRAP_OPTION).getEnum();
+        disableColumnBalancing = pList.get(PR_X_DISABLE_COLUMN_BALANCING).getEnum();
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected void startOfNode() throws FOPException {
         super.startOfNode();
         getFOEventHandler().startBlock(this);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected void endOfNode() throws FOPException {
         super.endOfNode();
         getFOEventHandler().endBlock(this);
     }
 
-    /**
-     * @return the Common Margin Properties-Block.
-     */
+    /** @return the {@link CommonMarginBlock} */
     public CommonMarginBlock getCommonMarginBlock() {
         return commonMarginBlock;
     }
 
-    /**
-     * @return the Common Border, Padding, and Background Properties.
-     */
+    /** @return the {@link CommonBorderPaddingBackground} */
     public CommonBorderPaddingBackground getCommonBorderPaddingBackground() {
         return commonBorderPaddingBackground;
     }
 
     /**
-     * @return the Common Font Properties.
-     */
+     * @return the {@link CommonFont} */
     public CommonFont getCommonFont() {
         return commonFont;
     }
 
-    /**
-     * @return the Common Hyphenation Properties.
-     */
+    /** @return the {@link CommonHyphenation} */
     public CommonHyphenation getCommonHyphenation() {
         return commonHyphenation;
     }
@@ -180,6 +175,11 @@ public class Block extends FObjMixed implements BreakPropertySet {
         return breakAfter;
     }
 
+    /** {@inheritDoc} */
+    public String getPtr() {
+        return ptr;
+    }
+
     /** @return the "break-before" property. */
     public int getBreakBefore() {
         return breakBefore;
@@ -220,83 +220,67 @@ public class Block extends FObjMixed implements BreakPropertySet {
         return lineStackingStrategy;
     }
 
-    /**
-     * @return the "color" property.
-     */
+    /** @return the "color" property */
     public Color getColor() {
         return color;
     }
 
-    /**
-     * @return the "line-height" property.
-     */
+    /** @return the "line-height" property */
     public SpaceProperty getLineHeight() {
         return lineHeight;
     }
 
-    /**
-     * @return the "span" property.
-     */
+    /** @return the "span" property */
     public int getSpan() {
         return this.span;
     }
 
-    /**
-     * @return the "text-align" property.
-     */
+    /** @return the "text-align" property */
     public int getTextAlign() {
         return textAlign;
     }
 
-    /**
-     * @return the "text-align-last" property.
-     */
+    /** @return the "text-align-last" property */
     public int getTextAlignLast() {
         return textAlignLast;
     }
 
-    /**
-     * @return the "text-indent" property.
-     */
+    /** @return the "text-indent" property */
     public Length getTextIndent() {
         return textIndent;
     }
 
-    /**
-     * @return the "last-line-end-indent" property.
-     */
+    /** @return the "last-line-end-indent" property */
     public Length getLastLineEndIndent() {
         return lastLineEndIndent;
     }
 
-    /**
-     * @return the "wrap-option" property.
-     */
+    /** @return the "wrap-option" property */
     public int getWrapOption() {
         return wrapOption;
     }
 
     /**
      * {@inheritDoc}
-     * XSL Content Model: marker* initial-property-set? (#PCDATA|%inline;|%block;)*
-     * Additionally: "An fo:bidi-override that is a descendant of an fo:leader
+     * <br>XSL Content Model: marker* initial-property-set? (#PCDATA|%inline;|%block;)*
+     * <br><i>Additionally: "An fo:bidi-override that is a descendant of an fo:leader
      *  or of the fo:inline child of an fo:footnote may not have block-level
-     *  children, unless it has a nearer ancestor that is an 
-     *  fo:inline-container."
+     *  children, unless it has a nearer ancestor that is an
+     *  fo:inline-container."</i>
      */
-    protected void validateChildNode(Locator loc, String nsURI, String localName) 
+    protected void validateChildNode(Locator loc, String nsURI, String localName)
             throws ValidationException {
         if (FO_URI.equals(nsURI)) {
-            if (localName.equals("marker")) {
+            if ("marker".equals(localName)) {
                 if (blockOrInlineItemFound || initialPropertySetFound) {
-                   nodesOutOfOrderError(loc, "fo:marker", 
+                   nodesOutOfOrderError(loc, "fo:marker",
                         "initial-property-set? (#PCDATA|%inline;|%block;)");
                 }
-            } else if (localName.equals("initial-property-set")) {
+            } else if ("initial-property-set".equals(localName)) {
                 if (initialPropertySetFound) {
                     tooManyNodesError(loc, "fo:initial-property-set");
                 } else if (blockOrInlineItemFound) {
-                    nodesOutOfOrderError(loc, "fo:initial-property-set", 
+                    nodesOutOfOrderError(loc, "fo:initial-property-set",
                         "(#PCDATA|%inline;|%block;)");
                 } else {
                     initialPropertySetFound = true;
@@ -309,61 +293,50 @@ public class Block extends FObjMixed implements BreakPropertySet {
         }
     }
 
-    /**
-     * Accessor for the linefeed-treatment property
-     * 
-     * @return the enum value of linefeed-treatment
-     */
+    /** @return the "linefeed-treatment" property */
     public int getLinefeedTreatment() {
         return linefeedTreatment;
     }
-    
-    /**
-     * Accessor for the white-space-treatment property
-     * 
-     * @return the enum value of white-space-treatment
-     */
+
+    /** @return the "white-space-treatment" property */
     public int getWhitespaceTreatment() {
         return whiteSpaceTreatment;
     }
-    
-    /**
-     * Accessor for the white-space-collapse property
-     * 
-     * @return the enum value of white-space-collapse
-     */
+
+    /** @return the "white-space-collapse" property */
     public int getWhitespaceCollapse() {
         return whiteSpaceCollapse;
     }
-    
-    /**
-     * @return Returns the commonRelativePosition.
-     */
+
+    /** @return the {@link CommonRelativePosition} */
     public CommonRelativePosition getCommonRelativePosition() {
         return this.commonRelativePosition;
     }
 
-    /**
-     * @return Returns the hyphenationKeep.
-     */
+    /** @return the "hyphenation-keep" property */
     public int getHyphenationKeep() {
         return this.hyphenationKeep;
     }
 
-    /**
-     * @return Returns the intrusionDisplace.
-     */
+    /** @return the "intrusion-displace" property */
     public int getIntrusionDisplace() {
         return this.intrusionDisplace;
     }
 
-    /**
-     * @return Returns the lineHeightShiftAdjustment.
-     */
+    /** @return the "line-height-shift-adjustment" property */
     public int getLineHeightShiftAdjustment() {
         return this.lineHeightShiftAdjustment;
     }
 
+     /**
+     * @return the "fox:disable-column-balancing" property, one of
+     * {@link Constants#EN_TRUE}, {@link Constants#EN_FALSE}
+     */
+     public int getDisableColumnBalancing() {
+         return disableColumnBalancing;
+     }
+
+
     /** {@inheritDoc} */
     public CharIterator charIterator() {
         return NullCharIterator.getInstance();
@@ -374,11 +347,12 @@ public class Block extends FObjMixed implements BreakPropertySet {
         return "block";
     }
 
-    /**
-     * {@inheritDoc}
-     */
+     /**
+      * {@inheritDoc}
+      * @return {@link org.apache.fop.fo.Constants#FO_BLOCK}
+      */
     public int getNameId() {
         return FO_BLOCK;
     }
-    
+
 }
