@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Footnote.java 557337 2007-07-18 17:37:14Z adelmelle $ */
+/* $Id: Footnote.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fo.flow;
 
@@ -28,7 +28,8 @@ import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.ValidationException;
 
 /**
- * Class modelling the fo:footnote object.
+ * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_footnote">
+ * <code>fo:footnote</code></a> object.
  */
 public class Footnote extends FObj {
     // The value of properties relevant for fo:footnote (commented out for performance).
@@ -39,29 +40,29 @@ public class Footnote extends FObj {
     private FootnoteBody footnoteBody;
 
     /**
-     * @param parent FONode that is the parent of this object
+     * Create a Footnote instance that is a child of the
+     * given {@link FONode}
+     *
+     * @param parent {@link FONode} that is the parent of this object
      */
     public Footnote(FONode parent) {
         super(parent);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void bind(PropertyList pList) throws FOPException {
         // No active properties -> do nothing.
     }
-    
-    /**
-     * {@inheritDoc}
-     */
+
+    /** {@inheritDoc} */
     protected void startOfNode() throws FOPException {
         getFOEventHandler().startFootnote(this);
     }
 
     /**
      * Make sure content model satisfied, if so then tell the
-     * FOEventHandler that we are at the end of the flow.
+     * {@link org.apache.fop.fo.FOEventHandler} that we are at the end of the footnote.
+     *
      * {@inheritDoc}
      */
     protected void endOfNode() throws FOPException {
@@ -74,44 +75,45 @@ public class Footnote extends FObj {
 
     /**
      * {@inheritDoc}
-     * XSL Content Model: (inline,footnote-body)
+     * <br>XSL Content Model: (inline,footnote-body)
      * @todo implement additional constraint: A fo:footnote is not permitted
      *      to have a fo:float, fo:footnote, or fo:marker as a descendant.
-     * @todo implement additional constraint: A fo:footnote is not 
-     *      permitted to have as a descendant a fo:block-container that 
+     * @todo implement additional constraint: A fo:footnote is not
+     *      permitted to have as a descendant a fo:block-container that
      *      generates an absolutely positioned area.
      */
-    protected void validateChildNode(Locator loc, String nsURI, String localName) 
-        throws ValidationException {
-            if (FO_URI.equals(nsURI) && localName.equals("inline")) {
+    protected void validateChildNode(Locator loc, String nsURI, String localName)
+                throws ValidationException {
+        if (FO_URI.equals(nsURI)) {
+            if (localName.equals("inline")) {
                 if (footnoteCitation != null) {
                     tooManyNodesError(loc, "fo:inline");
                 }
-            } else if (FO_URI.equals(nsURI) && localName.equals("footnote-body")) {
+            } else if (localName.equals("footnote-body")) {
                 if (footnoteCitation == null) {
                     nodesOutOfOrderError(loc, "fo:inline", "fo:footnote-body");
                 } else if (footnoteBody != null) {
                     tooManyNodesError(loc, "fo:footnote-body");
-                }                
+                }
             } else {
                 invalidChildError(loc, nsURI, localName);
             }
+        }
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void addChildNode(FONode child) {
-        if (((FObj)child).getNameId() == FO_INLINE) {
+        if (child.getNameId() == FO_INLINE) {
             footnoteCitation = (Inline) child;
-        } else if (((FObj)child).getNameId() == FO_FOOTNOTE_BODY) {
+        } else if (child.getNameId() == FO_FOOTNOTE_BODY) {
             footnoteBody = (FootnoteBody) child;
         }
     }
 
     /**
      * Public accessor for inline FO
-     * @return the Inline child
+     *
+     * @return the {@link Inline} child
      */
     public Inline getFootnoteCitation() {
         return footnoteCitation;
@@ -119,7 +121,8 @@ public class Footnote extends FObj {
 
     /**
      * Public accessor for footnote-body FO
-     * @return the FootnoteBody child
+     *
+     * @return the {@link FootnoteBody} child
      */
     public FootnoteBody getFootnoteBody() {
         return footnoteBody;
@@ -129,9 +132,10 @@ public class Footnote extends FObj {
     public String getLocalName() {
         return "footnote";
     }
-    
+
     /**
      * {@inheritDoc}
+     * @return {@link org.apache.fop.fo.Constants#FO_FOOTNOTE}
      */
     public int getNameId() {
         return FO_FOOTNOTE;
