@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: InlineLevel.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: InlineLevel.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fo.flow;
 
@@ -25,8 +25,6 @@ import org.apache.fop.apps.FOPException;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.FObjMixed;
 import org.apache.fop.fo.PropertyList;
-import org.apache.fop.fo.properties.CommonAccessibility;
-import org.apache.fop.fo.properties.CommonAural;
 import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
 import org.apache.fop.fo.properties.CommonFont;
 import org.apache.fop.fo.properties.CommonMarginInline;
@@ -38,73 +36,71 @@ import org.apache.fop.fo.properties.SpaceProperty;
  * formatting objects.
  */
 public abstract class InlineLevel extends FObjMixed {
-    
+
     // The value of properties relevant for inline-level FOs.
-    protected CommonBorderPaddingBackground commonBorderPaddingBackground;
-    protected CommonAccessibility commonAccessibility;
-    protected CommonMarginInline commonMarginInline;
-    protected CommonAural commonAural;
-    protected CommonFont commonFont;
-    protected Color color;
-    protected SpaceProperty lineHeight;
-    protected int visibility;
+    private CommonBorderPaddingBackground commonBorderPaddingBackground;
+    private CommonMarginInline commonMarginInline;
+    private CommonFont commonFont;
+    private Color color;
+    private KeepProperty keepWithNext;
+    private KeepProperty keepWithPrevious;
+    private SpaceProperty lineHeight;
     // End of property values
 
     /**
-     * @param parent FONode that is the parent of this object
+     * Base constructor
+     *
+     * @param parent {@link FONode} that is the parent of this object
      */
     protected InlineLevel(FONode parent) {
         super(parent);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void bind(PropertyList pList) throws FOPException {
         super.bind(pList);
         commonBorderPaddingBackground = pList.getBorderPaddingBackgroundProps();
-        commonAccessibility = pList.getAccessibilityProps();
         commonMarginInline = pList.getMarginInlineProps();
-        commonAural = pList.getAuralProps();
         commonFont = pList.getFontProps();
         color = pList.get(PR_COLOR).getColor(getUserAgent());
+        keepWithNext = pList.get(PR_KEEP_WITH_NEXT).getKeep();
+        keepWithPrevious = pList.get(PR_KEEP_WITH_PREVIOUS).getKeep();
         lineHeight = pList.get(PR_LINE_HEIGHT).getSpace();
-        visibility = pList.get(PR_VISIBILITY).getEnum();
     }
 
-    /**
-     * @return the Common Margin Properties-Inline.
-     */
+    /** @return the {@link CommonMarginInline} */
     public CommonMarginInline getCommonMarginInline() {
         return commonMarginInline;
     }
 
-    /**
-     * @return the Common Border, Padding, and Background Properties.
-     */
+    /** @return the {@link CommonBorderPaddingBackground} */
     public CommonBorderPaddingBackground getCommonBorderPaddingBackground() {
         return commonBorderPaddingBackground;
-    } 
+    }
 
-    /**
-     * @return the Common Font Properties.
-     */
+    /** @return the {@link CommonFont} */
     public CommonFont getCommonFont() {
         return commonFont;
     }
 
-    /**
-     * @return the "color" property.
-     */
+    /** @return the "color" property */
     public Color getColor() {
         return color;
     }
 
-    /**
-     * @return the "line-height" property
-     */
+    /** @return the "line-height" property */
     public SpaceProperty getLineHeight() {
         return lineHeight;
     }
-    
+
+    /** @return the "keep-with-next" property */
+    public KeepProperty getKeepWithNext() {
+        return keepWithNext;
+    }
+
+    /** @return the "keep-with-previous" property */
+    public KeepProperty getKeepWithPrevious() {
+        return keepWithPrevious;
+    }
+
 }
