@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: MultiSwitch.java 557337 2007-07-18 17:37:14Z adelmelle $ */
+/* $Id: MultiSwitch.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fo.flow;
 
@@ -29,7 +29,8 @@ import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.ValidationException;
 
 /**
- * Class modelling the fo:multi-switch object.
+ * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_multi-switch">
+ * <code>fo:multi-switch</code></a> object.
  */
 public class MultiSwitch extends FObj {
     // The value of properties relevant for fo:multi-switch.
@@ -41,29 +42,28 @@ public class MultiSwitch extends FObj {
     static boolean notImplementedWarningGiven = false;
 
     /**
-     * @param parent FONode that is the parent of this object
+     * Base constructor
+     *
+     * @param parent {@link FONode} that is the parent of this object
      */
     public MultiSwitch(FONode parent) {
         super(parent);
 
         if (!notImplementedWarningGiven) {
-            log.warn("fo:multi-switch is not yet implemented.");
+            getFOValidationEventProducer().unimplementedFeature(this, getName(),
+                    getName(), getLocator());
             notImplementedWarningGiven = true;
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void bind(PropertyList pList) throws FOPException {
         super.bind(pList);
         // autoRestore = pList.get(PR_AUTO_RESTORE);
     }
 
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected void endOfNode() throws FOPException {
         if (firstChild == null) {
             missingChildElementError("(multi-case+)");
@@ -72,12 +72,14 @@ public class MultiSwitch extends FObj {
 
     /**
      * {@inheritDoc}
-     * XSL Content Model: (multi-case+)
+     * <br>XSL Content Model: (multi-case+)
      */
-    protected void validateChildNode(Locator loc, String nsURI, String localName) 
-        throws ValidationException {
-        if (!(FO_URI.equals(nsURI) && localName.equals("multi-case"))) {
-            invalidChildError(loc, nsURI, localName);
+    protected void validateChildNode(Locator loc, String nsURI, String localName)
+                throws ValidationException {
+        if (FO_URI.equals(nsURI)) {
+            if (!localName.equals("multi-case")) {
+                invalidChildError(loc, nsURI, localName);
+            }
         }
     }
 
@@ -88,6 +90,7 @@ public class MultiSwitch extends FObj {
 
     /**
      * {@inheritDoc}
+     * @return {@link org.apache.fop.fo.Constants#FO_MULTI_SWITCH}
      */
     public int getNameId() {
         return FO_MULTI_SWITCH;
