@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Table.java 669118 2008-06-18 09:02:45Z jeremias $ */
+/* $Id: Table.java 681307 2008-07-31 09:06:10Z jeremias $ */
 
 package org.apache.fop.fo.flow.table;
 
@@ -40,7 +40,8 @@ import org.apache.fop.fo.properties.LengthRangeProperty;
 import org.apache.fop.fo.properties.TableColLength;
 
 /**
- * Class modelling the fo:table object.
+ * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_table">
+ * <code>fo:table</code></a> object.
  */
 public class Table extends TableFObj implements ColumnNumberManagerHolder, BreakPropertySet {
 
@@ -76,8 +77,8 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
     private ColumnNumberManager columnNumberManager = new ColumnNumberManager();
 
     /** the table-header and -footer */
-    private TableBody tableHeader = null;
-    private TableBody tableFooter = null;
+    private TableHeader tableHeader = null;
+    private TableFooter tableFooter = null;
 
     /** used for validation */
     private boolean tableColumnFound = false;
@@ -97,7 +98,10 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
     private PropertyList propList;
 
     /**
-     * @param parent FONode that is the parent of this object
+     * Construct a Table instance with the given {@link FONode}
+     * as parent.
+     *
+     * @param parent {@link FONode} that is the parent of this object
      */
     public Table(FONode parent) {
         super(parent);
@@ -128,20 +132,22 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
         orphanContentLimit = pList.get(PR_X_ORPHAN_CONTENT_LIMIT).getLength();
 
         if (!blockProgressionDimension.getOptimum(null).isAuto()) {
-            attributeWarning("only a value of \"auto\" for block-progression-dimension has a"
-                    + " well-specified behavior on fo:table. Falling back to \"auto\"");
+            TableEventProducer eventProducer = TableEventProducer.Provider.get(
+                    getUserAgent().getEventBroadcaster());
+            eventProducer.nonAutoBPDOnTable(this, getLocator());
             // Anyway, the bpd of a table is not used by the layout code
         }
         if (tableLayout == EN_AUTO) {
-            attributeWarning("table-layout=\"auto\" is currently not supported by FOP");
+            getFOValidationEventProducer().unimplementedFeature(this, getName(),
+                    "table-layout=\"auto\"", getLocator());
         }
         if (!isSeparateBorderModel()
                 && getCommonBorderPaddingBackground().hasPadding(
                         ValidationPercentBaseContext.getPseudoContext())) {
             //See "17.6.2 The collapsing border model" in CSS2
-            attributeWarning("In collapsing border model a table does not have padding"
-                    + " (see http://www.w3.org/TR/REC-CSS2/tables.html#collapsing-borders)"
-                    + ", but a non-zero value for padding was found. The padding will be ignored.");
+            TableEventProducer eventProducer = TableEventProducer.Provider.get(
+                    getUserAgent().getEventBroadcaster());
+            eventProducer.noTablePaddingWithCollapsingBorderModel(this, getLocator());
         }
 
         /* Store reference to the property list, so
@@ -152,20 +158,18 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
         this.propList = pList;
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public void startOfNode() throws FOPException {
+    /** {@inheritDoc} */
+    protected void startOfNode() throws FOPException {
         super.startOfNode();
         getFOEventHandler().startTable(this);
     }
 
     /**
      * {@inheritDoc}
-     * XSL Content Model: (marker*,table-column*,table-header?,table-footer?,table-body+)
+     * <br>XSL Content Model: (marker*,table-column*,table-header?,table-footer?,table-body+)
      */
     protected void validateChildNode(Locator loc, String nsURI, String localName)
-        throws ValidationException {
+                throws ValidationException {
         if (FO_URI.equals(nsURI)) {
             if ("marker".equals(localName)) {
                 if (tableColumnFound || tableHeaderFound || tableFooterFound
@@ -196,14 +200,12 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
                     tableFooterFound = true;
                     if (tableBodyFound) {
                         if (getUserAgent().validateStrictly()) {
-                            nodesOutOfOrderError(loc, "fo:table-footer", "(table-body+)");
-                        } else if (!isSeparateBorderModel()) {
-                            nodesOutOfOrderError(loc, "fo:table-footer", "(table-body+)."
-                                    + " This table uses the collapsing border"
-                                    + " model. In order to resolve borders in an efficient way"
-                                    + " the table-footer must be known before any table-body"
-                                    + " is parsed. Either put the footer at the correct place"
-                                    + " or switch to the separate border model");
+                            nodesOutOfOrderError(loc, "fo:table-footer", "(table-body+)", true);
+                        }
+                        if (!isSeparateBorderModel()) {
+                            TableEventProducer eventProducer = TableEventProducer.Provider.get(
+                                    getUserAgent().getEventBroadcaster());
+                            eventProducer.footerOrderCannotRecover(this, getName(), getLocator());
                         }
                     }
                 }
@@ -212,21 +214,27 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
             } else {
                 invalidChildError(loc, nsURI, localName);
             }
-        } else {
-            invalidChildError(loc, nsURI, localName);
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public void endOfNode() throws FOPException {
+    /** {@inheritDoc} */
+    protected void endOfNode() throws FOPException {
+        super.endOfNode();
+        getFOEventHandler().endTable(this);
+    }
+
+    /** {@inheritDoc} */
+    public void finalizeNode() throws FOPException {
 
         if (!tableBodyFound) {
            missingChildElementError(
                    "(marker*,table-column*,table-header?,table-footer?"
                        + ",table-body+)");
         }
+        if (!hasChildren()) {
+            getParent().removeChild(this);
+            return;
+        }
         if (!inMarker()) {
             rowGroupBuilder.endTable();
             /* clean up */
@@ -239,13 +247,10 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
             this.propList = null;
             rowGroupBuilder = null;
         }
-        getFOEventHandler().endTable(this);
 
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected void addChildNode(FONode child) throws FOPException {
 
         int childId = child.getNameId();
@@ -256,13 +261,13 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
             if (!inMarker()) {
                 addColumnNode((TableColumn) child);
             } else {
-                columns.add((TableColumn) child);
+                columns.add(child);
             }
             break;
         case FO_TABLE_HEADER:
         case FO_TABLE_FOOTER:
         case FO_TABLE_BODY:
-            if (!columnsFinalized) {
+            if (!inMarker() && !columnsFinalized) {
                 columnsFinalized = true;
                 if (hasExplicitColumns) {
                     finalizeColumns();
@@ -270,14 +275,14 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
                 } else {
                     rowGroupBuilder = new VariableColRowGroupBuilder(this);
                 }
-                
+
             }
             switch (childId) {
             case FO_TABLE_FOOTER:
-                tableFooter = (TableBody) child;
+                tableFooter = (TableFooter) child;
                 break;
             case FO_TABLE_HEADER:
-                tableHeader = (TableBody) child;
+                tableHeader = (TableHeader) child;
                 break;
             default:
                 super.addChildNode(child);
@@ -288,14 +293,6 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
         }
     }
 
-    /** {@inheritDoc} */
-    protected void setCollapsedBorders() {
-        createBorder(CommonBorderPaddingBackground.START);
-        createBorder(CommonBorderPaddingBackground.END);
-        createBorder(CommonBorderPaddingBackground.BEFORE);
-        createBorder(CommonBorderPaddingBackground.AFTER);
-    }
-
     private void finalizeColumns() throws FOPException {
         for (int i = 0; i < columns.size(); i++) {
             if (columns.get(i) == null) {
@@ -313,7 +310,7 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
      * Creates the appropriate number of additional implicit columns to match the given
      * column number. Used when the table has no explicit column: the number of columns is
      * then determined by the row that has the most columns.
-     * 
+     *
      * @param columnNumber the table must at least have this number of column
      * @throws FOPException if there was an error creating the property list for implicit
      * columns
@@ -343,9 +340,8 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
     /**
      * Adds a column to the columns List, and updates the columnIndex
      * used for determining initial values for column-number
-     * 
+     *
      * @param col   the column to add
-     * @throws FOPException
      */
     private void addColumnNode(TableColumn col) {
 
@@ -390,7 +386,7 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
 
     /**
      * Returns the column at the given index.
-     * 
+     *
      * @param index index of the column to be retrieved, 0-based
      * @return the corresponding column (may be an implicitly created column)
      */
@@ -400,7 +396,7 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
 
     /**
      * Returns the number of columns of this table.
-     * 
+     *
      * @return the number of columns, implicit or explicit, in this table
      */
     public int getNumberOfColumns() {
@@ -408,12 +404,12 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
     }
 
     /** @return the body for the table-header. */
-    public TableBody getTableHeader() {
+    public TableHeader getTableHeader() {
         return tableHeader;
     }
 
     /** @return the body for the table-footer. */
-    public TableBody getTableFooter() {
+    public TableFooter getTableFooter() {
         return tableFooter;
     }
 
@@ -519,22 +515,25 @@ public class Table extends TableFObj implements ColumnNumberManagerHolder, Break
         return "table";
     }
 
-    /** {@inheritDoc} */
+    /**
+     * {@inheritDoc}
+     * @return {@link org.apache.fop.fo.Constants#FO_TABLE}
+     */
     public int getNameId() {
         return FO_TABLE;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public FONode clone(FONode parent, boolean removeChildren)
         throws FOPException {
         Table clone = (Table) super.clone(parent, removeChildren);
-        clone.columnsFinalized = false;
         if (removeChildren) {
             clone.columns = new ArrayList();
+            clone.columnsFinalized = false;
+            clone.columnNumberManager = new ColumnNumberManager();
             clone.tableHeader = null;
             clone.tableFooter = null;
+            clone.rowGroupBuilder = null;
         }
         return clone;
     }
