@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ColorProfile.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: ColorProfile.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fo.pagination;
 
@@ -28,7 +28,9 @@ import org.apache.fop.fo.ValidationException;
 import org.xml.sax.Locator;
 
 /**
- * The fo:color-profile formatting object.
+ * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_color-profile">
+ * <code>fo:color-profile</code></a> object.
+ *
  * This loads the color profile when needed and resolves a requested color.
  */
 public class ColorProfile extends FObj {
@@ -39,15 +41,15 @@ public class ColorProfile extends FObj {
     // End of property values
 
     /**
-     * @see org.apache.fop.fo.FONode#FONode(FONode)
+     * Base constructor
+     *
+     * @param parent {@link FONode} that is the parent of this object
      */
     public ColorProfile(FONode parent) {
         super(parent);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void bind(PropertyList pList) throws FOPException {
         src = pList.get(PR_SRC).getString();
         colorProfileName = pList.get(PR_COLOR_PROFILE_NAME).getString();
@@ -56,15 +58,17 @@ public class ColorProfile extends FObj {
 
     /**
      * {@inheritDoc}
-        XSL 1.0/FOP: EMPTY (no child nodes permitted)
+     * <br>XSL 1.0/FOP: EMPTY (no child nodes permitted)
      */
-    protected void validateChildNode(Locator loc, String nsURI, String localName) 
-        throws ValidationException {
-        invalidChildError(loc, nsURI, localName);
+    protected void validateChildNode(Locator loc, String nsURI, String localName)
+                throws ValidationException {
+        if (FO_URI.equals(nsURI)) {
+            invalidChildError(loc, nsURI, localName);
+        }
     }
 
     /**
-     * Return the "color-profile-name" property.
+     * @return the "color-profile-name" property.
      */
     public String getColorProfileName() {
         return colorProfileName;
@@ -74,33 +78,34 @@ public class ColorProfile extends FObj {
     public String getLocalName() {
         return "color-profile";
     }
-    
+
     /**
      * {@inheritDoc}
+     * @return {@link org.apache.fop.fo.Constants#FO_COLOR_PROFILE}
      */
     public int getNameId() {
         return FO_COLOR_PROFILE;
     }
-    
-    /** 
+
+    /**
      * Get src attribute
-     * 
-     * @return Value of color-profile src attribute
+     *
+     * @return value of color-profile src attribute
      */
     public String getSrc() {
         return this.src;
     }
-    
+
     /**
      * Get rendering-intent attribute
-     * 
+     *
      * Returned value is one of
      *   Constants.EN_AUTO
      *   Constants.EN_PERCEPTUAL
      *   Constants.EN_RELATIVE_COLOMETRIC
      *   Constants.EN_SATURATION
      *   Constants.EN_ABSOLUTE_COLORMETRIC
-     *    
+     *
      * @return Rendering intent attribute
      */
     public int getRenderingIntent() {
