@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RegionAfter.java 557337 2007-07-18 17:37:14Z adelmelle $ */
+/* $Id: RegionAfter.java 757256 2009-03-22 21:08:48Z adelmelle $ */
 
 package org.apache.fop.fo.pagination;
 
@@ -26,46 +26,31 @@ import java.awt.Rectangle;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.datatypes.FODimension;
 import org.apache.fop.datatypes.LengthBase;
-import org.apache.fop.datatypes.SimplePercentBaseContext;
+import org.apache.fop.datatypes.PercentBaseContext;
 
 /**
- * The fo:region-after element.
+ * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_region-after">
+ * <code>fo:region-after</code></a> object.
  */
 public class RegionAfter extends RegionBA {
 
     /**
-     * @see org.apache.fop.fo.FONode#FONode(FONode)
+     * Create a RegionAfter instance that is a child of the
+     * given parent {@link FONode}.
+     * @param parent    the {@link FONode} that is to be the parent
      */
     public RegionAfter(FONode parent) {
         super(parent);
     }
 
-    /**
-     * {@inheritDoc} 
-     */
+    /** {@inheritDoc} */
     public Rectangle getViewportRectangle (FODimension reldims, SimplePageMaster spm) {
-        /* Special rules apply to resolving extent as values are resolved relative 
+        /* Special rules apply to resolving extent as values are resolved relative
          * to the page size and reference orientation.
          */
-        SimplePercentBaseContext pageWidthContext;
-        SimplePercentBaseContext pageHeightContext;
-        if (spm.getReferenceOrientation() % 180 == 0) {
-            pageWidthContext = new SimplePercentBaseContext(null, 
-                                                            LengthBase.CUSTOM_BASE,
-                                                            spm.getPageWidth().getValue());
-            pageHeightContext = new SimplePercentBaseContext(null,
-                                                             LengthBase.CUSTOM_BASE,
-                                                             spm.getPageHeight().getValue());
-        } else {
-            // invert width and height since top left are rotated by 90 (cl or ccl)
-            pageWidthContext = new SimplePercentBaseContext(null, 
-                                                            LengthBase.CUSTOM_BASE,
-                                                            spm.getPageHeight().getValue());
-            pageHeightContext = new SimplePercentBaseContext(null,
-                                                             LengthBase.CUSTOM_BASE,
-                                                             spm.getPageWidth().getValue());
-        }
-        SimplePercentBaseContext neighbourContext;
+        PercentBaseContext pageWidthContext = getPageWidthContext(LengthBase.CUSTOM_BASE);
+        PercentBaseContext pageHeightContext = getPageHeightContext(LengthBase.CUSTOM_BASE);
+        PercentBaseContext neighbourContext;
         Rectangle vpRect;
         if (spm.getWritingMode() == EN_LR_TB || spm.getWritingMode() == EN_RL_TB) {
             neighbourContext = pageWidthContext;
@@ -82,9 +67,7 @@ public class RegionAfter extends RegionBA {
         return vpRect;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected String getDefaultRegionName() {
         return "xsl-region-after";
     }
@@ -96,6 +79,7 @@ public class RegionAfter extends RegionBA {
 
     /**
      * {@inheritDoc}
+     * @return {@link org.apache.fop.fo.Constants#FO_REGION_AFTER}
      */
     public int getNameId() {
         return FO_REGION_AFTER;
