@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,34 +15,31 @@
  * limitations under the License.
  */
 
-/* $Id: SubSequenceSpecifier.java 607032 2007-12-27 10:34:15Z jeremias $ */
- 
+/* $Id: SubSequenceSpecifier.java 698280 2008-09-23 18:48:34Z adelmelle $ */
+
 package org.apache.fop.fo.pagination;
 
-import org.apache.fop.apps.FOPException;
 
 /**
- * Classes that implement this interface can be added to a PageSequenceMaster,
- * and are capable of looking up an appropriate PageMaster.
+ * Classes that implement this interface can be added to a {@link PageSequenceMaster},
+ * and are capable of looking up an appropriate {@link SimplePageMaster}.
  */
 public interface SubSequenceSpecifier {
-    
+
     /**
      * Returns the name of the next page master.
      * @param isOddPage True if the next page number is odd
      * @param isFirstPage True if the next page is the first
      * @param isLastPage True if the next page is the last
-     * @param isOnlyPage True if the next page is the only page
      * @param isBlankPage True if the next page is blank
      * @return the page master name
-     * @throws FOPException if there's a problem determining the next page master
+     * @throws PageProductionException if there's a problem determining the next page master
      */
     String getNextPageMasterName(boolean isOddPage,
                                  boolean isFirstPage,
                                  boolean isLastPage,
-                                 boolean isOnlyPage,
                                  boolean isBlankPage)
-                                    throws FOPException;
+                                    throws PageProductionException;
 
     /**
      * Called before a new page sequence is rendered so subsequences can reset
@@ -58,9 +55,9 @@ public interface SubSequenceSpecifier {
 
     /** @return true if the subsequence has a page master for page-position "last" */
     boolean hasPagePositionLast();
-    
+
     /** @return true if the subsequence has a page master for page-position "only" */
     boolean hasPagePositionOnly();
-    
+
 }
 
