@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -20,6 +20,7 @@
 package org.apache.fop.fo.pagination.bookmarks;
 
 import java.util.ArrayList;
+
 import org.xml.sax.Locator;
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.fo.FObj;
@@ -29,9 +30,9 @@ import org.apache.fop.fo.ValidationException;
 
 
 /**
- * The fo:bookmark formatting object, first introduced in the 
- * XSL 1.1 WD.  Prototype version only, subject to change as
- * XSL 1.1 WD evolves.
+ * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_bookmark">
+ * <code>fo:bookmark</code></a> object, first introduced in the
+ * XSL 1.1 WD.
  */
 public class Bookmark extends FObj {
     private BookmarkTitle bookmarkTitle;
@@ -41,13 +42,14 @@ public class Bookmark extends FObj {
     private String internalDestination;
     private String externalDestination;
     private boolean bShow = true; // from starting-state property
-    
+
     // Valid, but unused properties. Commented out for performance
     // private CommonAccessibility commonAccessibility;
 
 
     /**
-     * Create a new bookmark object.
+     * Create a new Bookmark object that is a child of the
+     * given {@link FONode}.
      *
      * @param parent the parent fo node
      */
@@ -55,57 +57,53 @@ public class Bookmark extends FObj {
         super(parent);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void bind(PropertyList pList) throws FOPException {
         externalDestination = pList.get(PR_EXTERNAL_DESTINATION).getString();
         internalDestination = pList.get(PR_INTERNAL_DESTINATION).getString();
         bShow = (pList.get(PR_STARTING_STATE).getEnum() == EN_SHOW);
 
-        // per spec, internal takes precedence if both specified        
-        if (internalDestination.length() > 0) { 
+        // per spec, internal takes precedence if both specified
+        if (internalDestination.length() > 0) {
             externalDestination = null;
         } else if (externalDestination.length() == 0) {
             // slightly stronger than spec "should be specified"
-            attributeError("Missing attribute:  Either external-destination or " +
-                "internal-destination must be specified.");
+            getFOValidationEventProducer().missingLinkDestination(this, getName(), locator);
         } else {
-            attributeWarning("external-destination property not currently supported");
+            getFOValidationEventProducer().unimplementedFeature(this, getName(),
+                    "external-destination", getLocator());
         }
     }
 
     /**
      * {@inheritDoc}
-        XSL/FOP: (bookmark-title, bookmark*)
+     * <br>XSL/FOP: (bookmark-title, bookmark*)
      */
-    protected void validateChildNode(Locator loc, String nsURI, String localName) 
-        throws ValidationException {
-            if (FO_URI.equals(nsURI) && localName.equals("bookmark-title")) {
+    protected void validateChildNode(Locator loc, String nsURI, String localName)
+                throws ValidationException {
+        if (FO_URI.equals(nsURI)) {
+            if (localName.equals("bookmark-title")) {
                 if (bookmarkTitle != null) {
                     tooManyNodesError(loc, "fo:bookmark-title");
                 }
-            } else if (FO_URI.equals(nsURI) && localName.equals("bookmark")) {
+            } else if (localName.equals("bookmark")) {
                 if (bookmarkTitle == null) {
                     nodesOutOfOrderError(loc, "fo:bookmark-title", "fo:bookmark");
-                }                
+                }
             } else {
                 invalidChildError(loc, nsURI, localName);
             }
+        }
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected void endOfNode() throws FOPException {
         if (bookmarkTitle == null) {
            missingChildElementError("(bookmark-title, bookmark*)");
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected void addChildNode(FONode obj) {
         if (obj instanceof BookmarkTitle) {
             bookmarkTitle = (BookmarkTitle)obj;
@@ -123,10 +121,18 @@ public class Bookmark extends FObj {
         return bookmarkTitle == null ? "" : bookmarkTitle.getTitle();
     }
 
+    /**
+     * Returns the value of the internal-destination property.
+     * @return the internal-destination
+     */
     public String getInternalDestination() {
         return internalDestination;
     }
 
+    /**
+     * Returns the value of the external-destination property.
+     * @return the external-destination
+     */
     public String getExternalDestination() {
         return externalDestination;
     }
@@ -141,6 +147,10 @@ public class Bookmark extends FObj {
         return bShow;
     }
 
+    /**
+     * Get the child <code>Bookmark</code>s in an <code>java.util.ArrayList</code>.
+     * @return an <code>ArrayList</code> containing the child Bookmarks
+     */
     public ArrayList getChildBookmarks() {
         return childBookmarks;
     }
@@ -152,6 +162,7 @@ public class Bookmark extends FObj {
 
     /**
      * {@inheritDoc}
+     * @return {@link org.apache.fop.fo.Constants#FO_BOOKMARK}
      */
     public int getNameId() {
         return FO_BOOKMARK;
