@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: CharacterProperty.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: CharacterProperty.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fo.properties;
 
@@ -26,7 +26,7 @@ import org.apache.fop.fo.PropertyList;
  * Superclass for properties that wrap a character value
  */
 public final class CharacterProperty extends Property {
-    
+
     /**
      * Inner class for creating instances of CharacterProperty
      */
@@ -48,7 +48,7 @@ public final class CharacterProperty extends Property {
     }
 
     /** cache containing all canonical CharacterProperty instances */
-    private static final PropertyCache cache = new PropertyCache();
+    private static final PropertyCache cache = new PropertyCache(CharacterProperty.class);
 
     private final char character;
 
@@ -58,7 +58,7 @@ public final class CharacterProperty extends Property {
     private CharacterProperty(char character) {
         this.character = character;
     }
-    
+
     public static CharacterProperty getInstance(char character) {
         return (CharacterProperty) cache.fetch(
                         new CharacterProperty(character));
@@ -85,7 +85,7 @@ public final class CharacterProperty extends Property {
         return new Character(character).toString();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
     public boolean equals(Object obj) {
@@ -96,7 +96,7 @@ public final class CharacterProperty extends Property {
         }
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
     public int hashCode() {
