@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,12 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: CommonHyphenation.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: CommonHyphenation.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fo.properties;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.expr.PropertyException;
@@ -36,11 +37,11 @@ public final class CommonHyphenation {
 
     /** Logger */
     protected static Log log = LogFactory.getLog(CommonHyphenation.class);
-    
-    private static final PropertyCache cache = new PropertyCache();
-    
+
+    private static final PropertyCache cache = new PropertyCache(CommonHyphenation.class);
+
     private int hash = 0;
-    
+
     /** The "language" property */
     public final StringProperty language;
 
@@ -64,7 +65,7 @@ public final class CommonHyphenation {
 
     /**
      * Construct a CommonHyphenation object holding the given properties
-     * 
+     *
      */
     private CommonHyphenation(StringProperty language,
                               StringProperty country,
@@ -81,46 +82,46 @@ public final class CommonHyphenation {
         this.hyphenationPushCharacterCount = hyphenationPushCharacterCount;
         this.hyphenationRemainCharacterCount = hyphenationRemainCharacterCount;
     }
-    
+
     /**
      * Gets the canonical <code>CommonHyphenation</code> instance corresponding
-     * to the values of the related properties present on the given 
+     * to the values of the related properties present on the given
      * <code>PropertyList</code>
-     * 
+     *
      * @param propertyList  the <code>PropertyList</code>
      */
     public static CommonHyphenation getInstance(PropertyList propertyList) throws PropertyException {
-        StringProperty language = 
+        StringProperty language =
             (StringProperty) propertyList.get(Constants.PR_LANGUAGE);
-        StringProperty country = 
+        StringProperty country =
             (StringProperty) propertyList.get(Constants.PR_COUNTRY);
-        StringProperty script = 
+        StringProperty script =
             (StringProperty) propertyList.get(Constants.PR_SCRIPT);
-        EnumProperty hyphenate = 
+        EnumProperty hyphenate =
             (EnumProperty) propertyList.get(Constants.PR_HYPHENATE);
-        CharacterProperty hyphenationCharacter = 
+        CharacterProperty hyphenationCharacter =
             (CharacterProperty) propertyList.get(Constants.PR_HYPHENATION_CHARACTER);
-        NumberProperty hyphenationPushCharacterCount = 
+        NumberProperty hyphenationPushCharacterCount =
             (NumberProperty) propertyList.get(Constants.PR_HYPHENATION_PUSH_CHARACTER_COUNT);
-        NumberProperty hyphenationRemainCharacterCount = 
+        NumberProperty hyphenationRemainCharacterCount =
             (NumberProperty) propertyList.get(Constants.PR_HYPHENATION_REMAIN_CHARACTER_COUNT);
-        
+
         CommonHyphenation instance = new CommonHyphenation(
-                                language, 
-                                country, 
-                                script, 
-                                hyphenate, 
-                                hyphenationCharacter, 
-                                hyphenationPushCharacterCount, 
+                                language,
+                                country,
+                                script,
+                                hyphenate,
+                                hyphenationCharacter,
+                                hyphenationPushCharacterCount,
                                 hyphenationRemainCharacterCount);
-        
+
         return cache.fetch(instance);
-        
+
     }
-    
+
     private static final char HYPHEN_MINUS = '-';
     private static final char MINUS_SIGN = '\u2212';
-    
+
     /**
      * Returns the effective hyphenation character for a font. The hyphenation character specified
      * in XSL-FO may be substituted if it's not available in the font.
@@ -142,7 +143,7 @@ public final class CommonHyphenation {
             FontMetrics metrics = font.getFontMetrics();
             if (metrics instanceof Typeface) {
                 Typeface typeface = (Typeface)metrics;
-                if ("SymbolEncoding".equals(typeface.getEncoding())) {
+                if ("SymbolEncoding".equals(typeface.getEncodingName())) {
                     //SymbolEncoding doesn't have HYPHEN_MINUS, so replace by MINUS_SIGN
                 } else {
                     //only warn if the encoding is not SymbolEncoding
@@ -154,7 +155,7 @@ public final class CommonHyphenation {
             FontMetrics metrics = font.getFontMetrics();
             if (metrics instanceof Typeface) {
                 Typeface typeface = (Typeface)metrics;
-                if ("ZapfDingbatsEncoding".equals(typeface.getEncoding())) {
+                if ("ZapfDingbatsEncoding".equals(typeface.getEncodingName())) {
                     //ZapfDingbatsEncoding doesn't have HYPHEN_MINUS, so replace by ' '
                 } else {
                     //only warn if the encoding is not ZapfDingbatsEncoding
@@ -165,13 +166,13 @@ public final class CommonHyphenation {
         if (warn) {
             log.warn("Substituted specified hyphenation character (0x"
                     + Integer.toHexString(hyphChar)
-                    + ") with 0x" + Integer.toHexString(effHyphChar) 
-                    + " because the font doesn't have the specified hyphenation character: " 
+                    + ") with 0x" + Integer.toHexString(effHyphChar)
+                    + " because the font doesn't have the specified hyphenation character: "
                     + font.getFontTriplet());
         }
         return effHyphChar;
     }
-    
+
     /**
      * Returns the IPD for the hyphenation character for a font.
      * @param font the font
@@ -181,7 +182,7 @@ public final class CommonHyphenation {
         char hyphChar = getHyphChar(font);
         return font.getCharWidth(hyphChar);
     }
-    
+
     /** {@inheritDoc} */
     public boolean equals(Object obj) {
         if (obj == this) {
@@ -199,23 +200,24 @@ public final class CommonHyphenation {
         }
         return false;
     }
-    
+
     /** {@inheritDoc} */
     public int hashCode() {
-        if (hash == 0) {
+        if (this.hash == 0) {
             int hash = 17;
             hash = 37 * hash + (language == null ? 0 : language.hashCode());
             hash = 37 * hash + (script == null ? 0 : script.hashCode());
             hash = 37 * hash + (country == null ? 0 : country.hashCode());
             hash = 37 * hash + (hyphenate == null ? 0 : hyphenate.hashCode());
-            hash = 37 * hash + 
+            hash = 37 * hash +
                 (hyphenationCharacter == null ? 0 : hyphenationCharacter.hashCode());
-            hash = 37 * hash + 
+            hash = 37 * hash +
                 (hyphenationPushCharacterCount == null ? 0 : hyphenationPushCharacterCount.hashCode());
-            hash = 37 * hash + 
+            hash = 37 * hash +
                 (hyphenationRemainCharacterCount == null ? 0 : hyphenationRemainCharacterCount.hashCode());
+            this.hash = hash;
         }
-        return hash;
+        return this.hash;
     }
-    
+
 }
