@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: CondLengthProperty.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: CondLengthProperty.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fo.properties;
 
@@ -31,9 +31,17 @@ import org.apache.fop.fo.expr.PropertyException;
  * Superclass for properties that have conditional lengths
  */
 public class CondLengthProperty extends Property implements CompoundDatatype {
+
+    /** cache holding canonical instances (for absolute conditional lengths) */
+    private static final PropertyCache cache = new PropertyCache(CondLengthProperty.class);
+
+    /** components */
     private Property length;
     private EnumProperty conditionality;
 
+    private boolean isCached = false;
+    private int hash = -1;
+
     /**
      * Inner class for creating instances of CondLengthProperty
      */
@@ -48,7 +56,7 @@ public class CondLengthProperty extends Property implements CompoundDatatype {
 
         /**
          * Create a new empty instance of CondLengthProperty.
-         * @return the new instance. 
+         * @return the new instance.
          */
         public Property makeNewProperty() {
             return new CondLengthProperty();
@@ -56,7 +64,7 @@ public class CondLengthProperty extends Property implements CompoundDatatype {
 
         /**
          * {@inheritDoc}
-         */        
+         */
         public Property convertProperty(Property p, PropertyList propertyList, FObj fo)
                     throws PropertyException {
             if (p instanceof KeepProperty) {
@@ -67,10 +75,15 @@ public class CondLengthProperty extends Property implements CompoundDatatype {
     }
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
     public void setComponent(int cmpId, Property cmpnValue,
                              boolean bIsDefault) {
+        if (isCached) {
+            throw new IllegalStateException(
+                    "CondLengthProperty.setComponent() called on a cached value!");
+        }
+
         if (cmpId == CP_LENGTH) {
             length = cmpnValue;
         } else if (cmpId == CP_CONDITIONALITY) {
@@ -134,17 +147,25 @@ public class CondLengthProperty extends Property implements CompoundDatatype {
 
     /** {@inheritDoc} */
     public String toString() {
-        return "CondLength[" + length.getObject().toString() 
-                + ", " + (isDiscard() 
-                        ? conditionality.toString().toLowerCase() 
+        return "CondLength[" + length.getObject().toString()
+                + ", " + (isDiscard()
+                        ? conditionality.toString().toLowerCase()
                         : conditionality.toString()) + "]";
-    }    
+    }
 
     /**
      * @return this.condLength
      */
     public CondLengthProperty getCondLength() {
-        return this;
+        if (this.length.getLength().isAbsolute()) {
+            CondLengthProperty clp = (CondLengthProperty) cache.fetch(this);
+            if (clp == this) {
+                isCached = true;
+            }
+            return clp;
+        } else {
+            return this;
+        }
     }
 
     /**
@@ -162,4 +183,28 @@ public class CondLengthProperty extends Property implements CompoundDatatype {
         return this;
     }
 
+    /** {@inheritDoc} */
+    public boolean equals(Object obj) {
+        if (this == obj) {
+            return true;
+        }
+
+        if (obj instanceof CondLengthProperty) {
+            CondLengthProperty clp = (CondLengthProperty)obj;
+            return (this.length == clp.length
+                    && this.conditionality == clp.conditionality);
+        }
+        return false;
+    }
+
+    /** {@inheritDoc} */
+    public int hashCode() {
+        if (this.hash == -1) {
+            int hash = 17;
+            hash = 37 * hash + (length == null ? 0 : length.hashCode());
+            hash = 37 * hash + (conditionality == null ? 0 : conditionality.hashCode());
+            this.hash = hash;
+        }
+        return this.hash;
+    }
 }
