@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FixedLength.java 627719 2008-02-14 11:57:05Z maxberger $ */
+/* $Id: FixedLength.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fo.properties;
 
@@ -25,7 +25,7 @@ import org.apache.fop.datatypes.PercentBaseContext;
  * An absolute length quantity in XSL
  */
 public final class FixedLength extends LengthProperty {
-    
+
     /** Describes the unit pica. */
     public static final String PICA = "pc";
 
@@ -45,17 +45,17 @@ public final class FixedLength extends LengthProperty {
     public static final String MPT = "mpt";
 
     /** cache holding all canonical FixedLength instances */
-    private static final PropertyCache cache = new PropertyCache();
-    
+    private static final PropertyCache cache = new PropertyCache(FixedLength.class);
+
     /** canonical zero-length instance */
     public static final FixedLength ZERO_FIXED_LENGTH = new FixedLength(0, FixedLength.MPT, 1.0f);
-    
+
     private int millipoints;
 
     /**
      * Set the length given a number of units, a unit name and
      * an assumed resolution (used in case the units are pixels)
-     * 
+     *
      * @param numUnits  quantity of input units
      * @param units     input unit specifier
      * @param res       input/source resolution
@@ -63,11 +63,11 @@ public final class FixedLength extends LengthProperty {
     private FixedLength(double numUnits, String units, float res) {
         this.millipoints = convert(numUnits, units, res);
     }
-    
+
     /**
      * Return the cached {@link FixedLength} instance corresponding
      * to the computed value in base-units (millipoints).
-     * 
+     *
      * @param numUnits  quantity of input units
      * @param units     input unit specifier
      * @param sourceResolution input/source resolution (= ratio of pixels per pt)
@@ -75,7 +75,7 @@ public final class FixedLength extends LengthProperty {
      *          to the given number of units and unit specifier
      *          in the given resolution
      */
-    public static FixedLength getInstance(double numUnits, 
+    public static FixedLength getInstance(double numUnits,
                                           String units,
                                           float sourceResolution) {
         if (numUnits == 0.0) {
@@ -84,44 +84,44 @@ public final class FixedLength extends LengthProperty {
             return (FixedLength)cache.fetch(
                 new FixedLength(numUnits, units, sourceResolution));
         }
-        
+
     }
-    
+
     /**
      * Return the cached {@link FixedLength} instance corresponding
      * to the computed value
      * This method assumes a source-resolution of 1 (1px = 1pt)
-     * 
+     *
      * @param numUnits  input units
      * @param units     unit specifier
      * @return  the canonical FixedLength instance corresponding
      *          to the given number of units and unit specifier
      */
-    public static FixedLength getInstance(double numUnits, 
+    public static FixedLength getInstance(double numUnits,
                                           String units) {
         return getInstance(numUnits, units, 1.0f);
-        
+
     }
-    
+
     /**
      * Return the cached {@link FixedLength} instance corresponding
      * to the computed value.
-     * This method assumes 'millipoints' (non-standard) as units, 
+     * This method assumes 'millipoints' (non-standard) as units,
      * and an implied source-resolution of 1 (1px = 1pt).
-     * 
+     *
      * @param numUnits  input units
      * @return  the canonical FixedLength instance corresponding
      *          to the given number of units and unit specifier
      */
     public static FixedLength getInstance(double numUnits) {
         return getInstance(numUnits, FixedLength.MPT, 1.0f);
-        
+
     }
-    
+
     /**
      * Convert the given length to a dimensionless integer representing
      * a whole number of base units (milli-points).
-     * 
+     *
      * @param dvalue quantity of input units
      * @param unit input unit specifier (in, cm, etc.)
      * @param res   the input/source resolution (in case the unit spec is "px")
