@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FontFamilyProperty.java 578311 2007-09-21 22:51:22Z adelmelle $ */
+/* $Id: FontFamilyProperty.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fo.properties;
 
@@ -31,10 +31,10 @@ import org.apache.fop.fo.expr.PropertyException;
 public final class FontFamilyProperty extends ListProperty {
 
     /** cache holding all canonical FontFamilyProperty instances */
-    private static final PropertyCache cache = new PropertyCache();
-    
+    private static final PropertyCache cache = new PropertyCache(FontFamilyProperty.class);
+
     private int hash = 0;
-    
+
     /**
      * Inner class for creating instances of ListProperty
      */
@@ -126,7 +126,7 @@ public final class FontFamilyProperty extends ListProperty {
     private FontFamilyProperty() {
         super();
     }
-    
+
     /**
      * Add a new property to the list
      * @param prop Property to be added to the list
@@ -154,7 +154,7 @@ public final class FontFamilyProperty extends ListProperty {
         if (this == o) {
             return true;
         }
-        
+
         if (o instanceof FontFamilyProperty) {
             FontFamilyProperty ffp = (FontFamilyProperty) o;
             return (this.list != null
@@ -162,7 +162,7 @@ public final class FontFamilyProperty extends ListProperty {
         }
         return false;
     }
-    
+
     /** {@inheritDoc} */
     public int hashCode() {
         if (this.hash == 0) {
