@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: LengthRangeProperty.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: LengthRangeProperty.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.fo.properties;
 
@@ -25,6 +25,7 @@ import org.apache.fop.datatypes.PercentBaseContext;
 import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.expr.PropertyException;
+import org.apache.fop.traits.MinOptMax;
 
 /**
  * Superclass for properties that contain LengthRange values
@@ -39,6 +40,22 @@ public class LengthRangeProperty extends Property implements CompoundDatatype {
     private int bfSet = 0;    // bit field
     private boolean consistent = false;
 
+    /**
+     * Converts this <code>LengthRangeProperty</code> to a <code>MinOptMax</code>.
+     *
+     * @param context Percentage evaluation context
+     * @return the requested MinOptMax instance
+     */
+    public MinOptMax toMinOptMax(PercentBaseContext context) {
+        int min = getMinimum(context).isAuto() ? 0
+                : getMinimum(context).getLength().getValue(context);
+        int opt = getOptimum(context).isAuto() ? min
+                : getOptimum(context).getLength().getValue(context);
+        int max = getMaximum(context).isAuto() ? Integer.MAX_VALUE
+                : getMaximum(context).getLength().getValue(context);
+        return MinOptMax.getInstance(min, opt, max);
+    }
+
     /**
      * Inner class for a Maker for LengthProperty objects
      */
@@ -53,7 +70,7 @@ public class LengthRangeProperty extends Property implements CompoundDatatype {
 
         /**
          * Create a new empty instance of LengthRangeProperty.
-         * @return the new instance. 
+         * @return the new instance.
          */
         public Property makeNewProperty() {
             return new LengthRangeProperty();
@@ -64,16 +81,16 @@ public class LengthRangeProperty extends Property implements CompoundDatatype {
                         && ((PercentLength) len).getPercentage() < 0)
                     || (len.isAbsolute() && len.getValue() < 0));
         }
-        
-        /** {@inheritDoc} */        
-        public Property convertProperty(Property p, 
+
+        /** {@inheritDoc} */
+        public Property convertProperty(Property p,
                                 PropertyList propertyList, FObj fo)
                         throws PropertyException {
-            
+
             if (p instanceof LengthRangeProperty) {
                 return p;
             }
-            
+
             if (this.propId == PR_BLOCK_PROGRESSION_DIMENSION
                     || this.propId == PR_INLINE_PROGRESSION_DIMENSION) {
                 Length len = p.getLength();
@@ -86,13 +103,13 @@ public class LengthRangeProperty extends Property implements CompoundDatatype {
                     }
                 }
             }
-            
+
             return super.convertProperty(p, propertyList, fo);
         }
-        
-        
+
+
         /**
-         * {@inheritDoc} 
+         * {@inheritDoc}
          */
         protected Property setSubprop(Property baseProperty, int subpropertyId,
                                         Property subproperty) {
@@ -119,7 +136,7 @@ public class LengthRangeProperty extends Property implements CompoundDatatype {
 
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
     public void setComponent(int cmpId, Property cmpnValue,
                              boolean bIsDefault) {
@@ -191,7 +208,7 @@ public class LengthRangeProperty extends Property implements CompoundDatatype {
         }
         consistent = false;
     }
-    
+
     // Minimum is prioritaire, if explicit
     private void checkConsistency(PercentBaseContext context) {
         if (consistent) {
@@ -203,7 +220,7 @@ public class LengthRangeProperty extends Property implements CompoundDatatype {
         // Make sure max >= min
         // Must also control if have any allowed enum values!
 
-        if (!minimum.isAuto() && !maximum.isAuto() 
+        if (!minimum.isAuto() && !maximum.isAuto()
                 && minimum.getLength().getValue(context) > maximum.getLength().getValue(context)) {
             if ((bfSet & MINSET) != 0) {
                 // if minimum is explicit, force max to min
@@ -217,7 +234,7 @@ public class LengthRangeProperty extends Property implements CompoundDatatype {
             }
         }
         // Now make sure opt <= max and opt >= min
-        if (!optimum.isAuto() && !maximum.isAuto() 
+        if (!optimum.isAuto() && !maximum.isAuto()
                 && optimum.getLength().getValue(context) > maximum.getLength().getValue(context)) {
             if ((bfSet & OPTSET) != 0) {
                 if ((bfSet & MAXSET) != 0) {
@@ -231,8 +248,8 @@ public class LengthRangeProperty extends Property implements CompoundDatatype {
                 // opt is default and max is explicit or default
                 optimum = maximum;
             }
-        } else if (!optimum.isAuto() && !minimum.isAuto() 
-                    && optimum.getLength().getValue(context) 
+        } else if (!optimum.isAuto() && !minimum.isAuto()
+                    && optimum.getLength().getValue(context)
                         < minimum.getLength().getValue(context)) {
             if ((bfSet & MINSET) != 0) {
                 // if minimum is explicit, force opt to min
@@ -244,7 +261,7 @@ public class LengthRangeProperty extends Property implements CompoundDatatype {
                 minimum = optimum; // minimum was default value
             }
         }
-        
+
         consistent = true;
     }
 
@@ -278,8 +295,8 @@ public class LengthRangeProperty extends Property implements CompoundDatatype {
     /** {@inheritDoc} */
     public String toString() {
         return "LengthRange["
-            + "min:" + getMinimum(null).getObject() 
-            + ", max:" + getMaximum(null).getObject() 
+            + "min:" + getMinimum(null).getObject()
+            + ", max:" + getMaximum(null).getObject()
             + ", opt:" + getOptimum(null).getObject() + "]";
     }
 
