@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,23 +15,22 @@
  * limitations under the License.
  */
 
-/* $Id: FontInfo.java 629103 2008-02-19 14:16:08Z jeremias $ */
+/* $Id: FontInfo.java 891174 2009-12-16 10:06:16Z jeremias $ */
 
 package org.apache.fop.fonts;
 
-import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
-
 /**
- * The FontInfo for the layout and rendering of a fo document.
+ * The FontInfo holds font information for the layout and rendering of a fo document.
  * This stores the list of available fonts that are setup by
  * the renderer. The font name can be retrieved for the
  * family style and weight.
@@ -40,45 +39,56 @@ import org.apache.commons.logging.LogFactory;
  * implemented.
  */
 public class FontInfo {
-    
-    private static final FontTriplet[] TRIPLETS_TYPE = new FontTriplet[1];
-    
+
     /** logging instance */
     protected static Log log = LogFactory.getLog(FontInfo.class);
 
     /** Map containing fonts that have been used */
-    private Map usedFonts; //Map<String,FontMetrics> (String = font key)
-    
+    private Map/*<String,FontMetrics>*/ usedFonts = null; //(String = font key)
+
     /** look up a font-triplet to find a font-name */
-    private Map triplets; //Map<FontTriplet,String> (String = font key)
-    
+    private Map/*<FontTriplet,String>*/ triplets = null; //(String = font key)
+
     /** look up a font-triplet to find its priority
      *  (only used inside addFontProperties()) */
-    private Map tripletPriorities; //Map<FontTriplet,Integer>
+    private Map/*<FontTriplet,Integer>*/ tripletPriorities = null; //Map<FontTriplet,Integer>
 
     /** look up a font-name to get a font (that implements FontMetrics at least) */
-    private Map fonts; //Map<String,FontMetrics> (String = font key)
-    
-    /** collection of missing fonts; used to make sure the user gets 
+    private Map/*<String,FontMetrics>*/ fonts = null; //(String = font key)
+
+    /**
+     *  a collection of missing fonts; used to make sure the user gets
      *  a warning for a missing font only once (not every time the font is used)
      */
-    private Collection loggedFontKeys;
+    private Set/*<FontTriplet>*/ loggedFontKeys = null;
 
     /** Cache for Font instances. */
-    private Map fontInstanceCache = new java.util.HashMap();
-    
+    private Map/*<FontTriplet, Map>*/ fontInstanceCache = null;
+
+    /** Event listener for font events */
+    private FontEventListener eventListener = null;
+
     /**
      * Main constructor
      */
     public FontInfo() {
-        this.triplets = new java.util.HashMap();
-        this.tripletPriorities = new java.util.HashMap();
-        this.fonts = new java.util.HashMap();
-        this.usedFonts = new java.util.HashMap();
+        this.triplets = new java.util.HashMap/*<FontTriplet, String>*/();
+        this.tripletPriorities = new java.util.HashMap/*<FontTriplet, Integer>*/();
+        this.fonts = new java.util.HashMap/*<String, FontMetrics>*/();
+        this.usedFonts = new java.util.HashMap/*<String,FontMetrics>*/();
     }
 
     /**
-     * Checks if the font setup is valid (At least the ultimate fallback font 
+     * Sets the font event listener that can be used to receive events about particular events
+     * in this class.
+     * @param listener the font event listener
+     */
+    public void setEventListener(FontEventListener listener) {
+        this.eventListener = listener;
+    }
+
+    /**
+     * Checks if the font setup is valid (At least the ultimate fallback font
      * must be registered.)
      * @return True if valid
      */
@@ -100,17 +110,30 @@ public class FontInfo {
     }
 
     /**
-     * Adds a new font triplet.
+     * Adds a series of new font triplets given an array of font family names.
      * @param name internal key
+     * @param families an array of font family names
+     * @param style font style (normal, italic, oblique...)
+     * @param weight font weight
+     */
+    public void addFontProperties(String name, String[] families, String style, int weight) {
+        for (int i = 0; i < families.length; i++) {
+            addFontProperties(name, families[i], style, weight);
+        }
+    }
+
+    /**
+     * Adds a new font triplet.
+     * @param internalFontKey internal font key
      * @param triplet the font triplet to associate with the internal key
      */
-    public void addFontProperties(String name, FontTriplet triplet) {
+    public void addFontProperties(String internalFontKey, FontTriplet triplet) {
         /*
          * add the given family, style and weight as a lookup for the font
          * with the given name
          */
         if (log.isDebugEnabled()) {
-            log.debug("Registering: " + triplet + " under " + name);
+            log.debug("Registering: " + triplet + " under " + internalFontKey);
         }
         String oldName = (String)triplets.get(triplet);
         int newPriority = triplet.getPriority();
@@ -118,18 +141,20 @@ public class FontInfo {
             int oldPriority = ((Integer)tripletPriorities.get(triplet)).intValue();
             if (oldPriority < newPriority) {
                 logDuplicateFont(triplet, false, oldName, oldPriority,
-                            name, newPriority);
+                            internalFontKey, newPriority);
                 return;
             } else {
                 logDuplicateFont(triplet, true, oldName, oldPriority,
-                            name, newPriority);
+                            internalFontKey, newPriority);
             }
         }
-        this.triplets.put(triplet, name);
+        this.triplets.put(triplet, internalFontKey);
         this.tripletPriorities.put(triplet, new Integer(newPriority));
     }
 
-    /** Log warning about duplicate font triplets.
+    /**
+     * Log warning about duplicate font triplets.
+     *
      * @param triplet the duplicate font triplet
      * @param replacing true iff the new font will replace the old one
      * @param oldKey the old internal font name
@@ -144,21 +169,24 @@ public class FontInfo {
             log.debug(triplet
                     + (replacing ? ": Replacing " : ": Not replacing ")
                     + ((FontMetrics)fonts.get(triplets.get(triplet))).getFullName()
-                    + " (" + oldPriority + ") by "
+                    + " (priority=" + oldPriority + ") by "
                     + ((FontMetrics)fonts.get(newKey)).getFullName()
-                    + " (" + newPriority + ")");
+                    + " (priority=" + newPriority + ")");
         }
     }
 
     /**
      * Adds font metrics for a specific font.
-     * @param name internal key
+     * @param internalFontKey internal key
      * @param metrics metrics to register
      */
-    public void addMetrics(String name, FontMetrics metrics) {
+    public void addMetrics(String internalFontKey, FontMetrics metrics) {
         // add the given metrics as a font with the given name
 
-        this.fonts.put(name, metrics);
+        if (metrics instanceof Typeface) {
+            ((Typeface)metrics).setEventListener(this.eventListener);
+        }
+        this.fonts.put(internalFontKey, metrics);
     }
 
     /**
@@ -171,103 +199,91 @@ public class FontInfo {
      * @param family font family
      * @param style font style
      * @param weight font weight
-     * @param substFont true if the font may be substituted with the 
+     * @param substitutable true if the font may be substituted with the
      *                  default font if not found
-     * @return internal key
+     * @return internal font triplet key
      */
     private FontTriplet fontLookup(String family, String style,
-                             int weight, boolean substFont) {
+                             int weight, boolean substitutable) {
         if (log.isTraceEnabled()) {
-            log.trace("Font lookup: " + family + " " + style + " " + weight);
-        }
-        FontTriplet startKey = createFontKey(family, style, weight);
-        FontTriplet key = startKey;
-        // first try given parameters
-        String f = getInternalFontKey(key);
-        if (f == null) {
-            key = doAdjustedLookup(family, style, weight, startKey, substFont);
+            log.trace("Font lookup: " + family + " " + style + " " + weight
+                    + (substitutable ? " substitutable" : ""));
         }
 
-        if (key != null) {
-            if (key != startKey) {
-                notifyFontReplacement(startKey, key);
+        FontTriplet startKey = createFontKey(family, style, weight);
+        FontTriplet fontTriplet = startKey;
+        // first try given parameters
+        String internalFontKey = getInternalFontKey(fontTriplet);
+        if (internalFontKey == null) {
+            fontTriplet = fuzzyFontLookup(family, style, weight, startKey, substitutable);
+        }
+
+        if (fontTriplet != null) {
+            if (fontTriplet != startKey) {
+                notifyFontReplacement(startKey, fontTriplet);
             }
-            return key;
+            return fontTriplet;
         } else {
             return null;
         }
     }
 
-    private FontTriplet doAdjustedLookup(String family, String style,
-            int weight, FontTriplet startKey, boolean substFont) {
+    private FontTriplet fuzzyFontLookup(String family, String style,
+            int weight, FontTriplet startKey, boolean substitutable) {
         FontTriplet key;
-        String f;
+        String internalFontKey = null;
         if (!family.equals(startKey.getName())) {
             key = createFontKey(family, style, weight);
-            f = getInternalFontKey(key);
-            if (f != null) {
+            internalFontKey = getInternalFontKey(key);
+            if (internalFontKey != null) {
                 return key;
             }
         }
-        
+
         // adjust weight, favouring normal or bold
         key = findAdjustWeight(family, style, weight);
-        f = getInternalFontKey(key);
+        if (key != null) {
+            internalFontKey = getInternalFontKey(key);
+        }
 
-        if (!substFont && f == null) {
+        // return null if not found and not substitutable
+        if (!substitutable && internalFontKey == null) {
             return null;
         }
-        
+
         // only if the font may be substituted
         // fallback 1: try the same font-family and weight with default style
-        if (f == null && style != Font.STYLE_NORMAL) {
+        if (internalFontKey == null && style != Font.STYLE_NORMAL) {
             key = createFontKey(family, Font.STYLE_NORMAL, weight);
-            f = getInternalFontKey(key);
+            internalFontKey = getInternalFontKey(key);
         }
 
-        if (f == null && weight != Font.WEIGHT_NORMAL) {
-            int diffWeight = (Font.WEIGHT_NORMAL - weight) / 100;
-            int direction = diffWeight > 0 ? 1 : -1;
-            int tryWeight = weight;
-            while (tryWeight != Font.WEIGHT_NORMAL) {
-                tryWeight += 100 * direction;
-                key = createFontKey(family, style, weight);
-                f = getInternalFontKey(key);
-                if (f == null) {
-                    key = createFontKey(family, Font.STYLE_NORMAL, weight);
-                    f = getInternalFontKey(key);
-                }
-                if (f != null) {
-                    break;
-                }
+        // fallback 2: try the same font-family with default style and try to adjust weight
+        if (internalFontKey == null && style != Font.STYLE_NORMAL) {
+            key = findAdjustWeight(family, Font.STYLE_NORMAL, weight);
+            if (key != null) {
+                internalFontKey = getInternalFontKey(key);
             }
         }
-        
-        // fallback 2: try the same font-family with default style and weight
-        /* obsolete: replaced by the loop above
-        if (f == null) {
-            key = createFontKey(family, Font.STYLE_NORMAL, Font.WEIGHT_NORMAL);
-            f = getInternalFontKey(key);
-        }*/
-        
-        // fallback 3: try any family with orig style/weight
-        if (f == null) {
-            return doAdjustedLookup("any", style, weight, startKey, false);
+
+        // fallback 3: try any family with original style/weight
+        if (internalFontKey == null) {
+            return fuzzyFontLookup("any", style, weight, startKey, false);
         }
 
         // last resort: use default
-        if (f == null) {
+        if (key == null && internalFontKey == null) {
             key = Font.DEFAULT_FONT;
-            f = getInternalFontKey(key);
+            internalFontKey = getInternalFontKey(key);
         }
 
-        if (f != null) {
+        if (internalFontKey != null) {
             return key;
         } else {
             return null;
         }
     }
-    
+
     /**
      * Tells this class that the font with the given internal name has been used.
      * @param internalName the internal font name (F1, F2 etc.)
@@ -275,31 +291,91 @@ public class FontInfo {
     public void useFont(String internalName) {
         usedFonts.put(internalName, fonts.get(internalName));
     }
-    
+
+    private Map/*<FontTriplet,Map<Integer,Font>>*/ getFontInstanceCache() {
+        if (fontInstanceCache == null) {
+            fontInstanceCache = new java.util.HashMap/*<FontTriplet, Map<Integer,Font>>*/();
+        }
+        return fontInstanceCache;
+    }
+
     /**
-     * Retrieves a (possibly cached) Font instance based on a FontTriplet and a font size. 
+     * Retrieves a (possibly cached) Font instance based on a FontTriplet and a font size.
+     *
      * @param triplet the font triplet designating the requested font
      * @param fontSize the font size
      * @return the requested Font instance
      */
     public Font getFontInstance(FontTriplet triplet, int fontSize) {
-        Map sizes = (Map)fontInstanceCache.get(triplet);
+        Map/*<Integer,Font>*/ sizes
+            = (Map/*<Integer,Font>*/)getFontInstanceCache().get(triplet);
         if (sizes == null) {
-            sizes = new java.util.HashMap();
-            fontInstanceCache.put(triplet, sizes);
+            sizes = new java.util.HashMap/*<Integer,Font>*/();
+            getFontInstanceCache().put(triplet, sizes);
         }
         Integer size = new Integer(fontSize);
         Font font = (Font)sizes.get(size);
         if (font == null) {
-            String fname = getInternalFontKey(triplet);
-            useFont(fname);
-            FontMetrics metrics = getMetricsFor(fname);
-            font = new Font(fname, triplet, metrics, fontSize);
+            String fontKey = getInternalFontKey(triplet);
+            useFont(fontKey);
+            FontMetrics metrics = getMetricsFor(fontKey);
+            font = new Font(fontKey, triplet, metrics, fontSize);
             sizes.put(size, font);
         }
         return font;
     }
-    
+
+    private List/*<FontTriplet>*/ getTripletsForName(String fontName) {
+        List/*<FontTriplet>*/ matchedTriplets = new java.util.ArrayList/*<FontTriplet>*/();
+        Iterator it = triplets.keySet().iterator();
+        while (it.hasNext()) {
+            FontTriplet triplet = (FontTriplet)it.next();
+            String tripletName = triplet.getName();
+            if (tripletName.toLowerCase().equals(fontName.toLowerCase())) {
+                matchedTriplets.add(triplet);
+            }
+        }
+        return matchedTriplets;
+    }
+
+    /**
+     * Returns a suitable internal font given an AWT Font instance.
+     *
+     * @param awtFont the AWT font
+     * @return a best matching internal Font
+     */
+    public Font getFontInstanceForAWTFont(java.awt.Font awtFont) {
+        String awtFontName = awtFont.getName();
+        String awtFontFamily = awtFont.getFamily();
+        String awtFontStyle = awtFont.isItalic() ? Font.STYLE_ITALIC : Font.STYLE_NORMAL;
+        int awtFontWeight = awtFont.isBold() ? Font.WEIGHT_BOLD : Font.WEIGHT_NORMAL;
+
+        FontTriplet matchedTriplet = null;
+        List/*<FontTriplet>*/ triplets = getTripletsForName(awtFontName);
+        if (!triplets.isEmpty()) {
+            Iterator it = triplets.iterator();
+            while (it.hasNext()) {
+                FontTriplet triplet = (FontTriplet)it.next();
+                boolean styleMatched = triplet.getStyle().equals(awtFontStyle);
+                boolean weightMatched = triplet.getWeight() == awtFontWeight;
+                if (styleMatched && weightMatched) {
+                    matchedTriplet = triplet;
+                    break;
+                }
+            }
+        }
+
+        // not matched on font name so do a lookup using family
+        if (matchedTriplet == null) {
+            if (awtFontFamily.equals("sanserif")) {
+                awtFontFamily = "sans-serif";
+            }
+            matchedTriplet = fontLookup(awtFontFamily, awtFontStyle, awtFontWeight);
+        }
+        int fontSize = Math.round(awtFont.getSize2D() * 1000);
+        return getFontInstance(matchedTriplet, fontSize);
+    }
+
     /**
      * Lookup a font.
      * <br>
@@ -316,7 +392,20 @@ public class FontInfo {
                              int weight) {
         return fontLookup(family, style, weight, true);
     }
-    
+
+    private List/*<FontTriplet>*/ fontLookup(String[] families, String style,
+            int weight, boolean substitutable) {
+        List/*<FontTriplet>*/ matchingTriplets = new java.util.ArrayList/*<FontTriplet>*/();
+        FontTriplet triplet = null;
+        for (int i = 0; i < families.length; i++) {
+            triplet = fontLookup(families[i], style, weight, substitutable);
+            if (triplet != null) {
+                matchingTriplets.add(triplet);
+            }
+        }
+        return matchingTriplets;
+    }
+
     /**
      * Looks up a set of fonts.
      * <br>
@@ -335,40 +424,55 @@ public class FontInfo {
         if (families.length == 0) {
             throw new IllegalArgumentException("Specify at least one font family");
         }
-        FontTriplet triplet;
-        List tmpTriplets = new ArrayList();
-        for (int i = 0, c = families.length; i < c; i++) {
-            triplet = fontLookup(families[i], style, weight, (i >= families.length - 1));
-            if (triplet != null) {
-                tmpTriplets.add(triplet);
+
+        // try matching without substitutions
+        List/*<FontTriplet>*/ matchedTriplets = fontLookup(families, style, weight, false);
+
+        // if there are no matching font triplets found try with substitutions
+        if (matchedTriplets.size() == 0) {
+            matchedTriplets = fontLookup(families, style, weight, true);
+        }
+
+        // no matching font triplets found!
+        if (matchedTriplets.size() == 0) {
+            StringBuffer sb = new StringBuffer();
+            for (int i = 0, c = families.length; i < c; i++) {
+                if (i > 0) {
+                    sb.append(", ");
+                }
+                sb.append(families[i]);
             }
+            throw new IllegalStateException(
+                        "fontLookup must return an array with at least one "
+                        + "FontTriplet on the last call. Lookup: " + sb.toString());
+
         }
-        if (tmpTriplets.size() != 0) {
-            return (FontTriplet[]) tmpTriplets.toArray(TRIPLETS_TYPE);
-        }
-        StringBuffer sb = new StringBuffer();
-        for (int i = 0, c = families.length; i < c; i++) {
-            if (i > 0) {
-                sb.append(", ");
-            }
-            sb.append(families[i]);
-        }
-        throw new IllegalStateException(
-                    "fontLookup must return an array with at least one "
-                    + "FontTriplet on the last call. Lookup: " + sb.toString());
+        FontTriplet[] fontTriplets = new FontTriplet[matchedTriplets.size()];
+        matchedTriplets.toArray(fontTriplets);
+
+        // found some matching fonts so return them
+        return fontTriplets;
     }
-    
-    private void notifyFontReplacement(FontTriplet replacedKey, FontTriplet newKey) {
+
+    private Set/*<FontTriplet>*/ getLoggedFontKeys() {
         if (loggedFontKeys == null) {
-            loggedFontKeys = new java.util.HashSet();
+            loggedFontKeys = new java.util.HashSet/*<FontTriplet>*/();
         }
-        if (!loggedFontKeys.contains(replacedKey)) {
-            loggedFontKeys.add(replacedKey);
-            log.warn("Font '" + replacedKey + "' not found. "
-                    + "Substituting with '" + newKey + "'.");
+        return loggedFontKeys;
+    }
+
+    private void notifyFontReplacement(FontTriplet replacedKey, FontTriplet newKey) {
+        if (!getLoggedFontKeys().contains(replacedKey)) {
+            getLoggedFontKeys().add(replacedKey);
+            if (this.eventListener != null) {
+                this.eventListener.fontSubstituted(this, replacedKey, newKey);
+            } else {
+                log.warn("Font '" + replacedKey + "' not found. "
+                        + "Substituting with '" + newKey + "'.");
+            }
         }
     }
-    
+
     /**
      * Find a font with a given family and style by trying
      * different font weights according to the spec.
@@ -382,7 +486,7 @@ public class FontInfo {
         FontTriplet key = null;
         String f = null;
         int newWeight = weight;
-        if (newWeight < 400) {
+        if (newWeight <= 400) {
             while (f == null && newWeight > 100) {
                 newWeight -= 100;
                 key = createFontKey(family, style, newWeight);
@@ -442,7 +546,7 @@ public class FontInfo {
     public String getInternalFontKey(FontTriplet triplet) {
         return (String)triplets.get(triplet);
     }
-    
+
     /**
      * Creates a key from the given strings.
      * @param family font family
@@ -459,25 +563,25 @@ public class FontInfo {
      * Gets a Map of all registered fonts.
      * @return a read-only Map with font key/FontMetrics pairs
      */
-    public Map getFonts() {
+    public Map/*<String,FontMetrics>*/ getFonts() {
         return java.util.Collections.unmodifiableMap(this.fonts);
     }
 
     /**
      * Gets a Map of all registered font triplets.
-     * @return a read-only Map with FontTriplet/font key pairs
+     * @return a Map with FontTriplet/font key pairs
      */
-    public Map getFontTriplets() {
-        return java.util.Collections.unmodifiableMap(this.triplets);
+    public Map/*<FontTriplet,String>*/ getFontTriplets() {
+        return this.triplets;
     }
-    
+
     /**
      * This is used by the renderers to retrieve all the
      * fonts used in the document.
      * This is for embedded font or creating a list of used fonts.
      * @return a read-only Map with font key/FontMetrics pairs
      */
-    public Map getUsedFonts() {
+    public Map/*<String,FontMetrics>*/ getUsedFonts() {
         return this.usedFonts;
     }
 
@@ -492,6 +596,22 @@ public class FontInfo {
         return metrics;
     }
 
+    /**
+     * Returns all font triplet matching the given font name.
+     * @param fontName The font name we are looking for
+     * @return A list of matching font triplets
+     */
+    public List/*<FontTriplet>*/ getTripletsFor(String fontName) {
+        List/*<FontTriplet>*/ foundTriplets = new java.util.ArrayList();
+        for (Iterator iter = triplets.entrySet().iterator(); iter.hasNext();) {
+            Map.Entry tripletEntry = (Map.Entry) iter.next();
+            if (fontName.equals((tripletEntry.getValue()))) {
+                foundTriplets.add(tripletEntry.getKey());
+            }
+        }
+        return foundTriplets;
+    }
+
     /**
      * Returns the first triplet matching the given font name.
      * As there may be multiple triplets matching the font name
@@ -500,20 +620,14 @@ public class FontInfo {
      * @return The first triplet for the given font name
      */
     public FontTriplet getTripletFor(String fontName) {
-        List foundTriplets = new ArrayList();
-        for (Iterator iter = triplets.entrySet().iterator(); iter.hasNext();) {
-            Map.Entry tripletEntry = (Map.Entry) iter.next();
-            if (fontName.equals(((String)tripletEntry.getValue()))) {
-                foundTriplets.add(tripletEntry.getKey());
-            }
-        }
+        List/*<FontTriplet>*/ foundTriplets = getTripletsFor(fontName);
         if (foundTriplets.size() > 0) {
             Collections.sort(foundTriplets);
             return (FontTriplet)foundTriplets.get(0);
         }
         return null;
     }
-    
+
     /**
      * Returns the font style for a particular font.
      * There may be multiple font styles matching this font. Only the first
@@ -530,7 +644,7 @@ public class FontInfo {
             return "";
         }
     }
-    
+
     /**
      * Returns the font weight for a particular font.
      * There may be multiple font weights matching this font. Only the first
@@ -547,22 +661,30 @@ public class FontInfo {
             return 0;
         }
     }
-    
+
     /**
-     * Diagnostic method for logging all registered fonts to System.out. 
+     * Diagnostic method for logging all registered fonts to System.out.
      */
     public void dumpAllTripletsToSystemOut() {
+        System.out.print(toString());
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public String toString() {
         Collection entries = new java.util.TreeSet();
         Iterator iter = this.triplets.keySet().iterator();
         while (iter.hasNext()) {
             FontTriplet triplet = (FontTriplet)iter.next();
             String key = getInternalFontKey(triplet);
-            FontMetrics metrics = getMetricsFor(key); 
-            entries.add(triplet.toString() + " -> " + key + " -> " + metrics.getFontName());
+            FontMetrics metrics = getMetricsFor(key);
+            entries.add(triplet.toString() + " -> " + key + " -> " + metrics.getFontName() + "\n");
         }
-        iter = entries.iterator();
-        while (iter.hasNext()) {
-            System.out.println(iter.next());
+        StringBuffer stringBuffer = new StringBuffer();
+        for (iter = entries.iterator(); iter.hasNext();) {
+            stringBuffer.append(iter.next());
         }
+        return stringBuffer.toString();
     }
 }
