@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FontLoader.java 627679 2008-02-14 08:12:34Z jeremias $ */
+/* $Id: FontLoader.java 818424 2009-09-24 10:09:55Z maxberger $ */
 
 package org.apache.fop.fonts;
 
@@ -39,9 +39,7 @@ import org.apache.fop.fonts.type1.Type1FontLoader;
  */
 public abstract class FontLoader {
 
-    /**
-     * logging instance
-     */
+    /** logging instance */
     protected static Log log = LogFactory.getLog(FontLoader.class);
 
     /** URI representing the font file */
@@ -53,14 +51,23 @@ public abstract class FontLoader {
 
     /** true if the font has been loaded */
     protected boolean loaded = false;
+    /** true if the font will be embedded, false if it will be referenced only. */
+    protected boolean embedded = true;
+    /** true if kerning information shall be loaded if available. */
+    protected boolean useKerning = true;
 
     /**
      * Default constructor.
      * @param fontFileURI the URI to the PFB file of a Type 1 font
+     * @param embedded indicates whether the font is embedded or referenced
+     * @param useKerning indicates whether kerning information shall be loaded if available
      * @param resolver the font resolver used to resolve URIs
      */
-    public FontLoader(String fontFileURI, FontResolver resolver) {
+    public FontLoader(String fontFileURI, boolean embedded, boolean useKerning,
+            FontResolver resolver) {
         this.fontFileURI = fontFileURI;
+        this.embedded = embedded;
+        this.useKerning = useKerning;
         this.resolver = resolver;
     }
 
@@ -71,44 +78,63 @@ public abstract class FontLoader {
     /**
      * Loads a custom font from a File. In the case of Type 1 fonts, the PFB file must be specified.
      * @param fontFile the File representation of the font
+     * @param subFontName the sub-fontname of a font (for TrueType Collections, null otherwise)
+     * @param embedded indicates whether the font is embedded or referenced
+     * @param encodingMode the requested encoding mode
      * @param resolver the font resolver to use when resolving URIs
      * @return the newly loaded font
      * @throws IOException In case of an I/O error
      */
-    public static CustomFont loadFont(File fontFile, FontResolver resolver)
-                throws IOException {
-        return loadFont(fontFile.getAbsolutePath(), resolver);
+    public static CustomFont loadFont(File fontFile, String subFontName,
+            boolean embedded, EncodingMode encodingMode, FontResolver resolver) throws IOException {
+        return loadFont(fontFile.toURI().toURL(), subFontName,
+                embedded, encodingMode, resolver);
     }
 
     /**
      * Loads a custom font from an URL. In the case of Type 1 fonts, the PFB file must be specified.
      * @param fontUrl the URL representation of the font
+     * @param subFontName the sub-fontname of a font (for TrueType Collections, null otherwise)
+     * @param embedded indicates whether the font is embedded or referenced
+     * @param encodingMode the requested encoding mode
      * @param resolver the font resolver to use when resolving URIs
      * @return the newly loaded font
      * @throws IOException In case of an I/O error
      */
-    public static CustomFont loadFont(URL fontUrl, FontResolver resolver)
-                throws IOException {
-        return loadFont(fontUrl.toExternalForm(), resolver);
+    public static CustomFont loadFont(URL fontUrl, String subFontName,
+            boolean embedded, EncodingMode encodingMode,
+            FontResolver resolver) throws IOException {
+        return loadFont(fontUrl.toExternalForm(), subFontName,
+                embedded, encodingMode, true,
+                resolver);
     }
-    
-    
+
     /**
      * Loads a custom font from a URI. In the case of Type 1 fonts, the PFB file must be specified.
      * @param fontFileURI the URI to the font
+     * @param subFontName the sub-fontname of a font (for TrueType Collections, null otherwise)
+     * @param embedded indicates whether the font is embedded or referenced
+     * @param encodingMode the requested encoding mode
+     * @param useKerning indicates whether kerning information should be loaded if available
      * @param resolver the font resolver to use when resolving URIs
      * @return the newly loaded font
      * @throws IOException In case of an I/O error
      */
-    public static CustomFont loadFont(String fontFileURI, FontResolver resolver)
-                throws IOException {
+    public static CustomFont loadFont(String fontFileURI, String subFontName,
+            boolean embedded, EncodingMode encodingMode, boolean useKerning,
+            FontResolver resolver) throws IOException {
         fontFileURI = fontFileURI.trim();
         boolean type1 = isType1(fontFileURI);
         FontLoader loader;
         if (type1) {
-            loader = new Type1FontLoader(fontFileURI, resolver);
+            if (encodingMode == EncodingMode.CID) {
+                throw new IllegalArgumentException(
+                        "CID encoding mode not supported for Type 1 fonts");
+            }
+            loader = new Type1FontLoader(fontFileURI, embedded, useKerning, resolver);
         } else {
-            loader = new TTFFontLoader(fontFileURI, resolver);
+            loader = new TTFFontLoader(fontFileURI, subFontName,
+                    embedded, encodingMode, useKerning, resolver);
         }
         return loader.getFont();
     }
@@ -121,14 +147,14 @@ public abstract class FontLoader {
      * @throws IOException In case of an I/O error
      * @throws MalformedURLException If an invalid URL is built
      */
-    protected static InputStream openFontUri(FontResolver resolver, String uri) 
+    public static InputStream openFontUri(FontResolver resolver, String uri)
                     throws IOException, MalformedURLException {
         InputStream in = null;
         if (resolver != null) {
             Source source = resolver.resolve(uri);
             if (source == null) {
-                String err = "Cannot load font: failed to create Source for font file " 
-                    + uri; 
+                String err = "Cannot load font: failed to create Source for font file "
+                    + uri;
                 throw new IOException(err);
             }
             if (source instanceof StreamSource) {
@@ -139,7 +165,7 @@ public abstract class FontLoader {
             }
             if (in == null) {
                 String err = "Cannot load font: failed to create InputStream from"
-                    + " Source for font file " + uri; 
+                    + " Source for font file " + uri;
                 throw new IOException(err);
             }
         } else {
@@ -147,7 +173,7 @@ public abstract class FontLoader {
         }
         return in;
     }
-        
+
     /**
      * Reads/parses the font data.
      * @throws IOException In case of an I/O error
