@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,28 +15,37 @@
  * limitations under the License.
  */
 
-/* $Id: FontTriplet.java 629093 2008-02-19 13:08:39Z jeremias $ */
- 
+/* $Id: FontTriplet.java 721430 2008-11-28 11:13:12Z acumiskey $ */
+
 package org.apache.fop.fonts;
 
 import java.io.Serializable;
 
+
 /**
  * FontTriplet contains information on name, style and weight of one font
  */
 public class FontTriplet implements Comparable, Serializable {
-    
+
     /** serial version UID */
     private static final long serialVersionUID = 1168991106658033508L;
-    
+
     private String name;
     private String style;
     private int weight;
     private int priority; // priority of this triplet/font mapping
-    
+
     //This is only a cache
     private transient String key;
 
+    /**
+     * Creates a new font triplet (for base14 use).
+     * @param name font name
+     */
+    public FontTriplet(String name) {
+        this.name = name;
+    }
+
     /**
      * Creates a new font triplet.
      * @param name font name
@@ -44,7 +53,7 @@ public class FontTriplet implements Comparable, Serializable {
      * @param weight font weight (100, 200, 300...800, 900)
      */
     public FontTriplet(String name, String style, int weight) {
-        this(name, style, weight, 0);
+        this(name, style, weight, Font.PRIORITY_DEFAULT);
     }
 
     /**
@@ -55,7 +64,7 @@ public class FontTriplet implements Comparable, Serializable {
      * @param priority priority of this triplet/font mapping
      */
     public FontTriplet(String name, String style, int weight, int priority) {
-        this.name = name;
+        this(name);
         this.style = style;
         this.weight = weight;
         this.priority = priority;
@@ -70,7 +79,7 @@ public class FontTriplet implements Comparable, Serializable {
     public String getStyle() {
         return style;
     }
-    
+
     /** @return the font weight */
     public int getWeight() {
         return weight;
@@ -88,7 +97,7 @@ public class FontTriplet implements Comparable, Serializable {
         }
         return this.key;
     }
-    
+
     /** {@inheritDoc} */
     public int compareTo(Object o) {
         return getKey().compareTo(((FontTriplet)o).getKey());
@@ -109,7 +118,7 @@ public class FontTriplet implements Comparable, Serializable {
             if (obj instanceof FontTriplet) {
                 FontTriplet other = (FontTriplet)obj;
                 return (getName().equals(other.getName())
-                        && getStyle().equals(other.getStyle()) 
+                        && getStyle().equals(other.getStyle())
                         && (getWeight() == other.getWeight()));
             }
         }
@@ -121,5 +130,19 @@ public class FontTriplet implements Comparable, Serializable {
         return getKey();
     }
 
+
+    /**
+     * Matcher interface for {@link FontTriplet}.
+     */
+    public interface Matcher {
+
+        /**
+         * Indicates whether the given {@link FontTriplet} matches a particular criterium.
+         * @param triplet the font triplet
+         * @return true if the font triplet is a match
+         */
+        boolean matches(FontTriplet triplet);
+    }
+
 }
 
