@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FontFinder.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: FontFinder.java 756471 2009-03-20 14:08:04Z acumiskey $ */
 
 package org.apache.fop.fonts.autodetect;
 
@@ -27,16 +27,16 @@ import java.util.List;
  * for available fonts.
  */
 public interface FontFinder {
-    
+
     /**
      * Finds a list of font files.
-     * 
+     *
      * @return list of font files. List&lt;URL&gt; in the case of the
      *         FontFinder, and List&lt;File&gt; in the case of the
      *         FonrDirFinders.
      * @throws IOException
      *             In case of an I/O problem
      */
-    List find() throws IOException;
-    
+    List/*<URL>*/ find() throws IOException;
+
 }
