@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,13 +15,12 @@
  * limitations under the License.
  */
 
-/* $Id: FontInfoFinder.java 629131 2008-02-19 15:47:48Z jeremias $ */
+/* $Id: FontInfoFinder.java 787668 2009-06-23 13:18:44Z acumiskey $ */
 
 package org.apache.fop.fonts.autodetect;
 
-import java.io.IOException;
+import java.io.InputStream;
 import java.net.URL;
-import java.net.URLConnection;
 import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
@@ -32,36 +31,52 @@ import org.apache.commons.io.IOUtils;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
-import org.apache.fop.fonts.CachedFontInfo;
 import org.apache.fop.fonts.CustomFont;
 import org.apache.fop.fonts.EmbedFontInfo;
+import org.apache.fop.fonts.EncodingMode;
 import org.apache.fop.fonts.Font;
 import org.apache.fop.fonts.FontCache;
+import org.apache.fop.fonts.FontEventListener;
 import org.apache.fop.fonts.FontLoader;
 import org.apache.fop.fonts.FontResolver;
 import org.apache.fop.fonts.FontTriplet;
 import org.apache.fop.fonts.FontUtil;
+import org.apache.fop.fonts.MultiByteFont;
+import org.apache.fop.fonts.truetype.FontFileReader;
+import org.apache.fop.fonts.truetype.TTFFile;
+import org.apache.fop.fonts.truetype.TTFFontLoader;
 
 /**
  * Attempts to determine correct FontInfo
  */
 public class FontInfoFinder {
-    
+
     /** logging instance */
     private Log log = LogFactory.getLog(FontInfoFinder.class);
 
+    private FontEventListener eventListener;
+
+    /**
+     * Sets the font event listener that can be used to receive events about particular events
+     * in this class.
+     * @param listener the font event listener
+     */
+    public void setEventListener(FontEventListener listener) {
+        this.eventListener = listener;
+    }
+
     /**
      * Attempts to determine FontTriplets from a given CustomFont.
      * It seems to be fairly accurate but will probably require some tweaking over time
-     * 
+     *
      * @param customFont CustomFont
      * @param triplet Collection that will take the generated triplets
      */
     private void generateTripletsFromFont(CustomFont customFont, Collection triplets) {
         if (log.isTraceEnabled()) {
-            log.trace("Font: " + customFont.getFullName() 
-                    + ", family: " + customFont.getFamilyNames() 
-                    + ", PS: " + customFont.getFontName() 
+            log.trace("Font: " + customFont.getFullName()
+                    + ", family: " + customFont.getFamilyNames()
+                    + ", PS: " + customFont.getFontName()
                     + ", EmbedName: " + customFont.getEmbedFontName());
         }
 
@@ -73,16 +88,16 @@ public class FontInfoFinder {
 
         String style = guessStyle(customFont, searchName);
         int weight; //= customFont.getWeight();
-        int guessedWeight = FontUtil.guessWeight(searchName); 
+        int guessedWeight = FontUtil.guessWeight(searchName);
         //We always take the guessed weight for now since it yield much better results.
         //OpenType's OS/2 usWeightClass value proves to be unreliable.
         weight = guessedWeight;
 
         //Full Name usually includes style/weight info so don't use these traits
         //If we still want to use these traits, we have to make FontInfo.fontLookup() smarter
-        triplets.add(new FontTriplet(fullName, Font.STYLE_NORMAL, Font.WEIGHT_NORMAL, 0));
+        triplets.add(new FontTriplet(fullName, Font.STYLE_NORMAL, Font.WEIGHT_NORMAL));
         if (!fullName.equals(strippedName)) {
-            triplets.add(new FontTriplet(strippedName, Font.STYLE_NORMAL, Font.WEIGHT_NORMAL, 0));
+            triplets.add(new FontTriplet(strippedName, Font.STYLE_NORMAL, Font.WEIGHT_NORMAL));
         }
         Set familyNames = customFont.getFamilyNames();
         Iterator iter = familyNames.iterator();
@@ -100,9 +115,9 @@ public class FontInfoFinder {
             }
         }
     }
-    
+
     private final Pattern quotePattern = Pattern.compile("'");
-    
+
     private String stripQuotes(String name) {
         return quotePattern.matcher(name).replaceAll("");
     }
@@ -111,13 +126,13 @@ public class FontInfoFinder {
         // style
         String style = Font.STYLE_NORMAL;
         if (customFont.getItalicAngle() > 0) {
-            style = Font.STYLE_ITALIC;  
+            style = Font.STYLE_ITALIC;
         } else {
             style = FontUtil.guessStyle(fontName);
         }
         return style;
     }
-    
+
     /**
      * Attempts to determine FontInfo from a given custom font
      * @param fontUrl the font URL
@@ -125,55 +140,46 @@ public class FontInfoFinder {
      * @param fontCache font cache (may be null)
      * @return
      */
-    private EmbedFontInfo fontInfoFromCustomFont(
+    private EmbedFontInfo getFontInfoFromCustomFont(
             URL fontUrl, CustomFont customFont, FontCache fontCache) {
         List fontTripletList = new java.util.ArrayList();
         generateTripletsFromFont(customFont, fontTripletList);
         String embedUrl;
         embedUrl = fontUrl.toExternalForm();
+        String subFontName = null;
+        if (customFont instanceof MultiByteFont) {
+            subFontName = ((MultiByteFont)customFont).getTTCName();
+        }
         EmbedFontInfo fontInfo = new EmbedFontInfo(null, customFont.isKerningEnabled(),
-                fontTripletList, embedUrl);
+                fontTripletList, embedUrl, subFontName);
+        fontInfo.setPostScriptName(customFont.getFontName());
         if (fontCache != null) {
             fontCache.addFont(fontInfo);
         }
         return fontInfo;
     }
-        
+
     /**
      * Attempts to determine EmbedFontInfo from a given font file.
-     * 
+     *
      * @param fontUrl font URL. Assumed to be local.
      * @param resolver font resolver used to resolve font
      * @param fontCache font cache (may be null)
-     * @return newly created embed font info
+     * @return an array of newly created embed font info. Generally, this array
+     *         will have only one entry, unless the fontUrl is a TrueType Collection
      */
-    public EmbedFontInfo find(URL fontUrl, FontResolver resolver, FontCache fontCache) {
+    public EmbedFontInfo[] find(URL fontUrl, FontResolver resolver, FontCache fontCache) {
         String embedUrl = null;
         embedUrl = fontUrl.toExternalForm();
-        
+
         long fileLastModified = -1;
         if (fontCache != null) {
-            try {
-                URLConnection conn = fontUrl.openConnection();
-                try {
-                    fileLastModified = conn.getLastModified();
-                } finally {
-                    //An InputStream is created even if it's not accessed, but we need to close it.
-                    IOUtils.closeQuietly(conn.getInputStream());
-                }
-            } catch (IOException e) {
-                // Should never happen, because URL must be local
-                log.debug("IOError: " + e.getMessage());
-                fileLastModified = 0;
-            }
+            fileLastModified = FontCache.getLastModified(fontUrl);
             // firstly try and fetch it from cache before loading/parsing the font file
             if (fontCache.containsFont(embedUrl)) {
-                CachedFontInfo fontInfo = fontCache.getFont(embedUrl);
-                if (fontInfo.lastModified() == fileLastModified) {
-                    return fontInfo;
-                } else {
-                    // out of date cache item
-                    fontCache.removeFont(embedUrl);
+                EmbedFontInfo[] fontInfos = fontCache.getFontInfos(embedUrl, fileLastModified);
+                if (fontInfos != null) {
+                    return fontInfos;
                 }
             // is this a previously failed parsed font?
             } else if (fontCache.isFailedFont(embedUrl, fileLastModified)) {
@@ -183,21 +189,88 @@ public class FontInfoFinder {
                 return null;
             }
         }
-        
+
+
         // try to determine triplet information from font file
         CustomFont customFont = null;
-        try {
-            customFont = FontLoader.loadFont(fontUrl, resolver);
-        } catch (Exception e) {
-            //TODO Too verbose (it's an error but we don't care if some fonts can't be loaded)
-            if (log.isErrorEnabled()) {
-                log.error("Unable to load font file: " + embedUrl + ". Reason: " + e.getMessage());
+        if (fontUrl.toExternalForm().endsWith(".ttc")) {
+            // Get a list of the TTC Font names
+            List ttcNames = null; //List<String>
+            String fontFileURI = fontUrl.toExternalForm().trim();
+            InputStream in = null;
+            try {
+                in = FontLoader.openFontUri(resolver, fontFileURI);
+                TTFFile ttf = new TTFFile();
+                FontFileReader reader = new FontFileReader(in);
+                ttcNames = ttf.getTTCnames(reader);
+            } catch (Exception e) {
+                if (this.eventListener != null) {
+                    this.eventListener.fontLoadingErrorAtAutoDetection(this, fontFileURI, e);
+                }
+                return null;
+            } finally {
+                IOUtils.closeQuietly(in);
             }
-            if (fontCache != null) {
-                fontCache.registerFailedFont(embedUrl, fileLastModified);
+
+            List/*<EmbedFontInfo>*/ embedFontInfoList
+                = new java.util.ArrayList/*<EmbedFontInfo>*/();
+
+            // For each font name ...
+            //for (String fontName : ttcNames) {
+            Iterator ttcNamesIterator = ttcNames.iterator();
+            while (ttcNamesIterator.hasNext()) {
+                String fontName = (String)ttcNamesIterator.next();
+
+                if (log.isDebugEnabled()) {
+                    log.debug("Loading " + fontName);
+                }
+                try {
+                    TTFFontLoader ttfLoader = new TTFFontLoader(
+                            fontFileURI, fontName, true, EncodingMode.AUTO, true, resolver);
+                    customFont = ttfLoader.getFont();
+                    if (this.eventListener != null) {
+                        customFont.setEventListener(this.eventListener);
+                    }
+                } catch (Exception e) {
+                    if (fontCache != null) {
+                        fontCache.registerFailedFont(embedUrl, fileLastModified);
+                    }
+                    if (this.eventListener != null) {
+                        this.eventListener.fontLoadingErrorAtAutoDetection(this, embedUrl, e);
+                    }
+                    continue;
+                }
+                EmbedFontInfo fi = getFontInfoFromCustomFont(fontUrl, customFont, fontCache);
+                if (fi != null) {
+                    embedFontInfoList.add(fi);
+                }
+            }
+            return (EmbedFontInfo[])embedFontInfoList.toArray(
+                    new EmbedFontInfo[embedFontInfoList.size()]);
+        } else {
+            // The normal case
+            try {
+                customFont = FontLoader.loadFont(fontUrl, null, true, EncodingMode.AUTO, resolver);
+                if (this.eventListener != null) {
+                    customFont.setEventListener(this.eventListener);
+                }
+            } catch (Exception e) {
+                if (fontCache != null) {
+                    fontCache.registerFailedFont(embedUrl, fileLastModified);
+                }
+                if (this.eventListener != null) {
+                    this.eventListener.fontLoadingErrorAtAutoDetection(this, embedUrl, e);
+                }
+                return null;
+            }
+            EmbedFontInfo fi = getFontInfoFromCustomFont(fontUrl, customFont, fontCache);
+            if (fi != null) {
+                return new EmbedFontInfo[] {fi};
+            } else {
+                return null;
             }
-            return null;
         }
-        return fontInfoFromCustomFont(fontUrl, customFont, fontCache);     
+
     }
+
 }
