@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Type1FontLoader.java 632321 2008-02-29 14:03:01Z jeremias $ */
+/* $Id: Type1FontLoader.java 910445 2010-02-16 09:36:40Z jeremias $ */
 
 package org.apache.fop.fonts.type1;
 
@@ -32,6 +32,7 @@ import org.apache.fop.fonts.CodePointMapping;
 import org.apache.fop.fonts.FontLoader;
 import org.apache.fop.fonts.FontResolver;
 import org.apache.fop.fonts.FontType;
+import org.apache.fop.fonts.SingleByteEncoding;
 import org.apache.fop.fonts.SingleByteFont;
 
 /**
@@ -40,16 +41,18 @@ import org.apache.fop.fonts.SingleByteFont;
 public class Type1FontLoader extends FontLoader {
 
     private SingleByteFont singleFont;
-    
+
     /**
      * Constructs a new Type 1 font loader.
      * @param fontFileURI the URI to the PFB file of a Type 1 font
+     * @param embedded indicates whether the font is embedded or referenced
+     * @param useKerning indicates whether to load kerning information if available
      * @param resolver the font resolver used to resolve URIs
      * @throws IOException In case of an I/O error
      */
-    public Type1FontLoader(String fontFileURI, FontResolver resolver) 
-                throws IOException {
-        super(fontFileURI, resolver);
+    public Type1FontLoader(String fontFileURI, boolean embedded, boolean useKerning,
+            FontResolver resolver) throws IOException {
+        super(fontFileURI, embedded, useKerning, resolver);
     }
 
     private String getPFMURI(String pfbURI) {
@@ -58,14 +61,14 @@ public class Type1FontLoader extends FontLoader {
                 + (Character.isUpperCase(pfbExt.charAt(2)) ? "M" : "m");
         return pfbURI.substring(0, pfbURI.length() - 4) + "." + pfmExt;
     }
-    
+
     private static final String[] AFM_EXTENSIONS = new String[] {".AFM", ".afm", ".Afm"};
-    
+
     /** {@inheritDoc} */
     protected void read() throws IOException {
         AFMFile afm = null;
         PFMFile pfm = null;
-        
+
         InputStream afmIn = null;
         for (int i = 0; i < AFM_EXTENSIONS.length; i++) {
             try {
@@ -87,7 +90,7 @@ public class Type1FontLoader extends FontLoader {
                 IOUtils.closeQuietly(afmIn);
             }
         }
-        
+
         String pfmUri = getPFMURI(this.fontFileURI);
         InputStream pfmIn = null;
         try {
@@ -99,20 +102,20 @@ public class Type1FontLoader extends FontLoader {
             try {
                 pfm = new PFMFile();
                 pfm.load(pfmIn);
+            } catch (IOException ioe) {
+                if (afm == null) {
+                    //Ignore the exception if we have a valid PFM. PFM is only the fallback.
+                    throw ioe;
+                }
             } finally {
                 IOUtils.closeQuietly(pfmIn);
             }
         }
-        
+
         if (afm == null && pfm == null) {
             throw new java.io.FileNotFoundException(
                     "Neither an AFM nor a PFM file was found for " + this.fontFileURI);
         }
-        if (pfm == null) {      
-            //Cannot do without the PFM for now         
-            throw new java.io.FileNotFoundException(    
-                    "No PFM file was found for " + this.fontFileURI);   
-        }
         buildFont(afm, pfm);
         this.loaded = true;
     }
@@ -124,15 +127,26 @@ public class Type1FontLoader extends FontLoader {
         singleFont = new SingleByteFont();
         singleFont.setFontType(FontType.TYPE1);
         singleFont.setResolver(this.resolver);
-        singleFont.setEmbedFileName(this.fontFileURI);
+        if (this.embedded) {
+            singleFont.setEmbedFileName(this.fontFileURI);
+        }
         returnFont = singleFont;
-        
+
+        handleEncoding(afm, pfm);
+        handleFontName(afm, pfm);
+        handleMetrics(afm, pfm);
+    }
+
+    private void handleEncoding(AFMFile afm, PFMFile pfm) {
         //Encoding
         if (afm != null) {
             String encoding = afm.getEncodingScheme();
+            singleFont.setUseNativeEncoding(true);
             if ("AdobeStandardEncoding".equals(encoding)) {
-                //Use WinAnsi in this case as it better fits the usual character set people need
-                singleFont.setEncoding(CodePointMapping.WIN_ANSI_ENCODING);
+                singleFont.setEncoding(CodePointMapping.STANDARD_ENCODING);
+                addUnencodedBasedOnEncoding(afm);
+                //char codes in the AFM cannot be relied on in this case, so we override
+                afm.overridePrimaryEncoding(singleFont.getEncoding());
             } else {
                 String effEncodingName;
                 if ("FontSpecific".equals(encoding)) {
@@ -146,17 +160,64 @@ public class Type1FontLoader extends FontLoader {
                 }
                 CodePointMapping mapping = buildCustomEncoding(effEncodingName, afm);
                 singleFont.setEncoding(mapping);
+                addUnencodedBasedOnAFM(afm);
             }
         } else {
             if (pfm.getCharSet() >= 0 && pfm.getCharSet() <= 2) {
                 singleFont.setEncoding(pfm.getCharSetName() + "Encoding");
             } else {
-                log.warn("The PFM reports an unsupported encoding (" 
+                log.warn("The PFM reports an unsupported encoding ("
                         + pfm.getCharSetName() + "). The font may not work as expected.");
                 singleFont.setEncoding("WinAnsiEncoding"); //Try fallback, no guarantees!
             }
         }
-        
+    }
+
+    private Set toGlyphSet(String[] glyphNames) {
+        Set glyphSet = new java.util.HashSet();
+        for (int i = 0, c = glyphNames.length; i < c; i++) {
+            glyphSet.add(glyphNames[i]);
+        }
+        return glyphSet;
+    }
+
+    /**
+     * Adds characters not encoded in the font's primary encoding. This method is used when we
+     * don't trust the AFM to expose the same encoding as the primary font.
+     * @param afm the AFM file.
+     */
+    private void addUnencodedBasedOnEncoding(AFMFile afm) {
+        SingleByteEncoding encoding = singleFont.getEncoding();
+        Set glyphNames = toGlyphSet(encoding.getCharNameMap());
+        List charMetrics = afm.getCharMetrics();
+        for (int i = 0, c = afm.getCharCount(); i < c; i++) {
+            AFMCharMetrics metrics = (AFMCharMetrics)charMetrics.get(i);
+            String charName = metrics.getCharName();
+            if (charName != null && !glyphNames.contains(charName)) {
+                singleFont.addUnencodedCharacter(metrics.getCharacter(),
+                        (int)Math.round(metrics.getWidthX()));
+            }
+        }
+    }
+
+    /**
+     * Adds characters not encoded in the font's primary encoding. This method is used when
+     * the primary encoding is built based on the character codes in the AFM rather than
+     * the specified encoding (ex. with symbolic fonts).
+     * @param afm the AFM file
+     */
+    private void addUnencodedBasedOnAFM(AFMFile afm) {
+        List charMetrics = afm.getCharMetrics();
+        for (int i = 0, c = afm.getCharCount(); i < c; i++) {
+            AFMCharMetrics metrics = (AFMCharMetrics)charMetrics.get(i);
+            if (!metrics.hasCharCode() && metrics.getCharacter() != null) {
+                singleFont.addUnencodedCharacter(metrics.getCharacter(),
+                        (int)Math.round(metrics.getWidthX()));
+            }
+        }
+    }
+
+    private void handleFontName(AFMFile afm, PFMFile pfm) {
         //Font name
         if (afm != null) {
             returnFont.setFontName(afm.getFontName()); //PostScript font name
@@ -173,7 +234,9 @@ public class Type1FontLoader extends FontLoader {
             names.add(pfm.getWindowsName()); //emulate afm.getFamilyName()
             returnFont.setFamilyNames(names);
         }
-        
+    }
+
+    private void handleMetrics(AFMFile afm, PFMFile pfm) {
         //Basic metrics
         if (afm != null) {
             if (afm.getCapHeight() != null) {
@@ -188,7 +251,7 @@ public class Type1FontLoader extends FontLoader {
             if (afm.getDescender() != null) {
                 returnFont.setDescender(afm.getDescender().intValue());
             }
-            
+
             returnFont.setFontBBox(afm.getFontBBoxAsIntArray());
             if (afm.getStdVW() != null) {
                 returnFont.setStemV(afm.getStdVW().intValue());
@@ -216,7 +279,7 @@ public class Type1FontLoader extends FontLoader {
                 returnFont.setDescender(pfm.getLowerCaseDescent());
             }
         }
-        
+
         //Fallbacks when some crucial font metrics aren't available
         //(the following are all optional in AFM, but FontBBox is always available)
         if (returnFont.getXHeight(1) == 0) {
@@ -270,12 +333,28 @@ public class Type1FontLoader extends FontLoader {
         if (returnFont.getCapHeight() == 0) {
             returnFont.setCapHeight(returnFont.getAscender());
         }
-        
-        /* DISABLED because of mismatch with our using WinAnsiEncoding and the AFM
-        delivering the font's default encoding.
+
         if (afm != null) {
-            //TODO returnFont.setFlags(flags);
-             
+            String charSet = afm.getCharacterSet();
+            int flags = 0;
+            if ("Special".equals(charSet)) {
+                flags |= 4; //bit 3: Symbolic
+            } else {
+                if (singleFont.getEncoding().mapChar('A') == 'A') {
+                    //High likelyhood that the font is non-symbolic
+                    flags |= 32; //bit 6: Nonsymbolic
+                } else {
+                    flags |= 4; //bit 3: Symbolic
+                }
+            }
+            if (afm.getWritingDirectionMetrics(0).isFixedPitch()) {
+                flags |= 1; //bit 1: FixedPitch
+            }
+            if (afm.getWritingDirectionMetrics(0).getItalicAngle() != 0.0) {
+                flags |= 64; //bit 7: Italic
+            }
+            returnFont.setFlags(flags);
+
             returnFont.setFirstChar(afm.getFirstChar());
             returnFont.setLastChar(afm.getLastChar());
             Iterator iter = afm.getCharMetrics().iterator();
@@ -285,16 +364,20 @@ public class Type1FontLoader extends FontLoader {
                     singleFont.setWidth(chm.getCharCode(), (int)Math.round(chm.getWidthX()));
                 }
             }
-            returnFont.replaceKerningMap(afm.createXKerningMapEncoded());
-        } else {*/
+            if (useKerning) {
+                returnFont.replaceKerningMap(afm.createXKerningMapEncoded());
+            }
+        } else {
             returnFont.setFlags(pfm.getFlags());
             returnFont.setFirstChar(pfm.getFirstChar());
             returnFont.setLastChar(pfm.getLastChar());
             for (short i = pfm.getFirstChar(); i <= pfm.getLastChar(); i++) {
                 singleFont.setWidth(i, pfm.getCharWidth(i));
             }
-            returnFont.replaceKerningMap(pfm.getKerning());
-        //}
+            if (useKerning) {
+                returnFont.replaceKerningMap(pfm.getKerning());
+            }
+        }
     }
 
     private CodePointMapping buildCustomEncoding(String encodingName, AFMFile afm) {
