@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: HyphenationTree.java 627367 2008-02-13 12:03:30Z maxberger $ */
- 
+/* $Id: HyphenationTree.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+
 package org.apache.fop.hyphenation;
 
 import java.io.BufferedReader;
@@ -41,7 +41,7 @@ import org.xml.sax.InputSource;
  *
  * @author Carlos Villegas <cav@uniscope.co.jp>
  */
-public class HyphenationTree extends TernaryTree 
+public class HyphenationTree extends TernaryTree
             implements PatternConsumer, Serializable {
 
     private static final long serialVersionUID = -7842107987915665573L;
@@ -125,10 +125,10 @@ public class HyphenationTree extends TernaryTree
     public void loadPatterns(String filename) throws HyphenationException {
         File f = new File(filename);
         try {
-            InputSource src = new InputSource(f.toURL().toExternalForm());
+            InputSource src = new InputSource(f.toURI().toURL().toExternalForm());
             loadPatterns(src);
         } catch (MalformedURLException e) {
-            throw new HyphenationException("Error converting the File '" + f + "' to a URL: " 
+            throw new HyphenationException("Error converting the File '" + f + "' to a URL: "
                     + e.getMessage());
         }
     }
