@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: PatternParser.java 627367 2008-02-13 12:03:30Z maxberger $ */
- 
+/* $Id: PatternParser.java 808166 2009-08-26 20:00:24Z spepping $ */
+
 package org.apache.fop.hyphenation;
 
 // SAX
@@ -30,7 +30,10 @@ import org.xml.sax.Attributes;
 // Java
 import java.io.File;
 import java.io.FileNotFoundException;
+import java.io.FileOutputStream;
 import java.io.IOException;
+import java.io.InputStream;
+import java.io.PrintStream;
 import java.net.MalformedURLException;
 import java.util.ArrayList;
 
@@ -44,13 +47,14 @@ import javax.xml.parsers.SAXParserFactory;
  */
 public class PatternParser extends DefaultHandler implements PatternConsumer {
 
-    XMLReader parser;
-    int currElement;
-    PatternConsumer consumer;
-    StringBuffer token;
-    ArrayList exception;
-    char hyphenChar;
-    String errMsg;
+    private XMLReader parser;
+    private int currElement;
+    private PatternConsumer consumer;
+    private StringBuffer token;
+    private ArrayList exception;
+    private char hyphenChar;
+    private String errMsg;
+    private boolean hasClasses = false;
 
     static final int ELEM_CLASSES = 1;
     static final int ELEM_EXCEPTIONS = 2;
@@ -58,24 +62,19 @@ public class PatternParser extends DefaultHandler implements PatternConsumer {
     static final int ELEM_HYPHEN = 4;
 
     public PatternParser() throws HyphenationException {
+        this.consumer = this;
         token = new StringBuffer();
         parser = createParser();
         parser.setContentHandler(this);
         parser.setErrorHandler(this);
         hyphenChar = '-';    // default
-
     }
 
-    public PatternParser(PatternConsumer consumer)
-            throws HyphenationException {
+    public PatternParser(PatternConsumer consumer) throws HyphenationException {
         this();
         this.consumer = consumer;
     }
 
-    public void setConsumer(PatternConsumer consumer) {
-        this.consumer = consumer;
-    }
-
     /**
      * Parses a hyphenation pattern file.
      * @param filename the filename
@@ -84,7 +83,7 @@ public class PatternParser extends DefaultHandler implements PatternConsumer {
     public void parse(String filename) throws HyphenationException {
         parse(new File(filename));
     }
-    
+
     /**
      * Parses a hyphenation pattern file.
      * @param file the pattern file
@@ -92,10 +91,10 @@ public class PatternParser extends DefaultHandler implements PatternConsumer {
      */
     public void parse(File file) throws HyphenationException {
         try {
-            InputSource src = new InputSource(file.toURL().toExternalForm());
+            InputSource src = new InputSource(file.toURI().toURL().toExternalForm());
             parse(src);
         } catch (MalformedURLException e) {
-            throw new HyphenationException("Error converting the File '" + file + "' to a URL: " 
+            throw new HyphenationException("Error converting the File '" + file + "' to a URL: "
                     + e.getMessage());
         }
     }
@@ -116,7 +115,7 @@ public class PatternParser extends DefaultHandler implements PatternConsumer {
             throw new HyphenationException(errMsg);
         }
     }
-    
+
     /**
      * Creates a SAX parser using JAXP
      * @return the created SAX parser
@@ -249,15 +248,32 @@ public class PatternParser extends DefaultHandler implements PatternConsumer {
         return il.toString();
     }
 
+    protected void getExternalClasses() throws SAXException {
+        XMLReader mainParser = parser;
+        parser = createParser();
+        parser.setContentHandler(this);
+        parser.setErrorHandler(this);
+        InputStream stream = this.getClass().getResourceAsStream("classes.xml");
+        InputSource source = new InputSource(stream);
+        try {
+            parser.parse(source);
+        } catch (IOException ioe) {
+            throw new SAXException(ioe.getMessage());
+        } finally {
+            parser = mainParser;
+        }
+    }
+    
     //
     // ContentHandler methods
     //
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
+     * @throws SAXException 
      */
     public void startElement(String uri, String local, String raw,
-                             Attributes attrs) {
+                             Attributes attrs) throws SAXException {
         if (local.equals("hyphen-char")) {
             String h = attrs.getValue("value");
             if (h != null && h.length() == 1) {
@@ -266,8 +282,14 @@ public class PatternParser extends DefaultHandler implements PatternConsumer {
         } else if (local.equals("classes")) {
             currElement = ELEM_CLASSES;
         } else if (local.equals("patterns")) {
+            if (!hasClasses) {
+                getExternalClasses();
+            }
             currElement = ELEM_PATTERNS;
         } else if (local.equals("exceptions")) {
+            if (!hasClasses) {
+                getExternalClasses();
+            }
             currElement = ELEM_EXCEPTIONS;
             exception = new ArrayList();
         } else if (local.equals("hyphen")) {
@@ -283,7 +305,7 @@ public class PatternParser extends DefaultHandler implements PatternConsumer {
     }
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
     public void endElement(String uri, String local, String raw) {
 
@@ -311,6 +333,9 @@ public class PatternParser extends DefaultHandler implements PatternConsumer {
                 token.setLength(0);
             }
         }
+        if (currElement == ELEM_CLASSES) {
+            hasClasses = true;
+        }
         if (currElement == ELEM_HYPHEN) {
             currElement = ELEM_EXCEPTIONS;
         } else {
@@ -320,7 +345,7 @@ public class PatternParser extends DefaultHandler implements PatternConsumer {
     }
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
     public void characters(char ch[], int start, int length) {
         StringBuffer chars = new StringBuffer(length);
@@ -403,23 +428,46 @@ public class PatternParser extends DefaultHandler implements PatternConsumer {
 
     // PatternConsumer implementation for testing purposes
     public void addClass(String c) {
-        System.out.println("class: " + c);
+        testOut.println("class: " + c);
     }
 
     public void addException(String w, ArrayList e) {
-        System.out.println("exception: " + w + " : " + e.toString());
+        testOut.println("exception: " + w + " : " + e.toString());
     }
 
     public void addPattern(String p, String v) {
-        System.out.println("pattern: " + p + " : " + v);
+        testOut.println("pattern: " + p + " : " + v);
+    }
+    
+    private PrintStream testOut = System.out;
+    
+    /**
+     * @param testOut the testOut to set
+     */
+    public void setTestOut(PrintStream testOut) {
+        this.testOut = testOut;
+    }
+    
+    public void closeTestOut() {
+        testOut.flush();
+        testOut.close();
     }
 
     public static void main(String[] args) throws Exception {
         if (args.length > 0) {
             PatternParser pp = new PatternParser();
-            pp.setConsumer(pp);
+            PrintStream p = null;
+            if (args.length > 1) {
+                FileOutputStream f = new FileOutputStream(args[1]);
+                p = new PrintStream(f, false, "utf-8");
+                pp.setTestOut(p);
+            }
             pp.parse(args[0]);
+            if (pp != null) {
+                pp.closeTestOut();
+            }
         }
     }
 
+ 
 }
