@@ -15,13 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id: BlockLayoutManager.java 635508 2008-03-10 10:06:37Z jeremias $ */
+/* $Id: BlockLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr;
 
 import java.util.LinkedList;
 import java.util.List;
 import java.util.ListIterator;
+import java.util.Stack;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -30,10 +31,10 @@ import org.apache.fop.area.Area;
 import org.apache.fop.area.Block;
 import org.apache.fop.area.LineArea;
 import org.apache.fop.datatypes.Length;
+import org.apache.fop.fo.properties.KeepProperty;
 import org.apache.fop.fonts.Font;
 import org.apache.fop.fonts.FontInfo;
 import org.apache.fop.fonts.FontTriplet;
-import org.apache.fop.layoutmgr.inline.InlineLayoutManager;
 import org.apache.fop.layoutmgr.inline.InlineLevelLayoutManager;
 import org.apache.fop.layoutmgr.inline.LineLayoutManager;
 import org.apache.fop.traits.MinOptMax;
@@ -42,14 +43,14 @@ import org.apache.fop.traits.SpaceVal;
 /**
  * LayoutManager for a block FO.
  */
-public class BlockLayoutManager extends BlockStackingLayoutManager 
+public class BlockLayoutManager extends BlockStackingLayoutManager
             implements ConditionalElementListener {
 
     /**
      * logging instance
      */
     private static Log log = LogFactory.getLog(BlockLayoutManager.class);
-    
+
     private Block curBlockArea;
 
     /** Iterator over the child layout managers. */
@@ -59,7 +60,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     private Length lineHeight;
     private int follow = 2000;
     //private int middleShift = 0;
-    
+
     private boolean discardBorderBefore;
     private boolean discardBorderAfter;
     private boolean discardPaddingBefore;
@@ -67,9 +68,6 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     private MinOptMax effSpaceBefore;
     private MinOptMax effSpaceAfter;
 
-    /** The list of child BreakPoss instances. */
-    protected List childBreaks = new java.util.ArrayList();
-
     /**
      * Creates a new BlockLayoutManager.
      * @param inBlock the block FO object to create the layout manager for.
@@ -79,17 +77,19 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         proxyLMiter = new ProxyLMiter();
     }
 
+    /** {@inheritDoc} */
     public void initialize() {
         super.initialize();
         FontInfo fi = getBlockFO().getFOEventHandler().getFontInfo();
         FontTriplet[] fontkeys = getBlockFO().getCommonFont().getFontState(fi);
-        Font initFont = fi.getFontInstance(fontkeys[0], getBlockFO().getCommonFont().fontSize.getValue(this));
+        Font initFont = fi.getFontInstance(fontkeys[0],
+                getBlockFO().getCommonFont().fontSize.getValue(this));
         lead = initFont.getAscender();
         follow = -initFont.getDescender();
         //middleShift = -fs.getXHeight() / 2;
         lineHeight = getBlockFO().getLineHeight().getOptimum(this).getLength();
         startIndent = getBlockFO().getCommonMarginBlock().startIndent.getValue(this);
-        endIndent = getBlockFO().getCommonMarginBlock().endIndent.getValue(this); 
+        endIndent = getBlockFO().getCommonMarginBlock().endIndent.getValue(this);
         foSpaceBefore = new SpaceVal(getBlockFO().getCommonMarginBlock().spaceBefore, this)
                             .getSpace();
         foSpaceAfter = new SpaceVal(getBlockFO().getCommonMarginBlock().spaceAfter, this)
@@ -111,20 +111,31 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     }
 
     /** {@inheritDoc} */
-    public LinkedList getNextKnuthElements(LayoutContext context, int alignment) {
-        resetSpaces(); 
-        return super.getNextKnuthElements(context, alignment);
+    public List getNextKnuthElements(LayoutContext context, int alignment) {
+        return getNextKnuthElements(context, alignment, null, null, null);
     }
-   
+
+    /** {@inheritDoc} */
+    public List getNextKnuthElements(LayoutContext context, int alignment, Stack lmStack,
+            Position restartPosition, LayoutManager restartAtLM) {
+        resetSpaces();
+        if (lmStack == null) {
+            return super.getNextKnuthElements(context, alignment);
+        } else {
+            return super.getNextKnuthElements(context, alignment, lmStack, restartPosition,
+                    restartAtLM);
+        }
+    }
+
     private void resetSpaces() {
-        this.discardBorderBefore = false;        
-        this.discardBorderAfter = false;        
-        this.discardPaddingBefore = false;        
+        this.discardBorderBefore = false;
+        this.discardBorderAfter = false;
+        this.discardPaddingBefore = false;
         this.discardPaddingAfter = false;
         this.effSpaceBefore = null;
         this.effSpaceAfter = null;
     }
-    
+
     /**
      * Proxy iterator for Block LM.
      * This iterator creates and holds the complete list
@@ -135,7 +146,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
      */
     protected class ProxyLMiter extends LMiter {
 
-        /*
+        /**
          * Constructs a proxy iterator for Block LM.
          */
         public ProxyLMiter() {
@@ -147,10 +158,11 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
          * @return true if there are more child lms
          */
         public boolean hasNext() {
-            return (curPos < listLMs.size()) ? true : createNextChildLMs(curPos);
+            return (curPos < listLMs.size()) || createNextChildLMs(curPos);
         }
 
         /**
+         * @param pos ...
          * @return true if new child lms were added
          */
         protected boolean createNextChildLMs(int pos) {
@@ -206,40 +218,22 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         return llm;
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public boolean mustKeepTogether() {
-        // TODO Keeps will have to be more sophisticated sooner or later
-        // TODO This is a quick fix for the fact that the parent is not always a BlockLevelLM;
-        // eventually mustKeepTogether() must be moved up to the LM interface
-        return (!getBlockFO().getKeepTogether().getWithinPage().isAuto()
-                || !getBlockFO().getKeepTogether().getWithinColumn().isAuto()
-                || (getParent() instanceof BlockLevelLayoutManager
-                    && ((BlockLevelLayoutManager) getParent()).mustKeepTogether())
-                || (getParent() instanceof InlineLayoutManager
-                    && ((InlineLayoutManager) getParent()).mustKeepTogether()));
+    /** {@inheritDoc} */
+    public KeepProperty getKeepTogetherProperty() {
+        return getBlockFO().getKeepTogether();
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public boolean mustKeepWithPrevious() {
-        return !getBlockFO().getKeepWithPrevious().getWithinPage().isAuto()
-                || !getBlockFO().getKeepWithPrevious().getWithinColumn().isAuto();
+    /** {@inheritDoc} */
+    public KeepProperty getKeepWithPreviousProperty() {
+        return getBlockFO().getKeepWithPrevious();
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public boolean mustKeepWithNext() {
-        return !getBlockFO().getKeepWithNext().getWithinPage().isAuto()
-                || !getBlockFO().getKeepWithNext().getWithinColumn().isAuto();
+    /** {@inheritDoc} */
+    public KeepProperty getKeepWithNextProperty() {
+        return getBlockFO().getKeepWithNext();
     }
 
-    /**
-     * {@inheritDoc} 
-     */
+    /** {@inheritDoc} */
     public void addAreas(PositionIterator parentIter,
             LayoutContext layoutContext) {
         getParentArea(null);
@@ -247,10 +241,10 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         // if this will create the first block area in a page
         // and display-align is after or center, add space before
         if (layoutContext.getSpaceBefore() > 0) {
-            addBlockSpacing(0.0, new MinOptMax(layoutContext.getSpaceBefore()));
+            addBlockSpacing(0.0, MinOptMax.getInstance(layoutContext.getSpaceBefore()));
         }
 
-        LayoutManager childLM = null;
+        LayoutManager childLM;
         LayoutManager lastLM = null;
         LayoutContext lc = new LayoutContext(0);
         lc.setSpaceAdjust(layoutContext.getSpaceAdjust());
@@ -264,8 +258,8 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         // and put them in a new list;
         LinkedList positionList = new LinkedList();
         Position pos;
-        boolean bSpaceBefore = false;
-        boolean bSpaceAfter = false;
+        boolean spaceBefore = false;
+        boolean spaceAfter = false;
         Position firstPos = null;
         Position lastPos = null;
         while (parentIter.hasNext()) {
@@ -280,7 +274,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
             Position innerPosition = pos;
             if (pos instanceof NonLeafPosition) {
                 //Not all elements are wrapped
-                innerPosition = ((NonLeafPosition) pos).getPosition();
+                innerPosition = pos.getPosition();
             }
             if (innerPosition == null) {
                 // pos was created by this BlockLM and was inside an element
@@ -288,11 +282,11 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
                 // this means the space was not discarded
                 if (positionList.size() == 0) {
                     // pos was in the element representing space-before
-                    bSpaceBefore = true;
+                    spaceBefore = true;
                     //log.trace(" space before");
                 } else {
                     // pos was in the element representing space-after
-                    bSpaceAfter = true;
+                    spaceAfter = true;
                     //log.trace(" space-after");
                 }
             } else if (innerPosition.getLM() == this
@@ -309,15 +303,15 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
             }
         }
 
-        getPSLM().addIDToPage(getBlockFO().getId());
-        
+        addId();
+
         addMarkersToPage(true, isFirst(firstPos), isLast(lastPos));
 
         if (bpUnit == 0) {
             // the Positions in positionList were inside the elements
             // created by the LineLM
             childPosIter = new StackingIter(positionList.listIterator());
-            } else {
+        } else {
             // the Positions in positionList were inside the elements
             // created by the BlockLM in the createUnitElements() method
             //if (((Position) positionList.getLast()) instanceof
@@ -345,37 +339,37 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
                 // by this BlockLM, and must be ignored
                 if (element.getLayoutManager() != this) {
                     splitList.add(element);
-                    splitLength += element.getW();
+                    splitLength += element.getWidth();
                     lastLM = element.getLayoutManager();
                 }
             }
             //log.debug("Adding areas from " + iFirst + " to " + iLast);
             //log.debug("splitLength= " + splitLength
             //                   + " (" + neededUnits(splitLength) + " units') "
-            //                   + (neededUnits(splitLength) * bpUnit - splitLength) 
+            //                   + (neededUnits(splitLength) * bpUnit - splitLength)
             //                   + " spacing");
             // add space before and / or after the paragraph
             // to reach a multiple of bpUnit
-            if (bSpaceBefore && bSpaceAfter) {
+            if (spaceBefore && spaceAfter) {
                 foSpaceBefore = new SpaceVal(getBlockFO()
                                     .getCommonMarginBlock().spaceBefore, this).getSpace();
                 foSpaceAfter = new SpaceVal(getBlockFO()
                                     .getCommonMarginBlock().spaceAfter, this).getSpace();
                 adjustedSpaceBefore = (neededUnits(splitLength
-                        + foSpaceBefore.min
-                        + foSpaceAfter.min)
+                        + foSpaceBefore.getMin()
+                        + foSpaceAfter.getMin())
                         * bpUnit - splitLength) / 2;
                 adjustedSpaceAfter = neededUnits(splitLength
-                        + foSpaceBefore.min
-                        + foSpaceAfter.min)
+                        + foSpaceBefore.getMin()
+                        + foSpaceAfter.getMin())
                         * bpUnit - splitLength - adjustedSpaceBefore;
-                } else if (bSpaceBefore) {
+                } else if (spaceBefore) {
                 adjustedSpaceBefore = neededUnits(splitLength
-                        + foSpaceBefore.min)
+                        + foSpaceBefore.getMin())
                         * bpUnit - splitLength;
                 } else {
                 adjustedSpaceAfter = neededUnits(splitLength
-                        + foSpaceAfter.min)
+                        + foSpaceAfter.getMin())
                         * bpUnit - splitLength;
                 }
             //log.debug("spazio prima = " + adjustedSpaceBefore
@@ -397,15 +391,16 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
 
         addMarkersToPage(false, isFirst(firstPos), isLast(lastPos));
 
-        TraitSetter.addSpaceBeforeAfter(curBlockArea, layoutContext.getSpaceAdjust(), 
+        TraitSetter.addPtr(curBlockArea, getBlockFO().getPtr());  // used for accessibility
+        TraitSetter.addSpaceBeforeAfter(curBlockArea, layoutContext.getSpaceAdjust(),
                 effSpaceBefore, effSpaceAfter);
         flush();
 
         curBlockArea = null;
         resetSpaces();
-        
-        // Notify end of block layout manager to the PSLM
-        getPSLM().notifyEndOfLayout(getBlockFO().getId());
+
+        //Notify end of block layout manager to the PSLM
+        checkEndOfLayout(lastPos);
     }
 
     /**
@@ -426,23 +421,23 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
 
             curBlockArea.setIPD(super.getContentAreaIPD());
 
-            TraitSetter.addBreaks(curBlockArea, 
+            TraitSetter.addBreaks(curBlockArea,
                     getBlockFO().getBreakBefore(), getBlockFO().getBreakAfter());
 
             // Must get dimensions from parent area
             //Don't optimize this line away. It can have ugly side-effects.
-            /*Area parentArea =*/ parentLM.getParentArea(curBlockArea);
+            /*Area parentArea =*/ parentLayoutManager.getParentArea(curBlockArea);
 
             // set traits
             TraitSetter.setProducerID(curBlockArea, getBlockFO().getId());
-            TraitSetter.addBorders(curBlockArea, 
-                    getBlockFO().getCommonBorderPaddingBackground(), 
+            TraitSetter.addBorders(curBlockArea,
+                    getBlockFO().getCommonBorderPaddingBackground(),
                     discardBorderBefore, discardBorderAfter, false, false, this);
-            TraitSetter.addPadding(curBlockArea, 
-                    getBlockFO().getCommonBorderPaddingBackground(), 
+            TraitSetter.addPadding(curBlockArea,
+                    getBlockFO().getCommonBorderPaddingBackground(),
                     discardPaddingBefore, discardPaddingAfter, false, false, this);
             TraitSetter.addMargins(curBlockArea,
-                    getBlockFO().getCommonBorderPaddingBackground(), 
+                    getBlockFO().getCommonBorderPaddingBackground(),
                     startIndent, endIndent,
                     this);
 
@@ -470,7 +465,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
      */
     protected void flush() {
         if (curBlockArea != null) {
-            TraitSetter.addBackground(curBlockArea, 
+            TraitSetter.addBackground(curBlockArea,
                     getBlockFO().getCommonBorderPaddingBackground(),
                     this);
             super.flush();
@@ -484,9 +479,9 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     protected org.apache.fop.fo.flow.Block getBlockFO() {
         return (org.apache.fop.fo.flow.Block) fobj;
     }
-    
+
     // --------- Property Resolution related functions --------- //
-    
+
     /**
      * Returns the IPD of the content area
      * @return the IPD of the content area
@@ -497,7 +492,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         }
         return super.getContentAreaIPD();
     }
-   
+
 
     /**
      * Returns the BPD of the content area
@@ -509,7 +504,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         }
         return -1;
     }
-   
+
     /**
      * {@inheritDoc}
      */
@@ -521,13 +516,13 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     public void notifySpace(RelSide side, MinOptMax effectiveLength) {
         if (RelSide.BEFORE == side) {
             if (log.isDebugEnabled()) {
-                log.debug(this + ": Space " + side + ", " 
+                log.debug(this + ": Space " + side + ", "
                         + this.effSpaceBefore + "-> " + effectiveLength);
             }
             this.effSpaceBefore = effectiveLength;
         } else {
             if (log.isDebugEnabled()) {
-                log.debug(this + ": Space " + side + ", " 
+                log.debug(this + ": Space " + side + ", "
                         + this.effSpaceAfter + "-> " + effectiveLength);
             }
             this.effSpaceAfter = effectiveLength;
@@ -562,5 +557,10 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         }
     }
 
+    /** {@inheritDoc} */
+    public boolean isRestartable() {
+        return true;
+    }
+
 }
 
