@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,15 +15,15 @@
  * limitations under the License.
  */
 
-/* $Id: FootnoteBodyLayoutManager.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: FootnoteBodyLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr;
 
+import java.util.LinkedList;
+
 import org.apache.fop.area.Area;
 import org.apache.fop.fo.flow.FootnoteBody;
 
-import java.util.LinkedList;
-
 /**
  * Layout manager for footnote bodies.
  */
@@ -83,7 +83,7 @@ public class FootnoteBodyLayoutManager extends BlockStackingLayoutManager {
     /** {@inheritDoc} */
     public void addChildArea(Area childArea) {
         childArea.setAreaClass(Area.CLASS_FOOTNOTE);
-        parentLM.addChildArea(childArea);
+        parentLayoutManager.addChildArea(childArea);
     }
 
     /** @return the FootnoteBody node */
@@ -91,4 +91,19 @@ public class FootnoteBodyLayoutManager extends BlockStackingLayoutManager {
         return (FootnoteBody) fobj;
     }
 
+    /** {@inheritDoc} */
+    public Keep getKeepTogether() {
+        return getParentKeepTogether();
+    }
+
+    /** {@inheritDoc} */
+    public Keep getKeepWithNext() {
+        return Keep.KEEP_AUTO;
+    }
+
+    /** {@inheritDoc} */
+    public Keep getKeepWithPrevious() {
+        return Keep.KEEP_AUTO;
+    }
+
 }
