@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,21 +15,23 @@
  * limitations under the License.
  */
 
-/* $Id: KnuthSequence.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: KnuthSequence.java 825646 2009-10-15 20:43:13Z acumiskey $ */
 
 package org.apache.fop.layoutmgr;
 
+import org.apache.fop.util.ListUtil;
+
 import java.util.ArrayList;
 import java.util.List;
 import java.util.ListIterator;
 
 /**
- * Represents a list of Knuth elements.
- */
-/**
- * 
+ * Represents a list of {@link KnuthElement Knuth elements}.
  */
 public abstract class KnuthSequence extends ArrayList {
+
+    //TODO: do not extend ArrayList
+
     /**
      * Creates a new and empty list.
      */
@@ -68,19 +70,19 @@ public abstract class KnuthSequence extends ArrayList {
      * Append sequence to this sequence if it can be appended.
      * @param sequence The sequence that is to be appended.
      * @param keepTogether Whether the two sequences must be kept together.
-     * @param breakElement The BreakElement that may be inserted between the two sequences. 
+     * @param breakElement The BreakElement that may be inserted between the two sequences.
      * @return whether the sequence was succesfully appended to this sequence.
      */
     public abstract boolean appendSequence(KnuthSequence sequence, boolean keepTogether,
                                            BreakElement breakElement);
-    
+
     /**
      * Append sequence to this sequence if it can be appended.
      * @param sequence The sequence that is to be appended.
      * @return whether the sequence was succesfully appended to this sequence.
      */
     public abstract boolean appendSequence(KnuthSequence sequence);
-    
+
     /**
      * Append sequence to this sequence if it can be appended.
      * If that is not possible, close this sequence.
@@ -95,13 +97,13 @@ public abstract class KnuthSequence extends ArrayList {
             return true;
         }
     }
-    
+
     /**
      * Append sequence to this sequence if it can be appended.
      * If that is not possible, close this sequence.
      * @param sequence The sequence that is to be appended.
      * @param keepTogether Whether the two sequences must be kept together.
-     * @param breakElement The BreakElement that may be inserted between the two sequences. 
+     * @param breakElement The BreakElement that may be inserted between the two sequences.
      * @return whether the sequence was succesfully appended to this sequence.
      */
     public boolean appendSequenceOrClose(KnuthSequence sequence, boolean keepTogether,
@@ -113,7 +115,7 @@ public abstract class KnuthSequence extends ArrayList {
             return true;
         }
     }
-    
+
     /**
      * Wrap the Positions of the elements of this sequence in a Position for LayoutManager lm.
      * @param lm The LayoutManager for the Positions that will be created.
@@ -127,16 +129,14 @@ public abstract class KnuthSequence extends ArrayList {
             (lm.notifyPos(new NonLeafPosition(lm, element.getPosition())));
         }
     }
-    
+
     /**
      * @return the last element of this sequence.
      */
     public ListElement getLast() {
-        int idx = size();
-        if (idx == 0) {
-            return null; 
-        }
-        return (ListElement) get(idx - 1);
+        return (isEmpty()
+                ? null
+                : (ListElement) ListUtil.getLast(this));
     }
 
     /**
@@ -144,11 +144,9 @@ public abstract class KnuthSequence extends ArrayList {
      * @return the removed element.
      */
     public ListElement removeLast() {
-        int idx = size();
-        if (idx == 0) {
-            return null; 
-        }
-        return (ListElement) remove(idx - 1);
+        return (isEmpty()
+                ? null
+                : (ListElement) ListUtil.removeLast(this));
     }
 
     /**
@@ -156,7 +154,45 @@ public abstract class KnuthSequence extends ArrayList {
      * @return the element at index index.
      */
     public ListElement getElement(int index) {
-        return (ListElement) get(index);
+        return (index >= size() || index < 0)
+                ? null
+                : (ListElement) get(index);
+    }
+
+    /** @return the position index of the first box in this sequence */
+    protected int getFirstBoxIndex() {
+        if (isEmpty()) {
+            return -1;
+        } else {
+            return getFirstBoxIndex(0);
+        }
+    }
+
+    /**
+     * Get the position index of the first box in this sequence,
+     * starting at the given index. If there is no box after the
+     * passed {@code startIndex}, the starting index itself is returned.
+     * @param startIndex    the starting index for the lookup
+     * @return  the absolute position index of the next box element
+     */
+    protected int getFirstBoxIndex(int startIndex) {
+        if (isEmpty() || startIndex < 0 || startIndex >= size()) {
+            return -1;
+        } else {
+            ListElement element = null;
+            int posIndex = startIndex;
+            int lastIndex = size();
+            while (posIndex < lastIndex
+                    && !(element = getElement(posIndex)).isBox()) {
+                posIndex++;
+            }
+            if (posIndex != startIndex
+                    && element.isBox()) {
+                return posIndex - 1;
+            } else {
+                return startIndex;
+            }
+        }
     }
 
     /**
@@ -165,4 +201,9 @@ public abstract class KnuthSequence extends ArrayList {
      */
     public abstract boolean isInlineSequence();
 
+    /** {@inheritDoc} */
+    public String toString() {
+        return "<KnuthSequence " + super.toString() + ">";
+    }
+
 }
