@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: LayoutContext.java 669118 2008-06-18 09:02:45Z jeremias $ */
+/* $Id: LayoutContext.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -61,12 +61,12 @@ public class LayoutContext {
      * This flag indicates that there's a keep-with-next that hasn't
      * been processed, yet.
      */
-    public static final int KEEP_WITH_NEXT_PENDING = 0x200;
+    //public static final int KEEP_WITH_NEXT_PENDING = 0x200;
     /**
      * This flag indicates that there's a keep-with-previous that hasn't
      * been processed, yet.
      */
-    public static final int KEEP_WITH_PREVIOUS_PENDING = 0x400;
+    //public static final int KEEP_WITH_PREVIOUS_PENDING = 0x400;
 
 
     private int flags; // Contains some set of flags defined above
@@ -78,17 +78,9 @@ public class LayoutContext {
      * level LM to allow them to optimize returned break possibilities.
      */
     private MinOptMax stackLimitBP;
-    /**
-     * Total available stacking dimension for a "galley-level" layout
-     * manager in inline-progression-direction. It is passed by the
-     * parent LM. For LineLM, the block LM determines this based on
-     * indent properties.
-     * These LM <b>may</b> wish to pass this information down to lower
-     * level LM to allow them to optimize returned break possibilities.
-     */
-    private MinOptMax stackLimitIP;
 
-    /** True if current element list is spanning in multi-column layout. */
+    /** to keep track of spanning in multi-column layout */
+    private int currentSpan = Constants.NOT_SET;
     private int nextSpan = Constants.NOT_SET;
 
     /** inline-progression-dimension of nearest ancestor reference area */
@@ -104,15 +96,15 @@ public class LayoutContext {
 
     /** Current pending space-before or space-start from ancestor areas */
     private SpaceSpecifier leadingSpace;
-    
+
     /**
-     * A list of pending marks (border and padding) on the after edge when a page break occurs. 
+     * A list of pending marks (border and padding) on the after edge when a page break occurs.
      * May be null.
      */
     private List pendingAfterMarks;
-    
+
     /**
-     * A list of pending marks (border and padding) on the before edge when a page break occurs. 
+     * A list of pending marks (border and padding) on the before edge when a page break occurs.
      * May be null.
      */
     private List pendingBeforeMarks;
@@ -122,7 +114,7 @@ public class LayoutContext {
 
     /** Alignment in BP direction */
     private int bpAlignment = Constants.EN_START;
-    
+
     /** Stretch or shrink value when making areas. */
     private double ipdAdjust = 0.0;
 
@@ -130,22 +122,25 @@ public class LayoutContext {
     private double dSpaceAdjust = 0.0;
 
     private AlignmentContext alignmentContext = null;
-    
+
     /** Amount of space before / start */
     private int spaceBefore = 0;
-    
     /** Amount of space after / end */
     private int spaceAfter = 0;
-    
+
     /** Amount of space to reserve at the beginning of each line */
     private int lineStartBorderAndPaddingWidth = 0;
     /** Amount of space to reserve at the end of each line */
     private int lineEndBorderAndPaddingWidth = 0;
 
     private int breakBefore;
-
     private int breakAfter;
 
+    private Keep pendingKeepWithNext = Keep.KEEP_AUTO;
+    private Keep pendingKeepWithPrevious = Keep.KEEP_AUTO;
+
+    private int disableColumnBalancing;
+
     /**
      * Copy constructor for creating child layout contexts.
      * @param parentLC the parent layout context to copy from
@@ -154,7 +149,7 @@ public class LayoutContext {
         this.flags = parentLC.flags;
         this.refIPD = parentLC.refIPD;
         this.writingMode = parentLC.writingMode;
-        setStackLimitsFrom(parentLC);
+        setStackLimitBP(parentLC.getStackLimitBP());
         this.leadingSpace = parentLC.leadingSpace; //???
         this.trailingSpace = parentLC.trailingSpace; //???
         this.hyphContext = parentLC.hyphContext;
@@ -165,7 +160,10 @@ public class LayoutContext {
         this.lineStartBorderAndPaddingWidth = parentLC.lineStartBorderAndPaddingWidth;
         this.lineEndBorderAndPaddingWidth = parentLC.lineEndBorderAndPaddingWidth;
         copyPendingMarksFrom(parentLC);
+        this.pendingKeepWithNext = parentLC.pendingKeepWithNext;
+        this.pendingKeepWithPrevious = parentLC.pendingKeepWithPrevious;
         // Copy other fields as necessary.
+        this.disableColumnBalancing = parentLC.disableColumnBalancing;
     }
 
     /**
@@ -175,21 +173,20 @@ public class LayoutContext {
     public LayoutContext(int flags) {
         this.flags = flags;
         this.refIPD = 0;
-        stackLimitBP = new MinOptMax(0);
-        stackLimitIP = new MinOptMax(0);
+        stackLimitBP = MinOptMax.ZERO;
         leadingSpace = null;
         trailingSpace = null;
     }
 
     public void copyPendingMarksFrom(LayoutContext source) {
         if (source.pendingAfterMarks != null) {
-            this.pendingAfterMarks = new java.util.ArrayList(source.pendingAfterMarks); 
+            this.pendingAfterMarks = new java.util.ArrayList(source.pendingAfterMarks);
         }
         if (source.pendingBeforeMarks != null) {
-            this.pendingBeforeMarks = new java.util.ArrayList(source.pendingBeforeMarks); 
+            this.pendingBeforeMarks = new java.util.ArrayList(source.pendingBeforeMarks);
         }
     }
-    
+
     public void setFlags(int flags) {
         setFlags(flags, true);
     }
@@ -226,14 +223,76 @@ public class LayoutContext {
         return ((this.flags & SUPPRESS_BREAK_BEFORE) != 0);
     }
 
+    /**
+     * Returns the strength of a keep-with-next currently pending.
+     * @return the keep-with-next strength
+     */
+    public Keep getKeepWithNextPending() {
+        return this.pendingKeepWithNext;
+    }
+
+    /**
+     * Returns the strength of a keep-with-previous currently pending.
+     * @return the keep-with-previous strength
+     */
+    public Keep getKeepWithPreviousPending() {
+        return this.pendingKeepWithPrevious;
+    }
+
+    /**
+     * Clears any pending keep-with-next strength.
+     */
+    public void clearKeepWithNextPending() {
+        this.pendingKeepWithNext = Keep.KEEP_AUTO;
+    }
+
+    /**
+     * Clears any pending keep-with-previous strength.
+     */
+    public void clearKeepWithPreviousPending() {
+        this.pendingKeepWithPrevious = Keep.KEEP_AUTO;
+    }
+
+    /**
+     * Clears both keep-with-previous and keep-with-next strengths.
+     */
+    public void clearKeepsPending() {
+        clearKeepWithPreviousPending();
+        clearKeepWithNextPending();
+    }
+
+    /**
+     * Updates the currently pending keep-with-next strength.
+     * @param keep the new strength to consider
+     */
+    public void updateKeepWithNextPending(Keep keep) {
+        this.pendingKeepWithNext = this.pendingKeepWithNext.compare(keep);
+    }
+
+    /**
+     * Updates the currently pending keep-with-previous strength.
+     * @param keep the new strength to consider
+     */
+    public void updateKeepWithPreviousPending(Keep keep) {
+        this.pendingKeepWithPrevious = this.pendingKeepWithPrevious.compare(keep);
+    }
+
+    /**
+     * Indicates whether a keep-with-next constraint is pending.
+     * @return true if a keep-with-next constraint is pending
+     */
     public boolean isKeepWithNextPending() {
-        return ((this.flags & KEEP_WITH_NEXT_PENDING) != 0);
+        return !getKeepWithNextPending().isAuto();
     }
-    
+
+    /**
+     * Indicates whether a keep-with-previous constraint is pending.
+     * @return true if a keep-with-previous constraint is pending
+     */
     public boolean isKeepWithPreviousPending() {
-        return ((this.flags & KEEP_WITH_PREVIOUS_PENDING) != 0);
+        return !getKeepWithPreviousPending().isAuto();
     }
-    
+
     public void setLeadingSpace(SpaceSpecifier space) {
         leadingSpace = space;
     }
@@ -266,7 +325,7 @@ public class LayoutContext {
         }
         this.pendingAfterMarks.add(element);
     }
-    
+
     /**
      * @return the pending border and padding elements at the after edge
      * @see #addPendingAfterMark(UnresolvedListElementWithLength)
@@ -278,7 +337,7 @@ public class LayoutContext {
             return null;
         }
     }
-    
+
     /**
      * Clears all pending marks on the LayoutContext.
      */
@@ -286,7 +345,7 @@ public class LayoutContext {
         this.pendingBeforeMarks = null;
         this.pendingAfterMarks = null;
     }
-    
+
     /**
      * Adds a border or padding element to the pending list which will be used to generate
      * the right element list for break possibilities. Conditionality resolution will be done
@@ -299,7 +358,7 @@ public class LayoutContext {
         }
         this.pendingBeforeMarks.add(element);
     }
-    
+
     /**
      * @return the pending border and padding elements at the before edge
      * @see #addPendingBeforeMark(UnresolvedListElementWithLength)
@@ -311,7 +370,7 @@ public class LayoutContext {
             return null;
         }
     }
-    
+
     /**
      * Sets the stack limit in block-progression-dimension.
      * @param limit the stack limit
@@ -328,31 +387,6 @@ public class LayoutContext {
         return stackLimitBP;
     }
 
-    /**
-     * Sets the stack limit in inline-progression-dimension.
-     * @param limit the stack limit
-     */
-    public void setStackLimitIP(MinOptMax limit) {
-        stackLimitIP = limit;
-    }
-
-    /**
-     * Returns the stack limit in inline-progression-dimension.
-     * @return the stack limit
-     */
-    public MinOptMax getStackLimitIP() {
-        return stackLimitIP;
-    }
-
-    /**
-     * Sets (Copies) the stack limits in both directions from another layout context.
-     * @param context the layout context to taje the values from
-     */
-    public void setStackLimitsFrom(LayoutContext context) {
-        setStackLimitBP(context.getStackLimitBP());
-        setStackLimitIP(context.getStackLimitIP());
-    }
-    
     /**
      * Sets the inline-progression-dimension of the nearest ancestor reference area.
      */
@@ -362,7 +396,7 @@ public class LayoutContext {
 
     /**
      * Returns the inline-progression-dimension of the nearest ancestor reference area.
-     * 
+     *
      * @return the inline-progression-dimension of the nearest ancestor reference area
      */
     public int getRefIPD() {
@@ -388,12 +422,12 @@ public class LayoutContext {
     public void setBPAlignment(int alignment) {
         this.bpAlignment = alignment;
     }
-    
+
     /** @return the currently applicable alignment in BP direction (EN_START, EN_JUSTIFY...) */
     public int getBPAlignment() {
         return this.bpAlignment;
     }
-    
+
     public void setSpaceAdjust(double adjust) {
         dSpaceAdjust = adjust;
     }
@@ -413,7 +447,7 @@ public class LayoutContext {
     public void setAlignmentContext(AlignmentContext alignmentContext) {
         this.alignmentContext = alignmentContext;
     }
-    
+
     public AlignmentContext getAlignmentContext() {
         return this.alignmentContext;
     }
@@ -423,7 +457,7 @@ public class LayoutContext {
             this.alignmentContext = this.alignmentContext.getParentAlignmentContext();
         }
     }
-    
+
     /**
      * Get the width to be reserved for border and padding at the start of the line.
      * @return the width to be reserved
@@ -431,7 +465,7 @@ public class LayoutContext {
     public int getLineStartBorderAndPaddingWidth() {
         return lineStartBorderAndPaddingWidth;
     }
-    
+
     /**
      * Set the width to be reserved for border and padding at the start of the line.
      * @param lineStartBorderAndPaddingWidth the width to be reserved
@@ -439,7 +473,7 @@ public class LayoutContext {
     public void setLineStartBorderAndPaddingWidth(int lineStartBorderAndPaddingWidth) {
         this.lineStartBorderAndPaddingWidth = lineStartBorderAndPaddingWidth;
     }
-    
+
     /**
      * Get the width to be reserved for border and padding at the end of the line.
      * @return the width to be reserved
@@ -447,7 +481,7 @@ public class LayoutContext {
     public int getLineEndBorderAndPaddingWidth() {
         return lineEndBorderAndPaddingWidth;
     }
-    
+
     /**
      * Set the width to be reserved for border and padding at the end of the line.
      * @param lineEndBorderAndPaddingWidth the width to be reserved
@@ -455,30 +489,45 @@ public class LayoutContext {
     public void setLineEndBorderAndPaddingWidth(int lineEndBorderAndPaddingWidth) {
         this.lineEndBorderAndPaddingWidth = lineEndBorderAndPaddingWidth;
     }
-    
+
     /**
-     * @return true if the current element list ends early because of a span change
-     * in multi-column layout.
+     * @return one of: {@link Constants#NOT_SET}, {@link Constants#EN_NONE}
+     *                  {@link Constants#EN_ALL}
      */
     public int getNextSpan() {
         return nextSpan;
     }
-    
+
+    /**
+     * @return one of: {@link Constants#NOT_SET}, {@link Constants#EN_NONE}
+     *                  {@link Constants#EN_ALL}
+     */
+    public int getCurrentSpan() {
+        return (currentSpan == Constants.NOT_SET)
+                ? Constants.EN_NONE : currentSpan;
+    }
+
     /**
      * Used to signal the PSLM that the element list ends early because of a span change in
      * multi-column layout.
      * @param span the new span value (legal values: NOT_SET, EN_NONE, EN_ALL)
      */
     public void signalSpanChange(int span) {
-        if (span == Constants.NOT_SET || span == Constants.EN_NONE || span == Constants.EN_ALL) {
+        switch (span) {
+        case Constants.NOT_SET:
+        case Constants.EN_NONE:
+        case Constants.EN_ALL:
+            this.currentSpan = this.nextSpan;
             this.nextSpan = span;
-        } else {
+            break;
+        default:
+            assert false;
             throw new IllegalArgumentException("Illegal value on signalSpanChange() for span: "
                     + span);
         }
     }
-    
-    /** 
+
+    /**
      * Get the writing mode of the relevant reference area.
      * @return the applicable writing mode
      */
@@ -486,7 +535,7 @@ public class LayoutContext {
         return writingMode;
     }
 
-    /** 
+    /**
      * Set the writing mode.
      * @param writingMode the writing mode
      */
@@ -530,7 +579,7 @@ public class LayoutContext {
      * Returns the value of the break before the element whose
      * {@link LayoutManager#getNextKnuthElements(LayoutContext, int)} method has just been
      * called.
-     * 
+     *
      * @return one of {@link Constants#EN_AUTO}, {@link Constants#EN_COLUMN},
      * {@link Constants#EN_PAGE}, {@link Constants#EN_EVEN_PAGE}, or
      * {@link Constants#EN_ODD_PAGE}
@@ -541,7 +590,7 @@ public class LayoutContext {
 
     /**
      * Sets the value of the break before the current element.
-     * 
+     *
      * @param breakBefore the value of the break-before
      * @see #getBreakBefore()
      */
@@ -553,7 +602,7 @@ public class LayoutContext {
      * Returns the value of the break after the element whose
      * {@link LayoutManager#getNextKnuthElements(LayoutContext, int)} method has just been
      * called.
-     * 
+     *
      * @return one of {@link Constants#EN_AUTO}, {@link Constants#EN_COLUMN},
      * {@link Constants#EN_PAGE}, {@link Constants#EN_EVEN_PAGE}, or
      * {@link Constants#EN_ODD_PAGE}
@@ -565,7 +614,7 @@ public class LayoutContext {
 
     /**
      * Sets the value of the break after the current element.
-     * 
+     *
      * @param breakAfter the value of the break-after
      * @see #getBreakAfter()
      */
@@ -578,12 +627,10 @@ public class LayoutContext {
         return "Layout Context:"
         + "\nStack Limit BPD: \t"
             + (getStackLimitBP() == null ? "null" : getStackLimitBP().toString())
-        + "\nStack Limit IPD: \t"
-            + (getStackLimitIP() == null ? "null" : getStackLimitIP().toString())
         + "\nTrailing Space: \t"
             + (getTrailingSpace() == null ? "null" : getTrailingSpace().toString())
         + "\nLeading Space: \t"
-            + (getLeadingSpace() == null ? "null" : getLeadingSpace().toString()) 
+            + (getLeadingSpace() == null ? "null" : getLeadingSpace().toString())
         + "\nReference IPD: \t" + getRefIPD()
         + "\nSpace Adjust: \t" + getSpaceAdjust()
         + "\nIPD Adjust: \t" + getIPDAdjust()
@@ -593,11 +640,29 @@ public class LayoutContext {
         + "\nStarts New Area: \t" + startsNewArea()
         + "\nIs Last Area: \t" + isLastArea()
         + "\nTry Hyphenate: \t" + tryHyphenate()
-        + "\nKeeps: \t[" + (isKeepWithNextPending() ? "keep-with-next" : "") + "][" 
-            + (isKeepWithPreviousPending() ? "keep-with-previous" : "") + "] pending"
-        + "\nBreaks: \tforced [" + (breakBefore != Constants.EN_AUTO ? "break-before" : "") + "][" 
+        + "\nKeeps: \t[keep-with-next=" + getKeepWithNextPending()
+                + "][keep-with-previous=" + getKeepWithPreviousPending() + "] pending"
+        + "\nBreaks: \tforced [" + (breakBefore != Constants.EN_AUTO ? "break-before" : "") + "]["
         + (breakAfter != Constants.EN_AUTO ? "break-after" : "") + "]";
     }
 
+    /**
+     * Returns whether the column balancer should be disabled before a spanning block
+     *
+     * @return one of {@link Constants#EN_TRUE}, {@link Constants#EN_FALSE}
+     */
+    public int getDisableColumnBalancing() {
+        return disableColumnBalancing;
+    }
+
+    /**
+     * Sets whether the column balancer should be disabled before a spanning block
+     *
+     * @param disableColumnBalancing the value of the fox:disable-column-balancing property
+     * @see #getDisableColumnBalancing()
+     */
+    public void setDisableColumnBalancing(int disableColumnBalancing) {
+        this.disableColumnBalancing = disableColumnBalancing;
+    }
 }
 
