@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,12 +15,12 @@
  * limitations under the License.
  */
 
-/* $Id: LayoutManager.java 604171 2007-12-14 11:32:51Z vhennebert $ */
- 
+/* $Id: LayoutManager.java 808157 2009-08-26 18:50:10Z vhennebert $ */
+
 package org.apache.fop.layoutmgr;
 
-import java.util.LinkedList;
 import java.util.List;
+import java.util.Stack;
 
 import org.apache.fop.area.Area;
 import org.apache.fop.datatypes.PercentBaseContext;
@@ -46,11 +46,11 @@ public interface LayoutManager extends PercentBaseContext {
     LayoutManager getParent();
 
     /**
-     * initialize the layout manager. Allows each layout manager 
+     * initialize the layout manager. Allows each layout manager
      * to calculate often used values.
      */
     void initialize();
-    
+
     /**
      * Get the active PageSequenceLayoutManager instance for this
      * layout process.
@@ -131,64 +131,64 @@ public interface LayoutManager extends PercentBaseContext {
     void addChildLMs(List newLMs);
 
     /**
-     * Get a sequence of KnuthElements representing the content 
+     * Get a sequence of KnuthElements representing the content
      * of the node assigned to the LM
-     * 
+     *
      * @param context   the LayoutContext used to store layout information
-     * @param alignment the desired text alignement
+     * @param alignment the desired text alignment
      * @return          the list of KnuthElements
      */
-    LinkedList getNextKnuthElements(LayoutContext context, int alignment);
+    List getNextKnuthElements(LayoutContext context, int alignment);
 
     /**
-     * Get a sequence of KnuthElements representing the content 
+     * Get a sequence of KnuthElements representing the content
      * of the node assigned to the LM, after changes have been applied
      *
      * In the context of line breaking, this method is called after hyphenation has
-     * been performed, in order to receive the sequence of elements representing the 
-     * text together with all possibile hyphenation points.
+     * been performed, in order to receive the sequence of elements representing the
+     * text together with all possible hyphenation points.
      * For example, if the text "representation" originates a single box element
      * when getNextKnuthElements() is called, it will be now split in syllables
      * (rep-re-sen-ta-tion) each one originating a box and divided by additional
      * elements allowing a line break.
-     * 
+     *
      * In the context of page breaking, this method is called only if the pages need
      * to be "vertically justified" modifying (also) the quantity of lines created by
      * the paragraphs, and after a first page breaking has been performed.
      * According to the result of the first page breaking, each paragraph now knows
-     * how many lines it must create (among the existing layout possibilities) and 
+     * how many lines it must create (among the existing layout possibilities) and
      * has to create a sequence of elements representing this layout; in particular,
      * each box, representing a line, will contain a LineBreakPositions that will be
      * used in the addAreas() phase.
-     * 
+     *
      * LMs having children look at the old list of elements in order to know which
      * ones they must get the new elements from, as break conditions of preserved
-     * linefeeds can divide children into smaller groups (page sequences or 
+     * linefeeds can divide children into smaller groups (page sequences or
      * paragraphs).
      * LMs having no children can simply return the old elements if they have nothing
      * to change.
      *
      * Inline LMs need to know the text alignment because it affects the elements
      * representing feasible breaks between syllables.
-     * 
+     *
      * @param oldList        the elements to replace
      * @param alignment      the desired text alignment
      * @return               the updated list of KnuthElements
      */
-    LinkedList getChangedKnuthElements(List oldList, int alignment);
-    
+    List getChangedKnuthElements(List oldList, int alignment);
+
     /**
      * Returns the IPD of the content area
      * @return the IPD of the content area
      */
     int getContentAreaIPD();
-   
+
     /**
      * Returns the BPD of the content area
      * @return the BPD of the content area
      */
     int getContentAreaBPD();
-   
+
     /**
      * Returns an indication if the layout manager generates a reference area.
      * @return True if the layout manager generates a reference area
@@ -206,13 +206,13 @@ public interface LayoutManager extends PercentBaseContext {
      * @return True if the layout manager generates a line area
      */
     boolean getGeneratesLineArea();
-    
+
     /**
      * Returns the fo this layout manager is associated with.
      * @return The fo for this layout manager or null.
      */
     FObj getFObj();
-    
+
     /**
      * Adds a Position to the Position participating in the first|last determination by assigning
      * it a unique position index.
@@ -220,4 +220,36 @@ public interface LayoutManager extends PercentBaseContext {
      * @return the same Position but with a position index
      */
     Position notifyPos(Position pos);
+
+    /**
+     * Re-initializes this layout manager in order to re-generate its Knuth
+     * elements according to a new IPD value.
+     */
+    void reset();
+
+    /**
+     * Returns {@code true} if this layout manager is able to re-generate its
+     * Knuth elements after an IPD change.
+     *
+     * @return {@code true} if this layout manager can be restarted after an IPD
+     * change
+     */
+    boolean isRestartable();
+
+    /**
+     * Returns an updated list of Knuth elements corresponding to this layout
+     * manager, after a change of IPD has been detected.
+     *
+     * @param context the layout context
+     * @param alignment the alignment
+     * @param lmStack the stack of LMs that are active at the IPD change
+     * @param positionAtIPDChange the position corresponding to the element
+     * finishing the page before the IPD change
+     * @param restartAtLM if not null, the layout manager from which to restart.
+     * That is, the IPD change occurs between two block elements and not inside
+     * a paragraph
+     * @return an updated list of elements, taking the new IPD into account
+     */
+    List getNextKnuthElements(LayoutContext context, int alignment, Stack lmStack,
+            Position positionAtIPDChange, LayoutManager restartAtLM);
 }
