@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,27 +15,32 @@
  * limitations under the License.
  */
 
-/* $Id: LeafPosition.java 557337 2007-07-18 17:37:14Z adelmelle $ */
- 
+/* $Id: LeafPosition.java 893238 2009-12-22 17:20:51Z vhennebert $ */
+
 package org.apache.fop.layoutmgr;
 
 public class LeafPosition extends Position {
 
-    private int iLeafPos;
+    private int leafPos;
 
-    public LeafPosition(LayoutManager lm, int pos) {
-        super(lm);
-        iLeafPos = pos;
+    public LeafPosition(LayoutManager layoutManager, int pos) {
+        super(layoutManager);
+        leafPos = pos;
+    }
+
+    public LeafPosition(LayoutManager layoutManager, int pos, int index) {
+        super(layoutManager, index);
+        leafPos = pos;
     }
 
     public int getLeafPos() {
-        return iLeafPos;
+        return leafPos;
     }
-    
+
     public boolean generatesAreas() {
         return getLM() != null;
     }
-    
+
     /** {@inheritDoc} */
     public String toString() {
         StringBuffer sb = new StringBuffer();
