@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,21 +15,19 @@
  * limitations under the License.
  */
 
-/* $Id: PageBreaker.java 635508 2008-03-10 10:06:37Z jeremias $ */
+/* $Id: PageBreaker.java 900364 2010-01-18 12:28:32Z vhennebert $ */
 
 package org.apache.fop.layoutmgr;
 
-import java.util.LinkedList;
 import java.util.List;
 import java.util.ListIterator;
 
 import org.apache.fop.area.Block;
+import org.apache.fop.area.BodyRegion;
 import org.apache.fop.area.Footnote;
 import org.apache.fop.area.PageViewport;
 import org.apache.fop.fo.Constants;
-import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.FObj;
-import org.apache.fop.fo.pagination.PageSequence;
 import org.apache.fop.fo.pagination.Region;
 import org.apache.fop.fo.pagination.RegionBody;
 import org.apache.fop.fo.pagination.StaticContent;
@@ -40,14 +38,14 @@ import org.apache.fop.traits.MinOptMax;
  * Handles the breaking of pages in an fo:flow
  */
 public class PageBreaker extends AbstractBreaker {
-    
+
     private PageSequenceLayoutManager pslm;
     private boolean firstPart = true;
     private boolean pageBreakHandled;
     private boolean needColumnBalancing;
     private PageProvider pageProvider;
     private Block separatorArea;
-    
+
     /**
      * The FlowLayoutManager object, which processes
      * the single fo:flow of the fo:page-sequence
@@ -62,46 +60,52 @@ public class PageBreaker extends AbstractBreaker {
         this.childFLM = pslm.getLayoutManagerMaker().makeFlowLayoutManager(
                 pslm, pslm.getPageSequence().getMainFlow());
     }
-    
+
     /** {@inheritDoc} */
     protected void updateLayoutContext(LayoutContext context) {
         int flowIPD = pslm.getCurrentPV().getCurrentSpan().getColumnWidth();
         context.setRefIPD(flowIPD);
     }
-    
+
     /** {@inheritDoc} */
     protected LayoutManager getTopLevelLM() {
         return pslm;
     }
-    
+
     /** {@inheritDoc} */
     protected PageProvider getPageProvider() {
         return pslm.getPageProvider();
     }
-    
+
     /**
-     * {@inheritDoc}
+     * Starts the page breaking process.
+     * @param flowBPD the constant available block-progression-dimension (used for every part)
      */
-    protected PageBreakingLayoutListener getLayoutListener() {
+    void doLayout(int flowBPD) {
+        doLayout(flowBPD, false);
+    }
+
+    /** {@inheritDoc} */
+    protected PageBreakingLayoutListener createLayoutListener() {
         return new PageBreakingLayoutListener() {
 
-            public void notifyOverflow(int part, FObj obj) {
+            public void notifyOverflow(int part, int amount, FObj obj) {
                 Page p = pageProvider.getPage(
                             false, part, PageProvider.RELTO_CURRENT_ELEMENT_LIST);
                 RegionBody body = (RegionBody)p.getSimplePageMaster().getRegion(
                         Region.FO_REGION_BODY);
-                String err = FONode.decorateWithContextInfo(
-                        "Content of the region-body on page " 
-                        + p.getPageViewport().getPageNumberString() 
-                        + " overflows the available area in block-progression dimension.", 
-                        obj);
-                if (body.getOverflow() == Constants.EN_ERROR_IF_OVERFLOW) {
-                    throw new RuntimeException(err);
-                } else {
-                    log.warn(err);
-                }
+                BlockLevelEventProducer eventProducer = BlockLevelEventProducer.Provider.get(
+                        body.getUserAgent().getEventBroadcaster());
+
+                boolean canRecover = (body.getOverflow() != Constants.EN_ERROR_IF_OVERFLOW);
+                boolean needClip = (body.getOverflow() == Constants.EN_HIDDEN
+                        || body.getOverflow() == Constants.EN_ERROR_IF_OVERFLOW);
+                eventProducer.regionOverflow(this, body.getName(),
+                        p.getPageViewport().getPageNumberString(),
+                        amount, needClip, canRecover,
+                        body.getLocator());
             }
-            
+
         };
     }
 
@@ -111,7 +115,9 @@ public class PageBreaker extends AbstractBreaker {
         if (childLC.getNextSpan() != Constants.NOT_SET) {
             //Next block list will have a different span.
             nextSequenceStartsOn = childLC.getNextSpan();
-            needColumnBalancing = (childLC.getNextSpan() == Constants.EN_ALL);
+            needColumnBalancing = childLC.getNextSpan() == Constants.EN_ALL
+                    && childLC.getDisableColumnBalancing() == Constants.EN_FALSE;
+
         }
         if (needColumnBalancing) {
             AbstractBreaker.log.debug(
@@ -121,8 +127,14 @@ public class PageBreaker extends AbstractBreaker {
     }
 
     /** {@inheritDoc} */
-    protected int getNextBlockList(LayoutContext childLC, 
+    protected int getNextBlockList(LayoutContext childLC,
             int nextSequenceStartsOn) {
+        return getNextBlockList(childLC, nextSequenceStartsOn, null, null, null);
+    }
+
+    /** {@inheritDoc} */
+    protected int getNextBlockList(LayoutContext childLC, int nextSequenceStartsOn,
+            Position positionAtIPDChange, LayoutManager restartLM, List firstElements) {
         if (!firstPart) {
             // if this is the first page that will be created by
             // the current BlockSequence, it could have a break
@@ -132,21 +144,15 @@ public class PageBreaker extends AbstractBreaker {
         }
         firstPart = false;
         pageBreakHandled = true;
-        pageProvider.setStartOfNextElementList(pslm.getCurrentPageNum(), 
+        pageProvider.setStartOfNextElementList(pslm.getCurrentPageNum(),
                 pslm.getCurrentPV().getCurrentSpan().getCurrentFlowIndex());
-        return super.getNextBlockList(childLC, nextSequenceStartsOn);
+        return super.getNextBlockList(childLC, nextSequenceStartsOn, positionAtIPDChange,
+                restartLM, firstElements);
     }
-    
-    /** {@inheritDoc} */
-    protected LinkedList getNextKnuthElements(LayoutContext context, int alignment) {
-        LinkedList contentList = null;
-        
-        while (!childFLM.isFinished() && contentList == null) {
-            contentList = childFLM.getNextKnuthElements(context, alignment);
-        }
 
-        // scan contentList, searching for footnotes
-        boolean bFootnotesPresent = false;
+    private boolean containsFootnotes(List contentList, LayoutContext context) {
+
+        boolean containsFootnotes = false;
         if (contentList != null) {
             ListIterator contentListIterator = contentList.listIterator();
             while (contentListIterator.hasNext()) {
@@ -154,17 +160,17 @@ public class PageBreaker extends AbstractBreaker {
                 if (element instanceof KnuthBlockBox
                     && ((KnuthBlockBox) element).hasAnchors()) {
                     // element represents a line with footnote citations
-                    bFootnotesPresent = true;
+                    containsFootnotes = true;
                     LayoutContext footnoteContext = new LayoutContext(context);
                     footnoteContext.setStackLimitBP(context.getStackLimitBP());
                     footnoteContext.setRefIPD(pslm.getCurrentPV()
                             .getRegionReference(Constants.FO_REGION_BODY).getIPD());
-                    LinkedList footnoteBodyLMs = ((KnuthBlockBox) element).getFootnoteBodyLMs();
+                    List footnoteBodyLMs = ((KnuthBlockBox) element).getFootnoteBodyLMs();
                     ListIterator footnoteBodyIterator = footnoteBodyLMs.listIterator();
                     // store the lists of elements representing the footnote bodies
                     // in the box representing the line containing their references
                     while (footnoteBodyIterator.hasNext()) {
-                        FootnoteBodyLayoutManager fblm 
+                        FootnoteBodyLayoutManager fblm
                             = (FootnoteBodyLayoutManager) footnoteBodyIterator.next();
                         fblm.setParent(childFLM);
                         fblm.initialize();
@@ -174,35 +180,68 @@ public class PageBreaker extends AbstractBreaker {
                 }
             }
         }
+        return containsFootnotes;
+    }
 
-        if (bFootnotesPresent) {
+    private void handleFootnoteSeparator() {
+        StaticContent footnoteSeparator;
+        footnoteSeparator = pslm.getPageSequence().getStaticContent("xsl-footnote-separator");
+        if (footnoteSeparator != null) {
+            // the footnote separator can contain page-dependent content such as
+            // page numbers or retrieve markers, so its areas cannot simply be
+            // obtained now and repeated in each page;
+            // we need to know in advance the separator bpd: the actual separator
+            // could be different from page to page, but its bpd would likely be
+            // always the same
+
+            // create a Block area that will contain the separator areas
+            separatorArea = new Block();
+            separatorArea.setIPD(pslm.getCurrentPV()
+                        .getRegionReference(Constants.FO_REGION_BODY).getIPD());
+            // create a StaticContentLM for the footnote separator
+            footnoteSeparatorLM
+                    = pslm.getLayoutManagerMaker().makeStaticContentLayoutManager(
+                        pslm, footnoteSeparator, separatorArea);
+            footnoteSeparatorLM.doLayout();
+
+            footnoteSeparatorLength = MinOptMax.getInstance(separatorArea.getBPD());
+        }
+    }
+
+    /** {@inheritDoc} */
+    protected List getNextKnuthElements(LayoutContext context, int alignment) {
+        List contentList = null;
+
+        while (!childFLM.isFinished() && contentList == null) {
+            contentList = childFLM.getNextKnuthElements(context, alignment);
+        }
+
+        // scan contentList, searching for footnotes
+        if (containsFootnotes(contentList, context)) {
             // handle the footnote separator
-            StaticContent footnoteSeparator;
-            footnoteSeparator = pslm.getPageSequence().getStaticContent("xsl-footnote-separator");
-            if (footnoteSeparator != null) {
-                // the footnote separator can contain page-dependent content such as
-                // page numbers or retrieve markers, so its areas cannot simply be 
-                // obtained now and repeated in each page;
-                // we need to know in advance the separator bpd: the actual separator
-                // could be different from page to page, but its bpd would likely be
-                // always the same
-
-                // create a Block area that will contain the separator areas
-                separatorArea = new Block();
-                separatorArea.setIPD(pslm.getCurrentPV()
-                            .getRegionReference(Constants.FO_REGION_BODY).getIPD());
-                // create a StaticContentLM for the footnote separator
-                footnoteSeparatorLM = (StaticContentLayoutManager)
-                    pslm.getLayoutManagerMaker().makeStaticContentLayoutManager(
-                    pslm, footnoteSeparator, separatorArea);
-                footnoteSeparatorLM.doLayout();
-
-                footnoteSeparatorLength = new MinOptMax(separatorArea.getBPD());
-            }
+            handleFootnoteSeparator();
         }
         return contentList;
     }
-    
+
+    /** {@inheritDoc} */
+    protected List getNextKnuthElements(LayoutContext context, int alignment,
+            Position positionAtIPDChange, LayoutManager restartAtLM) {
+        List contentList = null;
+
+        do {
+            contentList = childFLM.getNextKnuthElements(context, alignment, positionAtIPDChange,
+                    restartAtLM);
+        } while (!childFLM.isFinished() && contentList == null);
+
+        // scan contentList, searching for footnotes
+        if (containsFootnotes(contentList, context)) {
+            // handle the footnote separator
+            handleFootnoteSeparator();
+        }
+        return contentList;
+    }
+
     /**
      * @return current display alignment
      */
@@ -210,14 +249,14 @@ public class PageBreaker extends AbstractBreaker {
         return pslm.getCurrentPage().getSimplePageMaster().getRegion(
                 Constants.FO_REGION_BODY).getDisplayAlign();
     }
-    
+
     /**
      * @return whether or not this flow has more page break opportunities
      */
     protected boolean hasMoreContent() {
         return !childFLM.isFinished();
     }
-    
+
     /**
      * Adds an area to the flow layout manager
      * @param posIter the position iterator
@@ -238,156 +277,148 @@ public class PageBreaker extends AbstractBreaker {
             footnoteSeparatorLM.doLayout();
         }
 
-        childFLM.addAreas(posIter, context);    
-    }
-    
-    /**
-     * Performs phase 3 operation
-     * 
-     * @param alg page breaking algorithm
-     * @param partCount part count
-     * @param originalList the block sequence original list
-     * @param effectiveList the block sequence effective list
-     */
-    protected void doPhase3(PageBreakingAlgorithm alg, int partCount, 
-            BlockSequence originalList, BlockSequence effectiveList) {
-        if (needColumnBalancing) {
-            doPhase3WithColumnBalancing(alg, partCount, originalList, effectiveList);
-        } else {
-            if (!hasMoreContent() && pslm.getPageSequence().hasPagePositionLast()) {
-                //last part is reached and we have a "last page" condition
-                doPhase3WithLastPage(alg, partCount, originalList, effectiveList);
-            } else {
-                //Directly add areas after finding the breaks
-                addAreas(alg, partCount, originalList, effectiveList);
-            }
-        }
+        childFLM.addAreas(posIter, context);
     }
 
-    private void doPhase3WithLastPage(PageBreakingAlgorithm alg, int partCount, 
+    /**
+     * {@inheritDoc}
+     * This implementation checks whether to trigger column-balancing,
+     * or whether to take into account a 'last-page' condition.
+     */
+    protected void doPhase3(PageBreakingAlgorithm alg, int partCount,
             BlockSequence originalList, BlockSequence effectiveList) {
-        int newStartPos;
+
+        if (needColumnBalancing) {
+            //column balancing for the last part
+            redoLayout(alg, partCount, originalList, effectiveList);
+            return;
+        }
+
+        boolean lastPageMasterDefined = pslm.getPageSequence().hasPagePositionLast();
+        if (!hasMoreContent()) {
+            //last part is reached
+            if (lastPageMasterDefined) {
+                //last-page condition
+                redoLayout(alg, partCount, originalList, effectiveList);
+                return;
+            }
+        }
+
+        //nothing special: just add the areas now
+        addAreas(alg, partCount, originalList, effectiveList);
+    }
+
+    /**
+     * Restart the algorithm at the break corresponding to the given partCount. Used to
+     * re-do the part after the last break in case of either column-balancing or a last
+     * page-master.
+     */
+    private void redoLayout(PageBreakingAlgorithm alg, int partCount,
+            BlockSequence originalList, BlockSequence effectiveList) {
+
+        int newStartPos = 0;
         int restartPoint = pageProvider.getStartingPartIndexForLastPage(partCount);
         if (restartPoint > 0) {
-            //Add definitive areas before last page
+            //Add definitive areas for the parts before the
+            //restarting point
             addAreas(alg, restartPoint, originalList, effectiveList);
             //Get page break from which we restart
             PageBreakPosition pbp = (PageBreakPosition)
                     alg.getPageBreaks().get(restartPoint - 1);
-            newStartPos = pbp.getLeafPos();
+            newStartPos = pbp.getLeafPos() + 1;
             //Handle page break right here to avoid any side-effects
             if (newStartPos > 0) {
                 handleBreakTrait(Constants.EN_PAGE);
             }
-        } else {
-            newStartPos = 0;
         }
-        AbstractBreaker.log.debug("Last page handling now!!!");
-        AbstractBreaker.log.debug("===================================================");
-        AbstractBreaker.log.debug("Restarting at " + restartPoint 
+
+        AbstractBreaker.log.debug("Restarting at " + restartPoint
                 + ", new start position: " + newStartPos);
 
         pageBreakHandled = true;
         //Update so the available BPD is reported correctly
         int currentPageNum = pslm.getCurrentPageNum();
-        pageProvider.setStartOfNextElementList(currentPageNum, 
-                pslm.getCurrentPV().getCurrentSpan().getCurrentFlowIndex());
-        pageProvider.setLastPageIndex(currentPageNum);
-
-        //Restart last page
-        PageBreakingAlgorithm algRestart = new PageBreakingAlgorithm(
-                getTopLevelLM(),
-                getPageProvider(), getLayoutListener(),
-                alg.getAlignment(), alg.getAlignmentLast(), 
-                footnoteSeparatorLength,
-                isPartOverflowRecoveryActivated(), false, false);
-        //alg.setConstantLineWidth(flowBPD);
-        int iOptPageCount = algRestart.findBreakingPoints(effectiveList,
-                    newStartPos,
-                    1, true, BreakingAlgorithm.ALL_BREAKS);
-        AbstractBreaker.log.debug("restart: iOptPageCount= " + iOptPageCount
-                + " pageBreaks.size()= " + algRestart.getPageBreaks().size());
-        boolean replaceLastPage 
-                = iOptPageCount <= pslm.getCurrentPV().getBodyRegion().getColumnCount(); 
-        if (replaceLastPage) {
-            //Replace last page
-            pslm.setCurrentPage(pageProvider.getPage(false, currentPageNum));
-            //Make sure we only add the areas we haven't added already
-            effectiveList.ignoreAtStart = newStartPos;
-            addAreas(algRestart, iOptPageCount, originalList, effectiveList);
-        } else {
-            effectiveList.ignoreAtStart = newStartPos;
-            addAreas(alg, restartPoint, partCount - restartPoint, originalList, effectiveList);
-            //Add blank last page
-            pageProvider.setLastPageIndex(currentPageNum + 1);
-            pslm.setCurrentPage(pslm.makeNewPage(true, true));
-        }
-        AbstractBreaker.log.debug("===================================================");
-    }
-
-    private void doPhase3WithColumnBalancing(PageBreakingAlgorithm alg, int partCount, 
-            BlockSequence originalList, BlockSequence effectiveList) {
-        AbstractBreaker.log.debug("Column balancing now!!!");
-        AbstractBreaker.log.debug("===================================================");
-        int newStartPos;
-        int restartPoint = pageProvider.getStartingPartIndexForLastPage(partCount);
-        if (restartPoint > 0) {
-            //Add definitive areas
-            addAreas(alg, restartPoint, originalList, effectiveList);
-            //Get page break from which we restart
-            PageBreakPosition pbp = (PageBreakPosition)
-                    alg.getPageBreaks().get(restartPoint - 1);
-            newStartPos = pbp.getLeafPos();
-            //Handle page break right here to avoid any side-effects
-            if (newStartPos > 0) {
-                handleBreakTrait(Constants.EN_PAGE);
-            }
-        } else {
-            newStartPos = 0;
-        }
-        AbstractBreaker.log.debug("Restarting at " + restartPoint 
-                + ", new start position: " + newStartPos);
-
-        pageBreakHandled = true;
-        //Update so the available BPD is reported correctly
-        pageProvider.setStartOfNextElementList(pslm.getCurrentPageNum(), 
+        pageProvider.setStartOfNextElementList(currentPageNum,
                 pslm.getCurrentPV().getCurrentSpan().getCurrentFlowIndex());
 
-        //Restart last page
-        PageBreakingAlgorithm algRestart = new BalancingColumnBreakingAlgorithm(
-                getTopLevelLM(),
-                getPageProvider(), getLayoutListener(),
-                alignment, Constants.EN_START, footnoteSeparatorLength,
-                isPartOverflowRecoveryActivated(),
-                pslm.getCurrentPV().getBodyRegion().getColumnCount());
-        //alg.setConstantLineWidth(flowBPD);
-        int iOptPageCount = algRestart.findBreakingPoints(effectiveList,
-                    newStartPos,
-                    1, true, BreakingAlgorithm.ALL_BREAKS);
-        AbstractBreaker.log.debug("restart: iOptPageCount= " + iOptPageCount
-                + " pageBreaks.size()= " + algRestart.getPageBreaks().size());
-        if (iOptPageCount > pslm.getCurrentPV().getBodyRegion().getColumnCount()) {
-            AbstractBreaker.log.warn(
-                    "Breaking algorithm produced more columns than are available.");
-            /* reenable when everything works
-            throw new IllegalStateException(
-                    "Breaking algorithm must not produce more columns than available.");
-            */
-        }
         //Make sure we only add the areas we haven't added already
         effectiveList.ignoreAtStart = newStartPos;
-        addAreas(algRestart, iOptPageCount, originalList, effectiveList);
-        AbstractBreaker.log.debug("===================================================");
+
+        PageBreakingAlgorithm algRestart;
+        if (needColumnBalancing) {
+            AbstractBreaker.log.debug("Column balancing now!!!");
+            AbstractBreaker.log.debug("===================================================");
+
+            //Restart last page
+            algRestart = new BalancingColumnBreakingAlgorithm(
+                    getTopLevelLM(), getPageProvider(), createLayoutListener(),
+                    alignment, Constants.EN_START, footnoteSeparatorLength,
+                    isPartOverflowRecoveryActivated(),
+                    pslm.getCurrentPV().getBodyRegion().getColumnCount());
+            AbstractBreaker.log.debug("===================================================");
+        } else  {
+            // Handle special page-master for last page
+            BodyRegion currentBody = pageProvider.getPage(false, currentPageNum)
+                    .getPageViewport().getBodyRegion();
+            pageProvider.setLastPageIndex(currentPageNum);
+            BodyRegion lastBody = pageProvider.getPage(false, currentPageNum)
+                    .getPageViewport().getBodyRegion();
+            lastBody.getMainReference().setSpans(currentBody.getMainReference().getSpans());
+            AbstractBreaker.log.debug("Last page handling now!!!");
+            AbstractBreaker.log.debug("===================================================");
+            //Restart last page
+            algRestart = new PageBreakingAlgorithm(
+                    getTopLevelLM(), getPageProvider(), createLayoutListener(),
+                    alg.getAlignment(), alg.getAlignmentLast(),
+                    footnoteSeparatorLength,
+                    isPartOverflowRecoveryActivated(), false, false);
+            AbstractBreaker.log.debug("===================================================");
+        }
+
+        int optimalPageCount = algRestart.findBreakingPoints(effectiveList,
+                    newStartPos,
+                    1, true, BreakingAlgorithm.ALL_BREAKS);
+        AbstractBreaker.log.debug("restart: optimalPageCount= " + optimalPageCount
+                + " pageBreaks.size()= " + algRestart.getPageBreaks().size());
+
+        boolean fitsOnePage
+                = optimalPageCount <= pslm.getCurrentPV().getBodyRegion().getMainReference().getCurrentSpan().getColumnCount();
+
+        if (needColumnBalancing) {
+            if (!fitsOnePage) {
+                AbstractBreaker.log.warn(
+                        "Breaking algorithm produced more columns than are available.");
+                /* reenable when everything works
+                throw new IllegalStateException(
+                        "Breaking algorithm must not produce more columns than available.");
+                */
+            }
+        } else {
+            if (fitsOnePage) {
+                //Replace last page
+                pslm.setCurrentPage(pageProvider.getPage(false, currentPageNum));
+            } else {
+                //Last page-master cannot hold the content.
+                //Add areas now...
+                addAreas(alg, restartPoint, partCount - restartPoint, originalList, effectiveList);
+                //...and add a blank last page
+                pageProvider.setLastPageIndex(currentPageNum + 1);
+                pslm.setCurrentPage(pslm.makeNewPage(true, true));
+                return;
+            }
+        }
+
+        addAreas(algRestart, optimalPageCount, originalList, effectiveList);
     }
-    
+
     protected void startPart(BlockSequence list, int breakClass) {
-        AbstractBreaker.log.debug("startPart() breakClass=" + breakClass);
+        AbstractBreaker.log.debug("startPart() breakClass=" + getBreakClassName(breakClass));
         if (pslm.getCurrentPage() == null) {
             throw new IllegalStateException("curPage must not be null");
         }
         if (!pageBreakHandled) {
-            
-            //firstPart is necessary because we need the first page before we start the 
+
+            //firstPart is necessary because we need the first page before we start the
             //algorithm so we have a BPD and IPD. This may subject to change later when we
             //start handling more complex cases.
             if (!firstPart) {
@@ -397,7 +428,7 @@ public class PageBreaker extends AbstractBreaker {
                 // otherwise, we may simply need a new page
                 handleBreakTrait(breakClass);
             }
-            pageProvider.setStartOfNextElementList(pslm.getCurrentPageNum(), 
+            pageProvider.setStartOfNextElementList(pslm.getCurrentPageNum(),
                     pslm.getCurrentPV().getCurrentSpan().getCurrentFlowIndex());
         }
         pageBreakHandled = false;
@@ -405,34 +436,34 @@ public class PageBreaker extends AbstractBreaker {
         // finish page and add to area tree
         firstPart = false;
     }
-    
+
     /** {@inheritDoc} */
     protected void handleEmptyContent() {
         pslm.getCurrentPV().getPage().fakeNonEmpty();
     }
-    
+
     protected void finishPart(PageBreakingAlgorithm alg, PageBreakPosition pbp) {
         // add footnote areas
         if (pbp.footnoteFirstListIndex < pbp.footnoteLastListIndex
             || pbp.footnoteFirstElementIndex <= pbp.footnoteLastElementIndex) {
             // call addAreas() for each FootnoteBodyLM
             for (int i = pbp.footnoteFirstListIndex; i <= pbp.footnoteLastListIndex; i++) {
-                LinkedList elementList = alg.getFootnoteList(i);
-                int firstIndex = (i == pbp.footnoteFirstListIndex 
+                List elementList = alg.getFootnoteList(i);
+                int firstIndex = (i == pbp.footnoteFirstListIndex
                         ? pbp.footnoteFirstElementIndex : 0);
-                int lastIndex = (i == pbp.footnoteLastListIndex 
+                int lastIndex = (i == pbp.footnoteLastListIndex
                         ? pbp.footnoteLastElementIndex : elementList.size() - 1);
 
-                SpaceResolver.performConditionalsNotification(elementList, 
+                SpaceResolver.performConditionalsNotification(elementList,
                         firstIndex, lastIndex, -1);
                 LayoutContext childLC = new LayoutContext(0);
-                AreaAdditionUtil.addAreas(null, 
-                        new KnuthPossPosIter(elementList, firstIndex, lastIndex + 1), 
+                AreaAdditionUtil.addAreas(null,
+                        new KnuthPossPosIter(elementList, firstIndex, lastIndex + 1),
                         childLC);
             }
             // set the offset from the top margin
-            Footnote parentArea = (Footnote) pslm.getCurrentPV().getBodyRegion().getFootnote();
-            int topOffset = (int) pslm.getCurrentPV().getBodyRegion().getBPD() - parentArea.getBPD();
+            Footnote parentArea = pslm.getCurrentPV().getBodyRegion().getFootnote();
+            int topOffset = pslm.getCurrentPV().getBodyRegion().getBPD() - parentArea.getBPD();
             if (separatorArea != null) {
                 topOffset -= separatorArea.getBPD();
             }
@@ -441,20 +472,18 @@ public class PageBreaker extends AbstractBreaker {
         }
         pslm.getCurrentPV().getCurrentSpan().notifyFlowsFinished();
     }
-    
-    /**
-     * @return the current child flow layout manager
-     */
+
+    /** @return the current child flow layout manager */
     protected LayoutManager getCurrentChildLM() {
         return childFLM;
     }
-    
+
     /** {@inheritDoc} */
     protected void observeElementList(List elementList) {
-        ElementListObserver.observe(elementList, "breaker", 
-                ((PageSequence)pslm.getFObj()).getId());
+        ElementListObserver.observe(elementList, "breaker",
+                pslm.getFObj().getId());
     }
-    
+
     /**
      * Depending on the kind of break condition, move to next column
      * or page. May need to make an empty page if next page would
@@ -463,48 +492,54 @@ public class PageBreaker extends AbstractBreaker {
      */
     private void handleBreakTrait(int breakVal) {
         Page curPage = pslm.getCurrentPage();
-        if (breakVal == Constants.EN_ALL) {
+        switch (breakVal) {
+        case Constants.EN_ALL:
             //break due to span change in multi-column layout
             curPage.getPageViewport().createSpan(true);
             return;
-        } else if (breakVal == Constants.EN_NONE) {
+        case Constants.EN_NONE:
             curPage.getPageViewport().createSpan(false);
             return;
-        } else if (breakVal == Constants.EN_COLUMN
-                || breakVal <= 0
-                || breakVal == Constants.EN_AUTO) {
+        case Constants.EN_COLUMN:
+        case Constants.EN_AUTO:
+        case Constants.EN_PAGE:
+        case -1:
             PageViewport pv = curPage.getPageViewport();
-            
+
             //Check if previous page was spanned
             boolean forceNewPageWithSpan = false;
             RegionBody rb = (RegionBody)curPage.getSimplePageMaster().getRegion(
                     Constants.FO_REGION_BODY);
-            if (breakVal < 0 
-                    && rb.getColumnCount() > 1 
-                    && pv.getCurrentSpan().getColumnCount() == 1) {
-                forceNewPageWithSpan = true;
-            }
-            
+            forceNewPageWithSpan
+                    = (rb.getColumnCount() > 1
+                        && pv.getCurrentSpan().getColumnCount() == 1);
+
             if (forceNewPageWithSpan) {
+                log.trace("Forcing new page with span");
                 curPage = pslm.makeNewPage(false, false);
                 curPage.getPageViewport().createSpan(true);
             } else if (pv.getCurrentSpan().hasMoreFlows()) {
+                log.trace("Moving to next flow");
                 pv.getCurrentSpan().moveToNextFlow();
             } else {
-                curPage = pslm.makeNewPage(false, false);
+                log.trace("Making new page");
+                /*curPage = */pslm.makeNewPage(false, false);
             }
             return;
-        }
-        log.debug("handling break-before after page " + pslm.getCurrentPageNum() 
-            + " breakVal=" + breakVal);
-        if (needBlankPageBeforeNew(breakVal)) {
-            curPage = pslm.makeNewPage(true, false);
-        }
-        if (needNewPage(breakVal)) {
-            curPage = pslm.makeNewPage(false, false);
+        default:
+            log.debug("handling break-before after page " + pslm.getCurrentPageNum()
+                + " breakVal=" + getBreakClassName(breakVal));
+            if (needBlankPageBeforeNew(breakVal)) {
+                log.trace("Inserting blank page");
+                /*curPage = */pslm.makeNewPage(true, false);
+            }
+            if (needNewPage(breakVal)) {
+                log.trace("Making new page");
+                /*curPage = */pslm.makeNewPage(false, false);
+            }
         }
     }
-    
+
     /**
      * Check if a blank page is needed to accomodate
      * desired even or odd page number.
@@ -523,7 +558,7 @@ public class PageBreaker extends AbstractBreaker {
             }
         }
     }
-    
+
     /**
      * See if need to generate a new page
      * @param breakVal - value of break-before or break-after trait.
@@ -541,4 +576,4 @@ public class PageBreaker extends AbstractBreaker {
             return true;
         }
     }
-}
\ No newline at end of file
+}
