@@ -15,12 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: TraitSetter.java 633961 2008-03-05 18:22:06Z vhennebert $ */
+/* $Id: TraitSetter.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.area.Area;
 import org.apache.fop.area.Trait;
 import org.apache.fop.datatypes.LengthBase;
@@ -42,14 +43,15 @@ public class TraitSetter {
 
     /** logger */
     protected static Log log = LogFactory.getLog(TraitSetter.class);
-    
+
     /**
      * Sets border and padding traits on areas.
-     * @param area area to set the traits on
-     * @param bpProps border and padding properties
+     *
+     * @param area      area to set the traits on
+     * @param bpProps   border and padding properties
      * @param bNotFirst True if the area is not the first area
-     * @param bNotLast True if the area is not the last area
-     * @param context Property evaluation context
+     * @param bNotLast  True if the area is not the last area
+     * @param context   Property evaluation context
      */
     public static void setBorderPaddingTraits(Area area,
             CommonBorderPaddingBackground bpProps, boolean bNotFirst, boolean bNotLast,
@@ -73,27 +75,28 @@ public class TraitSetter {
         }
 
         addBorderTrait(area, bpProps, bNotFirst,
-                       CommonBorderPaddingBackground.START, 
-                       BorderProps.SEPARATE, Trait.BORDER_START);
+                CommonBorderPaddingBackground.START,
+                BorderProps.SEPARATE, Trait.BORDER_START);
 
-        addBorderTrait(area, bpProps, bNotLast, 
-                       CommonBorderPaddingBackground.END,
-                       BorderProps.SEPARATE, Trait.BORDER_END);
+        addBorderTrait(area, bpProps, bNotLast,
+                CommonBorderPaddingBackground.END,
+                BorderProps.SEPARATE, Trait.BORDER_END);
 
-        addBorderTrait(area, bpProps, false, 
-                       CommonBorderPaddingBackground.BEFORE,
-                       BorderProps.SEPARATE, Trait.BORDER_BEFORE);
+        addBorderTrait(area, bpProps, false,
+                CommonBorderPaddingBackground.BEFORE,
+                BorderProps.SEPARATE, Trait.BORDER_BEFORE);
 
-        addBorderTrait(area, bpProps, false, 
-                       CommonBorderPaddingBackground.AFTER,
-                       BorderProps.SEPARATE, Trait.BORDER_AFTER);
+        addBorderTrait(area, bpProps, false,
+                CommonBorderPaddingBackground.AFTER,
+                BorderProps.SEPARATE, Trait.BORDER_AFTER);
     }
 
     /**
      * Sets border traits on an area.
-     * @param area area to set the traits on
+     *
+     * @param area    area to set the traits on
      * @param bpProps border and padding properties
-     * @param mode the border paint mode (see BorderProps)
+     * @param mode    the border paint mode (see BorderProps)
      */
     private static void addBorderTrait(Area area,
                                        CommonBorderPaddingBackground bpProps,
@@ -102,9 +105,9 @@ public class TraitSetter {
         int iBP = bpProps.getBorderWidth(iSide, bDiscard);
         if (iBP > 0) {
             area.addTrait(oTrait,
-                          new BorderProps(bpProps.getBorderStyle(iSide),
-                                          iBP, bpProps.getBorderColor(iSide),
-                                          mode));
+                    new BorderProps(bpProps.getBorderStyle(iSide),
+                            iBP, bpProps.getBorderColor(iSide),
+                            mode));
         }
     }
 
@@ -118,7 +121,7 @@ public class TraitSetter {
      * @deprecated Call the other addBorders() method and addPadding separately.
      */
     public static void addBorders(Area area, CommonBorderPaddingBackground bordProps,
-                                PercentBaseContext context) {
+                                  PercentBaseContext context) {
         BorderProps bps = getBorderProps(bordProps, CommonBorderPaddingBackground.BEFORE);
         if (bps != null) {
             area.addTrait(Trait.BORDER_BEFORE, bps);
@@ -182,7 +185,7 @@ public class TraitSetter {
      * @param borderAfter the resolved after border
      * @param borderStart the resolved start border
      * @param borderEnd the resolved end border
-     * @param outer 4 boolean values indicating if the side represents the 
+     * @param outer 4 boolean values indicating if the side represents the
      *     table's outer border. Order: before, after, start, end
      */
     public static void addCollapsingBorders(Area area,
@@ -207,7 +210,7 @@ public class TraitSetter {
         }
     }
 
-    private static void addPadding(Area area, CommonBorderPaddingBackground bordProps, 
+    private static void addPadding(Area area, CommonBorderPaddingBackground bordProps,
                                 PercentBaseContext context) {
         addPadding(area, bordProps, false, false, false, false, context);
     }
@@ -228,32 +231,32 @@ public class TraitSetter {
                 boolean discardBefore, boolean discardAfter,
                 boolean discardStart, boolean discardEnd,
                 PercentBaseContext context) {
-        int padding = bordProps.getPadding(CommonBorderPaddingBackground.BEFORE, 
+        int padding = bordProps.getPadding(CommonBorderPaddingBackground.BEFORE,
                 discardBefore, context);
         if (padding != 0) {
             area.addTrait(Trait.PADDING_BEFORE, new java.lang.Integer(padding));
         }
 
-        padding = bordProps.getPadding(CommonBorderPaddingBackground.AFTER, 
+        padding = bordProps.getPadding(CommonBorderPaddingBackground.AFTER,
                 discardAfter, context);
         if (padding != 0) {
             area.addTrait(Trait.PADDING_AFTER, new java.lang.Integer(padding));
         }
 
-        padding = bordProps.getPadding(CommonBorderPaddingBackground.START, 
+        padding = bordProps.getPadding(CommonBorderPaddingBackground.START,
                 discardStart, context);
         if (padding != 0) {
             area.addTrait(Trait.PADDING_START, new java.lang.Integer(padding));
         }
 
-        padding = bordProps.getPadding(CommonBorderPaddingBackground.END, 
+        padding = bordProps.getPadding(CommonBorderPaddingBackground.END,
                 discardEnd, context);
         if (padding != 0) {
             area.addTrait(Trait.PADDING_END, new java.lang.Integer(padding));
         }
 
     }
-    
+
     private static BorderProps getBorderProps(CommonBorderPaddingBackground bordProps, int side) {
         int width = bordProps.getBorderWidth(side, false);
         if (width != 0) {
@@ -289,17 +292,17 @@ public class TraitSetter {
      * background-position-horizontal/vertical to ensure the background images are
      * correctly placed. Indeed the placement of images must be made WRT the
      * column/body/row and not the cell.
-     * 
+     *
      * <p>Note: The area's IPD and BPD must be set before calling this method.</p>
-     * 
+     *
      * <p>TODO the regular
      * {@link #addBackground(Area, CommonBorderPaddingBackground, PercentBaseContext)}
      * method should be used instead, and a means to retrieve the original area's
      * dimensions must be found.</p>
-     * 
+     *
      * <p>TODO the placement of images in the x- or y-direction will be incorrect if
      * background-repeat is set for that direction.</p>
-     * 
+     *
      * @param area the area to set the traits on
      * @param backProps the background properties
      * @param context Property evaluation context
@@ -310,7 +313,7 @@ public class TraitSetter {
      * @param referenceIPD value to use as a reference for percentage calculation
      * @param referenceBPD value to use as a reference for percentage calculation
      */
-    public static void addBackground(Area area, 
+    public static void addBackground(Area area,
             CommonBorderPaddingBackground backProps,
             PercentBaseContext context,
             int ipdShift, int bpdShift, int referenceIPD, int referenceBPD) {
@@ -325,7 +328,7 @@ public class TraitSetter {
             back.setImageInfo(backProps.getImageInfo());
             back.setRepeat(backProps.backgroundRepeat);
             if (backProps.backgroundPositionHorizontal != null) {
-                if (back.getRepeat() == Constants.EN_NOREPEAT 
+                if (back.getRepeat() == Constants.EN_NOREPEAT
                         || back.getRepeat() == Constants.EN_REPEATY) {
                     if (area.getIPD() > 0) {
                         PercentBaseContext refContext = new SimplePercentBaseContext(context,
@@ -343,7 +346,7 @@ public class TraitSetter {
                 }
             }
             if (backProps.backgroundPositionVertical != null) {
-                if (back.getRepeat() == Constants.EN_NOREPEAT 
+                if (back.getRepeat() == Constants.EN_NOREPEAT
                         || back.getRepeat() == Constants.EN_REPEATX) {
                     if (area.getBPD() > 0) {
                         PercentBaseContext refContext = new SimplePercentBaseContext(context,
@@ -373,7 +376,7 @@ public class TraitSetter {
      * @param backProps the background properties
      * @param context Property evaluation context
      */
-    public static void addBackground(Area area, 
+    public static void addBackground(Area area,
                                      CommonBorderPaddingBackground backProps,
                                      PercentBaseContext context) {
         if (!backProps.hasBackground()) {
@@ -387,18 +390,21 @@ public class TraitSetter {
             back.setImageInfo(backProps.getImageInfo());
             back.setRepeat(backProps.backgroundRepeat);
             if (backProps.backgroundPositionHorizontal != null) {
-                if (back.getRepeat() == Constants.EN_NOREPEAT 
+                if (back.getRepeat() == Constants.EN_NOREPEAT
                         || back.getRepeat() == Constants.EN_REPEATY) {
                     if (area.getIPD() > 0) {
                         int width = area.getIPD();
                         width += backProps.getPaddingStart(false, context);
                         width += backProps.getPaddingEnd(false, context);
-                        back.setHoriz(backProps.backgroundPositionHorizontal.getValue(
-                                new SimplePercentBaseContext(context, 
-                                    LengthBase.IMAGE_BACKGROUND_POSITION_HORIZONTAL,
-                                    (width - back.getImageInfo().getSize().getWidthMpt())
-                                )
-                            ));
+                        int imageWidthMpt = back.getImageInfo().getSize().getWidthMpt();
+                        int lengthBaseValue = width - imageWidthMpt;
+                        SimplePercentBaseContext simplePercentBaseContext
+                                = new SimplePercentBaseContext(context,
+                                LengthBase.IMAGE_BACKGROUND_POSITION_HORIZONTAL,
+                                lengthBaseValue);
+                        int horizontal = backProps.backgroundPositionHorizontal.getValue(
+                                simplePercentBaseContext);
+                        back.setHoriz(horizontal);
                     } else {
                         //TODO Area IPD has to be set for this to work
                         log.warn("Horizontal background image positioning ignored"
@@ -408,18 +414,21 @@ public class TraitSetter {
                 }
             }
             if (backProps.backgroundPositionVertical != null) {
-                if (back.getRepeat() == Constants.EN_NOREPEAT 
+                if (back.getRepeat() == Constants.EN_NOREPEAT
                         || back.getRepeat() == Constants.EN_REPEATX) {
                     if (area.getBPD() > 0) {
                         int height = area.getBPD();
                         height += backProps.getPaddingBefore(false, context);
                         height += backProps.getPaddingAfter(false, context);
-                        back.setVertical(backProps.backgroundPositionVertical.getValue(
-                                new SimplePercentBaseContext(context, 
-                                     LengthBase.IMAGE_BACKGROUND_POSITION_VERTICAL,
-                                     (height - back.getImageInfo().getSize().getHeightMpt())
-                                )
-                            ));
+                        int imageHeightMpt = back.getImageInfo().getSize().getHeightMpt();
+                        int lengthBaseValue = height - imageHeightMpt;
+                        SimplePercentBaseContext simplePercentBaseContext
+                                = new SimplePercentBaseContext(context,
+                                LengthBase.IMAGE_BACKGROUND_POSITION_VERTICAL,
+                                lengthBaseValue);
+                        int vertical = backProps.backgroundPositionVertical.getValue(
+                                simplePercentBaseContext);
+                        back.setVertical(vertical);
                     } else {
                         //TODO Area BPD has to be set for this to work
                         log.warn("Vertical background image positioning ignored"
@@ -450,10 +459,10 @@ public class TraitSetter {
         if (startIndent != 0) {
             area.addTrait(Trait.START_INDENT, new Integer(startIndent));
         }
-        
+
         int spaceStart = startIndent
-                            - bpProps.getBorderStartWidth(false)
-                            - bpProps.getPaddingStart(false, context);
+                - bpProps.getBorderStartWidth(false)
+                - bpProps.getPaddingStart(false, context);
         if (spaceStart != 0) {
             area.addTrait(Trait.SPACE_START, new Integer(spaceStart));
         }
@@ -462,8 +471,8 @@ public class TraitSetter {
             area.addTrait(Trait.END_INDENT, new Integer(endIndent));
         }
         int spaceEnd = endIndent
-                            - bpProps.getBorderEndWidth(false)
-                            - bpProps.getPaddingEnd(false, context);
+                - bpProps.getBorderEndWidth(false)
+                - bpProps.getPaddingEnd(false, context);
         if (spaceEnd != 0) {
             area.addTrait(Trait.SPACE_END, new Integer(spaceEnd));
         }
@@ -488,7 +497,7 @@ public class TraitSetter {
     }
 
     /**
-     * Returns the effective space length of a resolved space specifier based on the adjustment 
+     * Returns the effective space length of a resolved space specifier based on the adjustment
      * value.
      * @param adjust the adjustment value
      * @param space the space specifier
@@ -497,16 +506,17 @@ public class TraitSetter {
     public static int getEffectiveSpace(double adjust, MinOptMax space) {
         if (space == null) {
             return 0;
-        }
-        int sp = space.opt;
-        if (adjust > 0) {
-            sp = sp + (int)(adjust * (space.max - space.opt));
         } else {
-            sp = sp + (int)(adjust * (space.opt - space.min));
+            int spaceOpt = space.getOpt();
+            if (adjust > 0) {
+                spaceOpt += (int) (adjust * space.getStretch());
+            } else {
+                spaceOpt += (int) (adjust * space.getShrink());
+            }
+            return spaceOpt;
         }
-        return sp;
     }
-    
+
     /**
      * Adds traits for space-before and space-after to an area.
      * @param area the target area
@@ -514,32 +524,33 @@ public class TraitSetter {
      * @param spaceBefore the space-before space specifier
      * @param spaceAfter the space-after space specifier
      */
-    public static void addSpaceBeforeAfter(Area area, double adjust, 
-            MinOptMax spaceBefore, MinOptMax spaceAfter) {
-        int space;
-        space = getEffectiveSpace(adjust, spaceBefore);
-        if (space != 0) {
-            area.addTrait(Trait.SPACE_BEFORE, new Integer(space));
-        }
-        space = getEffectiveSpace(adjust, spaceAfter);
-        if (space != 0) {
-            area.addTrait(Trait.SPACE_AFTER, new Integer(space));
+    public static void addSpaceBeforeAfter(Area area, double adjust, MinOptMax spaceBefore,
+                                           MinOptMax spaceAfter) {
+        addSpaceTrait(area, Trait.SPACE_BEFORE, spaceBefore, adjust);
+        addSpaceTrait(area, Trait.SPACE_AFTER, spaceAfter, adjust);
+    }
+
+    private static void addSpaceTrait(Area area, Integer spaceTrait,
+            MinOptMax space, double adjust) {
+        int effectiveSpace = getEffectiveSpace(adjust, space);
+        if (effectiveSpace != 0) {
+            area.addTrait(spaceTrait, new Integer(effectiveSpace));
         }
     }
-    
+
     /**
      * Sets the traits for breaks on an area.
      * @param area the area to set the traits on.
      * @param breakBefore the value for break-before
      * @param breakAfter the value for break-after
      */
-    public static void addBreaks(Area area,  int breakBefore, int breakAfter) {
+    public static void addBreaks(Area area, int breakBefore, int breakAfter) {
         /* Currently disabled as these traits are never used by the renderers
         area.addTrait(Trait.BREAK_AFTER, new Integer(breakAfter));
         area.addTrait(Trait.BREAK_BEFORE, new Integer(breakBefore));
         */
     }
-    
+
     /**
      * Adds font traits to an area
      * @param area the target are
@@ -549,7 +560,7 @@ public class TraitSetter {
         area.addTrait(Trait.FONT, font.getFontTriplet());
         area.addTrait(Trait.FONT_SIZE, new Integer(font.getFontSize()));
     }
-    
+
     /**
      * Adds the text-decoration traits to the area.
      * @param area the area to set the traits on
@@ -575,9 +586,20 @@ public class TraitSetter {
             }
         }
     }
-    
+
     /**
-     * Sets the producer's ID as a trait on the area. This can be used to track back the 
+     * Adds the ptr trait to the area.
+     * @param area the area to set the traits on
+     * @param ptr string
+     */
+    public static void addPtr(Area area, String ptr) {
+        if (ptr != null && ptr.length() > 0) {
+            area.addTrait(Trait.PTR, ptr);
+        }
+    }
+
+    /**
+     * Sets the producer's ID as a trait on the area. This can be used to track back the
      * generating FO node.
      * @param area the area to set the traits on
      * @param id the ID to set
