@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,23 +15,23 @@
  * limitations under the License.
  */
 
-/* $Id: BasicLinkLayoutManager.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: BasicLinkLayoutManager.java 830293 2009-10-27 19:07:52Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
-import org.apache.fop.datatypes.URISpecification;
-import org.apache.fop.fo.flow.BasicLink;
-import org.apache.fop.layoutmgr.LayoutManager;
-import org.apache.fop.layoutmgr.PageSequenceLayoutManager;
-import org.apache.fop.area.inline.InlineArea;
-import org.apache.fop.area.Trait;
 import org.apache.fop.area.LinkResolver;
+import org.apache.fop.area.Trait;
+import org.apache.fop.area.inline.InlineArea;
+import org.apache.fop.datatypes.URISpecification;
+import org.apache.fop.fo.Constants;
+import org.apache.fop.fo.flow.BasicLink;
+import org.apache.fop.layoutmgr.PageSequenceLayoutManager;
+import org.apache.fop.layoutmgr.TraitSetter;
 
 /**
  * LayoutManager for the fo:basic-link formatting object
  */
 public class BasicLinkLayoutManager extends InlineLayoutManager {
-    private BasicLink fobj;
 
     /**
      * Create an fo:basic-link layout manager.
@@ -40,24 +40,24 @@ public class BasicLinkLayoutManager extends InlineLayoutManager {
      */
     public BasicLinkLayoutManager(BasicLink node) {
         super(node);
-        fobj = node;
     }
 
     /** {@inheritDoc} */
     protected InlineArea createArea(boolean bInlineParent) {
         InlineArea area = super.createArea(bInlineParent);
-        setupBasicLinkArea(parentLM, area);
+        setupBasicLinkArea(area);
         return area;
     }
 
     /*
      * Detect internal or external link and add it as an area trait
      *
-     * @param parentLM the parent LayoutManager
      * @param area the basic-link's area
      */
-    private void setupBasicLinkArea(LayoutManager parentLM, InlineArea area) {
+    private void setupBasicLinkArea(InlineArea area) {
+        BasicLink fobj = (BasicLink) this.fobj;
         // internal destinations take precedence:
+        TraitSetter.addPtr(area, fobj.getPtr()); // used for accessibility
         if (fobj.hasInternalDestination()) {
             String idref = fobj.getInternalDestination();
             PageSequenceLayoutManager pslm = getPSLM();
@@ -70,8 +70,10 @@ public class BasicLinkLayoutManager extends InlineLayoutManager {
             }
         } else if (fobj.hasExternalDestination()) {
             String url = URISpecification.getURL(fobj.getExternalDestination());
+            boolean newWindow = (fobj.getShowDestination() == Constants.EN_NEW);
             if (url.length() > 0) {
-                area.addTrait(Trait.EXTERNAL_LINK, url);
+                area.addTrait(Trait.EXTERNAL_LINK,
+                        new Trait.ExternalLink(url, newWindow));
             }
         }
     }
