@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ICLayoutManager.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: ICLayoutManager.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
@@ -31,12 +31,10 @@ import org.apache.fop.fo.flow.InlineContainer;
  * and id areas are maintained for later retrieval.
  */
 public class ICLayoutManager extends LeafNodeLayoutManager {
-    private InlineContainer fobj;
     private List childrenLM;
 
     public ICLayoutManager(InlineContainer node, List childLM) {
         super(node);
-        fobj = node;
         childrenLM = childLM;
     }
 
@@ -44,7 +42,4 @@ public class ICLayoutManager extends LeafNodeLayoutManager {
         return null;
     }
 
-    protected void addId() {
-        getPSLM().addIDToPage(fobj.getId());
-    }
 }
