@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,10 +15,11 @@
  * limitations under the License.
  */
 
-/* $Id: InlineLayoutManager.java 635508 2008-03-10 10:06:37Z jeremias $ */
+/* $Id: InlineLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
+import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.ListIterator;
@@ -36,6 +37,7 @@ import org.apache.fop.fo.flow.InlineLevel;
 import org.apache.fop.fo.flow.Leader;
 import org.apache.fop.fo.pagination.Title;
 import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
+import org.apache.fop.fo.properties.CommonFont;
 import org.apache.fop.fo.properties.CommonMarginInline;
 import org.apache.fop.fo.properties.SpaceProperty;
 import org.apache.fop.fonts.Font;
@@ -56,6 +58,7 @@ import org.apache.fop.layoutmgr.SpaceSpecifier;
 import org.apache.fop.layoutmgr.TraitSetter;
 import org.apache.fop.traits.MinOptMax;
 import org.apache.fop.traits.SpaceVal;
+import org.apache.fop.util.ListUtil;
 
 /**
  * LayoutManager for objects which stack children in the inline direction,
@@ -68,8 +71,6 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
      */
     private static Log log = LogFactory.getLog(InlineLayoutManager.class);
 
-    private InlineLevel fobj;
-
     private CommonMarginInline inlineProps = null;
     private CommonBorderPaddingBackground borderProps = null;
 
@@ -92,7 +93,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
     protected SpaceProperty lineHeight;
     /** The keep-together property */
     //private KeepProperty keepTogether;
-    
+
     private AlignmentContext alignmentContext = null;
 
     /**
@@ -105,23 +106,27 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
     // The node should be FObjMixed
     public InlineLayoutManager(InlineLevel node) {
         super(node);
-        fobj = node;
     }
-    
+
     private Inline getInlineFO() {
         return (Inline) fobj;
     }
-    
+
     /** {@inheritDoc} */
     public void initialize() {
+        InlineLevel fobj = (InlineLevel) this.fobj;
+
         int padding = 0;
+
         FontInfo fi = fobj.getFOEventHandler().getFontInfo();
-        FontTriplet[] fontkeys = fobj.getCommonFont().getFontState(fi);
-        font = fi.getFontInstance(fontkeys[0], fobj.getCommonFont().fontSize.getValue(this));
+        CommonFont commonFont = fobj.getCommonFont();
+        FontTriplet[] fontkeys = commonFont.getFontState(fi);
+        font = fi.getFontInstance(fontkeys[0], commonFont.fontSize.getValue(this));
+
         lineHeight = fobj.getLineHeight();
         borderProps = fobj.getCommonBorderPaddingBackground();
         inlineProps = fobj.getCommonMarginInline();
-        
+
         if (fobj instanceof Inline) {
             alignmentAdjust = ((Inline)fobj).getAlignmentAdjust();
             alignmentBaseline = ((Inline)fobj).getAlignmentBaseline();
@@ -140,7 +145,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
             padding += borderProps.getPadding(CommonBorderPaddingBackground.AFTER, false, this);
             padding += borderProps.getBorderWidth(CommonBorderPaddingBackground.AFTER, false);
         }
-        extraBPD = new MinOptMax(padding);
+        extraBPD = MinOptMax.getInstance(padding);
 
     }
 
@@ -148,16 +153,16 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
     protected MinOptMax getExtraIPD(boolean isNotFirst, boolean isNotLast) {
         int borderAndPadding = 0;
         if (borderProps != null) {
-            borderAndPadding 
+            borderAndPadding
                 = borderProps.getPadding(CommonBorderPaddingBackground.START, isNotFirst, this);
-            borderAndPadding 
+            borderAndPadding
                 += borderProps.getBorderWidth(CommonBorderPaddingBackground.START, isNotFirst);
-            borderAndPadding 
+            borderAndPadding
                 += borderProps.getPadding(CommonBorderPaddingBackground.END, isNotLast, this);
-            borderAndPadding 
+            borderAndPadding
                 += borderProps.getBorderWidth(CommonBorderPaddingBackground.END, isNotLast);
         }
-        return new MinOptMax(borderAndPadding);
+        return MinOptMax.getInstance(borderAndPadding);
     }
 
 
@@ -185,11 +190,11 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
     protected SpaceProperty getSpaceEnd() {
         return inlineProps != null ? inlineProps.spaceEnd : null;
     }
-    
-    /** 
-     * Create and initialize an <code>InlineArea</code> 
-     * 
-     * @param hasInlineParent   true if the parent is an inline 
+
+    /**
+     * Create and initialize an <code>InlineArea</code>
+     *
+     * @param hasInlineParent   true if the parent is an inline
      * @return the area
      */
     protected InlineArea createArea(boolean hasInlineParent) {
@@ -205,7 +210,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         }
         return area;
     }
-    
+
     /** {@inheritDoc} */
     protected void setTraits(boolean isNotFirst, boolean isNotLast) {
         if (borderProps != null) {
@@ -228,29 +233,27 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
             return ((BlockLevelLayoutManager) lm).mustKeepTogether();
         } else if (lm instanceof InlineLayoutManager) {
             return ((InlineLayoutManager) lm).mustKeepTogether();
-        } else { 
+        } else {
             return mustKeepTogether(lm.getParent());
         }
     }
 
     /** {@inheritDoc} */
-    public LinkedList getNextKnuthElements(LayoutContext context, int alignment) {
+    public List getNextKnuthElements(LayoutContext context, int alignment) {
         LayoutManager curLM;
 
         // the list returned by child LM
-        LinkedList returnedList;
+        List returnedList;
 
         // the list which will be returned to the parent LM
-        LinkedList returnList = new LinkedList();
+        List returnList = new LinkedList();
         KnuthSequence lastSequence = null;
 
-        SpaceSpecifier leadingSpace = context.getLeadingSpace();
-        
         if (fobj instanceof Title) {
             alignmentContext = new AlignmentContext(font,
                                     lineHeight.getOptimum(this).getLength().getValue(this),
                                     context.getWritingMode());
-                                                    
+
         } else {
             alignmentContext = new AlignmentContext(font
                                     , lineHeight.getOptimum(this).getLength().getValue(this)
@@ -260,7 +263,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
                                     , dominantBaseline
                                     , context.getAlignmentContext());
         }
-        
+
         childLC = new LayoutContext(context);
         childLC.setAlignmentContext(alignmentContext);
 
@@ -270,14 +273,6 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
             if (getSpaceStart() != null) {
                 context.getLeadingSpace().addSpace(new SpaceVal(getSpaceStart(), this));
             }
-
-            // Check for "fence"
-            if (hasLeadingFence(!context.isFirstArea())) {
-                // Reset leading space sequence for child areas
-                leadingSpace = new SpaceSpecifier(false);
-            }
-            // Reset state variables
-            clearPrevIPD(); // Clear stored prev content dimensions
         }
 
         StringBuffer trace = new StringBuffer("InlineLM:");
@@ -296,9 +291,9 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
                 + borderProps.getBorderEndWidth(true)
              );
         }
-        
-        while ((curLM = (LayoutManager) getChildLM()) != null) {
-            
+
+        while ((curLM = getChildLM()) != null) {
+
             if (!(curLM instanceof InlineLevelLayoutManager)) {
                 // A block LM
                 // Leave room for start/end border and padding
@@ -310,37 +305,41 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
                             - borderProps.getBorderEndWidth(hasNextChildLM()));
                 }
             }
-            
+
             // get KnuthElements from curLM
             returnedList = curLM.getNextKnuthElements(childLC, alignment);
-            if (returnList.size() == 0 && childLC.isKeepWithPreviousPending()) {
-                childLC.setFlags(LayoutContext.KEEP_WITH_PREVIOUS_PENDING, false);
+            if (returnList.isEmpty() && childLC.isKeepWithPreviousPending()) {
+                childLC.clearKeepWithPreviousPending();
             }
             if (returnedList == null
-                    || returnedList.size() == 0) {
+                    || returnedList.isEmpty()) {
                 // curLM returned null or an empty list, because it finished;
                 // just iterate once more to see if there is another child
                 continue;
             }
-            
+
             if (curLM instanceof InlineLevelLayoutManager) {
-                context.setFlags(LayoutContext.KEEP_WITH_NEXT_PENDING, false);
+                context.clearKeepWithNextPending();
                 // "wrap" the Position stored in each element of returnedList
                 ListIterator seqIter = returnedList.listIterator();
                 while (seqIter.hasNext()) {
                     KnuthSequence sequence = (KnuthSequence) seqIter.next();
                     sequence.wrapPositions(this);
                 }
+                int insertionStartIndex = 0;
                 if (lastSequence != null && lastSequence.appendSequenceOrClose
                         ((KnuthSequence) returnedList.get(0))) {
-                    returnedList.remove(0);
+                    insertionStartIndex = 1;
                 }
                 // add border and padding to the first complete sequence of this LM
-                if (!borderAdded && returnedList.size() != 0) {
+                if (!borderAdded && !returnedList.isEmpty()) {
                     addKnuthElementsForBorderPaddingStart((KnuthSequence) returnedList.get(0));
                     borderAdded = true;
                 }
-                returnList.addAll(returnedList);
+                for (Iterator iter = returnedList.listIterator(insertionStartIndex);
+                        iter.hasNext();) {
+                    returnList.add(iter.next());
+                }
             } else { // A block LM
                 BlockKnuthSequence sequence = new BlockKnuthSequence(returnedList);
                 sequence.wrapPositions(this);
@@ -365,40 +364,38 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
                     returnList.add(sequence);
                 }
                 // propagate and clear
-                context.setFlags(LayoutContext.KEEP_WITH_NEXT_PENDING,
-                                 childLC.isKeepWithNextPending());
-                childLC.setFlags(LayoutContext.KEEP_WITH_NEXT_PENDING, false);
-                childLC.setFlags(LayoutContext.KEEP_WITH_PREVIOUS_PENDING, false);
+                context.updateKeepWithNextPending(childLC.getKeepWithNextPending());
+                childLC.clearKeepsPending();
             }
-            lastSequence = (KnuthSequence) returnList.getLast();
+            lastSequence = (KnuthSequence) ListUtil.getLast(returnList);
             lastChildLM = curLM;
         }
-        
+
         if (lastSequence != null) {
             addKnuthElementsForBorderPaddingEnd(lastSequence);
         }
 
         setFinished(true);
         log.trace(trace);
-        
-        if (returnList.size() == 0) {
+
+        if (returnList.isEmpty()) {
             /*
-             * if the FO itself is empty, but has an id specified 
+             * if the FO itself is empty, but has an id specified
              * or associated fo:markers, then we still need a dummy
              * sequence to register its position in the area tree
              */
             if (fobj.hasId() || fobj.hasMarkers()) {
                 InlineKnuthSequence emptySeq = new InlineKnuthSequence();
                 emptySeq.add(new KnuthInlineBox(
-                                0, 
-                                alignmentContext, 
-                                notifyPos(getAuxiliaryPosition()), 
+                                0,
+                                alignmentContext,
+                                notifyPos(getAuxiliaryPosition()),
                                 true));
                 returnList.add(emptySeq);
             }
         }
-        
-        return returnList.size() == 0 ? null : returnList;
+
+        return returnList.isEmpty() ? null : returnList;
     }
 
     /**
@@ -412,7 +409,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
      */
     public void addAreas(PositionIterator parentIter,
                          LayoutContext context) {
-        
+
         addId();
 
         setChildContext(new LayoutContext(context)); // Store current value
@@ -435,8 +432,8 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         // set in the layout context, it must be also set in the
         // layout context given to lastLM, but must be cleared in the
         // layout context given to the other LMs.
-        LinkedList positionList = new LinkedList();
-        NonLeafPosition pos = null;
+        List positionList = new LinkedList();
+        NonLeafPosition pos;
         LayoutManager lastLM = null;// last child LM in this iterator
         Position lastPos = null;
         while (parentIter.hasNext()) {
@@ -452,11 +449,11 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         }*/
 
         addMarkersToPage(
-                true, 
-                !areaCreated, 
+                true,
+                !areaCreated,
                 lastPos == null || isLast(lastPos));
-        
-        InlineArea parent = createArea(lastLM == null 
+
+        InlineArea parent = createArea(lastLM == null
                                         || lastLM instanceof InlineLevelLayoutManager);
         parent.setBPD(alignmentContext.getHeight());
         if (parent instanceof InlineParent) {
@@ -470,7 +467,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
             }
         }
         setCurrentArea(parent);
-        
+
         StackingIter childPosIter
             = new StackingIter(positionList.listIterator());
 
@@ -485,7 +482,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
             prevLM = childLM;
         }
 
-        
+
         /* If this LM has a trailing fence, resolve trailing space
          * specs from descendants.  Otherwise, propagate any trailing
          * space specs to the parent LM via the layout context.  If
@@ -494,10 +491,9 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
          * must be the last area for the current LM too.
          */
         boolean isLast = (getContext().isLastArea() && prevLM == lastChildLM);
-        
+
         if (hasTrailingFence(isLast)) {
-            addSpace(getCurrentArea(),
-                     getContext().getTrailingSpace().resolve(false),
+            addSpace(getCurrentArea(), getContext().getTrailingSpace().resolve(false),
                      getContext().getSpaceAdjust());
             context.setTrailingSpace(new SpaceSpecifier(false));
         } else {
@@ -508,41 +504,41 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         if (context.getTrailingSpace() != null  && getSpaceEnd() != null) {
             context.getTrailingSpace().addSpace(new SpaceVal(getSpaceEnd(), this));
         }
-        
+
         // Not sure if lastPos can legally be null or if that masks a different problem.
         // But it seems to fix bug 38053.
         setTraits(areaCreated, lastPos == null || !isLast(lastPos));
-        parentLM.addChildArea(getCurrentArea());
+        parentLayoutManager.addChildArea(getCurrentArea());
 
         addMarkersToPage(
-                false, 
-                !areaCreated, 
+                false,
+                !areaCreated,
                 lastPos == null || isLast(lastPos));
-        
+
         context.setFlags(LayoutContext.LAST_AREA, isLast);
         areaCreated = true;
+        checkEndOfLayout(lastPos);
     }
 
     /** {@inheritDoc} */
     public void addChildArea(Area childArea) {
         Area parent = getCurrentArea();
         if (getContext().resolveLeadingSpace()) {
-            addSpace(parent,
-                    getContext().getLeadingSpace().resolve(false),
+            addSpace(parent, getContext().getLeadingSpace().resolve(false),
                     getContext().getSpaceAdjust());
         }
         parent.addChildArea(childArea);
     }
 
     /** {@inheritDoc} */
-    public LinkedList getChangedKnuthElements(List oldList, int alignment) {
-        LinkedList returnedList = new LinkedList();
+    public List getChangedKnuthElements(List oldList, int alignment) {
+        List returnedList = new LinkedList();
         addKnuthElementsForBorderPaddingStart(returnedList);
         returnedList.addAll(super.getChangedKnuthElements(oldList, alignment));
         addKnuthElementsForBorderPaddingEnd(returnedList);
         return returnedList;
     }
-    
+
     /**
      * Creates Knuth elements for start border padding and adds them to the return list.
      * @param returnList return list to add the additional elements to
@@ -557,7 +553,8 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         if (returnList instanceof BlockKnuthSequence) {
             return;
         }
-        CommonBorderPaddingBackground borderAndPadding = fobj.getCommonBorderPaddingBackground();
+        CommonBorderPaddingBackground borderAndPadding =
+                ((InlineLevel)fobj).getCommonBorderPaddingBackground();
         if (borderAndPadding != null) {
             int ipStart = borderAndPadding.getBorderStartWidth(false)
                          + borderAndPadding.getPaddingStart(false, this);
@@ -581,7 +578,8 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         if (returnList instanceof BlockKnuthSequence) {
             return;
         }
-        CommonBorderPaddingBackground borderAndPadding = fobj.getCommonBorderPaddingBackground();
+        CommonBorderPaddingBackground borderAndPadding =
+                ((InlineLevel)fobj).getCommonBorderPaddingBackground();
         if (borderAndPadding != null) {
             int ipEnd = borderAndPadding.getBorderEndWidth(false)
                         + borderAndPadding.getPaddingEnd(false, this);
@@ -599,10 +597,5 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         //}
         return this.auxiliaryPosition;
     }
-    
-    /** {@inheritDoc} */
-    protected void addId() {
-        getPSLM().addIDToPage(fobj.getId());
-    }
-    
+
 }
