@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: InlineLevelLayoutManager.java 627367 2008-02-13 12:03:30Z maxberger $ */
- 
+/* $Id: InlineLevelLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
+
 package org.apache.fop.layoutmgr.inline;
 
 import java.util.List;
@@ -30,7 +30,7 @@ import org.apache.fop.layoutmgr.Position;
 public interface InlineLevelLayoutManager extends LayoutManager {
 
     /**
-     * Tell the LM to modify its data, adding a letter space 
+     * Tell the LM to modify its data, adding a letter space
      * to the word fragment represented by the given elements,
      * and returning the corrected elements
      *
@@ -40,7 +40,7 @@ public interface InlineLevelLayoutManager extends LayoutManager {
     List addALetterSpaceTo(List oldList);
 
     /**
-     * Tell the LM to modify its data, removing the word space 
+     * Tell the LM to modify its data, removing the word space
      * represented by the given elements
      *
      * @param oldList the elements representing the word space
@@ -48,20 +48,19 @@ public interface InlineLevelLayoutManager extends LayoutManager {
     void removeWordSpace(List oldList);
 
     /**
-     * Get the word chars corresponding to the given position
+     * Get the word chars corresponding to the given position.
      *
-     * @param sbChars the StringBuffer used to append word chars
-     * @param pos     the Position referring to the needed word chars
+     * @param pos     the position referring to the needed word chars.
      */
-    void getWordChars(StringBuffer sbChars, Position pos);
+    String getWordChars(Position pos);
 
     /**
      * Tell the LM to hyphenate a word
      *
      * @param pos the Position referring to the word
-     * @param hc  the HyphContext storing hyphenation information
+     * @param hyphContext  the HyphContext storing hyphenation information
      */
-    void hyphenate(Position pos, HyphContext hc);
+    void hyphenate(Position pos, HyphContext hyphContext);
 
     /**
      * Tell the LM to apply the changes due to hyphenation
