@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,16 +15,17 @@
  * limitations under the License.
  */
 
-/* $Id: InlineStackingLayoutManager.java 604171 2007-12-14 11:32:51Z vhennebert $ */
+/* $Id: InlineStackingLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
-import java.util.LinkedList;
 import java.util.Iterator;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.ListIterator;
-import java.util.HashMap;
 
+import org.apache.fop.area.Area;
+import org.apache.fop.area.inline.Space;
 import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.properties.SpaceProperty;
 import org.apache.fop.layoutmgr.AbstractLayoutManager;
@@ -34,8 +35,6 @@ import org.apache.fop.layoutmgr.LayoutManager;
 import org.apache.fop.layoutmgr.NonLeafPosition;
 import org.apache.fop.layoutmgr.Position;
 import org.apache.fop.layoutmgr.PositionIterator;
-import org.apache.fop.area.Area;
-import org.apache.fop.area.inline.Space;
 import org.apache.fop.traits.MinOptMax;
 
 /**
@@ -43,7 +42,7 @@ import org.apache.fop.traits.MinOptMax;
  * which stack children in the inline direction, such as Inline or
  * Line. It should not be instantiated directly.
  */
-public abstract class InlineStackingLayoutManager extends AbstractLayoutManager 
+public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
                                          implements InlineLevelLayoutManager {
 
 
@@ -62,12 +61,6 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
         }
     }
 
-
-    /**
-     * Size of any start or end borders and padding.
-     */
-    private MinOptMax allocIPD = new MinOptMax(0);
-
     /**
      * Size of border and padding in BPD (ie, before and after).
      */
@@ -75,18 +68,9 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
 
     private Area currentArea; // LineArea or InlineParent
 
-    //private BreakPoss prevBP;
-    
     /** The child layout context */
     protected LayoutContext childLC;
 
-    private boolean bAreaCreated = false;
-
-    //private LayoutManager currentLM = null;
-
-    /** Used to store previous content IPD for each child LM. */
-    private HashMap hmPrevIPD = new HashMap();
-
     /**
      * Create an inline stacking layout manager.
      * This is used for fo's that create areas that
@@ -96,7 +80,7 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
      */
     protected InlineStackingLayoutManager(FObj node) {
         super(node);
-        extraBPD = new MinOptMax(0);
+        extraBPD = MinOptMax.ZERO;
     }
 
     /**
@@ -109,14 +93,14 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
     }
 
     /**
-     * Returns the extra IPD needed for any leading or trailing fences for the 
+     * Returns the extra IPD needed for any leading or trailing fences for the
      * current area.
      * @param bNotFirst true if not the first area for this layout manager
      * @param bNotLast true if not the last area for this layout manager
      * @return the extra IPD as a MinOptMax spec
      */
     protected MinOptMax getExtraIPD(boolean bNotFirst, boolean bNotLast) {
-        return new MinOptMax(0);
+        return MinOptMax.ZERO;
     }
 
 
@@ -145,7 +129,7 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
     protected SpaceProperty getSpaceStart() {
         return null;
     }
-    
+
     /**
      * Get the space at the end of the inline area.
      * @return the space property describing the space
@@ -154,30 +138,6 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
         return null;
     }
 
-    /**
-     * TODO: Explain this method
-     * @param lm ???
-     * @return ???
-     */
-    protected MinOptMax getPrevIPD(LayoutManager lm) {
-        return (MinOptMax) hmPrevIPD.get(lm);
-    }
-
-    /**
-     * Clear the previous IPD calculation.
-     */
-    protected void clearPrevIPD() {
-        hmPrevIPD.clear();
-    }
-
-    /**
-     * This method is called by addAreas() so IDs can be added to a page for FOs that 
-     * support the 'id' property.
-     */
-    protected void addId() {
-        // Do nothing here, overriden in subclasses that have an 'id' property.
-    }
-    
     /**
      * Returns the current area.
      * @return the current area
@@ -219,23 +179,21 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
     }
 
     /**
-     * Adds a space to the area
+     * Adds a space to the area.
+     *
      * @param parentArea the area to which to add the space
      * @param spaceRange the space range specifier
-     * @param dSpaceAdjust the factor by which to stretch or shrink the space
+     * @param spaceAdjust the factor by which to stretch or shrink the space
      */
-    protected void addSpace(Area parentArea, MinOptMax spaceRange,
-                            double dSpaceAdjust) {
+    protected void addSpace(Area parentArea, MinOptMax spaceRange, double spaceAdjust) {
         if (spaceRange != null) {
-            int iAdjust = spaceRange.opt;
-            if (dSpaceAdjust > 0.0) {
+            int iAdjust = spaceRange.getOpt();
+            if (spaceAdjust > 0.0) {
                 // Stretch by factor
-                iAdjust += (int) ((double) (spaceRange.max
-                                          - spaceRange.opt) * dSpaceAdjust);
-            } else if (dSpaceAdjust < 0.0) {
+                iAdjust += (int) (spaceRange.getStretch() * spaceAdjust);
+            } else if (spaceAdjust < 0.0) {
                 // Shrink by factor
-                iAdjust += (int) ((double) (spaceRange.opt
-                                          - spaceRange.min) * dSpaceAdjust);
+                iAdjust += (int) (spaceRange.getShrink() * spaceAdjust);
             }
             if (iAdjust != 0) {
                 //getLogger().debug("Add leading space: " + iAdjust);
@@ -255,12 +213,12 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
         // "unwrap" the Position stored in each element of oldList
         while (oldListIterator.hasNext()) {
             element = (KnuthElement) oldListIterator.next();
-            element.setPosition(((NonLeafPosition)element.getPosition()).getPosition());
+            element.setPosition(element.getPosition().getPosition());
         }
 
         // The last element may not have a layout manager (its position == null);
         // this may happen if it is a padding box; see bug 39571.
-        InlineLevelLayoutManager LM = 
+        InlineLevelLayoutManager LM =
             (InlineLevelLayoutManager) element.getLayoutManager();
         if (LM != null) {
             oldList = LM.addALetterSpaceTo(oldList);
@@ -288,7 +246,7 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
         // "unwrap" the Position stored in each element of oldList
         while (oldListIterator.hasNext()) {
             element = (KnuthElement) oldListIterator.next();
-            element.setPosition(((NonLeafPosition)element.getPosition()).getPosition());
+            element.setPosition(element.getPosition().getPosition());
         }
 
         ((InlineLevelLayoutManager)
@@ -297,15 +255,14 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
     }
 
     /** {@inheritDoc} */
-    public void getWordChars(StringBuffer sbChars, Position pos) {
-        Position newPos = ((NonLeafPosition) pos).getPosition();
-        ((InlineLevelLayoutManager)
-         newPos.getLM()).getWordChars(sbChars, newPos);
+    public String getWordChars(Position pos) {
+        Position newPos = pos.getPosition();
+        return ((InlineLevelLayoutManager) newPos.getLM()).getWordChars(newPos);
     }
 
     /** {@inheritDoc} */
     public void hyphenate(Position pos, HyphContext hc) {
-        Position newPos = ((NonLeafPosition) pos).getPosition();
+        Position newPos = pos.getPosition();
         ((InlineLevelLayoutManager)
          newPos.getLM()).hyphenate(newPos, hc);
     }
@@ -318,7 +275,7 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
         while (oldListIterator.hasNext()) {
             oldElement = (KnuthElement) oldListIterator.next();
             oldElement.setPosition
-                (((NonLeafPosition) oldElement.getPosition()).getPosition());
+                (oldElement.getPosition().getPosition());
         }
         // reset the iterator
         oldListIterator = oldList.listIterator();
@@ -348,7 +305,8 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
                     fromIndex = oldListIterator.previousIndex();
                 } else if (currLM == prevLM) {
                     bSomethingChanged
-                        = prevLM.applyChanges(oldList.subList(fromIndex, oldList.size()))
+                        = (prevLM != null)
+                            && prevLM.applyChanges(oldList.subList(fromIndex, oldList.size()))
                             || bSomethingChanged;
                 } else {
                     bSomethingChanged
@@ -376,16 +334,16 @@ public abstract class InlineStackingLayoutManager extends AbstractLayoutManager
     }
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
-    public LinkedList getChangedKnuthElements(List oldList, int alignment) {
+    public List getChangedKnuthElements(List oldList, int alignment) {
         // "unwrap" the Positions stored in the elements
         ListIterator oldListIterator = oldList.listIterator();
         KnuthElement oldElement;
         while (oldListIterator.hasNext()) {
             oldElement = (KnuthElement) oldListIterator.next();
             oldElement.setPosition
-                (((NonLeafPosition) oldElement.getPosition()).getPosition());
+                (oldElement.getPosition().getPosition());
         }
         // reset the iterator
         oldListIterator = oldList.listIterator();
