@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,30 +15,29 @@
  * limitations under the License.
  */
 
-/* $Id: KnuthInlineBox.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: KnuthInlineBox.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
-import org.apache.fop.layoutmgr.inline.AlignmentContext;
 import org.apache.fop.layoutmgr.FootnoteBodyLayoutManager;
 import org.apache.fop.layoutmgr.KnuthBox;
 import org.apache.fop.layoutmgr.Position;
 
 public class KnuthInlineBox extends KnuthBox {
-    
+
     private FootnoteBodyLayoutManager footnoteBodyLM = null;
     private AlignmentContext alignmentContext = null;
 
     /**
      * Create a new KnuthBox.
      *
-     * @param w    the width of this box
+     * @param width            the width of this box
      * @param alignmentContext the alignmentContext for this box
-     * @param pos  the Position stored in this box
-     * @param bAux is this box auxiliary?
+     * @param pos              the Position stored in this box
+     * @param auxiliary        is this box auxiliary?
      */
-    public KnuthInlineBox(int w, AlignmentContext alignmentContext, Position pos, boolean bAux) {
-        super(w, pos, bAux);
+    public KnuthInlineBox(int width, AlignmentContext alignmentContext, Position pos, boolean auxiliary) {
+        super(width, pos, auxiliary);
         this.alignmentContext = alignmentContext;
     }
 
@@ -69,11 +68,4 @@ public class KnuthInlineBox extends KnuthBox {
     public boolean isAnchor() {
         return (footnoteBodyLM != null);
     }
-    
-    
-    /** {@inheritDoc} */
-    public String toString() {
-        StringBuffer sb = new StringBuffer(super.toString());
-        return sb.toString();
-    }
-}
\ No newline at end of file
+}
