@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,15 +15,20 @@
  * limitations under the License.
  */
 
-/* $Id: LeaderLayoutManager.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: LeaderLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
+import java.util.Collections;
+import java.util.LinkedList;
+import java.util.List;
+
 import org.apache.fop.area.Trait;
 import org.apache.fop.area.inline.FilledArea;
 import org.apache.fop.area.inline.InlineArea;
 import org.apache.fop.area.inline.Space;
 import org.apache.fop.area.inline.TextArea;
+import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.flow.Leader;
 import org.apache.fop.fonts.Font;
 import org.apache.fop.fonts.FontInfo;
@@ -41,20 +46,16 @@ import org.apache.fop.layoutmgr.PositionIterator;
 import org.apache.fop.layoutmgr.TraitSetter;
 import org.apache.fop.traits.MinOptMax;
 
-import java.util.List;
-import java.util.LinkedList;
-import org.apache.fop.fo.FObj;
-
 /**
  * LayoutManager for the fo:leader formatting object
  */
 public class LeaderLayoutManager extends LeafNodeLayoutManager {
     private Leader fobj;
     private Font font = null;
-    
-    private LinkedList contentList = null;
+
+    private List contentList = null;
     private ContentLayoutManager clm = null;
-    
+
     private int contentAreaIPD = 0;
 
     /**
@@ -66,7 +67,7 @@ public class LeaderLayoutManager extends LeafNodeLayoutManager {
         super(node);
         fobj = node;
     }
-    
+
     /** {@inheritDoc} */
     public void initialize() {
         FontInfo fi = fobj.getFOEventHandler().getFontInfo();
@@ -109,7 +110,7 @@ public class LeaderLayoutManager extends LeafNodeLayoutManager {
                     - borderPaddingWidth;
         int max = fobj.getLeaderLength().getMaximum(this).getLength().getValue(this)
                     - borderPaddingWidth;
-        return new MinOptMax(min, opt, max);
+        return MinOptMax.getInstance(min, opt, max);
     }
 
     private InlineArea getLeaderInlineArea(LayoutContext context) {
@@ -117,20 +118,19 @@ public class LeaderLayoutManager extends LeafNodeLayoutManager {
 
         if (fobj.getLeaderPattern() == EN_RULE) {
             if (fobj.getRuleStyle() != EN_NONE) {
-                org.apache.fop.area.inline.Leader leader 
+                org.apache.fop.area.inline.Leader leader
                     = new org.apache.fop.area.inline.Leader();
                 leader.setRuleStyle(fobj.getRuleStyle());
                 leader.setRuleThickness(fobj.getRuleThickness().getValue(this));
-                leader.setBPD(fobj.getRuleThickness().getValue(this));
                 leaderArea = leader;
             } else {
                 leaderArea = new Space();
-                leaderArea.setBPD(1);
             }
+            leaderArea.setBPD(fobj.getRuleThickness().getValue(this));
             leaderArea.addTrait(Trait.COLOR, fobj.getColor());
         } else if (fobj.getLeaderPattern() == EN_SPACE) {
             leaderArea = new Space();
-            leaderArea.setBPD(1);
+            leaderArea.setBPD(fobj.getRuleThickness().getValue(this));
         } else if (fobj.getLeaderPattern() == EN_DOTS) {
             TextArea t = new TextArea();
             char dot = '.'; // userAgent.getLeaderDotCharacter();
@@ -159,13 +159,15 @@ public class LeaderLayoutManager extends LeafNodeLayoutManager {
             leaderArea = fa;
         } else if (fobj.getLeaderPattern() == EN_USECONTENT) {
             if (fobj.getChildNodes() == null) {
-                fobj.getLogger().error("Leader use-content with no content");
+                InlineLevelEventProducer eventProducer = InlineLevelEventProducer.Provider.get(
+                        getFObj().getUserAgent().getEventBroadcaster());
+                eventProducer.leaderWithoutContent(this, getFObj().getLocator());
                 return null;
             }
 
             // child FOs are assigned to the InlineStackingLM
             fobjIter = null;
-            
+
             // get breaks then add areas to FilledArea
             FilledArea fa = new FilledArea();
 
@@ -181,17 +183,23 @@ public class LeaderLayoutManager extends LeafNodeLayoutManager {
             childContext.setAlignmentContext(context.getAlignmentContext());
             contentList = clm.getNextKnuthElements(childContext, 0);
             int width = clm.getStackingSize();
-            Space spacer = null;
-            if (fobj.getLeaderPatternWidth().getValue(this) > width) {
-                spacer = new Space();
-                spacer.setIPD(fobj.getLeaderPatternWidth().getValue(this) - width);
-                width = fobj.getLeaderPatternWidth().getValue(this);
+            if (width != 0) {
+                Space spacer = null;
+                if (fobj.getLeaderPatternWidth().getValue(this) > width) {
+                    spacer = new Space();
+                    spacer.setIPD(fobj.getLeaderPatternWidth().getValue(this) - width);
+                    width = fobj.getLeaderPatternWidth().getValue(this);
+                }
+                fa.setUnitWidth(width);
+                if (spacer != null) {
+                    fa.addChildArea(spacer);
+                }
+                leaderArea = fa;
+            } else {
+                //Content collapsed to nothing, so use a space
+                leaderArea = new Space();
+                leaderArea.setBPD(fobj.getRuleThickness().getValue(this));
             }
-            fa.setUnitWidth(width);
-            if (spacer != null) {
-                fa.addChildArea(spacer);
-            }
-            leaderArea = fa;
         }
         TraitSetter.setProducerID(leaderArea, fobj.getId());
         return leaderArea;
@@ -219,7 +227,7 @@ public class LeaderLayoutManager extends LeafNodeLayoutManager {
             KnuthPossPosIter contentIter = new KnuthPossPosIter(contentList, 0, contentList.size());
             clm.addAreas(contentIter, context);
 
-            parentLM.addChildArea(curArea);
+            parentLayoutManager.addChildArea(curArea);
 
             while (posIter.hasNext()) {
                 posIter.next();
@@ -228,8 +236,7 @@ public class LeaderLayoutManager extends LeafNodeLayoutManager {
     }
 
     /** {@inheritDoc} */
-    public LinkedList getNextKnuthElements(LayoutContext context,
-                                           int alignment) {
+    public List getNextKnuthElements(LayoutContext context, int alignment) {
         MinOptMax ipd;
         curArea = get(context);
         KnuthSequence seq = new InlineKnuthSequence();
@@ -249,45 +256,34 @@ public class LeaderLayoutManager extends LeafNodeLayoutManager {
         ipd = getAllocationIPD(context.getRefIPD());
         if (fobj.getLeaderPattern() == EN_USECONTENT && curArea instanceof FilledArea) {
             // If we have user supplied content make it fit if we can
-            int unitWidth = ((FilledArea)curArea).getUnitWidth();
-            if (ipd.opt < unitWidth && ipd.max >= unitWidth) {
-                ipd.opt = unitWidth;
+            int unitWidth = ((FilledArea) curArea).getUnitWidth();
+            if (ipd.getOpt() < unitWidth && unitWidth <= ipd.getMax()) {
+                ipd = MinOptMax.getInstance(ipd.getMin(), unitWidth, ipd.getMax());
             }
         }
 
         // create the AreaInfo object to store the computed values
         areaInfo = new AreaInfo((short) 0, ipd, false, context.getAlignmentContext());
-        curArea.setAdjustingInfo(ipd.max - ipd.opt, ipd.opt - ipd.min, 0);
+        curArea.setAdjustingInfo(ipd.getStretch(), ipd.getShrink(), 0);
 
         addKnuthElementsForBorderPaddingStart(seq);
-        
+
         // node is a fo:Leader
-        seq.add(new KnuthInlineBox(0, alignmentContext,
-                                    new LeafPosition(this, -1), true));
+        seq.add(new KnuthInlineBox(0, alignmentContext, new LeafPosition(this, -1), true));
         seq.add(new KnuthPenalty(0, KnuthElement.INFINITE, false,
-                                        new LeafPosition(this, -1), true));
+                new LeafPosition(this, -1), true));
         if (alignment == EN_JUSTIFY || alignment == 0) {
-            seq.add
-                (new KnuthGlue(areaInfo.ipdArea.opt,
-                               areaInfo.ipdArea.max - areaInfo.ipdArea.opt,
-                               areaInfo.ipdArea.opt - areaInfo.ipdArea.min, 
-                               new LeafPosition(this, 0), false));
+            seq.add(new KnuthGlue(areaInfo.ipdArea, new LeafPosition(this, 0), false));
         } else {
-            seq.add
-                (new KnuthGlue(areaInfo.ipdArea.opt,
-                               0,
-                               0, 
-                               new LeafPosition(this, 0), false));
+            seq.add(new KnuthGlue(areaInfo.ipdArea.getOpt(), 0, 0,
+                    new LeafPosition(this, 0), false));
         }
-        seq.add(new KnuthInlineBox(0, alignmentContext,
-                                    new LeafPosition(this, -1), true));
+        seq.add(new KnuthInlineBox(0, alignmentContext, new LeafPosition(this, -1), true));
 
         addKnuthElementsForBorderPaddingEnd(seq);
-        
-        LinkedList returnList = new LinkedList();
-        returnList.add(seq);
+
         setFinished(true);
-        return returnList;
+        return Collections.singletonList(seq);
     }
 
     /** {@inheritDoc} */
@@ -303,50 +299,35 @@ public class LeaderLayoutManager extends LeafNodeLayoutManager {
     }
 
     /** {@inheritDoc} */
-    public LinkedList getChangedKnuthElements(List oldList,
-                                              int alignment) {
+    public List getChangedKnuthElements(List oldList, int alignment) {
         if (isFinished()) {
             return null;
         }
 
-        LinkedList returnList = new LinkedList();
+        List returnList = new LinkedList();
 
         addKnuthElementsForBorderPaddingStart(returnList);
-        
+
         returnList.add(new KnuthInlineBox(0, areaInfo.alignmentContext,
                                     new LeafPosition(this, -1), true));
         returnList.add(new KnuthPenalty(0, KnuthElement.INFINITE, false,
                                         new LeafPosition(this, -1), true));
         if (alignment == EN_JUSTIFY || alignment == 0) {
-            returnList.add
-                (new KnuthGlue(areaInfo.ipdArea.opt,
-                               areaInfo.ipdArea.max - areaInfo.ipdArea.opt,
-                               areaInfo.ipdArea.opt - areaInfo.ipdArea.min, 
-                               new LeafPosition(this, 0), false));
+            returnList.add(new KnuthGlue(areaInfo.ipdArea, new LeafPosition(this, 0), false));
         } else {
-            returnList.add
-                (new KnuthGlue(areaInfo.ipdArea.opt,
-                               0,
-                               0, 
-                               new LeafPosition(this, 0), false));
+            returnList.add(new KnuthGlue(areaInfo.ipdArea.getOpt(), 0, 0,
+                    new LeafPosition(this, 0), false));
         }
         returnList.add(new KnuthInlineBox(0, areaInfo.alignmentContext,
                                     new LeafPosition(this, -1), true));
 
         addKnuthElementsForBorderPaddingEnd(returnList);
-        
+
         setFinished(true);
         return returnList;
     }
 
     /** {@inheritDoc} */
-    protected void addId() {
-        getPSLM().addIDToPage(fobj.getId());
-    }
-
-    /**
-     * {@inheritDoc} 
-     */
     public int getBaseLength(int lengthBase, FObj fobj) {
         return getParent().getBaseLength(lengthBase, getParent().getFObj());
     }
@@ -358,9 +339,9 @@ public class LeaderLayoutManager extends LeafNodeLayoutManager {
     public int getContentAreaIPD() {
         return contentAreaIPD;
     }
-   
+
     private void setContentAreaIPD(int contentAreaIPD) {
         this.contentAreaIPD = contentAreaIPD;
     }
-    
+
 }
