@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,12 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: LeafNodeLayoutManager.java 557337 2007-07-18 17:37:14Z adelmelle $ */
+/* $Id: LeafNodeLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
 import java.util.LinkedList;
 import java.util.List;
+import java.util.Collections;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -48,7 +49,7 @@ import org.apache.fop.traits.MinOptMax;
  * This class can be extended to handle the creation and adding of the
  * inline area.
  */
-public abstract class LeafNodeLayoutManager extends AbstractLayoutManager 
+public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
                                    implements InlineLevelLayoutManager {
 
     /**
@@ -64,8 +65,6 @@ public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
     protected CommonBorderPaddingBackground commonBorderPaddingBackground = null;
     /** The alignment context applying to this area */
     protected AlignmentContext alignmentContext = null;
-    
-    private MinOptMax ipd;
 
     /** Flag to indicate if something was changed as part of the getChangeKnuthElements sequence */
     protected boolean isSomethingChanged = false;
@@ -88,12 +87,12 @@ public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
             bHyphenated = bHyph;
             this.alignmentContext = alignmentContext;
         }
-        
+
     }
 
 
     /**
-     * Create a Leaf node layout mananger.
+     * Create a Leaf node layout manager.
      * @param node the FObj to attach to this LM.
      */
     public LeafNodeLayoutManager(FObj node) {
@@ -101,7 +100,7 @@ public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
     }
 
     /**
-     * Create a Leaf node layout mananger.
+     * Create a Leaf node layout manager.
      */
     public LeafNodeLayoutManager() {
     }
@@ -165,7 +164,7 @@ public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
      * @return the min/opt/max ipd of the inline area
      */
     protected MinOptMax getAllocationIPD(int refIPD) {
-        return new MinOptMax(curArea.getIPD());
+        return MinOptMax.getInstance(curArea.getIPD());
     }
 
     /**
@@ -188,7 +187,7 @@ public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
                                                    false, false, this);
                 TraitSetter.addBackground(area, commonBorderPaddingBackground, this);
             }
-            parentLM.addChildArea(area);
+            parentLayoutManager.addChildArea(area);
         }
 
         while (posIter.hasNext()) {
@@ -203,15 +202,7 @@ public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
     protected InlineArea getEffectiveArea() {
         return curArea;
     }
-    
-    /**
-     * This method is called by addAreas() so IDs can be added to a page for FOs that 
-     * support the 'id' property.
-     */
-    protected void addId() {
-        // Do nothing here, overriden in subclasses that have an 'id' property.
-    }
-    
+
     /**
      * Offset this area.
      * Offset the inline area in the bpd direction when adding the
@@ -246,29 +237,27 @@ public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
      */
     protected void widthAdjustArea(InlineArea area, LayoutContext context) {
         double dAdjust = context.getIPDAdjust();
-        int width = areaInfo.ipdArea.opt;
+        int adjustment = 0;
         if (dAdjust < 0) {
-            width = (int) (width + dAdjust * (areaInfo.ipdArea.opt
-                                             - areaInfo.ipdArea.min));
+            adjustment += (int) (dAdjust * areaInfo.ipdArea.getShrink());
         } else if (dAdjust > 0) {
-            width = (int) (width + dAdjust * (areaInfo.ipdArea.max
-                                             - areaInfo.ipdArea.opt));
+            adjustment += (int) (dAdjust * areaInfo.ipdArea.getStretch());
         }
-        area.setIPD(width);
-        area.setAdjustment(width - areaInfo.ipdArea.opt);
+        area.setIPD(areaInfo.ipdArea.getOpt() + adjustment);
+        area.setAdjustment(adjustment);
     }
-    
+
     /** {@inheritDoc} */
-    public LinkedList getNextKnuthElements(LayoutContext context, int alignment) {
+    public List getNextKnuthElements(LayoutContext context, int alignment) {
         curArea = get(context);
-        
+
         if (curArea == null) {
             setFinished(true);
             return null;
         }
 
         alignmentContext = makeAlignmentContext(context);
-        
+
         MinOptMax ipd = getAllocationIPD(context.getRefIPD());
 
         // create the AreaInfo object to store the computed values
@@ -279,17 +268,14 @@ public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
         KnuthSequence seq = new InlineKnuthSequence();
 
         addKnuthElementsForBorderPaddingStart(seq);
-        
-        seq.add(new KnuthInlineBox(areaInfo.ipdArea.opt, alignmentContext,
+
+        seq.add(new KnuthInlineBox(areaInfo.ipdArea.getOpt(), alignmentContext,
                                     notifyPos(new LeafPosition(this, 0)), false));
 
         addKnuthElementsForBorderPaddingEnd(seq);
-        
-        LinkedList returnList = new LinkedList();
 
-        returnList.add(seq);
         setFinished(true);
-        return returnList;
+        return Collections.singletonList(seq);
     }
 
     /** {@inheritDoc} */
@@ -309,11 +295,12 @@ public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
     }
 
     /** {@inheritDoc} */
-    public void getWordChars(StringBuffer sbChars, Position pos) {
+    public String getWordChars(Position pos) {
+        return "";
     }
 
     /** {@inheritDoc} */
-    public void hyphenate(Position pos, HyphContext hc) {
+    public void hyphenate(Position pos, HyphContext hyphContext) {
     }
 
     /** {@inheritDoc} */
@@ -323,7 +310,7 @@ public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
     }
 
     /** {@inheritDoc} */
-    public LinkedList getChangedKnuthElements(List oldList,
+    public List getChangedKnuthElements(List oldList,
                                               int alignment) {
         if (isFinished()) {
             return null;
@@ -332,18 +319,18 @@ public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
         LinkedList returnList = new LinkedList();
 
         addKnuthElementsForBorderPaddingStart(returnList);
-        
+
         // fobj is a fo:ExternalGraphic, fo:InstreamForeignObject,
         // fo:PageNumber or fo:PageNumberCitation
-        returnList.add(new KnuthInlineBox(areaInfo.ipdArea.opt, areaInfo.alignmentContext, 
+        returnList.add(new KnuthInlineBox(areaInfo.ipdArea.getOpt(), areaInfo.alignmentContext,
                                           notifyPos(new LeafPosition(this, 0)), true));
 
         addKnuthElementsForBorderPaddingEnd(returnList);
-        
+
         setFinished(true);
         return returnList;
     }
-    
+
     /**
      * Creates Knuth elements for start border padding and adds them to the return list.
      * @param returnList return list to add the additional elements to
@@ -355,7 +342,7 @@ public abstract class LeafNodeLayoutManager extends AbstractLayoutManager
                          + commonBorderPaddingBackground.getPaddingStart(false, this);
             if (ipStart > 0) {
                 // Add a non breakable glue
-                returnList.add(new KnuthPenalty(0, KnuthPenalty.INFINITE, 
+                returnList.add(new KnuthPenalty(0, KnuthPenalty.INFINITE,
                                                 false, new LeafPosition(this, -1), true));
                 returnList.add(new KnuthGlue(ipStart, 0, 0, new LeafPosition(this, -1), true));
             }
