@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PageNumberCitationLastLayoutManager.java 596097 2007-11-18 16:56:09Z jeremias $ */
+/* $Id: PageNumberCitationLastLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
@@ -43,13 +43,13 @@ public class PageNumberCitationLastLayoutManager extends AbstractPageNumberCitat
         super(node);
         fobj = node;
     }
-    
+
     /** {@inheritDoc} */
     public InlineArea get(LayoutContext context) {
-        curArea = getPageNumberCitationLastInlineArea(parentLM);
+        curArea = getPageNumberCitationLastInlineArea(parentLayoutManager);
         return curArea;
     }
-    
+
     /**
      * if id can be resolved then simply return a word, otherwise
      * return a resolvable area
@@ -71,12 +71,12 @@ public class PageNumberCitationLastLayoutManager extends AbstractPageNumberCitat
             int width = getStringWidth(str);
             text.addWord(str, 0);
             text.setIPD(width);
-            
+
             resolved = true;
         }
-        
+
         updateTextAreaTraits(text);
-        
+
         return text;
     }
 }
