@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,17 +15,17 @@
  * limitations under the License.
  */
 
-/* $Id: PageNumberLayoutManager.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: PageNumberLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
 import org.apache.fop.fo.flow.PageNumber;
-import org.apache.fop.area.inline.InlineArea;
-import org.apache.fop.area.inline.TextArea;
-import org.apache.fop.area.Trait;
 import org.apache.fop.fonts.Font;
 import org.apache.fop.fonts.FontInfo;
 import org.apache.fop.fonts.FontTriplet;
+import org.apache.fop.area.inline.InlineArea;
+import org.apache.fop.area.inline.TextArea;
+import org.apache.fop.area.Trait;
 import org.apache.fop.layoutmgr.LayoutContext;
 import org.apache.fop.layoutmgr.TraitSetter;
 import org.apache.fop.traits.MinOptMax;
@@ -34,21 +34,21 @@ import org.apache.fop.traits.MinOptMax;
  * LayoutManager for the fo:page-number formatting object
  */
 public class PageNumberLayoutManager extends LeafNodeLayoutManager {
-    
+
     private PageNumber fobj;
     private Font font;
-    
+
     /**
      * Constructor
      *
      * @param node the fo:page-number formatting object that creates the area
-     * @todo better null checking of node, font
+     * TODO better null checking of node, font
      */
     public PageNumberLayoutManager(PageNumber node) {
         super(node);
         fobj = node;
     }
-    
+
     /** {@inheritDoc} */
     public void initialize() {
         FontInfo fi = fobj.getFOEventHandler().getFontInfo();
@@ -84,13 +84,13 @@ public class PageNumberLayoutManager extends LeafNodeLayoutManager {
         text.setBPD(font.getAscender() - font.getDescender());
         text.setBaselineOffset(font.getAscender());
         TraitSetter.addFontTraits(text, font);
-        text.addTrait(Trait.COLOR, fobj.getColor());        
-
+        text.addTrait(Trait.COLOR, fobj.getColor());
+        TraitSetter.addPtr(text, fobj.getPtr()); // used for accessibility
         TraitSetter.addTextDecoration(text, fobj.getTextDecoration());
 
         return text;
     }
-    
+
     /** {@inheritDoc} */
     protected InlineArea getEffectiveArea() {
         TextArea baseArea = (TextArea)curArea;
@@ -108,7 +108,7 @@ public class PageNumberLayoutManager extends LeafNodeLayoutManager {
         updateContent(ta);
         return ta;
     }
-    
+
     private void updateContent(TextArea area) {
         // get the page number of the page actually being built
         area.removeText();
@@ -116,7 +116,7 @@ public class PageNumberLayoutManager extends LeafNodeLayoutManager {
         // update the ipd of the area
         area.handleIPDVariation(getStringWidth(area.getText()) - area.getIPD());
         // update the width stored in the AreaInfo object
-        areaInfo.ipdArea = new MinOptMax(area.getIPD());
+        areaInfo.ipdArea = MinOptMax.getInstance(area.getIPD());
     }
 
     /**
@@ -130,10 +130,6 @@ public class PageNumberLayoutManager extends LeafNodeLayoutManager {
         }
         return width;
     }
-    
-    /** {@inheritDoc} */
-    protected void addId() {
-        getPSLM().addIDToPage(fobj.getId());
-    }
+
 }
 
