@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,12 +15,15 @@
  * limitations under the License.
  */
 
-/* $Id: WrapperLayoutManager.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: WrapperLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
 import org.apache.fop.area.inline.InlineArea;
+import org.apache.fop.area.Block;
 import org.apache.fop.fo.flow.Wrapper;
+import org.apache.fop.layoutmgr.BlockLayoutManager;
+import org.apache.fop.layoutmgr.BlockStackingLayoutManager;
 import org.apache.fop.layoutmgr.LayoutContext;
 import org.apache.fop.layoutmgr.PositionIterator;
 import org.apache.fop.layoutmgr.TraitSetter;
@@ -29,8 +32,6 @@ import org.apache.fop.layoutmgr.TraitSetter;
  * This is the layout manager for the fo:wrapper formatting object.
  */
 public class WrapperLayoutManager extends LeafNodeLayoutManager {
-    
-    private Wrapper fobj;
 
     /**
      * Creates a new LM for fo:wrapper.
@@ -38,7 +39,6 @@ public class WrapperLayoutManager extends LeafNodeLayoutManager {
      */
     public WrapperLayoutManager(Wrapper node) {
         super(node);
-        fobj = node;
     }
 
     /** {@inheritDoc} */
@@ -65,8 +65,15 @@ public class WrapperLayoutManager extends LeafNodeLayoutManager {
     public void addAreas(PositionIterator posIter, LayoutContext context) {
         if (fobj.hasId()) {
             addId();
-            InlineArea area = getEffectiveArea();
-            parentLM.addChildArea(area);
+            if (parentLayoutManager instanceof BlockStackingLayoutManager
+                    && !(parentLayoutManager instanceof BlockLayoutManager)) {
+                Block helperBlock = new Block();
+                TraitSetter.setProducerID(helperBlock, fobj.getId());
+                parentLayoutManager.addChildArea(helperBlock);
+            } else {
+                InlineArea area = getEffectiveArea();
+                parentLayoutManager.addChildArea(area);
+            }
         }
         while (posIter.hasNext()) {
             posIter.next();
