@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: ListItemContentLayoutManager.java 635508 2008-03-10 10:06:37Z jeremias $ */
- 
+/* $Id: ListItemContentLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
+
 package org.apache.fop.layoutmgr.list;
 
 import java.util.Iterator;
@@ -28,8 +28,9 @@ import org.apache.fop.area.Block;
 import org.apache.fop.fo.flow.AbstractListItemPart;
 import org.apache.fop.fo.flow.ListItemBody;
 import org.apache.fop.fo.flow.ListItemLabel;
-import org.apache.fop.layoutmgr.BlockLevelLayoutManager;
+import org.apache.fop.fo.properties.KeepProperty;
 import org.apache.fop.layoutmgr.BlockStackingLayoutManager;
+import org.apache.fop.layoutmgr.Keep;
 import org.apache.fop.layoutmgr.LayoutContext;
 import org.apache.fop.layoutmgr.LayoutManager;
 import org.apache.fop.layoutmgr.NonLeafPosition;
@@ -85,7 +86,7 @@ public class ListItemContentLayoutManager extends BlockStackingLayoutManager {
     protected AbstractListItemPart getPartFO() {
         return (AbstractListItemPart)fobj;
     }
-    
+
     /**
      * Set the x offset of this list item.
      * This offset is used to set the absolute position
@@ -98,7 +99,7 @@ public class ListItemContentLayoutManager extends BlockStackingLayoutManager {
     }
 
     /** {@inheritDoc} */
-    public LinkedList getChangedKnuthElements(List oldList, int alignment) {
+    public List getChangedKnuthElements(List oldList, int alignment) {
         //log.debug("  ListItemContentLayoutManager.getChanged>");
         return super.getChangedKnuthElements(oldList, alignment);
     }
@@ -114,10 +115,10 @@ public class ListItemContentLayoutManager extends BlockStackingLayoutManager {
     public void addAreas(PositionIterator parentIter,
                          LayoutContext layoutContext) {
         getParentArea(null);
-        
-        getPSLM().addIDToPage(getPartFO().getId());
 
-        LayoutManager childLM = null;
+        addId();
+
+        LayoutManager childLM;
         LayoutContext lc = new LayoutContext(0);
         LayoutManager firstLM = null;
         LayoutManager lastLM = null;
@@ -125,7 +126,7 @@ public class ListItemContentLayoutManager extends BlockStackingLayoutManager {
         Position lastPos = null;
 
         // "unwrap" the NonLeafPositions stored in parentIter
-        // and put them in a new list; 
+        // and put them in a new list;
         LinkedList positionList = new LinkedList();
         Position pos;
         while (parentIter.hasNext()) {
@@ -141,8 +142,8 @@ public class ListItemContentLayoutManager extends BlockStackingLayoutManager {
             }
             if (pos instanceof NonLeafPosition) {
                 // pos was created by a child of this ListBlockLM
-                positionList.add(((NonLeafPosition) pos).getPosition());
-                lastLM = ((NonLeafPosition) pos).getPosition().getLM();
+                positionList.add(pos.getPosition());
+                lastLM = pos.getPosition().getLM();
                 if (firstLM == null) {
                     firstLM = lastLM;
                 }
@@ -154,7 +155,7 @@ public class ListItemContentLayoutManager extends BlockStackingLayoutManager {
         }
 
         addMarkersToPage(true, isFirst(firstPos), isLast(lastPos));
-        
+
         StackingIter childPosIter = new StackingIter(positionList.listIterator());
         while ((childLM = childPosIter.getNextChildLM()) != null) {
             // Add the block areas to Area
@@ -167,12 +168,12 @@ public class ListItemContentLayoutManager extends BlockStackingLayoutManager {
         }
 
         addMarkersToPage(false, isFirst(firstPos), isLast(lastPos));
-        
+
         flush();
 
         curBlockArea = null;
-        
-        getPSLM().notifyEndOfLayout(((AbstractListItemPart)getFObj()).getId());
+
+        checkEndOfLayout(lastPos);
     }
 
     /**
@@ -198,9 +199,9 @@ public class ListItemContentLayoutManager extends BlockStackingLayoutManager {
             //curBlockArea.setHeight();
 
             TraitSetter.setProducerID(curBlockArea, getPartFO().getId());
-            
+
             // Set up dimensions
-            Area parentArea = parentLM.getParentArea(curBlockArea);
+            Area parentArea = parentLayoutManager.getParentArea(curBlockArea);
             int referenceIPD = parentArea.getIPD();
             curBlockArea.setIPD(referenceIPD);
             // Get reference IPD from parentArea
@@ -221,11 +222,18 @@ public class ListItemContentLayoutManager extends BlockStackingLayoutManager {
     }
 
     /** {@inheritDoc} */
-    public boolean mustKeepTogether() {
-        //TODO Keeps will have to be more sophisticated sooner or later
-        return ((BlockLevelLayoutManager)getParent()).mustKeepTogether() 
-                || !getPartFO().getKeepTogether().getWithinPage().isAuto()
-                || !getPartFO().getKeepTogether().getWithinColumn().isAuto();
+    public KeepProperty getKeepTogetherProperty() {
+        return getPartFO().getKeepTogether();
+    }
+
+    /** {@inheritDoc} */
+    public Keep getKeepWithNext() {
+        return Keep.KEEP_AUTO;
+    }
+
+    /** {@inheritDoc} */
+    public Keep getKeepWithPrevious() {
+        return Keep.KEEP_AUTO;
     }
 
 }
