@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ListItemLayoutManager.java 669118 2008-06-18 09:02:45Z jeremias $ */
+/* $Id: ListItemLayoutManager.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.list;
 
@@ -32,18 +32,21 @@ import org.apache.fop.area.Block;
 import org.apache.fop.fo.flow.ListItem;
 import org.apache.fop.fo.flow.ListItemBody;
 import org.apache.fop.fo.flow.ListItemLabel;
-import org.apache.fop.layoutmgr.BlockLevelLayoutManager;
+import org.apache.fop.fo.properties.KeepProperty;
 import org.apache.fop.layoutmgr.BlockStackingLayoutManager;
 import org.apache.fop.layoutmgr.BreakElement;
 import org.apache.fop.layoutmgr.ConditionalElementListener;
 import org.apache.fop.layoutmgr.ElementListObserver;
 import org.apache.fop.layoutmgr.ElementListUtils;
+import org.apache.fop.layoutmgr.Keep;
+import org.apache.fop.layoutmgr.KnuthBlockBox;
 import org.apache.fop.layoutmgr.KnuthBox;
 import org.apache.fop.layoutmgr.KnuthElement;
 import org.apache.fop.layoutmgr.KnuthPenalty;
 import org.apache.fop.layoutmgr.KnuthPossPosIter;
 import org.apache.fop.layoutmgr.LayoutContext;
 import org.apache.fop.layoutmgr.LayoutManager;
+import org.apache.fop.layoutmgr.ListElement;
 import org.apache.fop.layoutmgr.NonLeafPosition;
 import org.apache.fop.layoutmgr.Position;
 import org.apache.fop.layoutmgr.PositionIterator;
@@ -57,7 +60,7 @@ import org.apache.fop.traits.SpaceVal;
  * LayoutManager for a list-item FO.
  * The list item contains a list item label and a list item body.
  */
-public class ListItemLayoutManager extends BlockStackingLayoutManager 
+public class ListItemLayoutManager extends BlockStackingLayoutManager
                     implements ConditionalElementListener {
 
     /**
@@ -70,28 +73,28 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
 
     private Block curBlockArea = null;
 
-    private LinkedList labelList = null;
-    private LinkedList bodyList = null;
+    private List labelList = null;
+    private List bodyList = null;
 
-    private int listItemHeight;
-    
     private boolean discardBorderBefore;
     private boolean discardBorderAfter;
     private boolean discardPaddingBefore;
     private boolean discardPaddingAfter;
     private MinOptMax effSpaceBefore;
     private MinOptMax effSpaceAfter;
-    
-    private boolean keepWithNextPendingOnLabel;
-    private boolean keepWithNextPendingOnBody;
-  
+
+    private Keep keepWithNextPendingOnLabel;
+    private Keep keepWithNextPendingOnBody;
+
+    private int listItemHeight;
+
     private class ListItemPosition extends Position {
         private int iLabelFirstIndex;
         private int iLabelLastIndex;
         private int iBodyFirstIndex;
         private int iBodyLastIndex;
 
-        public ListItemPosition(LayoutManager lm, int labelFirst, int labelLast, 
+        public ListItemPosition(LayoutManager lm, int labelFirst, int labelLast,
                 int bodyFirst, int bodyLast) {
             super(lm);
             iLabelFirstIndex = labelFirst;
@@ -99,11 +102,11 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
             iBodyFirstIndex = bodyFirst;
             iBodyLastIndex = bodyLast;
         }
-        
+
         public int getLabelFirstIndex() {
             return iLabelFirstIndex;
         }
-        
+
         public int getLabelLastIndex() {
             return iLabelLastIndex;
         }
@@ -111,11 +114,11 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
         public int getBodyFirstIndex() {
             return iBodyFirstIndex;
         }
-        
+
         public int getBodyLastIndex() {
             return iBodyLastIndex;
         }
-        
+
         /** {@inheritDoc} */
         public boolean generatesAreas() {
             return true;
@@ -164,7 +167,7 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
      * @param node the fo:list-item-body FO
      */
     public void setBody(ListItemBody node) {
-        body = new ListItemContentLayoutManager(node); 
+        body = new ListItemContentLayoutManager(node);
         body.setParent(this);
     }
 
@@ -175,25 +178,25 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
         foSpaceAfter = new SpaceVal(
                 getListItemFO().getCommonMarginBlock().spaceAfter, this).getSpace();
         startIndent = getListItemFO().getCommonMarginBlock().startIndent.getValue(this);
-        endIndent = getListItemFO().getCommonMarginBlock().endIndent.getValue(this); 
+        endIndent = getListItemFO().getCommonMarginBlock().endIndent.getValue(this);
     }
 
     private void resetSpaces() {
-        this.discardBorderBefore = false;        
-        this.discardBorderAfter = false;        
-        this.discardPaddingBefore = false;        
+        this.discardBorderBefore = false;
+        this.discardBorderAfter = false;
+        this.discardPaddingBefore = false;
         this.discardPaddingAfter = false;
         this.effSpaceBefore = null;
         this.effSpaceAfter = null;
     }
-    
+
     /** {@inheritDoc} */
-    public LinkedList getNextKnuthElements(LayoutContext context, int alignment) {
+    public List getNextKnuthElements(LayoutContext context, int alignment) {
         referenceIPD = context.getRefIPD();
         LayoutContext childLC;
-        
-        LinkedList returnList = new LinkedList();
-        
+
+        List returnList = new LinkedList();
+
         if (!breakBeforeServed) {
             breakBeforeServed = true;
             if (!context.suppressBreakBefore()) {
@@ -204,7 +207,7 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
         }
 
         addKnuthElementsForSpaceBefore(returnList, alignment);
-        
+
         addKnuthElementsForBorderPaddingBefore(returnList, !firstVisibleMarkServed);
         firstVisibleMarkServed = true;
 
@@ -216,16 +219,14 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
         childLC.setRefIPD(context.getRefIPD());
         label.initialize();
         labelList = label.getNextKnuthElements(childLC, alignment);
-        
+
         //Space resolution as if the contents were placed in a new reference area
         //(see 6.8.3, XSL 1.0, section on Constraints, last paragraph)
         SpaceResolver.resolveElementList(labelList);
         ElementListObserver.observe(labelList, "list-item-label", label.getPartFO().getId());
-        
-        if (childLC.isKeepWithPreviousPending()) {
-            context.setFlags(LayoutContext.KEEP_WITH_PREVIOUS_PENDING);
-        }
-        this.keepWithNextPendingOnLabel = childLC.isKeepWithNextPending();
+
+        context.updateKeepWithPreviousPending(childLC.getKeepWithPreviousPending());
+        this.keepWithNextPendingOnLabel = childLC.getKeepWithNextPending();
 
         // body
         childLC = new LayoutContext(0);
@@ -237,38 +238,33 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
         //(see 6.8.3, XSL 1.0, section on Constraints, last paragraph)
         SpaceResolver.resolveElementList(bodyList);
         ElementListObserver.observe(bodyList, "list-item-body", body.getPartFO().getId());
-        
-        if (childLC.isKeepWithPreviousPending()) {
-            context.setFlags(LayoutContext.KEEP_WITH_PREVIOUS_PENDING);
-        }
-        this.keepWithNextPendingOnBody = childLC.isKeepWithNextPending();
+
+        context.updateKeepWithPreviousPending(childLC.getKeepWithPreviousPending());
+        this.keepWithNextPendingOnBody = childLC.getKeepWithNextPending();
 
         // create a combined list
-        LinkedList returnedList = getCombinedKnuthElementsForListItem(labelList, bodyList, context);
+        List returnedList = getCombinedKnuthElementsForListItem(labelList, bodyList, context);
 
         // "wrap" the Position inside each element
         wrapPositionElements(returnedList, returnList, true);
-        
+
         addKnuthElementsForBorderPaddingAfter(returnList, true);
         addKnuthElementsForSpaceAfter(returnList, alignment);
         addKnuthElementsForBreakAfter(returnList, context);
 
-        if (keepWithNextPendingOnLabel || keepWithNextPendingOnBody || mustKeepWithNext()) {
-            context.setFlags(LayoutContext.KEEP_WITH_NEXT_PENDING);
-        }
-        if (mustKeepWithPrevious()) {
-            context.setFlags(LayoutContext.KEEP_WITH_PREVIOUS_PENDING);
-        }
+        context.updateKeepWithNextPending(this.keepWithNextPendingOnLabel);
+        context.updateKeepWithNextPending(this.keepWithNextPendingOnBody);
+        context.updateKeepWithNextPending(getKeepWithNext());
+        context.updateKeepWithPreviousPending(getKeepWithPrevious());
 
         setFinished(true);
         resetSpaces();
         return returnList;
     }
 
-    private LinkedList getCombinedKnuthElementsForListItem(LinkedList labelElements,
-                                                           LinkedList bodyElements,
-                                                           LayoutContext context) {
-        //Copy elements to array lists to improve element access performance
+    private List getCombinedKnuthElementsForListItem(List labelElements,
+            List bodyElements, LayoutContext context) {
+        // Copy elements to array lists to improve element access performance
         List[] elementLists = {new ArrayList(labelElements),
                                new ArrayList(bodyElements)};
         int[] fullHeights = {ElementListUtils.calcContentLength(elementLists[0]),
@@ -280,54 +276,76 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
         int totalHeight = Math.max(fullHeights[0], fullHeights[1]);
         int step;
         int addedBoxHeight = 0;
-        boolean keepWithNextActive = false;
+        Keep keepWithNextActive = Keep.KEEP_AUTO;
 
         LinkedList returnList = new LinkedList();
-        while ((step = getNextStep(elementLists, start, end, partialHeights))
-               > 0) {
-            
+        while ((step = getNextStep(elementLists, start, end, partialHeights)) > 0) {
+
             if (end[0] + 1 == elementLists[0].size()) {
-                if (keepWithNextPendingOnLabel) {
-                    keepWithNextActive = true;
-                }
+                keepWithNextActive = keepWithNextActive.compare(keepWithNextPendingOnLabel);
             }
             if (end[1] + 1 == elementLists[1].size()) {
-                if (keepWithNextPendingOnBody) {
-                    keepWithNextActive = true;
-                }
+                keepWithNextActive = keepWithNextActive.compare(keepWithNextPendingOnBody);
             }
-            
+
             // compute penalty height and box height
-            int penaltyHeight = step 
-                + getMaxRemainingHeight(fullHeights, partialHeights) 
+            int penaltyHeight = step
+                + getMaxRemainingHeight(fullHeights, partialHeights)
                 - totalHeight;
-            
+
             //Additional penalty height from penalties in the source lists
             int additionalPenaltyHeight = 0;
+            int stepPenalty = 0;
             KnuthElement endEl = (KnuthElement)elementLists[0].get(end[0]);
             if (endEl instanceof KnuthPenalty) {
-                additionalPenaltyHeight = ((KnuthPenalty)endEl).getW();
+                additionalPenaltyHeight = endEl.getWidth();
+                stepPenalty = Math.max(stepPenalty, endEl.getPenalty());
             }
             endEl = (KnuthElement)elementLists[1].get(end[1]);
             if (endEl instanceof KnuthPenalty) {
                 additionalPenaltyHeight = Math.max(
-                        additionalPenaltyHeight, ((KnuthPenalty)endEl).getW());
+                        additionalPenaltyHeight, endEl.getWidth());
+                stepPenalty = Math.max(stepPenalty, endEl.getPenalty());
             }
-            
+
             int boxHeight = step - addedBoxHeight - penaltyHeight;
             penaltyHeight += additionalPenaltyHeight; //Add AFTER calculating boxHeight!
 
+            // collect footnote information
+            // TODO this should really not be done like this. ListItemLM should remain as
+            // footnote-agnostic as possible
+            LinkedList footnoteList = null;
+            ListElement el;
+            for (int i = 0; i < elementLists.length; i++) {
+                for (int j = start[i]; j <= end[i]; j++) {
+                    el = (ListElement) elementLists[i].get(j);
+                    if (el instanceof KnuthBlockBox && ((KnuthBlockBox) el).hasAnchors()) {
+                        if (footnoteList == null) {
+                            footnoteList = new LinkedList();
+                        }
+                        footnoteList.addAll(((KnuthBlockBox) el).getFootnoteBodyLMs());
+                    }
+                }
+            }
+
             // add the new elements
             addedBoxHeight += boxHeight;
-            ListItemPosition stepPosition = new ListItemPosition(this, 
+            ListItemPosition stepPosition = new ListItemPosition(this,
                     start[0], end[0], start[1], end[1]);
-            returnList.add(new KnuthBox(boxHeight, stepPosition, false));
+            if (footnoteList == null) {
+                returnList.add(new KnuthBox(boxHeight, stepPosition, false));
+            } else {
+                returnList.add(new KnuthBlockBox(boxHeight, footnoteList, stepPosition, false));
+            }
+
             if (addedBoxHeight < totalHeight) {
-                int p = 0;
-                if (keepWithNextActive || mustKeepTogether()) {
-                    p = KnuthPenalty.INFINITE;
+                Keep keep = keepWithNextActive.compare(getKeepTogether());
+                int p = stepPenalty;
+                if (p > -KnuthElement.INFINITE) {
+                    p = Math.max(p, keep.getPenalty());
                 }
-                returnList.add(new BreakElement(stepPosition, penaltyHeight, p, -1, context));
+                returnList.add(new BreakElement(stepPosition, penaltyHeight, p, keep.getContext(),
+                        context));
             }
         }
 
@@ -349,7 +367,7 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
                 end[i]++;
                 KnuthElement el = (KnuthElement)elementLists[i].get(end[i]);
                 if (el.isPenalty()) {
-                    if (el.getP() < KnuthElement.INFINITE) {
+                    if (el.getPenalty() < KnuthElement.INFINITE) {
                         //First legal break point
                         break;
                     }
@@ -361,9 +379,9 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
                             break;
                         }
                     }
-                    partialHeights[i] += el.getW();
+                    partialHeights[i] += el.getWidth();
                 } else {
-                    partialHeights[i] += el.getW();
+                    partialHeights[i] += el.getWidth();
                 }
             }
             if (end[i] < start[i]) {
@@ -375,7 +393,7 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
         if (seqCount == 0) {
             return 0;
         }
-        
+
         // determine next step
         int step;
         if (backupHeights[0] == 0 && backupHeights[1] == 0) {
@@ -407,9 +425,9 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
     }
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
-    public LinkedList getChangedKnuthElements(List oldList, int alignment) {
+    public List getChangedKnuthElements(List oldList, int alignment) {
         //log.debug(" LILM.getChanged> label");
         // label
         labelList = label.getChangedKnuthElements(labelList, alignment);
@@ -418,14 +436,14 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
         // body
         // "unwrap" the Positions stored in the elements
         ListIterator oldListIterator = oldList.listIterator();
-        KnuthElement oldElement = null;
+        KnuthElement oldElement;
         while (oldListIterator.hasNext()) {
             oldElement = (KnuthElement)oldListIterator.next();
-            Position innerPosition = ((NonLeafPosition) oldElement.getPosition()).getPosition();
-            //log.debug(" BLM> unwrapping: " + (oldElement.isBox() 
-            //  ? "box    " : (oldElement.isGlue() ? "glue   " : "penalty")) 
+            Position innerPosition = oldElement.getPosition().getPosition();
+            //log.debug(" BLM> unwrapping: " + (oldElement.isBox()
+            //  ? "box    " : (oldElement.isGlue() ? "glue   " : "penalty"))
             //  + " creato da " + oldElement.getLayoutManager().getClass().getName());
-            //log.debug(" BLM> unwrapping:         " 
+            //log.debug(" BLM> unwrapping:         "
             //  + oldElement.getPosition().getClass().getName());
             if (innerPosition != null) {
                 // oldElement was created by a descendant of this BlockLM
@@ -438,9 +456,9 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
             }
         }
 
-        LinkedList returnedList = body.getChangedKnuthElements(oldList, alignment);
+        List returnedList = body.getChangedKnuthElements(oldList, alignment);
         // "wrap" the Position inside each element
-        LinkedList tempList = returnedList;
+        List tempList = returnedList;
         KnuthElement tempElement;
         returnedList = new LinkedList();
         ListIterator listIter = tempList.listIterator();
@@ -464,7 +482,7 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
                          LayoutContext layoutContext) {
         getParentArea(null);
 
-        getPSLM().addIDToPage(getListItemFO().getId());
+        addId();
 
         LayoutContext lc = new LayoutContext(0);
         Position firstPos = null;
@@ -483,13 +501,13 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
             }
             if (pos instanceof NonLeafPosition && pos.getPosition() != null) {
                 // pos contains a ListItemPosition created by this ListBlockLM
-                positionList.add(((NonLeafPosition) pos).getPosition());
+                positionList.add(pos.getPosition());
             }
         }
 
         addMarkersToPage(true, isFirst(firstPos), isLast(lastPos));
 
-        // use the first and the last ListItemPosition to determine the 
+        // use the first and the last ListItemPosition to determine the
         // corresponding indexes in the original labelList and bodyList
         int labelFirstIndex = ((ListItemPosition) positionList.getFirst()).getLabelFirstIndex();
         int labelLastIndex = ((ListItemPosition) positionList.getLast()).getLabelLastIndex();
@@ -498,17 +516,17 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
 
         //Determine previous break if any
         int previousBreak = ElementListUtils.determinePreviousBreak(labelList, labelFirstIndex);
-        SpaceResolver.performConditionalsNotification(labelList, 
+        SpaceResolver.performConditionalsNotification(labelList,
                 labelFirstIndex, labelLastIndex, previousBreak);
 
         //Determine previous break if any
         previousBreak = ElementListUtils.determinePreviousBreak(bodyList, bodyFirstIndex);
-        SpaceResolver.performConditionalsNotification(bodyList, 
+        SpaceResolver.performConditionalsNotification(bodyList,
                 bodyFirstIndex, bodyLastIndex, previousBreak);
-        
+
         // add label areas
         if (labelFirstIndex <= labelLastIndex) {
-            KnuthPossPosIter labelIter = new KnuthPossPosIter(labelList, 
+            KnuthPossPosIter labelIter = new KnuthPossPosIter(labelList,
                     labelFirstIndex, labelLastIndex + 1);
             lc.setFlags(LayoutContext.FIRST_AREA, layoutContext.isFirstArea());
             lc.setFlags(LayoutContext.LAST_AREA, layoutContext.isLastArea());
@@ -519,17 +537,9 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
             label.addAreas(labelIter, lc);
         }
 
-        // reset the area bpd after adding the label areas and before adding the body areas
-        int savedBPD = 0;
-        if (labelFirstIndex <= labelLastIndex
-            && bodyFirstIndex <= bodyLastIndex) {
-            savedBPD = curBlockArea.getBPD();
-            curBlockArea.setBPD(0);
-        }
-
         // add body areas
         if (bodyFirstIndex <= bodyLastIndex) {
-            KnuthPossPosIter bodyIter = new KnuthPossPosIter(bodyList, 
+            KnuthPossPosIter bodyIter = new KnuthPossPosIter(bodyList,
                     bodyFirstIndex, bodyLastIndex + 1);
             lc.setFlags(LayoutContext.FIRST_AREA, layoutContext.isFirstArea());
             lc.setFlags(LayoutContext.LAST_AREA, layoutContext.isLastArea());
@@ -541,25 +551,29 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
         }
 
         // after adding body areas, set the maximum area bpd
-        if (curBlockArea.getBPD() < savedBPD) {
-            curBlockArea.setBPD(savedBPD);
+        int childCount = curBlockArea.getChildAreas().size();
+        assert childCount >= 1 && childCount <= 2;
+        int itemBPD = ((Block)curBlockArea.getChildAreas().get(0)).getAllocBPD();
+        if (childCount == 2) {
+            itemBPD = Math.max(itemBPD, ((Block)curBlockArea.getChildAreas().get(1)).getAllocBPD());
         }
+        curBlockArea.setBPD(itemBPD);
 
         addMarkersToPage(false, isFirst(firstPos), isLast(lastPos));
 
         // We are done with this area add the background
-        TraitSetter.addBackground(curBlockArea, 
+        TraitSetter.addBackground(curBlockArea,
                 getListItemFO().getCommonBorderPaddingBackground(),
                 this);
-        TraitSetter.addSpaceBeforeAfter(curBlockArea, layoutContext.getSpaceAdjust(), 
+        TraitSetter.addSpaceBeforeAfter(curBlockArea, layoutContext.getSpaceAdjust(),
                 effSpaceBefore, effSpaceAfter);
 
         flush();
 
         curBlockArea = null;
         resetSpaces();
-        
-        getPSLM().notifyEndOfLayout(((ListItem)getFObj()).getId());
+
+        checkEndOfLayout(lastPos);
     }
 
     /**
@@ -590,23 +604,23 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
             curBlockArea = new Block();
 
             // Set up dimensions
-            /*Area parentArea =*/ parentLM.getParentArea(curBlockArea);
-            
+            /*Area parentArea =*/ parentLayoutManager.getParentArea(curBlockArea);
+
             // set traits
             TraitSetter.setProducerID(curBlockArea, getListItemFO().getId());
-            TraitSetter.addBorders(curBlockArea, 
-                    getListItemFO().getCommonBorderPaddingBackground(), 
+            TraitSetter.addBorders(curBlockArea,
+                    getListItemFO().getCommonBorderPaddingBackground(),
                     discardBorderBefore, discardBorderAfter, false, false, this);
-            TraitSetter.addPadding(curBlockArea, 
-                    getListItemFO().getCommonBorderPaddingBackground(), 
+            TraitSetter.addPadding(curBlockArea,
+                    getListItemFO().getCommonBorderPaddingBackground(),
                     discardPaddingBefore, discardPaddingAfter, false, false, this);
             TraitSetter.addMargins(curBlockArea,
-                    getListItemFO().getCommonBorderPaddingBackground(), 
+                    getListItemFO().getCommonBorderPaddingBackground(),
                     getListItemFO().getCommonMarginBlock(), this);
-            TraitSetter.addBreaks(curBlockArea, 
-                    getListItemFO().getBreakBefore(), 
+            TraitSetter.addBreaks(curBlockArea,
+                    getListItemFO().getBreakBefore(),
                     getListItemFO().getBreakAfter());
-            
+
             int contentIPD = referenceIPD - getIPIndents();
             curBlockArea.setIPD(contentIPD);
 
@@ -629,36 +643,31 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
     }
 
     /** {@inheritDoc} */
-    public boolean mustKeepTogether() {
-        //TODO Keeps will have to be more sophisticated sooner or later
-        return ((BlockLevelLayoutManager)getParent()).mustKeepTogether() 
-                || !getListItemFO().getKeepTogether().getWithinPage().isAuto()
-                || !getListItemFO().getKeepTogether().getWithinColumn().isAuto();
+    public KeepProperty getKeepTogetherProperty() {
+        return getListItemFO().getKeepTogether();
     }
 
     /** {@inheritDoc} */
-    public boolean mustKeepWithPrevious() {
-        return !getListItemFO().getKeepWithPrevious().getWithinPage().isAuto()
-            || !getListItemFO().getKeepWithPrevious().getWithinColumn().isAuto();
+    public KeepProperty getKeepWithPreviousProperty() {
+        return getListItemFO().getKeepWithPrevious();
     }
 
     /** {@inheritDoc} */
-    public boolean mustKeepWithNext() {
-        return !getListItemFO().getKeepWithNext().getWithinPage().isAuto()
-                || !getListItemFO().getKeepWithNext().getWithinColumn().isAuto();
+    public KeepProperty getKeepWithNextProperty() {
+        return getListItemFO().getKeepWithNext();
     }
 
     /** {@inheritDoc} */
     public void notifySpace(RelSide side, MinOptMax effectiveLength) {
         if (RelSide.BEFORE == side) {
             if (log.isDebugEnabled()) {
-                log.debug(this + ": Space " + side + ", " 
+                log.debug(this + ": Space " + side + ", "
                         + this.effSpaceBefore + "-> " + effectiveLength);
             }
             this.effSpaceBefore = effectiveLength;
         } else {
             if (log.isDebugEnabled()) {
-                log.debug(this + ": Space " + side + ", " 
+                log.debug(this + ": Space " + side + ", "
                         + this.effSpaceAfter + "-> " + effectiveLength);
             }
             this.effSpaceAfter = effectiveLength;
@@ -693,6 +702,13 @@ public class ListItemLayoutManager extends BlockStackingLayoutManager
         }
     }
 
-    
+    /** {@inheritDoc} */
+    public void reset() {
+        super.reset();
+        label.reset();
+        body.reset();
+    }
+
+
 }
 
