@@ -15,25 +15,29 @@
  * limitations under the License.
  */
 
-/* $Id: ActiveCell.java 637993 2008-03-17 17:48:44Z vhennebert $ */
+/* $Id: ActiveCell.java 893238 2009-12-22 17:20:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.table;
 
+import java.util.ArrayList;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.ListIterator;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.flow.table.ConditionalBorder;
 import org.apache.fop.fo.flow.table.EffRow;
 import org.apache.fop.fo.flow.table.PrimaryGridUnit;
 import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
 import org.apache.fop.layoutmgr.ElementListUtils;
+import org.apache.fop.layoutmgr.Keep;
+import org.apache.fop.layoutmgr.KnuthBlockBox;
 import org.apache.fop.layoutmgr.KnuthBox;
 import org.apache.fop.layoutmgr.KnuthElement;
 import org.apache.fop.layoutmgr.KnuthPenalty;
-import org.apache.fop.layoutmgr.MinOptMaxUtil;
 import org.apache.fop.traits.MinOptMax;
 
 /**
@@ -70,7 +74,7 @@ class ActiveCell {
     /** True if the next CellPart that will be created will be the last one for this cell. */
     private boolean lastCellPart;
 
-    private boolean keepWithNextSignal;
+    private Keep keepWithNext;
 
     private int spanIndex = 0;
 
@@ -97,6 +101,10 @@ class ActiveCell {
         private int totalLength;
         /** Length of the penalty ending this step, if any. */
         private int penaltyLength;
+        /** Value of the penalty ending this step, 0 if the step does not end on a penalty. */
+        private int penaltyValue;
+        /** List of footnotes for this step. */
+        private List footnoteList;
         /**
          * One of {@link Constants#EN_AUTO}, {@link Constants#EN_COLUMN},
          * {@link Constants#EN_PAGE}, {@link Constants#EN_EVEN_PAGE},
@@ -125,6 +133,13 @@ class ActiveCell {
             this.contentLength = other.contentLength;
             this.totalLength   = other.totalLength;
             this.penaltyLength = other.penaltyLength;
+            this.penaltyValue  = other.penaltyValue;
+            if (other.footnoteList != null) {
+                if (this.footnoteList == null) {
+                    this.footnoteList = new ArrayList();
+                }
+                this.footnoteList.addAll(other.footnoteList);
+            }
             this.condBeforeContentLength = other.condBeforeContentLength;
             this.breakClass    = other.breakClass;
         }
@@ -142,9 +157,9 @@ class ActiveCell {
         private int contentLength;
 
         FillerPenalty(KnuthPenalty p, int length) {
-            super(length, p.getP(), p.isFlagged(), p.getBreakClass(),
+            super(length, p.getPenalty(), p.isPenaltyFlagged(), p.getBreakClass(),
                     p.getPosition(), p.isAuxiliary());
-            contentLength = p.getW();
+            contentLength = p.getWidth();
         }
 
         FillerPenalty(int length) {
@@ -164,7 +179,7 @@ class ActiveCell {
      * Returns the actual length of the content represented by the given element. In the
      * case where this element is used as a filler to match a row's fixed height, the
      * value returned by the getW() method will be higher than the actual content.
-     * 
+     *
      * @param el an element
      * @return the actual content length corresponding to the element
      */
@@ -174,7 +189,7 @@ class ActiveCell {
         } else if (el instanceof FillerBox) {
             return 0;
         } else {
-            return el.getW();
+            return el.getWidth();
         }
     }
 
@@ -195,14 +210,13 @@ class ActiveCell {
         bpAfterNormal = paddingAfterNormal + pgu.getAfterBorderWidth(ConditionalBorder.NORMAL);
         bpAfterTrailing = paddingAfterTrailing + pgu.getAfterBorderWidth(0, ConditionalBorder.REST);
         elementList = pgu.getElements();
-        handleExplicitHeight(
-                MinOptMaxUtil.toMinOptMax(pgu.getCell().getBlockProgressionDimension(), tableLM),
+        handleExplicitHeight(pgu.getCell().getBlockProgressionDimension().toMinOptMax(tableLM),
                 row.getExplicitHeight());
         knuthIter = elementList.listIterator();
         includedLength = -1;  // Avoid troubles with cells having content of zero length
         totalLength = previousRowsLength + ElementListUtils.calcContentLength(elementList);
         endRowIndex = rowIndex + pgu.getCell().getNumberRowsSpanned() - 1;
-        keepWithNextSignal = false;
+        keepWithNext = Keep.KEEP_AUTO;
         remainingLength = totalLength - previousRowsLength;
 
         afterNextStep = new Step(previousRowsLength);
@@ -217,13 +231,13 @@ class ActiveCell {
     /**
      * Modifies the cell's element list by putting filler elements, so that the cell's or
      * row's explicit height is always reached.
-     * 
+     *
      * TODO this will work properly only for the first break. Then the limitation
      * explained on http://wiki.apache.org/xmlgraphics-fop/TableLayout/KnownProblems
      * occurs. The list of elements needs to be re-adjusted after each break.
      */
     private void handleExplicitHeight(MinOptMax cellBPD, MinOptMax rowBPD) {
-        int minBPD = Math.max(cellBPD.min, rowBPD.min);
+        int minBPD = Math.max(cellBPD.getMin(), rowBPD.getMin());
         if (minBPD > 0) {
             ListIterator iter = elementList.listIterator();
             int cumulateLength = 0;
@@ -232,23 +246,23 @@ class ActiveCell {
                 KnuthElement el = (KnuthElement) iter.next();
                 if (el.isBox()) {
                     prevIsBox = true;
-                    cumulateLength += el.getW();
+                    cumulateLength += el.getWidth();
                 } else if (el.isGlue()) {
                     if (prevIsBox) {
                         elementList.add(iter.nextIndex() - 1,
                                 new FillerPenalty(minBPD - cumulateLength));
                     }
                     prevIsBox = false;
-                    cumulateLength += el.getW();
+                    cumulateLength += el.getWidth();
                 } else {
                     prevIsBox = false;
-                    if (cumulateLength + el.getW() < minBPD) {
+                    if (cumulateLength + el.getWidth() < minBPD) {
                         iter.set(new FillerPenalty((KnuthPenalty) el, minBPD - cumulateLength));
                     }
                 }
             }
         }
-        int optBPD = Math.max(minBPD, Math.max(cellBPD.opt, rowBPD.opt));
+        int optBPD = Math.max(minBPD, Math.max(cellBPD.getOpt(), rowBPD.getOpt()));
         if (pgu.getContentLength() < optBPD) {
             elementList.add(new FillerBox(optBPD - pgu.getContentLength()));
         }
@@ -260,7 +274,7 @@ class ActiveCell {
 
     /**
      * Returns true if this cell ends on the given row.
-     * 
+     *
      * @param rowIndex index of a row in the row-group, zero-based
      * @return true if this cell ends on the given row
      */
@@ -271,7 +285,7 @@ class ActiveCell {
     /**
      * Returns the length of this cell's content not yet included in the steps, plus the
      * cell's borders and paddings if applicable.
-     * 
+     *
      * @return the remaining length, zero if the cell is finished
      */
     int getRemainingLength() {
@@ -285,8 +299,12 @@ class ActiveCell {
 
     private void gotoNextLegalBreak() {
         afterNextStep.penaltyLength = 0;
+        afterNextStep.penaltyValue = 0;
         afterNextStep.condBeforeContentLength = 0;
         afterNextStep.breakClass = Constants.EN_AUTO;
+        if (afterNextStep.footnoteList != null) {
+            afterNextStep.footnoteList.clear();
+        }
         boolean breakFound = false;
         boolean prevIsBox = false;
         boolean boxFound = false;
@@ -294,11 +312,16 @@ class ActiveCell {
             KnuthElement el = (KnuthElement) knuthIter.next();
             if (el.isPenalty()) {
                 prevIsBox = false;
-                if (el.getP() < KnuthElement.INFINITE) {
+                if (el.getPenalty() < KnuthElement.INFINITE
+                        || ((KnuthPenalty) el).getBreakClass() == Constants.EN_PAGE) {
+                    // TODO too much is being done in that test, only to handle
+                    // keep.within-column properly.
+
                     // First legal break point
                     breakFound = true;
-                    afterNextStep.penaltyLength = el.getW();
                     KnuthPenalty p = (KnuthPenalty) el;
+                    afterNextStep.penaltyLength = p.getWidth();
+                    afterNextStep.penaltyValue = p.getPenalty();
                     if (p.isForcedBreak()) {
                         afterNextStep.breakClass = p.getBreakClass();
                     }
@@ -308,27 +331,33 @@ class ActiveCell {
                     // Second legal break point
                     breakFound = true;
                 } else {
-                    afterNextStep.contentLength += el.getW();
+                    afterNextStep.contentLength += el.getWidth();
                     if (!boxFound) {
-                        afterNextStep.condBeforeContentLength += el.getW();
+                        afterNextStep.condBeforeContentLength += el.getWidth();
                     }
                 }
                 prevIsBox = false;
             } else {
+                if (el instanceof KnuthBlockBox && ((KnuthBlockBox) el).hasAnchors()) {
+                    if (afterNextStep.footnoteList == null) {
+                        afterNextStep.footnoteList = new LinkedList();
+                    }
+                    afterNextStep.footnoteList.addAll(((KnuthBlockBox) el).getFootnoteBodyLMs());
+                }
                 prevIsBox = true;
                 boxFound = true;
-                afterNextStep.contentLength += el.getW();
+                afterNextStep.contentLength += el.getWidth();
             }
         }
         afterNextStep.end = knuthIter.nextIndex() - 1;
         afterNextStep.totalLength = bpBeforeNormal
                 + afterNextStep.contentLength + afterNextStep.penaltyLength
-                + bpAfterTrailing; 
+                + bpAfterTrailing;
     }
 
     /**
      * Returns the minimal step that is needed for this cell to contribute some content.
-     *  
+     *
      * @return the step for this cell's first legal break
      */
     int getFirstStep() {
@@ -345,7 +374,7 @@ class ActiveCell {
      * infinite penalty, plus the cell's content won't be taken into account since the
      * final step will be smaller than the current one (see {@link #signalNextStep(int)}).
      * This actually means that the content will be swallowed.
-     * 
+     *
      * @return the length of last step
      */
     int getLastStep() {
@@ -359,7 +388,7 @@ class ActiveCell {
 
     /**
      * Increases the next step up to the given limit.
-     * 
+     *
      * @param limit the length up to which the next step is allowed to increase
      * @see #signalRowFirstStep(int)
      * @see #signalRowLastStep(int)
@@ -367,7 +396,9 @@ class ActiveCell {
     private void increaseCurrentStep(int limit) {
         if (nextStep.end < elementList.size() - 1) {
             while (afterNextStep.totalLength <= limit && nextStep.breakClass == Constants.EN_AUTO) {
+                int condBeforeContentLength = nextStep.condBeforeContentLength;
                 nextStep.set(afterNextStep);
+                nextStep.condBeforeContentLength = condBeforeContentLength;
                 if (afterNextStep.end >= elementList.size() - 1) {
                     break;
                 }
@@ -380,7 +411,7 @@ class ActiveCell {
      * Gets the selected first step for the current row. If this cell's first step is
      * smaller, then it may be able to add some more of its content, since there will be
      * no break before the given step anyway.
-     * 
+     *
      * @param firstStep the current row's first step
      */
     void signalRowFirstStep(int firstStep) {
@@ -400,7 +431,7 @@ class ActiveCell {
 
     /**
      * Returns the total length up to the next legal break, not yet included in the steps.
-     * 
+     *
      * @return the total length up to the next legal break (-1 signals no further step)
      */
     int getNextStep() {
@@ -428,7 +459,7 @@ class ActiveCell {
     /**
      * Signals the length of the chosen next step, so that this cell determines whether
      * its own step may be included or not.
-     * 
+     *
      * @param minStep length of the chosen next step
      * @return the break class of the step, if any. One of {@link Constants#EN_AUTO},
      * {@link Constants#EN_COLUMN}, {@link Constants#EN_PAGE},
@@ -469,7 +500,7 @@ class ActiveCell {
     /**
      * Receives indication that the current row is ending, and that (collapse) borders
      * must be updated accordingly.
-     * 
+     *
      * @param rowIndex the index of the ending row
      */
     void endRow(int rowIndex) {
@@ -490,7 +521,7 @@ class ActiveCell {
     /**
      * Returns true if this cell would be finished after the given step. That is, it would
      * be included in the step and the end of its content would be reached.
-     * 
+     *
      * @param step the next step
      * @return true if this cell finishes at the given step
      */
@@ -501,19 +532,19 @@ class ActiveCell {
     /**
      * Creates and returns a CellPart instance for the content of this cell which
      * is included in the next step.
-     * 
+     *
      * @return a CellPart instance
      */
     CellPart createCellPart() {
         if (nextStep.end + 1 == elementList.size()) {
-            keepWithNextSignal = pgu.mustKeepWithNext();
+            keepWithNext = pgu.getKeepWithNext();
             // TODO if keep-with-next is set on the row, must every cell of the row
             // contribute some content from children blocks?
             // see http://mail-archives.apache.org/mod_mbox/xmlgraphics-fop-dev/200802.mbox/
             // %3c47BDA379.4050606@anyware-tech.com%3e
             // Assuming no, but if yes the following code should enable this behaviour
 //            if (pgu.getRow() != null && pgu.getRow().mustKeepWithNext()) {
-//                keepWithNextSignal = true;
+//                keepWithNextSignal = true; //to be converted to integer strengths
 //            }
         }
         int bpBeforeFirst;
@@ -536,11 +567,31 @@ class ActiveCell {
         }
     }
 
-    boolean keepWithNextSignal() {
-        return keepWithNextSignal;
+    /**
+     * Adds the footnotes (if any) that are part of the next step, if this cell
+     * contributes content to the next step.
+     *
+     * @param footnoteList the list to which this cell must add its footnotes
+     */
+    void addFootnotes(List footnoteList) {
+        if (includedInLastStep() && nextStep.footnoteList != null) {
+            footnoteList.addAll(nextStep.footnoteList);
+            nextStep.footnoteList.clear();
+        }
+    }
+
+    Keep getKeepWithNext() {
+        return keepWithNext;
+    }
+
+    int getPenaltyValue() {
+        if (includedInLastStep()) {
+            return nextStep.penaltyValue;
+        } else {
+            return previousStep.penaltyValue;
+        }
     }
 
-    
     /** {@inheritDoc} */
     public String toString() {
         return "Cell " + (pgu.getRowIndex() + 1) + "." + (pgu.getColIndex() + 1);
