@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RowPainter.java 653537 2008-05-05 17:47:28Z vhennebert $ */
+/* $Id: RowPainter.java 825646 2009-10-15 20:43:13Z acumiskey $ */
 
 package org.apache.fop.layoutmgr.table;
 
@@ -27,13 +27,19 @@ import java.util.ListIterator;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.area.Block;
+import org.apache.fop.area.Trait;
 import org.apache.fop.fo.flow.table.ConditionalBorder;
 import org.apache.fop.fo.flow.table.EffRow;
+import org.apache.fop.fo.flow.table.EmptyGridUnit;
 import org.apache.fop.fo.flow.table.GridUnit;
 import org.apache.fop.fo.flow.table.PrimaryGridUnit;
-import org.apache.fop.fo.flow.table.TableBody;
+import org.apache.fop.fo.flow.table.Table;
+import org.apache.fop.fo.flow.table.TableColumn;
+import org.apache.fop.fo.flow.table.TablePart;
 import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
+import org.apache.fop.fo.properties.CommonBorderPaddingBackground.BorderInfo;
 import org.apache.fop.layoutmgr.ElementListUtils;
 import org.apache.fop.layoutmgr.KnuthElement;
 import org.apache.fop.layoutmgr.KnuthPossPosIter;
@@ -94,7 +100,7 @@ class RowPainter {
         this.firstRowOnPageIndex = -1;
     }
 
-    void startTablePart(TableBody tablePart) {
+    void startTablePart(TablePart tablePart) {
         CommonBorderPaddingBackground background = tablePart.getCommonBorderPaddingBackground();
         if (background.hasBackground()) {
             tablePartBackground = background;
@@ -107,7 +113,7 @@ class RowPainter {
 
     /**
      * Signals that the end of the current table part is reached.
-     * 
+     *
      * @param lastInBody true if the part is the last table-body element to be displayed
      * on the current page. In which case all the cells must be flushed even if they
      * aren't finished, plus the proper collapsed borders must be selected (trailing
@@ -118,7 +124,7 @@ class RowPainter {
      */
     void endTablePart(boolean lastInBody, boolean lastOnPage) {
         addAreasAndFlushRow(lastInBody, lastOnPage);
-    
+
         if (tablePartBackground != null) {
             TableLayoutManager tableLM = tclm.getTableLM();
             for (Iterator iter = tablePartBackgroundAreas.iterator(); iter.hasNext();) {
@@ -139,7 +145,7 @@ class RowPainter {
     /**
      * Records the fragment of row represented by the given position. If it belongs to
      * another (grid) row than the current one, that latter is painted and flushed first.
-     * 
+     *
      * @param tcpos a position representing the row fragment
      */
     void handleTableContentPosition(TableContentPosition tcpos) {
@@ -173,7 +179,7 @@ class RowPainter {
                 firstCellParts[colIndex] = cellPart;
                 cellHeights[colIndex] = cellPart.getBorderPaddingBefore(firstCellOnPage[colIndex]);
             } else {
-                assert firstCellParts[colIndex].pgu == cellPart.pgu; 
+                assert firstCellParts[colIndex].pgu == cellPart.pgu;
                 cellHeights[colIndex] += cellPart.getConditionalBeforeContentLength();
             }
             cellHeights[colIndex] += cellPart.getLength();
@@ -185,7 +191,7 @@ class RowPainter {
      * Creates the areas corresponding to the last row. That is, an area with background
      * for the row, plus areas for all the cells that finish on the row (not spanning over
      * further rows).
-     * 
+     *
      * @param lastInPart true if the row is the last from its table part to be displayed
      * on the current page. In which case all the cells must be flushed even if they
      * aren't finished, plus the proper collapsed borders must be selected (trailing
@@ -202,10 +208,16 @@ class RowPainter {
         recordRowOffset(currentRow.getIndex(), currentRowOffset);
 
         // Need to compute the actual row height first
+        // and determine border behaviour for empty cells
+        boolean firstCellPart = true;
+        boolean lastCellPart = true;
         int actualRowHeight = 0;
         for (int i = 0; i < colCount; i++) {
-            GridUnit currentGU = currentRow.getGridUnit(i);            
-            if (!currentGU.isEmpty() && currentGU.getColSpanIndex() == 0
+            GridUnit currentGU = currentRow.getGridUnit(i);
+            if (currentGU.isEmpty()) {
+                continue;
+            }
+            if (currentGU.getColSpanIndex() == 0
                     && (lastInPart || currentGU.isLastGridUnitRowSpan())
                     && firstCellParts[i] != null) {
                 // TODO
@@ -225,21 +237,53 @@ class RowPainter {
                 actualRowHeight = Math.max(actualRowHeight, cellOffset + cellHeight
                         - currentRowOffset);
             }
+
+            if (firstCellParts[i] != null && !firstCellParts[i].isFirstPart()) {
+                firstCellPart = false;
+            }
+            if (lastCellParts[i] != null && !lastCellParts[i].isLastPart()) {
+                lastCellPart = false;
+            }
         }
 
         // Then add areas for cells finishing on the current row
         for (int i = 0; i < colCount; i++) {
             GridUnit currentGU = currentRow.getGridUnit(i);
-            if (currentGU.isEmpty()) {
-                // TODO remove once missing cells are properly implemented (i.e., replaced
-                // by an fo:table-cell element containing an empty fo:block)
+            if (currentGU.isEmpty() && !tclm.isSeparateBorderModel()) {
+                int borderBeforeWhich;
+                if (firstCellPart) {
+                    if (firstCellOnPage[i]) {
+                        borderBeforeWhich = ConditionalBorder.LEADING_TRAILING;
+                    } else {
+                        borderBeforeWhich = ConditionalBorder.NORMAL;
+                    }
+                } else {
+                    borderBeforeWhich = ConditionalBorder.REST;
+                }
+                int borderAfterWhich;
+                if (lastCellPart) {
+                    if (lastInPart) {
+                        borderAfterWhich = ConditionalBorder.LEADING_TRAILING;
+                    } else {
+                        borderAfterWhich = ConditionalBorder.NORMAL;
+                    }
+                } else {
+                    borderAfterWhich = ConditionalBorder.REST;
+                }
+                addAreaForEmptyGridUnit((EmptyGridUnit)currentGU,
+                        currentRow.getIndex(), i,
+                        actualRowHeight,
+                        borderBeforeWhich, borderAfterWhich,
+                        lastOnPage);
+
                 firstCellOnPage[i] = false;
             } else if (currentGU.getColSpanIndex() == 0
                     && (lastInPart || currentGU.isLastGridUnitRowSpan())
                     && firstCellParts[i] != null) {
                 assert firstCellParts[i].pgu == currentGU.getPrimary();
+
                 int borderBeforeWhich;
-                if (firstCellParts[i].start == 0) {
+                if (firstCellParts[i].isFirstPart()) {
                     if (firstCellOnPage[i]) {
                         borderBeforeWhich = ConditionalBorder.LEADING_TRAILING;
                     } else {
@@ -259,6 +303,7 @@ class RowPainter {
                 } else {
                     borderAfterWhich = ConditionalBorder.REST;
                 }
+
                 addAreasForCell(firstCellParts[i].pgu,
                         firstCellParts[i].start, lastCellParts[i].end,
                         actualRowHeight, borderBeforeWhich, borderAfterWhich,
@@ -316,7 +361,7 @@ class RowPainter {
                 while (iter.nextIndex() < endIndex) {
                     KnuthElement el = (KnuthElement) iter.next();
                     if (el.isBox() || el.isGlue()) {
-                        len += el.getW();
+                        len += el.getWidth();
                     }
                 }
                 len += ActiveCell.getElementContentLength((KnuthElement) iter.next());
@@ -387,6 +432,56 @@ class RowPainter {
                 startRowIndex == firstRowOnPageIndex, lastOnPage, this, firstRowHeight);
     }
 
+    private void addAreaForEmptyGridUnit(EmptyGridUnit gu, int rowIndex, int colIndex,
+            int actualRowHeight,
+            int borderBeforeWhich, int borderAfterWhich, boolean lastOnPage) {
+
+        //get effective borders
+        BorderInfo borderBefore = gu.getBorderBefore(borderBeforeWhich);
+        BorderInfo borderAfter = gu.getBorderAfter(borderAfterWhich);
+        BorderInfo borderStart = gu.getBorderStart();
+        BorderInfo borderEnd = gu.getBorderEnd();
+        if (borderBefore.getRetainedWidth() == 0
+                && borderAfter.getRetainedWidth() == 0
+                && borderStart.getRetainedWidth() == 0
+                && borderEnd.getRetainedWidth() == 0) {
+            return; //no borders, no area necessary
+        }
+
+        TableLayoutManager tableLM = tclm.getTableLM();
+        Table table = tableLM.getTable();
+        TableColumn col = tclm.getColumns().getColumn(colIndex + 1);
+
+        //position information
+        boolean firstOnPage = (rowIndex == firstRowOnPageIndex);
+        boolean inFirstColumn = (colIndex == 0);
+        boolean inLastColumn = (colIndex == table.getNumberOfColumns() - 1);
+
+        //determine the block area's size
+        int ipd = col.getColumnWidth().getValue(tableLM);
+        ipd -= (borderStart.getRetainedWidth() + borderEnd.getRetainedWidth()) / 2;
+        int bpd = actualRowHeight;
+        bpd -= (borderBefore.getRetainedWidth() + borderAfter.getRetainedWidth()) / 2;
+
+        //generate the block area
+        Block block = new Block();
+        block.setPositioning(Block.ABSOLUTE);
+        block.addTrait(Trait.IS_REFERENCE_AREA, Boolean.TRUE);
+        block.setIPD(ipd);
+        block.setBPD(bpd);
+        block.setXOffset(tclm.getXOffsetOfGridUnit(colIndex)
+                + (borderStart.getRetainedWidth() / 2));
+        block.setYOffset(getRowOffset(rowIndex)
+                - (borderBefore.getRetainedWidth() / 2));
+        boolean[] outer = new boolean[] {firstOnPage, lastOnPage, inFirstColumn,
+                inLastColumn};
+        TraitSetter.addCollapsingBorders(block,
+                borderBefore,
+                borderAfter,
+                borderStart,
+                borderEnd, outer);
+        tableLM.addChildArea(block);
+    }
 
     /**
      * Registers the given area, that will be used to render the part of
@@ -396,7 +491,7 @@ class RowPainter {
      * set when the areas for the cell are created since at that moment this bpd is yet
      * unknown. So they will instead be set in
      * {@link #addAreasAndFlushRow(boolean, boolean)}.
-     * 
+     *
      * @param backgroundArea the block of the cell's dimensions that will hold the part
      * background
      */
@@ -407,7 +502,7 @@ class RowPainter {
 
     /**
      * Records the y-offset of the row with the given index.
-     * 
+     *
      * @param rowIndex index of the row
      * @param offset y-offset of the row on the page
      */
@@ -419,7 +514,7 @@ class RowPainter {
          * considered as finished, since it contains no cell ending on this row. Thus no
          * TableContentPosition will be created for this row. Thus its index will never be
          * recorded by the #handleTableContentPosition method.
-         * 
+         *
          * The offset of such a row is the same as the next non-empty row. It's needed
          * to correctly offset blocks for cells starting on this row. Hence the loop
          * below.
@@ -431,7 +526,7 @@ class RowPainter {
 
     /**
      * Returns the offset of the row with the given index.
-     * 
+     *
      * @param rowIndex index of the row
      * @return its y-offset on the page
      */
