@@ -15,11 +15,10 @@
  * limitations under the License.
  */
 
-/* $Id: TableContentLayoutManager.java 650550 2008-04-22 15:22:31Z vhennebert $ */
+/* $Id: TableContentLayoutManager.java 807014 2009-08-23 20:27:48Z adelmelle $ */
 
 package org.apache.fop.layoutmgr.table;
 
-import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
@@ -28,15 +27,17 @@ import java.util.Map;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.datatypes.PercentBaseContext;
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.flow.table.EffRow;
 import org.apache.fop.fo.flow.table.PrimaryGridUnit;
 import org.apache.fop.fo.flow.table.Table;
-import org.apache.fop.fo.flow.table.TableBody;
+import org.apache.fop.fo.flow.table.TablePart;
 import org.apache.fop.layoutmgr.BreakElement;
 import org.apache.fop.layoutmgr.ElementListUtils;
+import org.apache.fop.layoutmgr.Keep;
 import org.apache.fop.layoutmgr.KnuthBox;
 import org.apache.fop.layoutmgr.KnuthElement;
 import org.apache.fop.layoutmgr.KnuthGlue;
@@ -67,9 +68,9 @@ public class TableContentLayoutManager implements PercentBaseContext {
 
     private int startXOffset;
     private int usedBPD;
-    
+
     private TableStepper stepper;
-        
+
     /**
      * Main constructor
      * @param parent Parent layout manager
@@ -86,19 +87,19 @@ public class TableContentLayoutManager implements PercentBaseContext {
         }
         stepper = new TableStepper(this);
     }
-    
+
     /**
      * @return the table layout manager
      */
     TableLayoutManager getTableLM() {
         return this.tableLM;
     }
-    
+
     /** @return true if the table uses the separate border model. */
     boolean isSeparateBorderModel() {
         return getTableLM().getTable().isSeparateBorderModel();
     }
-    
+
     /**
      * @return the column setup of this table
      */
@@ -140,14 +141,14 @@ public class TableContentLayoutManager implements PercentBaseContext {
             this.headerNetHeight
                     = ElementListUtils.calcContentLength(this.headerList);
             if (log.isDebugEnabled()) {
-                log.debug("==> Header: " 
+                log.debug("==> Header: "
                         + headerNetHeight + " - " + this.headerList);
             }
             TableHeaderFooterPosition pos = new TableHeaderFooterPosition(
                     getTableLM(), true, this.headerList);
             KnuthBox box = new KnuthBox(headerNetHeight, pos, false);
             if (getTableLM().getTable().omitHeaderAtBreak()) {
-                //We can simply add the table header at the start 
+                //We can simply add the table header at the start
                 //of the whole list
                 headerAsFirst = box;
             } else {
@@ -160,7 +161,7 @@ public class TableContentLayoutManager implements PercentBaseContext {
             this.footerNetHeight
                     = ElementListUtils.calcContentLength(this.footerList);
             if (log.isDebugEnabled()) {
-                log.debug("==> Footer: " 
+                log.debug("==> Footer: "
                         + footerNetHeight + " - " + this.footerList);
             }
             //We can simply add the table footer at the end of the whole list
@@ -193,59 +194,68 @@ public class TableContentLayoutManager implements PercentBaseContext {
         }
         return returnList;
     }
-    
+
     /**
      * Creates Knuth elements by iterating over a TableRowIterator.
      * @param iter TableRowIterator instance to fetch rows from
      * @param context Active LayoutContext
      * @param alignment alignment indicator
-     * @param bodyType Indicates what kind of body is being processed 
+     * @param bodyType Indicates what kind of body is being processed
      *                  (BODY, HEADER or FOOTER)
      * @return An element list
      */
-    private LinkedList getKnuthElementsForRowIterator(TableRowIterator iter, 
+    private LinkedList getKnuthElementsForRowIterator(TableRowIterator iter,
             LayoutContext context, int alignment, int bodyType) {
         LinkedList returnList = new LinkedList();
         EffRow[] rowGroup = iter.getNextRowGroup();
         // TODO homogenize the handling of keeps and breaks
-        context.unsetFlags(LayoutContext.KEEP_WITH_PREVIOUS_PENDING
-                | LayoutContext.KEEP_WITH_NEXT_PENDING);
+        context.clearKeepsPending();
         context.setBreakBefore(Constants.EN_AUTO);
         context.setBreakAfter(Constants.EN_AUTO);
-        boolean keepWithPrevious = false;
+        Keep keepWithPrevious = Keep.KEEP_AUTO;
         int breakBefore = Constants.EN_AUTO;
         if (rowGroup != null) {
             RowGroupLayoutManager rowGroupLM = new RowGroupLayoutManager(getTableLM(), rowGroup,
                     stepper);
             List nextRowGroupElems = rowGroupLM.getNextKnuthElements(context, alignment, bodyType);
-            keepWithPrevious = context.isKeepWithPreviousPending();
-            boolean keepBetween = context.isKeepWithNextPending();
+            keepWithPrevious = keepWithPrevious.compare(context.getKeepWithPreviousPending());
             breakBefore = context.getBreakBefore();
             int breakBetween = context.getBreakAfter();
             returnList.addAll(nextRowGroupElems);
             while ((rowGroup = iter.getNextRowGroup()) != null) {
                 rowGroupLM = new RowGroupLayoutManager(getTableLM(), rowGroup, stepper);
+
+                //Note previous pending keep-with-next and clear the strength
+                //(as the layout context is reused)
+                Keep keepWithNextPending = context.getKeepWithNextPending();
+                context.clearKeepWithNextPending();
+
+                //Get elements for next row group
                 nextRowGroupElems = rowGroupLM.getNextKnuthElements(context, alignment, bodyType);
                 /*
                  * The last break element produced by TableStepper (for the previous row
                  * group) may be used to represent the break between the two row groups.
                  * Its penalty value and break class must just be overridden by the
                  * characteristics of the keep or break between the two.
-                 * 
+                 *
                  * However, we mustn't forget that if the after border of the last row of
                  * the row group is thicker in the normal case than in the trailing case,
                  * an additional glue will be appended to the element list. So we may have
                  * to go two steps backwards in the list.
                  */
-                int penaltyValue = 0;
-                keepBetween |= context.isKeepWithPreviousPending();
-                if (keepBetween || tableLM.getTable().mustKeepTogether()) {
-                    penaltyValue = KnuthElement.INFINITE;
-                }
+
+                //Determine keep constraints
+                Keep keep = keepWithNextPending.compare(context.getKeepWithPreviousPending());
+                context.clearKeepWithPreviousPending();
+                keep = keep.compare(getTableLM().getKeepTogether());
+                int penaltyValue = keep.getPenalty();
+                int breakClass = keep.getContext();
+
                 breakBetween = BreakUtil.compareBreakClasses(breakBetween,
                         context.getBreakBefore());
                 if (breakBetween != Constants.EN_AUTO) {
                     penaltyValue = -KnuthElement.INFINITE;
+                    breakClass = breakBetween;
                 }
                 BreakElement breakElement;
                 ListIterator elemIter = returnList.listIterator(returnList.size());
@@ -256,10 +266,9 @@ public class TableContentLayoutManager implements PercentBaseContext {
                     breakElement = (BreakElement) elem;
                 }
                 breakElement.setPenaltyValue(penaltyValue);
-                breakElement.setBreakClass(breakBetween);
+                breakElement.setBreakClass(breakClass);
                 returnList.addAll(nextRowGroupElems);
                 breakBetween = context.getBreakAfter();
-                keepBetween = context.isKeepWithNextPending();
             }
         }
         /*
@@ -277,33 +286,41 @@ public class TableContentLayoutManager implements PercentBaseContext {
         } else {
             elemIter.remove();
         }
-        context.setFlags(LayoutContext.KEEP_WITH_PREVIOUS_PENDING, keepWithPrevious);
+        context.updateKeepWithPreviousPending(keepWithPrevious);
         context.setBreakBefore(breakBefore);
 
         //fox:widow-content-limit
-        int widowContentLimit = getTableLM().getTable().getWidowContentLimit().getValue(); 
+        int widowContentLimit = getTableLM().getTable().getWidowContentLimit().getValue();
         if (widowContentLimit != 0 && bodyType == TableRowIterator.BODY) {
             ElementListUtils.removeLegalBreaks(returnList, widowContentLimit);
         }
         //fox:orphan-content-limit
-        int orphanContentLimit = getTableLM().getTable().getOrphanContentLimit().getValue(); 
+        int orphanContentLimit = getTableLM().getTable().getOrphanContentLimit().getValue();
         if (orphanContentLimit != 0 && bodyType == TableRowIterator.BODY) {
             ElementListUtils.removeLegalBreaksFromEnd(returnList, orphanContentLimit);
         }
-        
+
         return returnList;
     }
 
     /**
-     * Retuns the X offset of the given grid unit.
+     * Returns the X offset of the given grid unit.
      * @param gu the grid unit
      * @return the requested X offset
      */
     protected int getXOffsetOfGridUnit(PrimaryGridUnit gu) {
-        int col = gu.getColIndex();
-        return startXOffset + getTableLM().getColumns().getXOffset(col + 1, getTableLM());
+        return getXOffsetOfGridUnit(gu.getColIndex());
     }
-    
+
+    /**
+     * Returns the X offset of the grid unit in the given column.
+     * @param colIndex the column index (zero-based)
+     * @return the requested X offset
+     */
+    protected int getXOffsetOfGridUnit(int colIndex) {
+        return startXOffset + getTableLM().getColumns().getXOffset(colIndex + 1, getTableLM());
+    }
+
     /**
      * Adds the areas generated by this layout manager to the area tree.
      * @param parentIter the position iterator
@@ -313,7 +330,7 @@ public class TableContentLayoutManager implements PercentBaseContext {
         this.usedBPD = 0;
         RowPainter painter = new RowPainter(this, layoutContext);
 
-        List tablePositions = new ArrayList();
+        List tablePositions = new java.util.ArrayList();
         List headerElements = null;
         List footerElements = null;
         Position firstPos = null;
@@ -348,7 +365,7 @@ public class TableContentLayoutManager implements PercentBaseContext {
                 }
             } else if (pos instanceof TableHFPenaltyPosition) {
                 //ignore for now, see special handling below if break is at a penalty
-                //Only if the last position in this part/page us such a position it will be used 
+                //Only if the last position in this part/page us such a position it will be used
             } else if (pos instanceof TableContentPosition) {
                 tablePositions.add(pos);
             } else {
@@ -366,23 +383,23 @@ public class TableContentLayoutManager implements PercentBaseContext {
                 headerElements = penaltyPos.headerElements;
             }
             if (penaltyPos.footerElements != null) {
-                footerElements = penaltyPos.footerElements; 
+                footerElements = penaltyPos.footerElements;
             }
         }
 
         Map markers = getTableLM().getTable().getMarkers();
         if (markers != null) {
-            getTableLM().getCurrentPV().addMarkers(markers, 
+            getTableLM().getCurrentPV().addMarkers(markers,
                     true, getTableLM().isFirst(firstPos), getTableLM().isLast(lastCheckPos));
         }
-        
+
         if (headerElements != null) {
             //header positions for the last part are the second-to-last element and need to
             //be handled first before all other TableContentPositions
             addHeaderFooterAreas(headerElements, tableLM.getTable().getTableHeader(), painter,
                     false);
         }
-        
+
         if (tablePositions.isEmpty()) {
             // TODO make sure this actually never happens
             log.error("tablePositions empty."
@@ -397,18 +414,18 @@ public class TableContentLayoutManager implements PercentBaseContext {
             addHeaderFooterAreas(footerElements, tableLM.getTable().getTableFooter(), painter,
                     true);
         }
-        
+
         this.usedBPD += painter.getAccumulatedBPD();
 
         if (markers != null) {
-            getTableLM().getCurrentPV().addMarkers(markers, 
+            getTableLM().getCurrentPV().addMarkers(markers,
                     false, getTableLM().isFirst(firstPos), getTableLM().isLast(lastCheckPos));
         }
     }
 
-    private void addHeaderFooterAreas(List elements, TableBody part, RowPainter painter,
+    private void addHeaderFooterAreas(List elements, TablePart part, RowPainter painter,
             boolean lastOnPage) {
-        List lst = new ArrayList(elements.size());
+        List lst = new java.util.ArrayList(elements.size());
         for (Iterator iter = new KnuthPossPosIter(elements); iter.hasNext();) {
             Position pos = (Position) iter.next();
             /*
@@ -425,7 +442,7 @@ public class TableContentLayoutManager implements PercentBaseContext {
     /**
      * Iterates over the positions corresponding to the table's body (which may contain
      * several table-body elements!) and adds the corresponding areas.
-     * 
+     *
      * @param iterator iterator over TableContentPosition elements. Those positions
      * correspond to the elements of the body present on the current page
      * @param painter
@@ -435,40 +452,40 @@ public class TableContentLayoutManager implements PercentBaseContext {
     private void addBodyAreas(Iterator iterator, RowPainter painter,
             boolean lastOnPage) {
         painter.startBody();
-        List lst = new ArrayList();
+        List lst = new java.util.ArrayList();
         TableContentPosition pos = (TableContentPosition) iterator.next();
         boolean isFirstPos = pos.getFlag(TableContentPosition.FIRST_IN_ROWGROUP)
                 && pos.getRow().getFlag(EffRow.FIRST_IN_PART);
-        TableBody body = pos.getTableBody();
+        TablePart part = pos.getTablePart();
         lst.add(pos);
         while (iterator.hasNext()) {
             pos = (TableContentPosition) iterator.next();
-            if (pos.getTableBody() != body) {
-                addTablePartAreas(lst, painter, body, isFirstPos, true, false, false);
+            if (pos.getTablePart() != part) {
+                addTablePartAreas(lst, painter, part, isFirstPos, true, false, false);
                 isFirstPos = true;
                 lst.clear();
-                body = pos.getTableBody();
+                part = pos.getTablePart();
             }
             lst.add(pos);
         }
         boolean isLastPos = pos.getFlag(TableContentPosition.LAST_IN_ROWGROUP)
                 && pos.getRow().getFlag(EffRow.LAST_IN_PART);
-        addTablePartAreas(lst, painter, body, isFirstPos, isLastPos, true, lastOnPage);
+        addTablePartAreas(lst, painter, part, isFirstPos, isLastPos, true, lastOnPage);
         painter.endBody();
     }
 
     /**
      * Adds the areas corresponding to a single fo:table-header/footer/body element.
      */
-    private void addTablePartAreas(List positions, RowPainter painter, TableBody body,
+    private void addTablePartAreas(List positions, RowPainter painter, TablePart body,
             boolean isFirstPos, boolean isLastPos, boolean lastInBody, boolean lastOnPage) {
-        getTableLM().getCurrentPV().addMarkers(body.getMarkers(), 
+        getTableLM().getCurrentPV().addMarkers(body.getMarkers(),
                 true, isFirstPos, isLastPos);
         painter.startTablePart(body);
         for (Iterator iter = positions.iterator(); iter.hasNext();) {
             painter.handleTableContentPosition((TableContentPosition) iter.next());
         }
-        getTableLM().getCurrentPV().addMarkers(body.getMarkers(), 
+        getTableLM().getCurrentPV().addMarkers(body.getMarkers(),
                 false, isFirstPos, isLastPos);
         painter.endTablePart(lastInBody, lastOnPage);
     }
@@ -491,7 +508,7 @@ public class TableContentLayoutManager implements PercentBaseContext {
     // --------- Property Resolution related functions --------- //
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
     public int getBaseLength(int lengthBase, FObj fobj) {
         return tableLM.getBaseLength(lengthBase, fobj);
