@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractPDFStream.java 611114 2008-01-11 09:04:28Z jeremias $ */
- 
+/* $Id: AbstractPDFStream.java 757712 2009-03-24 10:46:59Z vhennebert $ */
+
 package org.apache.fop.pdf;
 
 import java.io.IOException;
@@ -43,19 +43,29 @@ public abstract class AbstractPDFStream extends PDFDictionary {
     }
 
     /**
-     * Sets up the default filters for this stream if they haven't been set 
+     * Sets up the default filters for this stream if they haven't been set
      * from outside.
      */
     protected void setupFilterList() {
         if (!getFilterList().isInitialized()) {
             getFilterList().addDefaultFilters(
-                getDocumentSafely().getFilterMap(), 
-                PDFFilterList.DEFAULT_FILTER);
+                getDocumentSafely().getFilterMap(),
+                getDefaultFilterName());
         }
         prepareImplicitFilters();
         getDocument().applyEncryption(this);
     }
 
+    /**
+     * Returns the name of a suitable filter for this PDF object.
+     *
+     * @return the default filter
+     * @see PDFFilterList
+     */
+    protected String getDefaultFilterName() {
+        return PDFFilterList.DEFAULT_FILTER;
+    }
+
     /**
      * Returns the associated filter list.
      * @return the filter list
@@ -78,18 +88,18 @@ public abstract class AbstractPDFStream extends PDFDictionary {
     /**
      * Returns a value that hints at the size of the encoded stream. This is
      * used to optimize buffer allocation so fewer buffer reallocations are
-     * necessary. 
+     * necessary.
      * @return an estimated size (0 if no hint can be given)
      * @throws IOException in case of an I/O problem
      */
     protected abstract int getSizeHint() throws IOException;
-    
+
     /**
      * Sends the raw stream data to the target OutputStream.
      * @param out OutputStream to write to
      * @throws IOException In case of an I/O problem
      */
-    protected abstract void outputRawStreamData(OutputStream out) 
+    protected abstract void outputRawStreamData(OutputStream out)
             throws IOException;
 
     /**
@@ -104,10 +114,10 @@ public abstract class AbstractPDFStream extends PDFDictionary {
         byte[] p = encode("stream\n");
         out.write(p);
         length += p.length;
-        
+
         encodedStream.outputContents(out);
         length += encodedStream.getSize();
-        
+
         p = encode("\nendstream");
         out.write(p);
         length += p.length;
@@ -147,7 +157,7 @@ public abstract class AbstractPDFStream extends PDFDictionary {
         byte[] buf = encode("stream\n");
         out.write(buf);
         bytesWritten += buf.length;
-        
+
         //Stream contents
         CloseBlockerOutputStream cbout = new CloseBlockerOutputStream(out);
         CountingOutputStream cout = new CountingOutputStream(cbout);
@@ -156,12 +166,12 @@ public abstract class AbstractPDFStream extends PDFDictionary {
         filteredOutput.close();
         refLength.setNumber(new Integer(cout.getCount()));
         bytesWritten += cout.getCount();
-            
+
         //Stream trailer
         buf = encode("\nendstream");
         out.write(buf);
         bytesWritten += buf.length;
-        
+
         return bytesWritten;
     }
 
@@ -177,7 +187,7 @@ public abstract class AbstractPDFStream extends PDFDictionary {
         Writer writer = PDFDocument.getWriterFor(cout);
         writer.write(getObjectID());
         //int length = 0;
-        
+
         StreamCache encodedStream = null;
         PDFNumber refLength = null;
         final Object lengthEntry;
@@ -189,10 +199,10 @@ public abstract class AbstractPDFStream extends PDFDictionary {
             encodedStream = encodeStream();
             lengthEntry = new Integer(encodedStream.getSize() + 1);
         }
-        
+
         populateStreamDict(lengthEntry);
         writeDictionary(cout, writer);
-        
+
         //Send encoded stream to target OutputStream
         writer.flush();
         if (encodedStream == null) {
@@ -201,7 +211,7 @@ public abstract class AbstractPDFStream extends PDFDictionary {
             outputStreamData(encodedStream, cout);
             encodedStream.clear(); //Encoded stream can now be discarded
         }
-        
+
         writer.write("\nendobj\n");
         writer.flush();
         return cout.getCount();
