@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFFactory.java 649006 2008-04-17 08:42:52Z jeremias $ */
+/* $Id: PDFFactory.java 830293 2009-10-27 19:07:52Z vhennebert $ */
 
 package org.apache.fop.pdf;
 
@@ -43,6 +43,7 @@ import org.apache.commons.logging.LogFactory;
 import org.apache.xmlgraphics.xmp.Metadata;
 
 import org.apache.fop.fonts.CIDFont;
+import org.apache.fop.fonts.CIDSubset;
 import org.apache.fop.fonts.CodePointMapping;
 import org.apache.fop.fonts.CustomFont;
 import org.apache.fop.fonts.FontDescriptor;
@@ -50,6 +51,8 @@ import org.apache.fop.fonts.FontMetrics;
 import org.apache.fop.fonts.FontType;
 import org.apache.fop.fonts.LazyFont;
 import org.apache.fop.fonts.MultiByteFont;
+import org.apache.fop.fonts.SimpleSingleByteEncoding;
+import org.apache.fop.fonts.SingleByteEncoding;
 import org.apache.fop.fonts.SingleByteFont;
 import org.apache.fop.fonts.Typeface;
 import org.apache.fop.fonts.truetype.FontFileReader;
@@ -165,6 +168,34 @@ public class PDFFactory {
         return outputIntent;
     }
 
+    /**
+     * Make a /Page object. The page is assigned an object number immediately
+     * so references can already be made. The page must be added to the
+     * PDFDocument later using addObject().
+     *
+     * @param resources resources object to use
+     * @param pageIndex index of the page (zero-based)
+     * @param mediaBox the MediaBox area
+     * @param cropBox the CropBox area
+     * @param bleedBox the BleedBox area
+     * @param trimBox the TrimBox area
+     *
+     * @return the created /Page object
+     */
+    public PDFPage makePage(PDFResources resources, int pageIndex,
+                            Rectangle2D mediaBox, Rectangle2D cropBox,
+                            Rectangle2D bleedBox, Rectangle2D trimBox) {
+        /*
+         * create a PDFPage with the next object number, the given
+         * resources, contents and dimensions
+         */
+        PDFPage page = new PDFPage(resources, pageIndex, mediaBox, cropBox, bleedBox, trimBox);
+
+        getDocument().assignObjectNumber(page);
+        getDocument().getPages().addPage(page);
+        return page;
+    }
+
     /**
      * Make a /Page object. The page is assigned an object number immediately
      * so references can already be made. The page must be added to the
@@ -179,17 +210,8 @@ public class PDFFactory {
      */
     public PDFPage makePage(PDFResources resources,
                             int pageWidth, int pageHeight, int pageIndex) {
-
-        /*
-         * create a PDFPage with the next object number, the given
-         * resources, contents and dimensions
-         */
-        PDFPage page = new PDFPage(resources,
-                                   pageWidth, pageHeight, pageIndex);
-
-        getDocument().assignObjectNumber(page);
-        getDocument().getPages().addPage(page);
-        return page;
+        Rectangle2D mediaBox = new Rectangle2D.Double(0, 0, pageWidth, pageHeight);
+        return makePage(resources, pageIndex, mediaBox, mediaBox, mediaBox, mediaBox);
     }
 
     /**
@@ -721,6 +743,7 @@ public class PDFFactory {
      * @param theColors the list of colors for the gradient
      * @param theBounds the list of bounds associated with the colors
      * @param theCoords the coordinates for the gradient
+     * @param theMatrix the coordinate-transformation matrix
      * @return the PDF pattern that was created
      */
     public PDFPattern makeGradient(PDFResourceContext res, boolean radial,
@@ -734,7 +757,7 @@ public class PDFFactory {
         List theCone;
         PDFPattern myPattern;
         //PDFColorSpace theColorSpace;
-        double interpolation = (double)1.000;
+        double interpolation = 1.000;
         List theFunctions = new ArrayList();
 
         int currentPosition;
@@ -863,6 +886,35 @@ public class PDFFactory {
         return pageLabels;
     }
 
+    /**
+     * Creates and returns a StructTreeRoot object. Used for accessibility.
+     * @param parentTree the value of the ParenTree entry
+     * @return structure Tree Root element
+     */
+    public PDFStructTreeRoot makeStructTreeRoot(PDFParentTree parentTree) {
+        PDFStructTreeRoot structTreeRoot = new PDFStructTreeRoot(parentTree);
+        getDocument().assignObjectNumber(structTreeRoot);
+        getDocument().addTrailerObject(structTreeRoot);
+        getDocument().getRoot().setStructTreeRoot(structTreeRoot);
+        return structTreeRoot;
+    }
+
+    /**
+     * Creates and returns a StructElem object.
+     *
+     * @param structureType the structure type of the new element (value for the
+     * S entry)
+     * @param parent the parent of the new structure element in the structure
+     * hierarchy
+     * @return the newly created element
+     */
+    public PDFStructElem makeStructureElement(PDFName structureType, PDFObject parent) {
+        PDFStructElem structElem = new PDFStructElem(parent, structureType);
+        getDocument().assignObjectNumber(structElem);
+        getDocument().addTrailerObject(structElem);
+        return structElem;
+    }
+
     /**
      * Make a the head object of the name dictionary (the /Dests object).
      *
@@ -871,7 +923,8 @@ public class PDFFactory {
      */
     public PDFDests makeDests(List destinationList) {
         PDFDests dests;
-        
+
+        //TODO: Check why the below conditional branch is needed. Condition is always true...
         final boolean deep = true;
         //true for a "deep" structure (one node per entry), true for a "flat" structure
         if (deep) {
@@ -908,8 +961,8 @@ public class PDFFactory {
         PDFNameTreeNode node = new PDFNameTreeNode();
         getDocument().registerObject(node);
         return node;
-    }    
-    
+    }
+
     /* ========================= links ===================================== */
     // Some of the "yoffset-only" functions in this part are obsolete and can
     // possibly be removed or deprecated. Some are still called by PDFGraphics2D
@@ -950,7 +1003,7 @@ public class PDFFactory {
 
         PDFGoTo gt = new PDFGoTo(page);
         gt.setDestination(dest);
-        getDocument().addTrailerObject(gt);
+        getDocument().registerObject(gt);
         PDFInternalLink internalLink = new PDFInternalLink(gt.referencePDF());
         link.setAction(internalLink);
 
@@ -958,7 +1011,7 @@ public class PDFFactory {
     }
 
     /**
-     * make a link object
+     * Make a {@link PDFLink} object
      *
      * @param rect   the clickable rectangle
      * @param destination  the destination file
@@ -973,7 +1026,7 @@ public class PDFFactory {
         PDFLink link = new PDFLink(rect);
 
         if (linkType == PDFLink.EXTERNAL) {
-            link.setAction(getExternalAction(destination));
+            link.setAction(getExternalAction(destination, false));
         } else {
             // linkType is internal
             String goToReference = getGoToReference(destination, yoffset);
@@ -996,27 +1049,33 @@ public class PDFFactory {
      *
      * @param target The external target. This may be a PDF file name
      * (optionally with internal page number or destination) or any type of URI.
+     * @param newWindow boolean indicating whether the target should be
+     *                  displayed in a new window
      * @return the PDFAction thus created or found
      */
-    public PDFAction getExternalAction(String target) {
+    public PDFAction getExternalAction(String target, boolean newWindow) {
         int index;
         String targetLo = target.toLowerCase();
         // HTTP URL?
         if (targetLo.startsWith("http://")) {
             return new PDFUri(target);
+        // Non PDF files. Try to /Launch them.
+        } else if (targetLo.startsWith("file://")) {
+            target = target.substring("file://".length());
+            return getLaunchAction(target);
         // Bare PDF file name?
         } else if (targetLo.endsWith(".pdf")) {
-            return getGoToPDFAction(target, null, -1);
+            return getGoToPDFAction(target, null, -1, newWindow);
         // PDF file + page?
         } else if ((index = targetLo.indexOf(".pdf#page=")) > 0) {
             String filename = target.substring(0, index + 4);
             int page = Integer.parseInt(target.substring(index + 10));
-            return getGoToPDFAction(filename, null, page);
+            return getGoToPDFAction(filename, null, page, newWindow);
         // PDF file + destination?
         } else if ((index = targetLo.indexOf(".pdf#dest=")) > 0) {
             String filename = target.substring(0, index + 4);
             String dest = target.substring(index + 10);
-            return getGoToPDFAction(filename, dest, -1);
+            return getGoToPDFAction(filename, dest, -1, newWindow);
         // None of the above? Default to URI:
         } else {
             return new PDFUri(target);
@@ -1066,9 +1125,11 @@ public class PDFFactory {
      * @param file the pdf file name
      * @param dest the remote name destination, may be null
      * @param page the remote page number, -1 means not specified
+     * @param newWindow boolean indicating whether the target should be
+     *                  displayed in a new window
      * @return the pdf goto remote object
      */
-    private PDFGoToRemote getGoToPDFAction(String file, String dest, int page) {
+    private PDFGoToRemote getGoToPDFAction(String file, String dest, int page, boolean newWindow) {
         getDocument().getProfile().verifyActionAllowed();
         PDFFileSpec fileSpec = new PDFFileSpec(file);
         PDFFileSpec oldspec = getDocument().findFileSpec(fileSpec);
@@ -1080,11 +1141,11 @@ public class PDFFactory {
         PDFGoToRemote remote;
 
         if (dest == null && page == -1) {
-            remote = new PDFGoToRemote(fileSpec);
+            remote = new PDFGoToRemote(fileSpec, newWindow);
         } else if (dest != null) {
-            remote = new PDFGoToRemote(fileSpec, dest);
+            remote = new PDFGoToRemote(fileSpec, dest, newWindow);
         } else {
-            remote = new PDFGoToRemote(fileSpec, page);
+            remote = new PDFGoToRemote(fileSpec, page, newWindow);
         }
         PDFGoToRemote oldremote = getDocument().findGoToRemote(remote);
         if (oldremote == null) {
@@ -1095,6 +1156,37 @@ public class PDFFactory {
         return remote;
     }
 
+    /**
+     * Creates and returns a launch pdf document action using
+     * <code>file</code> to create a file spcifiaciton for
+     * the document/file to be opened with an external application.
+     *
+     * @param file the pdf file name
+     * @return the pdf launch object
+     */
+    private PDFLaunch getLaunchAction(String file) {
+        getDocument().getProfile().verifyActionAllowed();
+
+        PDFFileSpec fileSpec = new PDFFileSpec(file);
+        PDFFileSpec oldSpec = getDocument().findFileSpec(fileSpec);
+
+        if (oldSpec == null) {
+            getDocument().registerObject(fileSpec);
+        } else {
+            fileSpec = oldSpec;
+        }
+        PDFLaunch launch = new PDFLaunch(fileSpec);
+        PDFLaunch oldLaunch = getDocument().findLaunch(launch);
+
+        if (oldLaunch == null) {
+            getDocument().registerObject(launch);
+        } else {
+            launch = oldLaunch;
+        }
+
+        return launch;
+    }
+
     /**
      * Make an outline object and add it to the given parent
      *
@@ -1166,7 +1258,7 @@ public class PDFFactory {
     }
 
     /**
-     * make a Type1 /Font object
+     * Make a Type1 /Font object.
      *
      * @param fontname internal name to use for this font (eg "F1")
      * @param basefont name of the base font (eg "Helvetica")
@@ -1183,10 +1275,23 @@ public class PDFFactory {
             return preRegisteredfont;
         }
 
+        boolean forceToUnicode = true;
+
         if (descriptor == null) {
             //Usually Base 14 fonts
             PDFFont font = new PDFFont(fontname, FontType.TYPE1, basefont, encoding);
             getDocument().registerObject(font);
+            if (forceToUnicode && !PDFEncoding.isPredefinedEncoding(encoding)) {
+                SingleByteEncoding mapping;
+                if (encoding != null) {
+                    mapping = CodePointMapping.getMapping(encoding);
+                } else {
+                    //for Symbol and ZapfDingbats where encoding must be null in PDF
+                    Typeface tf = (Typeface)metrics;
+                    mapping = CodePointMapping.getMapping(tf.getEncodingName());
+                }
+                generateToUnicodeCmap(font, mapping);
+            }
             return font;
         } else {
             FontType fonttype = metrics.getFontType();
@@ -1194,8 +1299,7 @@ public class PDFFactory {
             PDFFontDescriptor pdfdesc = makeFontDescriptor(descriptor);
 
             PDFFont font = null;
-            font = (PDFFont)PDFFont.createFont(fontname, fonttype,
-                                                        basefont, null);
+            font = PDFFont.createFont(fontname, fonttype, basefont, null);
             getDocument().registerObject(font);
 
             if (fonttype == FontType.TYPE0) {
@@ -1218,10 +1322,12 @@ public class PDFFactory {
                                    (PDFCIDFontDescriptor)pdfdesc);
                 getDocument().registerObject(cidFont);
 
-                PDFCMap cmap = new PDFToUnicodeCMap(cidMetrics.getCharsUsed(), "fop-ucs-H",
-                    new PDFCIDSystemInfo("Adobe",
-                        "Identity",
-                        0));
+                PDFCMap cmap = new PDFToUnicodeCMap(
+                        cidMetrics.getCIDSubset().getSubsetChars(),
+                        "fop-ucs-H",
+                        new PDFCIDSystemInfo("Adobe",
+                            "Identity",
+                            0), false);
                 getDocument().registerObject(cmap);
                 ((PDFFontType0)font).setCMAP(cmap);
                 ((PDFFontType0)font).setDescendantFonts(cidFont);
@@ -1239,47 +1345,54 @@ public class PDFFactory {
                 int lastChar = singleByteFont.getLastChar();
                 nonBase14.setWidthMetrics(firstChar,
                                      lastChar,
-                                     makeArray(metrics.getWidths()));
-                
+                                     new PDFArray(null, metrics.getWidths()));
+
                 //Handle encoding
-                CodePointMapping mapping = singleByteFont.getCodePointMapping();
+                SingleByteEncoding mapping = singleByteFont.getEncoding();
                 if (singleByteFont.isSymbolicFont()) {
                     //no encoding, use the font's encoding
+                    if (forceToUnicode) {
+                        generateToUnicodeCmap(nonBase14, mapping);
+                    }
                 } else if (PDFEncoding.isPredefinedEncoding(mapping.getName())) {
                     font.setEncoding(mapping.getName());
+                    //No ToUnicode CMap necessary if PDF 1.4, chapter 5.9 (page 368) is to be
+                    //believed.
                 } else {
-                    CodePointMapping winansi = CodePointMapping.getMapping(
-                            CodePointMapping.WIN_ANSI_ENCODING);
-                    PDFEncoding pdfEncoding = new PDFEncoding(winansi.getName());
-                    PDFEncoding.DifferencesBuilder builder
-                            = pdfEncoding.createDifferencesBuilder();
-                    int start = -1;
-                    String[] winansiNames = winansi.getCharNameMap();
-                    String[] charNameMap = mapping.getCharNameMap();
-                    for (int i = 0; i < 256; i++) {
-                        String wac = winansiNames[i];
-                        String c = charNameMap[i];
-                        if (!wac.equals(c)) {
-                            if (start != i) {
-                                builder.addDifference(i);
-                                start = i;
-                            }
-                            builder.addName(c);
-                            start++;
+                    Object pdfEncoding = createPDFEncoding(mapping,
+                            singleByteFont.getFontName());
+                    if (pdfEncoding instanceof PDFEncoding) {
+                        font.setEncoding((PDFEncoding)pdfEncoding);
+                    } else {
+                        font.setEncoding((String)pdfEncoding);
+                    }
+                    if (forceToUnicode) {
+                        generateToUnicodeCmap(nonBase14, mapping);
+                    }
+                }
+
+                //Handle additional encodings (characters outside the primary encoding)
+                if (singleByteFont.hasAdditionalEncodings()) {
+                    for (int i = 0, c = singleByteFont.getAdditionalEncodingCount(); i < c; i++) {
+                        SimpleSingleByteEncoding addEncoding
+                            = singleByteFont.getAdditionalEncoding(i);
+                        String name = fontname + "_" + (i + 1);
+                        Object pdfenc = createPDFEncoding(addEncoding,
+                                singleByteFont.getFontName());
+                        PDFFontNonBase14 addFont = (PDFFontNonBase14)PDFFont.createFont(
+                                name, fonttype,
+                                basefont, pdfenc);
+                        addFont.setDescriptor(pdfdesc);
+                        addFont.setWidthMetrics(
+                                addEncoding.getFirstChar(),
+                                addEncoding.getLastChar(),
+                                new PDFArray(null, singleByteFont.getAdditionalWidths(i)));
+                        getDocument().registerObject(addFont);
+                        getDocument().getResources().addFont(addFont);
+                        if (forceToUnicode) {
+                            generateToUnicodeCmap(addFont, addEncoding);
                         }
                     }
-                    pdfEncoding.setDifferences(builder.toPDFArray());
-                    font.setEncoding(pdfEncoding);
-                    
-                    /* JM: What I thought would be a necessity with custom encodings turned out to
-                     * be a bug in Adobe Acrobat 8. The following section just demonstrates how
-                     * to generate a ToUnicode CMap for a Type 1 font. 
-                    PDFCMap cmap = new PDFToUnicodeCMap(mapping.getUnicodeCharMap(),
-                            "fop-ucs-H",
-                            new PDFCIDSystemInfo("Adobe", "Identity", 0));
-                    getDocument().registerObject(cmap);
-                    nonBase14.setToUnicode(cmap);
-                    */
                 }
             }
 
@@ -1287,15 +1400,70 @@ public class PDFFactory {
         }
     }
 
+    private void generateToUnicodeCmap(PDFFont font, SingleByteEncoding encoding) {
+        PDFCMap cmap = new PDFToUnicodeCMap(encoding.getUnicodeCharMap(),
+                "fop-ucs-H",
+                new PDFCIDSystemInfo("Adobe", "Identity", 0), true);
+        getDocument().registerObject(cmap);
+        font.setToUnicode(cmap);
+    }
+
+    /**
+     * Creates a PDFEncoding instance from a CodePointMapping instance.
+     * @param encoding the code point mapping (encoding)
+     * @param fontNameHint ...
+     * @return the PDF Encoding dictionary (or a String with the predefined encoding)
+     */
+    public Object createPDFEncoding(SingleByteEncoding encoding, String fontNameHint) {
+        SingleByteEncoding baseEncoding;
+        if (fontNameHint.indexOf("Symbol") >= 0) {
+            baseEncoding = CodePointMapping.getMapping(
+                    CodePointMapping.SYMBOL_ENCODING);
+        } else {
+            baseEncoding = CodePointMapping.getMapping(
+                    CodePointMapping.STANDARD_ENCODING);
+        }
+        PDFEncoding pdfEncoding = new PDFEncoding(baseEncoding.getName());
+        PDFEncoding.DifferencesBuilder builder
+                = pdfEncoding.createDifferencesBuilder();
+        int start = -1;
+        String[] baseNames = baseEncoding.getCharNameMap();
+        String[] charNameMap = encoding.getCharNameMap();
+        for (int i = 0, ci = charNameMap.length; i < ci; i++) {
+            String basec = baseNames[i];
+            String c = charNameMap[i];
+            if (!basec.equals(c)) {
+                if (start != i) {
+                    builder.addDifference(i);
+                    start = i;
+                }
+                builder.addName(c);
+                start++;
+            }
+        }
+        if (builder.hasDifferences()) {
+            pdfEncoding.setDifferences(builder.toPDFArray());
+            return pdfEncoding;
+        } else {
+            return baseEncoding.getName();
+        }
+    }
+
+    /**
+     * Creates and returns a width array with the widths of all the characters in the subset.
+     * @param cidFont the font
+     * @return the width array
+     */
     public PDFWArray getSubsetWidths(CIDFont cidFont) {
         // Create widths for reencoded chars
         PDFWArray warray = new PDFWArray();
-        int[] tmpWidth = new int[cidFont.usedGlyphsCount];
+        int[] widths = cidFont.getWidths();
+        CIDSubset subset = cidFont.getCIDSubset();
+        int[] tmpWidth = new int[subset.getSubsetSize()];
 
-        for (int i = 0; i < cidFont.usedGlyphsCount; i++) {
-            Integer nw = (Integer)cidFont.usedGlyphsIndex.get(new Integer(i));
-            int nwx = (nw == null) ? 0 : nw.intValue();
-            tmpWidth[i] = cidFont.width[nwx];
+        for (int i = 0, c = subset.getSubsetSize(); i < c; i++) {
+            int nwx = Math.max(0, subset.getGlyphIndexForSubsetIndex(i));
+            tmpWidth[i] = widths[nwx];
         }
         warray.addEntry(0, tmpWidth);
         return warray;
@@ -1348,12 +1516,7 @@ public class PDFFactory {
     }
 
     private void buildCIDSet(PDFFontDescriptor descriptor, CIDFont cidFont) {
-        BitSet cidSubset = new BitSet();
-        Iterator iter = cidFont.usedGlyphs.keySet().iterator();
-        while (iter.hasNext()) {
-            Integer cid = (Integer)iter.next();
-            cidSubset.set(cid.intValue());
-        }
+        BitSet cidSubset = cidFont.getCIDSubset().getGlyphIndexBitSet();
         PDFStream cidSet = makeStream(null, true);
         ByteArrayOutputStream baout = new ByteArrayOutputStream(cidSubset.length() / 8 + 1);
         int value = 0;
@@ -1409,6 +1572,7 @@ public class PDFFactory {
                 try {
                     in = new java.net.URL(source.getSystemId()).openStream();
                 } catch (MalformedURLException e) {
+                    //TODO: Why construct a new exception here, when it is not thrown?
                     new FileNotFoundException(
                             "File not found. URL could not be resolved: "
                                     + e.getMessage());
@@ -1465,7 +1629,7 @@ public class PDFFactory {
             log.error(
                     "Failed to embed font [" + desc + "] "
                     + desc.getEmbedFontName(), ioe);
-            return (PDFStream) null;
+            return null;
         }
     }
 
@@ -1514,8 +1678,7 @@ public class PDFFactory {
     /**
      * Create a PDFICCStream
      * @see PDFImageXObject
-     * @see org.apache.fop.image.JpegImage
-     * @see org.apache.fop.pdf.PDFDeviceColorSpace     
+     * @see org.apache.fop.pdf.PDFDeviceColorSpace
      * @return the new PDF ICC stream object
      */
     public PDFICCStream makePDFICCStream() {
@@ -1551,14 +1714,13 @@ public class PDFFactory {
     }
 
     /**
-     * make an Array object (ex. Widths array for a font)
+     * Make an Array object (ex. Widths array for a font).
      *
      * @param values the int array values
      * @return the PDF Array with the int values
      */
     public PDFArray makeArray(int[] values) {
         PDFArray array = new PDFArray(null, values);
-
         getDocument().registerObject(array);
         return array;
     }
