@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFFunction.java 426576 2006-07-28 15:44:37Z jeremias $ */
+/* $Id: PDFFunction.java 815358 2009-09-15 15:07:51Z maxberger $ */
 
 package org.apache.fop.pdf;
 
@@ -696,15 +696,8 @@ public class PDFFunction extends PDFObject {
 
     }
 
-    /**
-     * Check if this function is equal to another object.
-     * This is used to find if a particular function already exists
-     * in a document.
-     *
-     * @param obj the obj to compare
-     * @return true if the functions are equal
-     */
-    public boolean equals(Object obj) {
+    /** {@inheritDoc} */
+    protected boolean contentEquals(PDFObject obj) {
         if (obj == null) {
             return false;
         }
