@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: PDFGoTo.java 557337 2007-07-18 17:37:14Z adelmelle $ */
- 
+/* $Id: PDFGoTo.java 815358 2009-09-15 15:07:51Z maxberger $ */
+
 package org.apache.fop.pdf;
 
 import java.awt.geom.Point2D;
@@ -128,7 +128,7 @@ public class PDFGoTo extends PDFAction {
         } else {
             dest = "/D [" + this.pageReference + " " + destination + "]\n";
         }
-        return getObjectID() 
+        return getObjectID()
                     + "<< /Type /Action\n/S /GoTo\n" + dest
                     + ">>\nendobj\n";
     }
@@ -143,13 +143,8 @@ public class PDFGoTo extends PDFAction {
      * endobj
      */
 
-    /**
-     * Check if this equals another object.
-     *
-     * @param obj the object to compare
-     * @return true if this equals other object
-     */
-    public boolean equals(Object obj) {
+    /** {@inheritDoc} */
+    protected boolean contentEquals(PDFObject obj) {
         if (this == obj) {
             return true;
         }
