@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFICCBasedColorSpace.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: PDFICCBasedColorSpace.java 904467 2010-01-29 11:33:10Z vhennebert $ */
 
 package org.apache.fop.pdf;
 
@@ -33,7 +33,7 @@ public class PDFICCBasedColorSpace extends PDFObject implements PDFColorSpace {
 
     private PDFICCStream iccStream;
     private String explicitName;
-    
+
     /**
      * Constructs a the ICCBased color space with an explicit name (ex. "DefaultRGB").
      * @param explicitName an explicit name or null if a name should be generated
@@ -43,7 +43,7 @@ public class PDFICCBasedColorSpace extends PDFObject implements PDFColorSpace {
         this.explicitName = explicitName;
         this.iccStream = iccStream;
     }
-    
+
     /**
      * Constructs a the ICCBased color space.
      * @param iccStream the ICC stream to associate with this color space
@@ -51,12 +51,12 @@ public class PDFICCBasedColorSpace extends PDFObject implements PDFColorSpace {
     public PDFICCBasedColorSpace(PDFICCStream iccStream) {
         this(null, iccStream);
     }
-    
+
     /** @return the ICC stream associated with this color space */
     public PDFICCStream getICCStream() {
         return this.iccStream;
     }
-    
+
     /** {@inheritDoc} */
     public int getNumComponents() {
         return iccStream.getICCProfile().getNumComponents();
@@ -134,22 +134,24 @@ public class PDFICCBasedColorSpace extends PDFObject implements PDFColorSpace {
     public static PDFICCStream setupsRGBColorProfile(PDFDocument pdfDoc) {
         ICC_Profile profile;
         PDFICCStream sRGBProfile = pdfDoc.getFactory().makePDFICCStream();
-        InputStream in = PDFDocument.class.getResourceAsStream("sRGB Color Space Profile.icm");
-        if (in != null) {
-            try {
-                profile = ICC_Profile.getInstance(in);
-            } catch (IOException ioe) {
-                throw new RuntimeException(
-                        "Unexpected IOException loading the sRGB profile: " + ioe.getMessage());
-            } finally {
-                IOUtils.closeQuietly(in);
+        synchronized (PDFICCBasedColorSpace.class) {
+            InputStream in = PDFDocument.class.getResourceAsStream("sRGB Color Space Profile.icm");
+            if (in != null) {
+                try {
+                    profile = ICC_Profile.getInstance(in);
+                } catch (IOException ioe) {
+                    throw new RuntimeException(
+                            "Unexpected IOException loading the sRGB profile: " + ioe.getMessage());
+                } finally {
+                    IOUtils.closeQuietly(in);
+                }
+            } else {
+                // Fallback: Use the sRGB profile from the JRE (about 140KB)
+                profile = ICC_Profile.getInstance(ColorSpace.CS_sRGB);
             }
-        } else {
-            // Fallback: Use the sRGB profile from the JRE (about 140KB)
-            profile = ICC_Profile.getInstance(ColorSpace.CS_sRGB);
         }
         sRGBProfile.setColorSpace(profile, null);
         return sRGBProfile;
     }
-    
+
 }
