@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,12 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: PDFStream.java 588547 2007-10-26 07:48:14Z jeremias $ */
- 
+/* $Id: PDFStream.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+
 package org.apache.fop.pdf;
 
-import java.io.OutputStream;
 import java.io.IOException;
+import java.io.OutputStream;
+import java.io.Writer;
 
 /**
  * Class representing a PDF stream.
@@ -31,12 +32,14 @@ import java.io.IOException;
  * length.
  */
 public class PDFStream extends AbstractPDFStream {
-    
+
     /**
      * The stream of PDF commands
      */
     protected StreamCache data;
 
+    private transient Writer streamWriter;
+
     /**
      * Create an empty stream object
      */
@@ -44,6 +47,10 @@ public class PDFStream extends AbstractPDFStream {
         super();
         try {
             data = StreamCacheFactory.getInstance().createStreamCache();
+            this.streamWriter = new java.io.OutputStreamWriter(
+                    getBufferOutputStream(), PDFDocument.ENCODING);
+            //Buffer to minimize calls to the converter
+            this.streamWriter = new java.io.BufferedWriter(this.streamWriter);
         } catch (IOException ex) {
             //TODO throw the exception and catch it elsewhere
             ex.printStackTrace();
@@ -57,14 +64,25 @@ public class PDFStream extends AbstractPDFStream {
      */
     public void add(String s) {
         try {
-            data.getOutputStream().write(PDFDocument.encode(s));
+            this.streamWriter.write(s);
         } catch (IOException ex) {
             //TODO throw the exception and catch it elsewhere
             ex.printStackTrace();
         }
-
     }
-    
+
+    private void flush() throws IOException {
+        this.streamWriter.flush();
+    }
+
+    /**
+     * Returns a Writer that writes to the OutputStream of the buffer.
+     * @return the Writer
+     */
+    public Writer getBufferWriter() {
+        return this.streamWriter;
+    }
+
     /**
      * Returns an OutputStream that can be used to write to the buffer which is used
      * to build up the PDF stream.
@@ -72,9 +90,12 @@ public class PDFStream extends AbstractPDFStream {
      * @throws IOException In case of an I/O problem
      */
     public OutputStream getBufferOutputStream() throws IOException {
+        if (this.streamWriter != null) {
+            flush(); //Just to be sure
+        }
         return this.data.getOutputStream();
     }
-    
+
     /**
      * Used to set the contents of the PDF stream.
      * @param data the contents as a byte array
@@ -91,6 +112,7 @@ public class PDFStream extends AbstractPDFStream {
      */
     public int getDataLength() {
         try {
+            flush();
             return data.getSize();
         } catch (Exception e) {
             //TODO throw the exception and catch it elsewhere
@@ -99,17 +121,15 @@ public class PDFStream extends AbstractPDFStream {
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected int getSizeHint() throws IOException {
+        flush();
         return data.getSize();
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected void outputRawStreamData(OutputStream out) throws IOException {
+        flush();
         data.outputContents(out);
     }
 
@@ -118,7 +138,7 @@ public class PDFStream extends AbstractPDFStream {
      */
     protected int output(OutputStream stream) throws IOException {
         final int len = super.output(stream);
-        
+
         //Now that the data has been written, it can be discarded.
         this.data = null;
         return len;
