@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: PDFT1Stream.java 627679 2008-02-14 08:12:34Z jeremias $ */
- 
+/* $Id: PDFT1Stream.java 744851 2009-02-16 08:09:29Z jeremias $ */
+
 package org.apache.fop.pdf;
 
 // Java
@@ -28,13 +28,11 @@ import org.apache.fop.fonts.type1.PFBData;
 /**
  * Special PDFStream for embedding Type 1 fonts.
  */
-public class PDFT1Stream extends AbstractPDFStream {
-    
+public class PDFT1Stream extends AbstractPDFFontStream {
+
     private PFBData pfb;
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected int getSizeHint() throws IOException {
         if (this.pfb != null) {
             return pfb.getLength();
@@ -76,9 +74,9 @@ public class PDFT1Stream extends AbstractPDFStream {
     protected void outputRawStreamData(OutputStream out) throws IOException {
         this.pfb.outputAllParts(out);
     }
-    
+
     /**
-     * Used to set the PFBData object that represents the embeddable Type 1 
+     * Used to set the PFBData object that represents the embeddable Type 1
      * font.
      * @param pfb The PFB file
      * @throws IOException in case of an I/O problem
