@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: PDFUri.java 557337 2007-07-18 17:37:14Z adelmelle $ */
- 
+/* $Id: PDFUri.java 746664 2009-02-22 12:40:44Z jeremias $ */
+
 package org.apache.fop.pdf;
 
 /**
@@ -41,14 +41,22 @@ public class PDFUri extends PDFAction {
      * @return the action to place next to /A within a Link
      */
     public String getAction() {
+        if (hasObjectNumber()) {
+            return referencePDF();
+        } else {
+            return getDictString();
+        }
+    }
+
+    private String getDictString() {
         return "<< /URI (" + uri + ")\n/S /URI >>";
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public String toPDFString() {
-        throw new UnsupportedOperationException("This method should not be called");
+        //TODO Convert this class into a dictionary
+        return getObjectID() + getDictString() + "\nendobj\n";
+        //throw new UnsupportedOperationException("This method should not be called");
     }
 
 }
