@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractGraphics2DAdapter.java 656286 2008-05-14 14:16:49Z jeremias $ */
- 
+/* $Id: AbstractGraphics2DAdapter.java 721430 2008-11-28 11:13:12Z acumiskey $ */
+
 package org.apache.fop.render;
 
 import java.awt.Color;
@@ -35,7 +35,7 @@ import java.awt.image.WritableRaster;
 import java.io.IOException;
 
 import org.apache.fop.render.RendererContext.RendererContextWrapper;
-import org.apache.fop.util.UnitConv;
+import org.apache.xmlgraphics.util.UnitConv;
 
 /**
  * Abstract base class for Graphics2DAdapter implementations.
@@ -52,10 +52,10 @@ public abstract class AbstractGraphics2DAdapter implements Graphics2DAdapter {
      * @return the generated BufferedImage
      */
     protected BufferedImage paintToBufferedImage(
-            org.apache.xmlgraphics.java2d.Graphics2DImagePainter painter, 
+            org.apache.xmlgraphics.java2d.Graphics2DImagePainter painter,
              RendererContextWrapper context, int resolution, boolean gray, boolean withAlpha) {
-        int bmw = (int)Math.ceil(UnitConv.mpt2px(context.getWidth(), resolution));
-        int bmh = (int)Math.ceil(UnitConv.mpt2px(context.getHeight(), resolution));
+        int bmw = mpt2px(context.getWidth(), resolution);
+        int bmh = mpt2px(context.getHeight(), resolution);
         BufferedImage bi;
         if (gray) {
             if (withAlpha) {
@@ -72,10 +72,10 @@ public abstract class AbstractGraphics2DAdapter implements Graphics2DAdapter {
         }
         Graphics2D g2d = bi.createGraphics();
         try {
-            g2d.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, 
+            g2d.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS,
                     RenderingHints.VALUE_FRACTIONALMETRICS_ON);
             setRenderingHintsForBufferedImage(g2d);
-            
+
             g2d.setBackground(Color.white);
             g2d.setColor(Color.black);
             if (!withAlpha) {
@@ -102,6 +102,17 @@ public abstract class AbstractGraphics2DAdapter implements Graphics2DAdapter {
         return bi;
     }
 
+    /**
+     * Converts millipoints to pixels
+     *
+     * @param unit the unit to convert in mpts
+     * @param resolution the target resolution
+     * @return the converted unit in pixels
+     */
+    protected int mpt2px(int unit, int resolution) {
+        return (int)Math.ceil(UnitConv.mpt2px(unit, resolution));
+    }
+
     private static BufferedImage createGrayBufferedImageWithAlpha(int width, int height) {
         BufferedImage bi;
         boolean alphaPremultiplied = true;
@@ -130,18 +141,18 @@ public abstract class AbstractGraphics2DAdapter implements Graphics2DAdapter {
      * @param g2d the Graphics2D instance
      */
     protected void setRenderingHintsForBufferedImage(Graphics2D g2d) {
-        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
+        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
             RenderingHints.VALUE_ANTIALIAS_OFF);
-        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, 
+        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
             RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
     }
 
     /** {@inheritDoc} */
-    public void paintImage(Graphics2DImagePainter painter, 
+    public void paintImage(Graphics2DImagePainter painter,
             RendererContext context,
             int x, int y, int width, int height) throws IOException {
         paintImage((org.apache.xmlgraphics.java2d.Graphics2DImagePainter)painter,
                 context, x, y, width, height);
     }
-    
+
 }
