@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractPathOrientedRenderer.java 663482 2008-06-05 07:06:37Z jeremias $ */
+/* $Id: AbstractPathOrientedRenderer.java 746664 2009-02-22 12:40:44Z jeremias $ */
 
 package org.apache.fop.render;
 
@@ -31,12 +31,15 @@ import org.w3c.dom.Document;
 import org.apache.batik.parser.AWTTransformProducer;
 
 import org.apache.xmlgraphics.image.loader.ImageSize;
+import org.apache.xmlgraphics.util.QName;
+import org.apache.xmlgraphics.util.UnitConv;
 
 import org.apache.fop.area.Area;
 import org.apache.fop.area.Block;
 import org.apache.fop.area.BlockViewport;
 import org.apache.fop.area.CTM;
 import org.apache.fop.area.NormalFlow;
+import org.apache.fop.area.RegionReference;
 import org.apache.fop.area.RegionViewport;
 import org.apache.fop.area.Trait;
 import org.apache.fop.area.inline.ForeignObject;
@@ -46,7 +49,6 @@ import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.extensions.ExtensionElementMapping;
 import org.apache.fop.fonts.FontMetrics;
 import org.apache.fop.traits.BorderProps;
-import org.apache.fop.util.QName;
 
 /**
  * Abstract base class for renderers like PDF and PostScript where many painting operations
@@ -65,7 +67,7 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
     protected void handleBlockTraits(Block block) {
         int borderPaddingStart = block.getBorderAndPaddingWidthStart();
         int borderPaddingBefore = block.getBorderAndPaddingWidthBefore();
-        
+
         float startx = currentIPPosition / 1000f;
         float starty = currentBPPosition / 1000f;
         float width = block.getIPD() / 1000f;
@@ -96,19 +98,20 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
      */
     protected void handleRegionTraits(RegionViewport region) {
         Rectangle2D viewArea = region.getViewArea();
+        RegionReference referenceArea = region.getRegionReference();
         float startx = (float)(viewArea.getX() / 1000f);
         float starty = (float)(viewArea.getY() / 1000f);
         float width = (float)(viewArea.getWidth() / 1000f);
         float height = (float)(viewArea.getHeight() / 1000f);
 
-        if (region.getRegionReference().getRegionClass() == FO_REGION_BODY) {
-            currentBPPosition = region.getBorderAndPaddingWidthBefore();
-            currentIPPosition = region.getBorderAndPaddingWidthStart();
-        }
-        drawBackAndBorders(region, startx, starty, width, height);
+        // adjust the current position according to region borders and padding
+        currentBPPosition = referenceArea.getBorderAndPaddingWidthBefore();
+        currentIPPosition = referenceArea.getBorderAndPaddingWidthStart();
+        // draw background (traits are in the RegionViewport)
+        // and borders (traits are in the RegionReference)
+        drawBackAndBorders(region, referenceArea, startx, starty, width, height);
     }
 
-    
     /**
      * Draw the background and borders.
      * This draws the background and border traits for an area given
@@ -123,15 +126,57 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
     protected void drawBackAndBorders(Area area,
                     float startx, float starty,
                     float width, float height) {
+        drawBackAndBorders(area, area, startx, starty, width, height);
+    }
+
+    /**
+     * Draw the background and borders.
+     * This draws the background and border traits for an area given
+     * the position.
+     *
+     * @param backgroundArea the area to get the background traits from
+     * @param borderArea the area to get the border traits from
+     * @param startx the start x position
+     * @param starty the start y position
+     * @param width the width of the area
+     * @param height the height of the area
+     */
+    protected void drawBackAndBorders(Area backgroundArea, Area borderArea,
+                    float startx, float starty,
+                    float width, float height) {
         // draw background then border
 
-        BorderProps bpsBefore = (BorderProps)area.getTrait(Trait.BORDER_BEFORE);
-        BorderProps bpsAfter = (BorderProps)area.getTrait(Trait.BORDER_AFTER);
-        BorderProps bpsStart = (BorderProps)area.getTrait(Trait.BORDER_START);
-        BorderProps bpsEnd = (BorderProps)area.getTrait(Trait.BORDER_END);
+        BorderProps bpsBefore = (BorderProps)borderArea.getTrait(Trait.BORDER_BEFORE);
+        BorderProps bpsAfter = (BorderProps)borderArea.getTrait(Trait.BORDER_AFTER);
+        BorderProps bpsStart = (BorderProps)borderArea.getTrait(Trait.BORDER_START);
+        BorderProps bpsEnd = (BorderProps)borderArea.getTrait(Trait.BORDER_END);
 
-        Trait.Background back;
-        back = (Trait.Background)area.getTrait(Trait.BACKGROUND);
+        drawBackground(startx, starty, width, height,
+                (Trait.Background) backgroundArea.getTrait(Trait.BACKGROUND),
+                bpsBefore, bpsAfter, bpsStart, bpsEnd);
+        drawBorders(startx, starty, width, height,
+                bpsBefore, bpsAfter, bpsStart, bpsEnd);
+    }
+
+    /**
+     * Draw the background.
+     * This draws the background given the position and the traits.
+     *
+     * @param startx the start x position
+     * @param starty the start y position
+     * @param width the width of the area
+     * @param height the height of the area
+     * @param back the background traits
+     * @param bpsBefore the border-before traits
+     * @param bpsAfter the border-after traits
+     * @param bpsStart the border-start traits
+     * @param bpsEnd the border-end traits
+     */
+    protected void drawBackground(float startx, float starty,
+                    float width, float height,
+                    Trait.Background back,
+                    BorderProps bpsBefore, BorderProps bpsAfter,
+                    BorderProps bpsStart, BorderProps bpsEnd) {
         if (back != null) {
             endTextObject();
 
@@ -160,13 +205,13 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
                 fillRect(sx, sy, paddRectWidth, paddRectHeight);
             }
             if (back.getImageInfo() != null) {
-                ImageSize imageSize = back.getImageInfo().getSize(); 
+                ImageSize imageSize = back.getImageInfo().getSize();
                 saveGraphicsState();
                 clipRect(sx, sy, paddRectWidth, paddRectHeight);
-                int horzCount = (int)((paddRectWidth 
-                        * 1000 / imageSize.getWidthMpt()) + 1.0f); 
-                int vertCount = (int)((paddRectHeight 
-                        * 1000 / imageSize.getHeightMpt()) + 1.0f); 
+                int horzCount = (int)((paddRectWidth
+                        * 1000 / imageSize.getWidthMpt()) + 1.0f);
+                int vertCount = (int)((paddRectHeight
+                        * 1000 / imageSize.getHeightMpt()) + 1.0f);
                 if (back.getRepeat() == EN_NOREPEAT) {
                     horzCount = 1;
                     vertCount = 1;
@@ -189,7 +234,7 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
                         // place once
                         Rectangle2D pos;
                         // Image positions are relative to the currentIP/BP
-                        pos = new Rectangle2D.Float(sx - currentIPPosition 
+                        pos = new Rectangle2D.Float(sx - currentIPPosition
                                                         + (x * imageSize.getWidthMpt()),
                                                     sy - currentBPPosition
                                                         + (y * imageSize.getHeightMpt()),
@@ -198,15 +243,38 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
                         drawImage(back.getURL(), pos);
                     }
                 }
-                
+
                 restoreGraphicsState();
             }
         }
+    }
 
+    /**
+     * Draw the borders.
+     * This draws the border traits given the position and the traits.
+     *
+     * @param startx the start x position
+     * @param starty the start y position
+     * @param width the width of the area
+     * @param height the height of the area
+     * @param bpsBefore the border-before traits
+     * @param bpsAfter the border-after traits
+     * @param bpsStart the border-start traits
+     * @param bpsEnd the border-end traits
+     */
+    protected void drawBorders(float startx, float starty,
+                    float width, float height,
+                    BorderProps bpsBefore, BorderProps bpsAfter,
+                    BorderProps bpsStart, BorderProps bpsEnd) {
         Rectangle2D.Float borderRect = new Rectangle2D.Float(startx, starty, width, height);
         drawBorders(borderRect, bpsBefore, bpsAfter, bpsStart, bpsEnd);
     }
 
+    private static final int BEFORE = 0;
+    private static final int END = 1;
+    private static final int AFTER = 2;
+    private static final int START = 3;
+
     /**
      * Draws borders.
      * @param borderRect the border rectangle
@@ -215,47 +283,48 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
      * @param bpsStart the border specification on the start side
      * @param bpsEnd the border specification on the end side
      */
-    protected void drawBorders(Rectangle2D.Float borderRect, 
+    protected void drawBorders(Rectangle2D.Float borderRect,
             BorderProps bpsBefore, BorderProps bpsAfter, BorderProps bpsStart, BorderProps bpsEnd) {
+        //TODO generalize each of the four conditions into using a parameterized drawBorder()
+        boolean[] border = new boolean[] {
+                (bpsBefore != null), (bpsEnd != null),
+                (bpsAfter != null), (bpsStart != null)};
         float startx = borderRect.x;
         float starty = borderRect.y;
         float width = borderRect.width;
         float height = borderRect.height;
-        boolean[] b = new boolean[] {
-            (bpsBefore != null), (bpsEnd != null), 
-            (bpsAfter != null), (bpsStart != null)};
-        if (!b[0] && !b[1] && !b[2] && !b[3]) {
-            return;
-        }
-        float[] bw = new float[] {
-            (b[0] ? bpsBefore.width / 1000f : 0.0f),
-            (b[1] ? bpsEnd.width / 1000f : 0.0f),
-            (b[2] ? bpsAfter.width / 1000f : 0.0f),
-            (b[3] ? bpsStart.width / 1000f : 0.0f)};
+        float[] borderWidth = new float[] {
+            (border[BEFORE] ? bpsBefore.width / 1000f : 0.0f),
+            (border[END] ? bpsEnd.width / 1000f : 0.0f),
+            (border[AFTER] ? bpsAfter.width / 1000f : 0.0f),
+            (border[START] ? bpsStart.width / 1000f : 0.0f)};
         float[] clipw = new float[] {
-            BorderProps.getClippedWidth(bpsBefore) / 1000f,    
-            BorderProps.getClippedWidth(bpsEnd) / 1000f,    
-            BorderProps.getClippedWidth(bpsAfter) / 1000f,    
+            BorderProps.getClippedWidth(bpsBefore) / 1000f,
+            BorderProps.getClippedWidth(bpsEnd) / 1000f,
+            BorderProps.getClippedWidth(bpsAfter) / 1000f,
             BorderProps.getClippedWidth(bpsStart) / 1000f};
-        starty += clipw[0];
-        height -= clipw[0];
-        height -= clipw[2];
-        startx += clipw[3];
-        width -= clipw[3];
-        width -= clipw[1];
-        
+        starty += clipw[BEFORE];
+        height -= clipw[BEFORE];
+        height -= clipw[AFTER];
+        startx += clipw[START];
+        width -= clipw[START];
+        width -= clipw[END];
+
         boolean[] slant = new boolean[] {
-            (b[3] && b[0]), (b[0] && b[1]), (b[1] && b[2]), (b[2] && b[3])};
+            (border[START] && border[BEFORE]),
+            (border[BEFORE] && border[END]),
+            (border[END] && border[AFTER]),
+            (border[AFTER] && border[START])};
         if (bpsBefore != null) {
             endTextObject();
 
             float sx1 = startx;
-            float sx2 = (slant[0] ? sx1 + bw[3] - clipw[3] : sx1);
+            float sx2 = (slant[BEFORE] ? sx1 + borderWidth[START] - clipw[START] : sx1);
             float ex1 = startx + width;
-            float ex2 = (slant[1] ? ex1 - bw[1] + clipw[1] : ex1);
-            float outery = starty - clipw[0];
-            float clipy = outery + clipw[0];
-            float innery = outery + bw[0];
+            float ex2 = (slant[END] ? ex1 - borderWidth[END] + clipw[END] : ex1);
+            float outery = starty - clipw[BEFORE];
+            float clipy = outery + clipw[BEFORE];
+            float innery = outery + borderWidth[BEFORE];
 
             saveGraphicsState();
             moveTo(sx1, clipy);
@@ -263,10 +332,10 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
             float ex1a = ex1;
             if (bpsBefore.mode == BorderProps.COLLAPSE_OUTER) {
                 if (bpsStart != null && bpsStart.mode == BorderProps.COLLAPSE_OUTER) {
-                    sx1a -= clipw[3];
+                    sx1a -= clipw[START];
                 }
                 if (bpsEnd != null && bpsEnd.mode == BorderProps.COLLAPSE_OUTER) {
-                    ex1a += clipw[1];
+                    ex1a += clipw[END];
                 }
                 lineTo(sx1a, outery);
                 lineTo(ex1a, outery);
@@ -276,7 +345,7 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
             lineTo(sx2, innery);
             closePath();
             clip();
-            drawBorderLine(sx1a, outery, ex1a, innery, true, true, 
+            drawBorderLine(sx1a, outery, ex1a, innery, true, true,
                     bpsBefore.style, bpsBefore.color);
             restoreGraphicsState();
         }
@@ -284,23 +353,23 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
             endTextObject();
 
             float sy1 = starty;
-            float sy2 = (slant[1] ? sy1 + bw[0] - clipw[0] : sy1);
+            float sy2 = (slant[END] ? sy1 + borderWidth[BEFORE] - clipw[BEFORE] : sy1);
             float ey1 = starty + height;
-            float ey2 = (slant[2] ? ey1 - bw[2] + clipw[2] : ey1);
-            float outerx = startx + width + clipw[1];
-            float clipx = outerx - clipw[1];
-            float innerx = outerx - bw[1];
-            
+            float ey2 = (slant[AFTER] ? ey1 - borderWidth[AFTER] + clipw[AFTER] : ey1);
+            float outerx = startx + width + clipw[END];
+            float clipx = outerx - clipw[END];
+            float innerx = outerx - borderWidth[END];
+
             saveGraphicsState();
             moveTo(clipx, sy1);
             float sy1a = sy1;
             float ey1a = ey1;
             if (bpsEnd.mode == BorderProps.COLLAPSE_OUTER) {
                 if (bpsBefore != null && bpsBefore.mode == BorderProps.COLLAPSE_OUTER) {
-                    sy1a -= clipw[0];
+                    sy1a -= clipw[BEFORE];
                 }
                 if (bpsAfter != null && bpsAfter.mode == BorderProps.COLLAPSE_OUTER) {
-                    ey1a += clipw[2];
+                    ey1a += clipw[AFTER];
                 }
                 lineTo(outerx, sy1a);
                 lineTo(outerx, ey1a);
@@ -317,12 +386,12 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
             endTextObject();
 
             float sx1 = startx;
-            float sx2 = (slant[3] ? sx1 + bw[3] - clipw[3] : sx1);
+            float sx2 = (slant[START] ? sx1 + borderWidth[START] - clipw[START] : sx1);
             float ex1 = startx + width;
-            float ex2 = (slant[2] ? ex1 - bw[1] + clipw[1] : ex1);
-            float outery = starty + height + clipw[2];
-            float clipy = outery - clipw[2];
-            float innery = outery - bw[2];
+            float ex2 = (slant[AFTER] ? ex1 - borderWidth[END] + clipw[END] : ex1);
+            float outery = starty + height + clipw[AFTER];
+            float clipy = outery - clipw[AFTER];
+            float innery = outery - borderWidth[AFTER];
 
             saveGraphicsState();
             moveTo(ex1, clipy);
@@ -330,10 +399,10 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
             float ex1a = ex1;
             if (bpsAfter.mode == BorderProps.COLLAPSE_OUTER) {
                 if (bpsStart != null && bpsStart.mode == BorderProps.COLLAPSE_OUTER) {
-                    sx1a -= clipw[3];
+                    sx1a -= clipw[START];
                 }
                 if (bpsEnd != null && bpsEnd.mode == BorderProps.COLLAPSE_OUTER) {
-                    ex1a += clipw[1];
+                    ex1a += clipw[END];
                 }
                 lineTo(ex1a, outery);
                 lineTo(sx1a, outery);
@@ -350,12 +419,12 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
             endTextObject();
 
             float sy1 = starty;
-            float sy2 = (slant[0] ? sy1 + bw[0] - clipw[0] : sy1);
+            float sy2 = (slant[BEFORE] ? sy1 + borderWidth[BEFORE] - clipw[BEFORE] : sy1);
             float ey1 = sy1 + height;
-            float ey2 = (slant[3] ? ey1 - bw[2] + clipw[2] : ey1);
-            float outerx = startx - clipw[3];
-            float clipx = outerx + clipw[3];
-            float innerx = outerx + bw[3];
+            float ey2 = (slant[START] ? ey1 - borderWidth[AFTER] + clipw[AFTER] : ey1);
+            float outerx = startx - clipw[START];
+            float clipx = outerx + clipw[START];
+            float innerx = outerx + borderWidth[START];
 
             saveGraphicsState();
             moveTo(clipx, ey1);
@@ -363,10 +432,10 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
             float ey1a = ey1;
             if (bpsStart.mode == BorderProps.COLLAPSE_OUTER) {
                 if (bpsBefore != null && bpsBefore.mode == BorderProps.COLLAPSE_OUTER) {
-                    sy1a -= clipw[0];
+                    sy1a -= clipw[BEFORE];
                 }
                 if (bpsAfter != null && bpsAfter.mode == BorderProps.COLLAPSE_OUTER) {
-                    ey1a += clipw[2];
+                    ey1a += clipw[AFTER];
                 }
                 lineTo(outerx, ey1a);
                 lineTo(outerx, sy1a);
@@ -380,36 +449,36 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
             restoreGraphicsState();
         }
     }
-    
-    /** 
+
+    /**
      * Common method to render the background and borders for any inline area.
      * The all borders and padding are drawn outside the specified area.
      * @param area the inline area for which the background, border and padding is to be
      * rendered
      */
     protected void renderInlineAreaBackAndBorders(InlineArea area) {
-        float x = currentIPPosition / 1000f;
-        float y = (currentBPPosition + area.getOffset()) / 1000f;
-        float width = area.getIPD() / 1000f;
-        float height = area.getBPD() / 1000f;
         float borderPaddingStart = area.getBorderAndPaddingWidthStart() / 1000f;
         float borderPaddingBefore = area.getBorderAndPaddingWidthBefore() / 1000f;
-        float bpwidth = borderPaddingStart 
+        float bpwidth = borderPaddingStart
                 + (area.getBorderAndPaddingWidthEnd() / 1000f);
         float bpheight = borderPaddingBefore
                 + (area.getBorderAndPaddingWidthAfter() / 1000f);
-        
+
+        float height = area.getBPD() / 1000f;
         if (height != 0.0f || bpheight != 0.0f && bpwidth != 0.0f) {
+            float x = currentIPPosition / 1000f;
+            float y = (currentBPPosition + area.getOffset()) / 1000f;
+            float width = area.getIPD() / 1000f;
             drawBackAndBorders(area, x, y - borderPaddingBefore
                                 , width + bpwidth
                                 , height + bpheight);
         }
-        
     }
-    
-    private static final QName FOX_TRANSFORM
+
+    /** Constant for the fox:transform extension attribute */
+    protected static final QName FOX_TRANSFORM
             = new QName(ExtensionElementMapping.URI, "fox:transform");
-    
+
     /** {@inheritDoc} */
     protected void renderBlockViewport(BlockViewport bv, List children) {
         // clip and position viewport if necessary
@@ -419,26 +488,24 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
         int saveBP = currentBPPosition;
 
         CTM ctm = bv.getCTM();
-        int borderPaddingStart = bv.getBorderAndPaddingWidthStart();
         int borderPaddingBefore = bv.getBorderAndPaddingWidthBefore();
-        //This is the content-rect
-        float width = (float)bv.getIPD() / 1000f;
-        float height = (float)bv.getBPD() / 1000f;
 
-        if (bv.getPositioning() == Block.ABSOLUTE
-                || bv.getPositioning() == Block.FIXED) {
+        int positioning = bv.getPositioning();
+        if (positioning == Block.ABSOLUTE || positioning == Block.FIXED) {
 
             //For FIXED, we need to break out of the current viewports to the
             //one established by the page. We save the state stack for restoration
             //after the block-container has been painted. See below.
             List breakOutList = null;
-            if (bv.getPositioning() == Block.FIXED) {
+            if (positioning == Block.FIXED) {
                 breakOutList = breakOutOfStateStack();
             }
-            
+
             AffineTransform positionTransform = new AffineTransform();
             positionTransform.translate(bv.getXOffset(), bv.getYOffset());
-            
+
+            int borderPaddingStart = bv.getBorderAndPaddingWidthStart();
+
             //"left/"top" (bv.getX/YOffset()) specify the position of the content rectangle
             positionTransform.translate(-borderPaddingStart, -borderPaddingBefore);
 
@@ -450,41 +517,64 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
                 positionTransform.concatenate(freeTransform);
             }
 
-            saveGraphicsState();
             //Viewport position
-            concatenateTransformationMatrix(mptToPt(positionTransform));
-            
+            if (!positionTransform.isIdentity()) {
+                establishTransformationMatrix(positionTransform);
+            }
+
+            //This is the content-rect
+            float width = bv.getIPD() / 1000f;
+            float height = bv.getBPD() / 1000f;
+
             //Background and borders
-            float bpwidth = (borderPaddingStart + bv.getBorderAndPaddingWidthEnd()) / 1000f;
-            float bpheight = (borderPaddingBefore + bv.getBorderAndPaddingWidthAfter()) / 1000f;
-            drawBackAndBorders(bv, 0, 0, width + bpwidth, height + bpheight);
+            float borderPaddingWidth
+                = (borderPaddingStart + bv.getBorderAndPaddingWidthEnd()) / 1000f;
+            float borderPaddingHeight
+                = (borderPaddingBefore + bv.getBorderAndPaddingWidthAfter()) / 1000f;
+            drawBackAndBorders(bv, 0, 0, width + borderPaddingWidth, height + borderPaddingHeight);
 
             //Shift to content rectangle after border painting
             AffineTransform contentRectTransform = new AffineTransform();
             contentRectTransform.translate(borderPaddingStart, borderPaddingBefore);
-            concatenateTransformationMatrix(mptToPt(contentRectTransform));
-            
+
+            if (!contentRectTransform.isIdentity()) {
+                establishTransformationMatrix(contentRectTransform);
+            }
+
             //Clipping
             if (bv.getClip()) {
                 clipRect(0f, 0f, width, height);
             }
 
-            saveGraphicsState();
             //Set up coordinate system for content rectangle
             AffineTransform contentTransform = ctm.toAffineTransform();
-            concatenateTransformationMatrix(mptToPt(contentTransform));
-            
+            if (!contentTransform.isIdentity()) {
+                establishTransformationMatrix(contentTransform);
+            }
+
             currentIPPosition = 0;
             currentBPPosition = 0;
             renderBlocks(bv, children);
 
-            restoreGraphicsState();
-            restoreGraphicsState();
-            
-            if (breakOutList != null) {
-                restoreStateStackAfterBreakOut(breakOutList);
+            if (!contentTransform.isIdentity()) {
+                restoreGraphicsState();
             }
-            
+
+            if (!contentRectTransform.isIdentity()) {
+                restoreGraphicsState();
+            }
+
+            if (!positionTransform.isIdentity()) {
+                restoreGraphicsState();
+            }
+
+            //For FIXED, we need to restore break out now we are done
+            if (positioning == Block.FIXED) {
+                if (breakOutList != null) {
+                    restoreStateStackAfterBreakOut(breakOutList);
+                }
+            }
+
             currentIPPosition = saveIP;
             currentBPPosition = saveBP;
         } else {
@@ -499,16 +589,16 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
 
             CTM tempctm = new CTM(containingIPPosition, currentBPPosition);
             ctm = tempctm.multiply(ctm);
-            
+
             //Now adjust for border/padding
             currentBPPosition += borderPaddingBefore;
 
             Rectangle2D clippingRect = null;
             if (bv.getClip()) {
-                clippingRect = new Rectangle(currentIPPosition, currentBPPosition, 
+                clippingRect = new Rectangle(currentIPPosition, currentBPPosition,
                         bv.getIPD(), bv.getBPD());
             }
-            
+
             startVParea(ctm, clippingRect);
             currentIPPosition = 0;
             currentBPPosition = 0;
@@ -517,8 +607,8 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
 
             currentIPPosition = saveIP;
             currentBPPosition = saveBP;
-            
-            currentBPPosition += (int)(bv.getAllocBPD());
+
+            currentBPPosition += (bv.getAllocBPD());
         }
     }
 
@@ -533,10 +623,9 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
         at.translate(currentIPPosition, currentBPPosition);
         at.translate(block.getXOffset(), block.getYOffset());
         at.translate(0, block.getSpaceBefore());
-        
+
         if (!at.isIdentity()) {
-            saveGraphicsState();
-            concatenateTransformationMatrix(mptToPt(at));
+            establishTransformationMatrix(at);
         }
 
         currentIPPosition = 0;
@@ -551,12 +640,12 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
         if (!at.isIdentity()) {
             restoreGraphicsState();
         }
-        
+
         // stacked and relative blocks effect stacking
         currentIPPosition = saveIP;
         currentBPPosition = saveBP;
     }
-    
+
     /** {@inheritDoc} */
     protected void renderFlow(NormalFlow flow) {
         // save position and offset
@@ -566,32 +655,31 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
         //Establish a new coordinate system
         AffineTransform at = new AffineTransform();
         at.translate(currentIPPosition, currentBPPosition);
-        
+
         if (!at.isIdentity()) {
-            saveGraphicsState();
-            concatenateTransformationMatrix(mptToPt(at));
+            establishTransformationMatrix(at);
         }
 
         currentIPPosition = 0;
         currentBPPosition = 0;
         super.renderFlow(flow);
-        
+
         if (!at.isIdentity()) {
             restoreGraphicsState();
         }
-        
+
         // stacked and relative blocks effect stacking
         currentIPPosition = saveIP;
         currentBPPosition = saveBP;
     }
-    
+
     /**
      * Concatenates the current transformation matrix with the given one, therefore establishing
      * a new coordinate system.
      * @param at the transformation matrix to process (coordinates in points)
      */
     protected abstract void concatenateTransformationMatrix(AffineTransform at);
-    
+
     /**
      * Render an inline viewport.
      * This renders an inline viewport by clipping if necessary.
@@ -603,10 +691,10 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
         float y = (currentBPPosition + viewport.getOffset()) / 1000f;
         float width = viewport.getIPD() / 1000f;
         float height = viewport.getBPD() / 1000f;
-        // TODO: Calculate the border rect correctly. 
+        // TODO: Calculate the border rect correctly.
         float borderPaddingStart = viewport.getBorderAndPaddingWidthStart() / 1000f;
         float borderPaddingBefore = viewport.getBorderAndPaddingWidthBefore() / 1000f;
-        float bpwidth = borderPaddingStart 
+        float bpwidth = borderPaddingStart
                 + (viewport.getBorderAndPaddingWidthEnd() / 1000f);
         float bpheight = borderPaddingBefore
                 + (viewport.getBorderAndPaddingWidthAfter() / 1000f);
@@ -630,7 +718,7 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
      * @param breakOutList the state stack to restore.
      */
     protected abstract void restoreStateStackAfterBreakOut(List breakOutList);
-    
+
     /**
      * Breaks out of the state stack to handle fixed block-containers.
      * @return the saved state stack to recreate later
@@ -639,16 +727,16 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
 
     /** Saves the graphics state of the rendering engine. */
     protected abstract void saveGraphicsState();
-    
+
     /** Restores the last graphics state of the rendering engine. */
     protected abstract void restoreGraphicsState();
 
     /** Indicates the beginning of a text object. */
     protected abstract void beginTextObject();
-    
+
     /** Indicates the end of a text object. */
     protected abstract void endTextObject();
-    
+
     /**
      * Paints the text decoration marks.
      * @param fm Current typeface
@@ -657,10 +745,10 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
      * @param baseline position of the baseline
      * @param startx start IPD
      */
-    protected void renderTextDecoration(FontMetrics fm, int fontsize, InlineArea inline, 
+    protected void renderTextDecoration(FontMetrics fm, int fontsize, InlineArea inline,
                     int baseline, int startx) {
-        boolean hasTextDeco = inline.hasUnderline() 
-                || inline.hasOverline() 
+        boolean hasTextDeco = inline.hasUnderline()
+                || inline.hasOverline()
                 || inline.hasLineThrough();
         if (hasTextDeco) {
             endTextObject();
@@ -671,22 +759,22 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
             if (inline.hasUnderline()) {
                 Color ct = (Color) inline.getTrait(Trait.UNDERLINE_COLOR);
                 float y = baseline - descender / 2f;
-                drawBorderLine(startx / 1000f, (y - halfLineWidth) / 1000f, 
-                        endx, (y + halfLineWidth) / 1000f, 
+                drawBorderLine(startx / 1000f, (y - halfLineWidth) / 1000f,
+                        endx, (y + halfLineWidth) / 1000f,
                         true, true, Constants.EN_SOLID, ct);
             }
             if (inline.hasOverline()) {
                 Color ct = (Color) inline.getTrait(Trait.OVERLINE_COLOR);
                 float y = (float)(baseline - (1.1 * capHeight));
-                drawBorderLine(startx / 1000f, (y - halfLineWidth) / 1000f, 
-                        endx, (y + halfLineWidth) / 1000f, 
+                drawBorderLine(startx / 1000f, (y - halfLineWidth) / 1000f,
+                        endx, (y + halfLineWidth) / 1000f,
                         true, true, Constants.EN_SOLID, ct);
             }
             if (inline.hasLineThrough()) {
                 Color ct = (Color) inline.getTrait(Trait.LINETHROUGH_COLOR);
                 float y = (float)(baseline - (0.45 * capHeight));
-                drawBorderLine(startx / 1000f, (y - halfLineWidth) / 1000f, 
-                        endx, (y + halfLineWidth) / 1000f, 
+                drawBorderLine(startx / 1000f, (y - halfLineWidth) / 1000f,
+                        endx, (y + halfLineWidth) / 1000f,
                         true, true, Constants.EN_SOLID, ct);
             }
         }
@@ -694,7 +782,7 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
 
     /** Clip using the current path. */
     protected abstract void clip();
-        
+
     /**
      * Clip using a rectangular area.
      * @param x the x coordinate (in points)
@@ -703,28 +791,28 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
      * @param height the height of the rectangle (in points)
      */
     protected abstract void clipRect(float x, float y, float width, float height);
-    
+
     /**
-     * Moves the current point to (x, y), omitting any connecting line segment. 
+     * Moves the current point to (x, y), omitting any connecting line segment.
      * @param x x coordinate
      * @param y y coordinate
      */
     protected abstract void moveTo(float x, float y);
-    
+
     /**
-     * Appends a straight line segment from the current point to (x, y). The 
-     * new current point is (x, y). 
+     * Appends a straight line segment from the current point to (x, y). The
+     * new current point is (x, y).
      * @param x x coordinate
      * @param y y coordinate
      */
     protected abstract void lineTo(float x, float y);
-    
+
     /**
-     * Closes the current subpath by appending a straight line segment from 
+     * Closes the current subpath by appending a straight line segment from
      * the current point to the starting point of the subpath.
      */
     protected abstract void closePath();
-    
+
     /**
      * Fill a rectangular area.
      * @param x the x coordinate
@@ -740,7 +828,7 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
      * @param fill true to set the fill color, false for the foreground color
      */
     protected abstract void updateColor(Color col, boolean fill);
-    
+
     /**
      * Draw an image at the indicated location.
      * @param url the URI/URL of the image
@@ -748,7 +836,7 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
      * @param foreignAttributes an optional Map with foreign attributes, may be null
      */
     protected abstract void drawImage(String url, Rectangle2D pos, Map foreignAttributes);
-    
+
     /**
      * Draw an image at the indicated location.
      * @param url the URI/URL of the image
@@ -757,7 +845,7 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
     protected final void drawImage(String url, Rectangle2D pos) {
         drawImage(url, pos, null);
     }
-    
+
     /**
      * Draw a border segment of an XSL-FO style border.
      * @param x1 starting x coordinate
@@ -765,12 +853,12 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
      * @param x2 ending x coordinate
      * @param y2 ending y coordinate
      * @param horz true for horizontal border segments, false for vertical border segments
-     * @param startOrBefore true for border segments on the start or before edge, 
+     * @param startOrBefore true for border segments on the start or before edge,
      *                      false for end or after.
      * @param style the border style (one of Constants.EN_DASHED etc.)
      * @param col the color for the border segment
      */
-    protected abstract void drawBorderLine(float x1, float y1, float x2, float y2, 
+    protected abstract void drawBorderLine(float x1, float y1, float x2, float y2,
             boolean horz, boolean startOrBefore, int style, Color col);
 
     /** {@inheritDoc} */
@@ -780,5 +868,17 @@ public abstract class AbstractPathOrientedRenderer extends PrintRenderer {
         String ns = fo.getNameSpace();
         renderDocument(doc, ns, pos, fo.getForeignAttributes());
     }
-    
+
+    /**
+     * Establishes a new coordinate system with the given transformation matrix.
+     * The current graphics state is saved and the new coordinate system is concatenated.
+     * @param block
+     *
+     * @param at the transformation matrix
+     */
+    protected void establishTransformationMatrix(AffineTransform at) {
+        saveGraphicsState();
+        concatenateTransformationMatrix(UnitConv.mptToPt(at));
+    }
+
 }
