@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractRenderer.java 651302 2008-04-24 16:08:53Z jeremias $ */
+/* $Id: AbstractRenderer.java 932497 2010-04-09 16:34:29Z vhennebert $ */
 
 package org.apache.fop.render;
 
@@ -34,6 +34,7 @@ import org.w3c.dom.Document;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.fop.ResourceEventProducer;
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.area.Area;
@@ -54,7 +55,6 @@ import org.apache.fop.area.RegionReference;
 import org.apache.fop.area.RegionViewport;
 import org.apache.fop.area.Span;
 import org.apache.fop.area.Trait;
-import org.apache.fop.area.inline.Character;
 import org.apache.fop.area.inline.Container;
 import org.apache.fop.area.inline.ForeignObject;
 import org.apache.fop.area.inline.Image;
@@ -75,12 +75,12 @@ import org.apache.fop.fonts.FontInfo;
  * top level processing of the area tree and adds some abstract methods to
  * handle viewports. This keeps track of the current block and inline position.
  */
-public abstract class AbstractRenderer 
+public abstract class AbstractRenderer
          implements Renderer, Constants {
 
     /** logging instance */
     protected static Log log = LogFactory.getLog("org.apache.fop.render");
-    
+
     /**
      * user agent
      */
@@ -110,31 +110,32 @@ public abstract class AbstractRenderer
 
     /** the currently active PageViewport */
     protected PageViewport currentPageViewport;
-    
-    private Set warnedXMLHandlers;
-    
-    /**
-     *  {@inheritDoc}
-     */
-    public abstract void setupFontInfo(FontInfo fontInfo);
 
-    /**
-     *  {@inheritDoc}
-     */
+    private Set warnedXMLHandlers;
+
+    /** {@inheritDoc} */
+    public abstract void setupFontInfo(FontInfo fontInfo) throws FOPException;
+
+    /** {@inheritDoc} */
     public void setUserAgent(FOUserAgent agent) {
         userAgent = agent;
     }
 
-    /**
-     *  @return the associated <code>FOUserAgent</code>
-     */
+    /** {@inheritDoc} */
     public FOUserAgent getUserAgent() {
+        if (userAgent == null) {
+            throw new IllegalStateException("FOUserAgent has not been set on Renderer");
+        }
         return userAgent;
     }
 
     /** {@inheritDoc} */
     public void startRenderer(OutputStream outputStream)
-        throws IOException { }
+            throws IOException {
+        if (userAgent == null) {
+            throw new IllegalStateException("FOUserAgent has not been set on Renderer");
+        }
+    }
 
     /** {@inheritDoc} */
     public void stopRenderer()
@@ -152,7 +153,7 @@ public abstract class AbstractRenderer
     }
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
     public void processOffDocumentItem(OffDocumentItem odi) { }
 
@@ -160,17 +161,17 @@ public abstract class AbstractRenderer
     public Graphics2DAdapter getGraphics2DAdapter() {
         return null;
     }
-    
+
     /** {@inheritDoc} */
     public ImageAdapter getImageAdapter() {
         return null;
     }
-    
+
     /** @return the current PageViewport or null, if none is active */
     protected PageViewport getCurrentPageViewport() {
         return this.currentPageViewport;
     }
-    
+
     /** {@inheritDoc} */
     public void preparePage(PageViewport page) { }
 
@@ -211,7 +212,7 @@ public abstract class AbstractRenderer
     public void startPageSequence(LineArea seqTitle) {
         //do nothing
     }
-    
+
     /** {@inheritDoc} */
     public void startPageSequence(PageSequence pageSequence) {
         startPageSequence(pageSequence.getTitle());
@@ -244,15 +245,25 @@ public abstract class AbstractRenderer
         last here. */
         RegionViewport viewport;
         viewport = page.getRegionViewport(FO_REGION_BEFORE);
-        renderRegionViewport(viewport);
+        if (viewport != null) {
+            renderRegionViewport(viewport);
+        }
         viewport = page.getRegionViewport(FO_REGION_START);
-        renderRegionViewport(viewport);
+        if (viewport != null) {
+            renderRegionViewport(viewport);
+        }
         viewport = page.getRegionViewport(FO_REGION_END);
-        renderRegionViewport(viewport);
+        if (viewport != null) {
+            renderRegionViewport(viewport);
+        }
         viewport = page.getRegionViewport(FO_REGION_AFTER);
-        renderRegionViewport(viewport);
+        if (viewport != null) {
+            renderRegionViewport(viewport);
+        }
         viewport = page.getRegionViewport(FO_REGION_BODY);
-        renderRegionViewport(viewport);
+        if (viewport != null) {
+            renderRegionViewport(viewport);
+        }
     }
 
     /**
@@ -264,34 +275,32 @@ public abstract class AbstractRenderer
      * @param port  The region viewport to be rendered
      */
     protected void renderRegionViewport(RegionViewport port) {
-        if (port != null) {
-            Rectangle2D view = port.getViewArea();
-            // The CTM will transform coordinates relative to
-            // this region-reference area into page coords, so
-            // set origin for the region to 0,0.
-            currentBPPosition = 0;
-            currentIPPosition = 0;
+        Rectangle2D view = port.getViewArea();
+        // The CTM will transform coordinates relative to
+        // this region-reference area into page coords, so
+        // set origin for the region to 0,0.
+        currentBPPosition = 0;
+        currentIPPosition = 0;
 
-            RegionReference regionReference = port.getRegionReference();
-            handleRegionTraits(port);
+        RegionReference regionReference = port.getRegionReference();
+        handleRegionTraits(port);
 
-            //  shouldn't the viewport have the CTM
-            startVParea(regionReference.getCTM(), port.isClip() ? view : null);
-            // do after starting viewport area
-            if (regionReference.getRegionClass() == FO_REGION_BODY) {
-                renderBodyRegion((BodyRegion) regionReference);
-            } else {
-                renderRegion(regionReference);
-            }
-            endVParea();
+        //  shouldn't the viewport have the CTM
+        startVParea(regionReference.getCTM(), port.isClip() ? view : null);
+        // do after starting viewport area
+        if (regionReference.getRegionClass() == FO_REGION_BODY) {
+            renderBodyRegion((BodyRegion) regionReference);
+        } else {
+            renderRegion(regionReference);
         }
+        endVParea();
     }
 
     /**
      * Establishes a new viewport area.
      *
      * @param ctm the coordinate transformation matrix to use
-     * @param clippingRect the clipping rectangle if the viewport should be clipping, 
+     * @param clippingRect the clipping rectangle if the viewport should be clipping,
      *                     null if no clipping is performed.
      */
     protected abstract void startVParea(CTM ctm, Rectangle2D clippingRect);
@@ -318,8 +327,7 @@ public abstract class AbstractRenderer
      * @param region  The region reference area
      */
     protected void renderRegion(RegionReference region) {
-        List blocks = region.getBlocks();
-        renderBlocks(null, blocks);
+        renderBlocks(null, region.getBlocks());
     }
 
     /**
@@ -396,7 +404,7 @@ public abstract class AbstractRenderer
         for (int count = 0; count < spans.size(); count++) {
             span = (Span) spans.get(count);
             for (int c = 0; c < span.getColumnCount(); c++) {
-                NormalFlow flow = (NormalFlow) span.getNormalFlow(c);
+                NormalFlow flow = span.getNormalFlow(c);
 
                 if (flow != null) {
                     currentBPPosition = saveSpanBPPos;
@@ -454,7 +462,7 @@ public abstract class AbstractRenderer
             if (bv.getClip()) {
                 clippingRect = new Rectangle(saveIP, saveBP, bv.getIPD(), bv.getBPD());
             }
-            
+
             CTM ctm = bv.getCTM();
             currentIPPosition = 0;
             currentBPPosition = 0;
@@ -487,20 +495,20 @@ public abstract class AbstractRenderer
      * @param block the block area
      */
     protected abstract void renderReferenceArea(Block block);
-    
+
     /**
      * Renders a list of block areas.
      *
      * @param parent  the parent block if the parent is a block, otherwise
-     *                a null value. 
+     *                a null value.
      * @param blocks  The block areas
      */
     protected void renderBlocks(Block parent, List blocks) {
         int saveIP = currentIPPosition;
-        int saveBP = currentBPPosition;
+//        int saveBP = currentBPPosition;
 
         // Calculate the position of the content rectangle.
-        if (parent != null && !Boolean.TRUE.equals(parent.getTrait(Trait.IS_VIEWPORT_AREA))) {
+        if (parent != null && !parent.getTraitAsBoolean(Trait.IS_VIEWPORT_AREA)) {
             currentBPPosition += parent.getBorderAndPaddingWidthBefore();
             /* This is unnecessary now as we're going to use the *-indent traits
             currentIPPosition += parent.getBorderAndPaddingWidthStart();
@@ -509,7 +517,7 @@ public abstract class AbstractRenderer
                 currentIPPosition += spaceStart.intValue();
             }*/
         }
-        
+
         // the position of the containing block is used for
         // absolutely positioned areas
         int contBP = currentBPPosition;
@@ -530,8 +538,8 @@ public abstract class AbstractRenderer
                 // a line area is rendered from the top left position
                 // of the line, each inline object is offset from there
                 LineArea line = (LineArea) obj;
-                currentIPPosition = contIP 
-                        + parent.getStartIndent() 
+                currentIPPosition = contIP
+                        + parent.getStartIndent()
                         + line.getStartIndent();
                 renderLineArea(line);
                 //InlineArea child = (InlineArea) line.getInlineAreas().get(0);
@@ -556,7 +564,7 @@ public abstract class AbstractRenderer
                 // simply move position
                 currentBPPosition += block.getAllocBPD();
             }
-        } else if (Boolean.TRUE.equals(block.getTrait(Trait.IS_REFERENCE_AREA))) {
+        } else if (block.getTraitAsBoolean(Trait.IS_REFERENCE_AREA)) {
             renderReferenceArea(block);
         } else {
             // save position and offset
@@ -630,28 +638,18 @@ public abstract class AbstractRenderer
     }
 
     /**
-     * Render the given Character.
-     * @param ch the character to render
-     * @deprecated Only TextArea should be used. This method will be removed eventually.
-     */
-    protected void renderCharacter(Character ch) {
-        currentIPPosition += ch.getAllocIPD();
-    }
-
-    /** 
      * Common method to render the background and borders for any inline area.
      * The all borders and padding are drawn outside the specified area.
      * @param area the inline area for which the background, border and padding is to be
      * rendered
      */
     protected abstract void renderInlineAreaBackAndBorders(InlineArea area);
-    
+
     /**
      * Render the given Space.
      * @param space the space to render
      */
     protected void renderInlineSpace(Space space) {
-        space.setBPD(0);
         renderInlineAreaBackAndBorders(space);
         // an inline space moves the inline progression position
         // for the current block by the width or height of the space
@@ -677,7 +675,7 @@ public abstract class AbstractRenderer
         int saveBP = currentBPPosition;
         Iterator iter = text.getChildAreas().iterator();
         while (iter.hasNext()) {
-            renderInlineArea((InlineArea) iter.next()); 
+            renderInlineArea((InlineArea) iter.next());
         }
         currentIPPosition = saveIP + text.getAllocIPD();
     }
@@ -710,7 +708,7 @@ public abstract class AbstractRenderer
         currentBPPosition += ip.getOffset();
         Iterator iter = ip.getChildAreas().iterator();
         while (iter.hasNext()) {
-            renderInlineArea((InlineArea) iter.next()); 
+            renderInlineArea((InlineArea) iter.next());
         }
         currentIPPosition = saveIP + ip.getAllocIPD();
         currentBPPosition = saveBP;
@@ -745,6 +743,8 @@ public abstract class AbstractRenderer
             renderContainer((Container) content);
         } else if (content instanceof ForeignObject) {
             renderForeignObject((ForeignObject) content, contpos);
+        } else if (content instanceof InlineBlockParent) {
+            renderInlineBlockParent((InlineBlockParent) content);
         }
         currentIPPosition += viewport.getAllocIPD();
         currentBPPosition = saveBP;
@@ -783,7 +783,7 @@ public abstract class AbstractRenderer
      * @param pos  The target position of the foreign object
      * (todo) Make renderForeignObject() protected
      */
-    public void renderForeignObject(ForeignObject fo, Rectangle2D pos) {
+    protected void renderForeignObject(ForeignObject fo, Rectangle2D pos) {
         // Default: do nothing.
         // Some renderers (ex. Text) don't support foreign objects.
     }
@@ -806,10 +806,12 @@ public abstract class AbstractRenderer
                     = new XMLHandlerConfigurator(userAgent);
                 configurator.configure(ctx, namespace);
                 handler.handleXML(ctx, doc, namespace);
-            } catch (Throwable t) {
+            } catch (Exception e) {
                 // could not handle document
-                log.error("Some XML content will be ignored. "
-                        + "Could not render XML", t);
+                ResourceEventProducer eventProducer
+                        = ResourceEventProducer.Provider.get(
+                            ctx.getUserAgent().getEventBroadcaster());
+                eventProducer.foreignXMLProcessingError(this, doc, namespace, e);
             }
         } else {
             if (warnedXMLHandlers == null) {
@@ -818,21 +820,13 @@ public abstract class AbstractRenderer
             if (!warnedXMLHandlers.contains(namespace)) {
                 // no handler found for document
                 warnedXMLHandlers.add(namespace);
-                log.warn("Some XML content will be ignored. "
-                        + "No handler defined for XML: " + namespace);
+                ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
+                        ctx.getUserAgent().getEventBroadcaster());
+                eventProducer.foreignXMLNoHandler(this, doc, namespace);
             }
         }
     }
 
-    /**
-     * Get the MIME type of the renderer.
-     * 
-     * @return   The MIME type of the renderer
-     */
-    public String getMimeType() {
-        return null;
-    }
-
     /**
      * Converts a millipoint-based transformation matrix to points.
      * @param at a millipoint-based transformation matrix
@@ -856,8 +850,10 @@ public abstract class AbstractRenderer
         double[] matrix = new double[6];
         at.getMatrix(matrix);
         //Convert to millipoints
-        matrix[4] = matrix[4] * 1000;
-        matrix[5] = matrix[5] * 1000;
+        //Math.round() because things like this can happen: 65.6 * 1000 = 65.599999999999999
+        //which is bad for testing
+        matrix[4] = Math.round(matrix[4] * 1000);
+        matrix[5] = Math.round(matrix[5] * 1000);
         return new AffineTransform(matrix);
     }
 }
