@@ -15,22 +15,27 @@
  * limitations under the License.
  */
 
-/* $Id: AFPExtensionHandler.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: AFPExtensionHandler.java 896856 2010-01-07 12:07:23Z cbowditch $ */
 
 package org.apache.fop.render.afp.extensions;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-import org.apache.fop.util.ContentHandlerFactory;
-import org.apache.fop.util.ContentHandlerFactory.ObjectBuiltListener;
+import java.net.URI;
+import java.net.URISyntaxException;
+
 import org.xml.sax.Attributes;
 import org.xml.sax.SAXException;
 import org.xml.sax.helpers.DefaultHandler;
 
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+
+import org.apache.fop.util.ContentHandlerFactory;
+import org.apache.fop.util.ContentHandlerFactory.ObjectBuiltListener;
+
 /**
  * ContentHandler (parser) for restoring AFPExtension objects from XML.
  */
-public class AFPExtensionHandler extends DefaultHandler 
+public class AFPExtensionHandler extends DefaultHandler
             implements ContentHandlerFactory.ObjectSource {
 
     /** Logger instance */
@@ -38,34 +43,34 @@ public class AFPExtensionHandler extends DefaultHandler
 
     private StringBuffer content = new StringBuffer();
     private Attributes lastAttributes;
-    
-    private AFPPageSetup returnedObject;
+
+    private AFPExtensionAttachment returnedObject;
     private ObjectBuiltListener listener;
-    
+
     /** {@inheritDoc} */
-    public void startElement(String uri, String localName, String qName, Attributes attributes) 
+    public void startElement(String uri, String localName, String qName, Attributes attributes)
                 throws SAXException {
         boolean handled = false;
-        if (AFPPageSetup.CATEGORY.equals(uri)) {
+        if (AFPExtensionAttachment.CATEGORY.equals(uri)) {
             lastAttributes = attributes;
             handled = true;
             if (localName.equals(AFPElementMapping.NO_OPERATION)
                     || localName.equals(AFPElementMapping.TAG_LOGICAL_ELEMENT)
                     || localName.equals(AFPElementMapping.INCLUDE_PAGE_OVERLAY)
                     || localName.equals(AFPElementMapping.INCLUDE_PAGE_SEGMENT)
-                    || localName.equals(AFPElementMapping.PAGE)
-                    || localName.equals(AFPElementMapping.PAGE_GROUP)) {
+                    || localName.equals(AFPElementMapping.INCLUDE_FORM_MAP)
+                    || localName.equals(AFPElementMapping.INVOKE_MEDIUM_MAP)) {
                 //handled in endElement
             } else {
                 handled = false;
             }
         }
         if (!handled) {
-            if (AFPPageSetup.CATEGORY.equals(uri)) {
-                throw new SAXException("Unhandled element " + localName 
+            if (AFPExtensionAttachment.CATEGORY.equals(uri)) {
+                throw new SAXException("Unhandled element " + localName
                         + " in namespace: " + uri);
             } else {
-                log.warn("Unhandled element " + localName 
+                log.warn("Unhandled element " + localName
                         + " in namespace: " + uri);
             }
         }
@@ -73,21 +78,46 @@ public class AFPExtensionHandler extends DefaultHandler
 
     /** {@inheritDoc} */
     public void endElement(String uri, String localName, String qName) throws SAXException {
-        if (AFPPageSetup.CATEGORY.equals(uri)) {
-            this.returnedObject = new AFPPageSetup(localName);
-            String name = lastAttributes.getValue("name");
-            if (name != null) {
-                returnedObject.setName(name);
+        if (AFPExtensionAttachment.CATEGORY.equals(uri)) {
+            if (AFPElementMapping.INCLUDE_FORM_MAP.equals(localName)) {
+                AFPIncludeFormMap formMap = new AFPIncludeFormMap();
+                String name = lastAttributes.getValue("name");
+                formMap.setName(name);
+                String src = lastAttributes.getValue("src");
+                try {
+                    formMap.setSrc(new URI(src));
+                } catch (URISyntaxException e) {
+                    throw new SAXException("Invalid URI: " + src, e);
+                }
+                this.returnedObject = formMap;
+            } else if (AFPElementMapping.INCLUDE_PAGE_OVERLAY.equals(localName)) {
+                this.returnedObject = new AFPPageOverlay();
+                String name = lastAttributes.getValue("name");
+                if (name != null) {
+                    returnedObject.setName(name);
+                }
+            } else {
+                AFPPageSetup pageSetupExtn = null;
+                if (AFPElementMapping.INVOKE_MEDIUM_MAP.equals(localName)) {
+                    this.returnedObject = new AFPInvokeMediumMap();
+                } else {
+                    pageSetupExtn = new AFPPageSetup(localName);
+                    this.returnedObject = pageSetupExtn;
+                }
+                String name = lastAttributes.getValue("name");
+                if (name != null) {
+                    returnedObject.setName(name);
+                }
+                String value = lastAttributes.getValue("value");
+                if (value != null && pageSetupExtn != null) {
+                    pageSetupExtn.setValue(value);
+                }
+                if (content.length() > 0 && pageSetupExtn != null) {
+                    pageSetupExtn.setContent(content.toString());
+                    content.setLength(0); //Reset text buffer (see characters())
+                }
             }
-            String value = lastAttributes.getValue("value");
-            if (value != null) {
-                returnedObject.setValue(value);
-            }
-            if (content.length() > 0) {
-                returnedObject.setContent(content.toString());
-                content.setLength(0); //Reset text buffer (see characters())
-            }
-        }    
+        }
     }
 
     /** {@inheritDoc} */
