@@ -15,67 +15,46 @@
  * limitations under the License.
  */
 
-/* $Id: AFPPageSetup.java 609567 2008-01-07 10:52:09Z jeremias $ */
+/* $Id: AFPPageSetup.java 748981 2009-03-01 08:55:35Z jeremias $ */
 
 package org.apache.fop.render.afp.extensions;
 
-import java.io.Serializable;
-
 import org.xml.sax.ContentHandler;
 import org.xml.sax.SAXException;
 import org.xml.sax.helpers.AttributesImpl;
 
-import org.apache.xmlgraphics.util.XMLizable;
-
-import org.apache.fop.fo.extensions.ExtensionAttachment;
-
 /**
- * This is the pass-through value object for the PostScript extension.
+ * This is the pass-through value object for the AFP extension.
  */
-public class AFPPageSetup implements ExtensionAttachment, Serializable, XMLizable  {
+public class AFPPageSetup extends AFPExtensionAttachment {
 
-    private static final long serialVersionUID = 7190606822558332901L;
+    /** value attribute */
+    protected static final String ATT_VALUE = "value";
 
-    /** The category URI for this extension attachment. */
-    public static final String CATEGORY = "apache:fop:extensions:afp";
+    /**
+     * the extension content
+     */
+    protected String content;
 
-    private String elementName;
-
-    private String name;
-
-    private String value;
-
-    private String content;
+    /**
+     * the extension value attribute
+     */
+    protected String value;
 
     /**
      * Default constructor.
-     * 
+     *
      * @param elementName the name of the setup code object, may be null
      */
     public AFPPageSetup(String elementName) {
-        this.elementName = elementName;
+        super(elementName);
     }
 
-    /** @return the name */
-    public String getElementName() {
-        return elementName;
-    }
-
-    /** @return the name */
-    public String getName() {
-        return name;
-    }
+    private static final long serialVersionUID = -549941295384013190L;
 
     /**
-     * Sets the name of the setup code object.
-     * @param name The name to set.
-     */
-    public void setName(String name) {
-        this.name = name;
-    }
-
-    /**
-     * @return the value 
+     * Returns the value of the extension.
+     * @return the value
      */
     public String getValue() {
         return value;
@@ -88,13 +67,9 @@ public class AFPPageSetup implements ExtensionAttachment, Serializable, XMLizabl
     public void setValue(String source) {
         this.value = source;
     }
-    
-    /** {@inheritDoc} */
-    public String getCategory() {
-        return CATEGORY;
-    }
 
     /**
+     * Returns the content of the extension.
      * @return the data
      */
     public String getContent() {
@@ -109,15 +84,6 @@ public class AFPPageSetup implements ExtensionAttachment, Serializable, XMLizabl
         this.content = content;
     }
 
-    /** {@inheritDoc} */
-    public String toString() {
-        return "AFPPageSetup(element-name=" + getElementName() 
-            + " name=" + getName() + " value=" + getValue() + ")";
-    }
-
-    private static final String ATT_NAME = "name";
-    private static final String ATT_VALUE = "value";
-
     /** {@inheritDoc} */
     public void toSAX(ContentHandler handler) throws SAXException {
         AttributesImpl atts = new AttributesImpl();
@@ -134,4 +100,10 @@ public class AFPPageSetup implements ExtensionAttachment, Serializable, XMLizabl
         }
         handler.endElement(CATEGORY, elementName, elementName);
     }
+
+    /** {@inheritDoc} */
+    public String toString() {
+        return "AFPPageSetup(element-name=" + getElementName()
+            + " name=" + getName() + " value=" + getValue() + ")";
+    }
 }
