@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PreviewDialog.java 666635 2008-06-11 12:18:53Z jeremias $ */
+/* $Id: PreviewDialog.java 952770 2010-06-08 19:01:56Z jeremias $ */
 
 // Originally contributed by:
 // Juergen Verwohlt: Juergen.Verwohlt@jCatalog.com,
@@ -78,7 +78,7 @@ public class PreviewDialog extends JFrame implements StatusListener {
     /** The originally configured target resolution */
     protected float configuredTargetResolution;
     /**
-     * Renderable instance that can be used to reload and re-render a document after 
+     * Renderable instance that can be used to reload and re-render a document after
      * modifications.
      */
     protected Renderable renderable;
@@ -171,6 +171,11 @@ public class PreviewDialog extends JFrame implements StatusListener {
         //Page view stuff
         previewPanel = new PreviewPanel(foUserAgent, renderable, renderer);
         getContentPane().add(previewPanel, BorderLayout.CENTER);
+        previewPanel.addPageChangeListener(new PageChangeListener() {
+            public void pageChanged(PageChangeEvent pce) {
+              new ShowInfo().run();
+            }
+        });
 
         // Keyboard shortcuts - pgup/pgdn
         InputMap im = previewPanel.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
@@ -255,14 +260,15 @@ public class PreviewDialog extends JFrame implements StatusListener {
                                              new Insets(0, 0, 0, 0), 0, 0));
         getContentPane().add(statusBar, BorderLayout.SOUTH);
     }
-    
+
     /**
-     * Creates and initialize the AWT Viewer main window. 
+     * Creates and initialize the AWT Viewer main window.
      * @param foUserAgent the FO user agent
      * @param renderable the target for the rendering
+     * @param asMainWindow true if the window shall act as the main application window.
      * @return the newly initialized preview dialog
      */
-    public static PreviewDialog createPreviewDialog(FOUserAgent foUserAgent, 
+    public static PreviewDialog createPreviewDialog(FOUserAgent foUserAgent,
                 Renderable renderable, boolean asMainWindow) {
         PreviewDialog frame = new PreviewDialog(foUserAgent, renderable);
 
@@ -297,7 +303,7 @@ public class PreviewDialog extends JFrame implements StatusListener {
     public PreviewDialog(FOUserAgent foUserAgent) {
         this(foUserAgent, null);
     }
-    
+
     /**
      * Creates a new menubar to be shown in this window.
      * @return the newly created menubar
@@ -456,7 +462,7 @@ public class PreviewDialog extends JFrame implements StatusListener {
         setStatus(translator.getString("Status.Show"));
         previewPanel.reload();
     }
-    
+
     /**
      * Changes the current visible page
      * @param number the page number to go to
@@ -530,12 +536,18 @@ public class PreviewDialog extends JFrame implements StatusListener {
         goToPage(currentPage);
     }
 
-    /** Scales page image */
+    /**
+     * Scales page image.
+     * @param scaleFactor the scale factor
+     */
     public void setScale(double scaleFactor) {
         scale.setSelectedItem(percentFormat.format(scaleFactor) + "%");
         previewPanel.setScaleFactor(scaleFactor / 100d);
     }
 
+    /**
+     * Sets the scaling so the contents fit into the window.
+     */
     public void setScaleToFitWindow() {
         try {
             setScale(previewPanel.getScaleToFitWindow() * 100);
@@ -543,7 +555,10 @@ public class PreviewDialog extends JFrame implements StatusListener {
             fopEx.printStackTrace();
         }
     }
-    
+
+    /**
+     * Sets the scaling so the contents are spread over the whole width available.
+     */
     public void setScaleToFitWidth() {
         try {
             setScale(previewPanel.getScaleToFitWidth() * 100);
@@ -569,7 +584,7 @@ public class PreviewDialog extends JFrame implements StatusListener {
         //Restore originally configured target resolution
         float saveResolution = foUserAgent.getTargetResolution();
         foUserAgent.setTargetResolution(this.configuredTargetResolution);
-        
+
         PrinterJob pj = PrinterJob.getPrinterJob();
         pj.setPageable(renderer);
         if (!showDialog || pj.printDialog()) {
@@ -579,7 +594,7 @@ public class PreviewDialog extends JFrame implements StatusListener {
                 e.printStackTrace();
             }
         }
-        
+
         foUserAgent.setTargetResolution(saveResolution);
     }
 
