@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,18 +15,20 @@
  * limitations under the License.
  */
 
-/* $Id: Java2DFontMetrics.java 627367 2008-02-13 12:03:30Z maxberger $ */
- 
+/* $Id: Java2DFontMetrics.java 895012 2010-01-01 13:57:10Z jeremias $ */
+
 package org.apache.fop.render.java2d;
 
 // Java
 import java.awt.Font;
-import java.awt.Graphics2D;
-import java.awt.geom.Rectangle2D;
 import java.awt.FontMetrics;
+import java.awt.Graphics2D;
+import java.awt.RenderingHints;
 import java.awt.font.LineMetrics;
 import java.awt.font.TextAttribute;
 import java.awt.font.TextLayout;
+import java.awt.geom.Rectangle2D;
+import java.awt.image.BufferedImage;
 import java.util.Map;
 
 /**
@@ -63,17 +65,17 @@ public class Java2DFontMetrics {
      * The typical height of a small cap latter (often derived from "x", value in mpt)
      */
     private int xHeight = 0;
-    
+
     /**
      * The highest point of the font above the baseline (usually derived from "d", value in mpt)
      */
     private int ascender = 0;
-    
+
     /**
      * The lowest point of the font under the baseline (usually derived from "p", value in mpt)
      */
     private int descender = 0;
-    
+
     /**
      * Buffered font.
      * f1 is bufferd for metric measurements during layout.
@@ -103,12 +105,26 @@ public class Java2DFontMetrics {
 
     /** A LineMetrics to access high-resolution metrics information. */
     private LineMetrics lineMetrics;
-    
+
     /**
      * Temp graphics object needed to get the font metrics
      */
     private Graphics2D graphics;
 
+    /**
+     * Creates a Graphics2D object for the sole purpose of getting font metrics.
+     * @return a Graphics2D object
+     */
+    public static Graphics2D createFontMetricsGraphics2D() {
+        BufferedImage fontImage = new BufferedImage(100, 100,
+                BufferedImage.TYPE_INT_RGB);
+        Graphics2D graphics2D = fontImage.createGraphics();
+        //The next line is important to get accurate font metrics!
+        graphics2D.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS,
+                RenderingHints.VALUE_FRACTIONALMETRICS_ON);
+        return graphics2D;
+    }
+
     /**
      * Constructs a new Font-metrics.
      * @param graphics a temp graphics object - this is needed  so
@@ -126,11 +142,11 @@ public class Java2DFontMetrics {
      * @param size font size
      * @return ascent in milliponts
      */
-    public int getMaxAscent(String family, int style, int size) {
+    public synchronized int getMaxAscent(String family, int style, int size) {
         setFont(family, style, size);
         return Math.round(lineMetrics.getAscent() * FONT_FACTOR);
     }
-    
+
     /**
      * Determines the font ascent of the Font described by this
      * FontMetrics object
@@ -139,7 +155,7 @@ public class Java2DFontMetrics {
      * @param size font size
      * @return ascent in milliponts
      */
-    public int getAscender(String family, int style, int size) {
+    public synchronized int getAscender(String family, int style, int size) {
         setFont(family, style, size);
         return ascender * 1000;
 
@@ -159,7 +175,7 @@ public class Java2DFontMetrics {
          * Rectangle FontMetrics.getStringBounds(..) method can be called.
          * The y value of the rectangle is the offset from the origin
          * (baseline) apparently needed by the sample test program
-         * 
+         *
          * xxxxx@xxxxx 2001-05-15
          */
         /* I don't think this is right.
@@ -177,7 +193,7 @@ public class Java2DFontMetrics {
      * @param size font size
      * @return capital height in millipoints
      */
-    public int getCapHeight(String family, int style, int size) {
+    public synchronized int getCapHeight(String family, int style, int size) {
         // currently just gets Ascent value but maybe should use
         // getMaxAcent() at some stage
         return getAscender(family, style, size);
@@ -191,7 +207,7 @@ public class Java2DFontMetrics {
      * @param size font size
      * @return descent in milliponts
      */
-    public int getDescender(String family, int style, int size) {
+    public synchronized int getDescender(String family, int style, int size) {
         setFont(family, style, size);
         return descender * 1000;
     }
@@ -204,7 +220,7 @@ public class Java2DFontMetrics {
      * @param size font size
      * @return font height in milliponts
      */
-    public int getXHeight(String family, int style, int size) {
+    public synchronized int getXHeight(String family, int style, int size) {
         setFont(family, style, size);
         return xHeight * 1000;
     }
@@ -218,7 +234,7 @@ public class Java2DFontMetrics {
      * @param size font size
      * @return character width in millipoints
      */
-    public int width(int i, String family, int style, int size) {
+    public synchronized int width(int i, String family, int style, int size) {
         int w;
         setFont(family, style, size);
         w = internalCharWidth(i) * 1000;
@@ -231,7 +247,7 @@ public class Java2DFontMetrics {
         Rectangle2D rect = fmt.getStringBounds(ch, 0, 1, this.graphics);
         return (int)Math.round(rect.getWidth() * 1000);
     }
-    
+
     /**
      * Return widths (in 1/1000ths of point size) of all
      * characters
@@ -240,7 +256,7 @@ public class Java2DFontMetrics {
      * @param size font size
      * @return array of character widths in millipoints
      */
-    public int[] getWidths(String family, int style, int size) {
+    public synchronized int[] getWidths(String family, int style, int size) {
         int i;
 
         if (width == null) {
@@ -265,7 +281,7 @@ public class Java2DFontMetrics {
         atts.put(TextAttribute.SIZE, new Float(size)); //size in pt
         return new Font(atts);
     }
-    
+
     /**
      * Checks whether the font  for which values are
      * requested is the one used immediately before or
@@ -301,17 +317,17 @@ public class Java2DFontMetrics {
             TextLayout layout = new TextLayout("x", f1, graphics.getFontRenderContext());
             Rectangle2D rect = layout.getBounds();
             xHeight = (int)Math.round(-rect.getY() * 1000);
-            
+
             //PostScript-compatible ascent
             layout = new TextLayout("d", f1, graphics.getFontRenderContext());
             rect = layout.getBounds();
             ascender = (int)Math.round(-rect.getY() * 1000);
-            
+
             //PostScript-compatible descent
             layout = new TextLayout("p", f1, graphics.getFontRenderContext());
             rect = layout.getBounds();
             descender = (int)Math.round((rect.getY() + rect.getHeight()) * -1000);
-            
+
             //Alternative way to get metrics but the ascender is again wrong for our purposes
             lineMetrics = f1.getLineMetrics("", graphics.getFontRenderContext());
         }
@@ -335,7 +351,7 @@ public class Java2DFontMetrics {
      * @param size font size
      * @return font with the desired characeristics.
      */
-    public java.awt.Font getFont(String family, int style, int size) {
+    public synchronized java.awt.Font getFont(String family, int style, int size) {
         setFont(family, style, size);
         return f1;
         /*
@@ -356,7 +372,7 @@ public class Java2DFontMetrics {
      * @param c the glyph to check
      * @return true if the character is supported
      */
-    public boolean hasChar(String family, int style, int size, char c) {
+    public synchronized boolean hasChar(String family, int style, int size, char c) {
         setFont(family, style, size);
         return f1.canDisplay(c);
     }
