@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Java2DGraphicsState.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: Java2DGraphicsState.java 746664 2009-02-22 12:40:44Z jeremias $ */
 
 package org.apache.fop.render.java2d;
 
@@ -73,14 +73,14 @@ public class Java2DGraphicsState {
      * @param org the instance to copy
      */
     public Java2DGraphicsState(Java2DGraphicsState org) {
-        this.currentGraphics = (Graphics2D)org.currentGraphics.create(); 
+        this.currentGraphics = (Graphics2D)org.currentGraphics.create();
         this.fontInfo = org.fontInfo;
         this.initialTransform = org.initialTransform;
         this.currentStroke = org.currentStroke;
         this.currentStrokeStyle = org.currentStrokeStyle;
         this.currentStrokeWidth = org.currentStrokeWidth;
     }
-    
+
     /**
      * @return the currently valid state
      */
@@ -92,7 +92,7 @@ public class Java2DGraphicsState {
     public void dispose() {
         this.currentGraphics.dispose();
         this.currentGraphics = null;
-        
+
     }
 
     /**
@@ -222,7 +222,7 @@ public class Java2DGraphicsState {
                 getGraph().setPaint(p);
                 return true;
             }
-        } else if (p.equals(getGraph().getPaint())) {
+        } else if (!p.equals(getGraph().getPaint())) {
             getGraph().setPaint(p);
             return true;
         }
@@ -252,10 +252,12 @@ public class Java2DGraphicsState {
      * according to the rule last-specified-first-applied.
      * @see java.awt.Graphics2D#transform(AffineTransform)
      *
-     * @param tf the transform to concatonate to the current level transform
+     * @param tf the transform to concatenate to the current level transform
      */
     public void transform(AffineTransform tf) {
-        getGraph().transform(tf);
+        if (!tf.isIdentity()) {
+            getGraph().transform(tf);
+        }
     }
 
     /**
@@ -270,7 +272,7 @@ public class Java2DGraphicsState {
 
     /** {@inheritDoc} */
     public String toString() {
-        String s = "AWTGraphicsState " + currentGraphics.toString()
+        String s = "Java2DGraphicsState " + currentGraphics.toString()
                 + ", Stroke (width: " + currentStrokeWidth + " style: "
                 + currentStrokeStyle + "), " + getTransform();
         return s;
