@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Java2DRenderer.java 631984 2008-02-28 14:30:46Z jeremias $ */
+/* $Id: Java2DRenderer.java 932497 2010-04-09 16:34:29Z vhennebert $ */
 
 package org.apache.fop.render.java2d;
 
@@ -24,6 +24,7 @@ import java.awt.BasicStroke;
 import java.awt.Color;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
+import java.awt.Rectangle;
 import java.awt.RenderingHints;
 import java.awt.font.GlyphVector;
 import java.awt.geom.AffineTransform;
@@ -53,6 +54,7 @@ import org.apache.xmlgraphics.image.loader.impl.ImageRendered;
 import org.apache.xmlgraphics.image.loader.impl.ImageXMLDOM;
 import org.apache.xmlgraphics.image.loader.util.ImageUtil;
 
+import org.apache.fop.ResourceEventProducer;
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.apps.FopFactoryConfigurator;
@@ -68,13 +70,17 @@ import org.apache.fop.area.inline.WordArea;
 import org.apache.fop.datatypes.URISpecification;
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fonts.Font;
+import org.apache.fop.fonts.FontCollection;
 import org.apache.fop.fonts.FontInfo;
 import org.apache.fop.fonts.Typeface;
 import org.apache.fop.render.AbstractPathOrientedRenderer;
 import org.apache.fop.render.Graphics2DAdapter;
 import org.apache.fop.render.RendererContext;
+import org.apache.fop.render.extensions.prepress.PageBoundaries;
+import org.apache.fop.render.extensions.prepress.PageScale;
 import org.apache.fop.render.pdf.CTMHelper;
 import org.apache.fop.util.CharUtilities;
+import org.apache.fop.util.ColorUtil;
 
 /**
  * The <code>Java2DRenderer</code> class provides the abstract technical
@@ -120,10 +126,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
     /** The 0-based current page number */
     private int currentPageNumber = 0;
 
-    /** The 0-based total number of rendered pages */
-    private int numberOfPages;
-
-    /** true if antialiasing is set */
+    /** true if anti-aliasing is set */
     protected boolean antialiasing = true;
 
     /** true if qualityRendering is set */
@@ -131,28 +134,26 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
 
     /** false: paints a non-transparent white background, true: for a transparent background */
     protected boolean transparentPageBackground = false;
-    
+
     /** The current state, holds a Graphics2D and its context */
     protected Java2DGraphicsState state;
-    
-    private Stack stateStack = new Stack();
+
+    private final Stack stateStack = new Stack();
 
     /** true if the renderer has finished rendering all the pages */
     private boolean renderingDone;
 
     private GeneralPath currentPath = null;
-    
+
     /** Default constructor */
     public Java2DRenderer() {
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void setUserAgent(FOUserAgent foUserAgent) {
         super.setUserAgent(foUserAgent);
         userAgent.setRendererOverride(this); // for document regeneration
-        
+
         String s = (String)userAgent.getRendererOptions().get(JAVA2D_TRANSPARENT_PAGE_BACKGROUND);
         if (s != null) {
             this.transparentPageBackground = "true".equalsIgnoreCase(s);
@@ -164,20 +165,20 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         return userAgent;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void setupFontInfo(FontInfo inFontInfo) {
         //Don't call super.setupFontInfo() here! Java2D needs a special font setup
         // create a temp Image to test font metrics on
-        fontInfo = inFontInfo;
-        BufferedImage fontImage = new BufferedImage(100, 100,
-                BufferedImage.TYPE_INT_RGB);
-        Graphics2D g = fontImage.createGraphics();
-        //The next line is important to get accurate font metrics!
-        g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, 
-                RenderingHints.VALUE_FRACTIONALMETRICS_ON);
-        FontSetup.setup(fontInfo, fontList, fontResolver, g);
+        this.fontInfo = inFontInfo;
+        Graphics2D graphics2D = Java2DFontMetrics.createFontMetricsGraphics2D();
+
+        FontCollection[] fontCollections = new FontCollection[] {
+                new Base14FontCollection(graphics2D),
+                new InstalledFontCollection(graphics2D),
+                new ConfiguredFontCollection(getFontResolver(), getFontList())
+        };
+        userAgent.getFactory().getFontManager().setup(
+                getFontInfo(), fontCollections);
     }
 
     /** {@inheritDoc} */
@@ -190,16 +191,17 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
      * @param newScaleFactor ]0 ; 1]
      */
     public void setScaleFactor(double newScaleFactor) {
-        scaleFactor = newScaleFactor;
+        this.scaleFactor = newScaleFactor;
     }
 
     /** @return the scale factor */
     public double getScaleFactor() {
-        return scaleFactor;
+        return this.scaleFactor;
     }
 
     /** {@inheritDoc} */
     public void startRenderer(OutputStream out) throws IOException {
+        super.startRenderer(out);
         // do nothing by default
     }
 
@@ -207,7 +209,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
     public void stopRenderer() throws IOException {
         log.debug("Java2DRenderer stopped");
         renderingDone = true;
-        numberOfPages = currentPageNumber;
+        int numberOfPages = currentPageNumber;
         // TODO set all vars to null for gc
         if (numberOfPages == 0) {
             new FOPException("No page could be rendered");
@@ -218,7 +220,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
     public boolean isRenderingDone() {
         return this.renderingDone;
     }
-    
+
     /**
      * @return The 0-based current page number
      */
@@ -234,10 +236,12 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
     }
 
     /**
+     * Returns the number of pages available. This method is also part of the Pageable interface.
      * @return The 0-based total number of rendered pages
+     * @see java.awt.print.Pageable
      */
     public int getNumberOfPages() {
-            return numberOfPages;
+        return pageViewportList.size();
     }
 
     /**
@@ -252,7 +256,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
     /**
      * This method override only stores the PageViewport in a List. No actual
      * rendering is performed here. A renderer override renderPage() to get the
-     * freshly produced PageViewport, and rendere them on the fly (producing the
+     * freshly produced PageViewport, and render them on the fly (producing the
      * desired BufferedImages by calling getPageImage(), which lazily starts the
      * rendering process).
      *
@@ -262,11 +266,22 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
      * @see org.apache.fop.render.Renderer
      */
     public void renderPage(PageViewport pageViewport) throws IOException {
-        // TODO clone?
-        pageViewportList.add(pageViewport.clone());
+        rememberPage((PageViewport)pageViewport.clone());
+        //The clone() call is necessary as we store the page for later. Otherwise, the
+        //RenderPagesModel calls PageViewport.clear() to release memory as early as possible.
         currentPageNumber++;
     }
 
+    /**
+     * Stores the pageViewport in a list of page viewports so they can be rendered later.
+     * Subclasses can override this method to filter pages, for example.
+     * @param pageViewport the page viewport
+     */
+    protected void rememberPage(PageViewport pageViewport) {
+        assert pageViewport.getPageIndex() >= 0;
+        pageViewportList.add(pageViewport);
+    }
+
     /**
      * Generates a desired page from the renderer's page viewport list.
      *
@@ -278,27 +293,41 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
 
         this.currentPageViewport = pageViewport;
         try {
-            Rectangle2D bounds = pageViewport.getViewArea();
-            pageWidth = (int) Math.round(bounds.getWidth() / 1000f);
-            pageHeight = (int) Math.round(bounds.getHeight() / 1000f);
+            PageBoundaries boundaries = new PageBoundaries(
+                    pageViewport.getViewArea().getSize(), pageViewport.getForeignAttributes());
+            Rectangle bounds = boundaries.getCropBox();
+            Rectangle bleedBox = boundaries.getBleedBox();
+            this.pageWidth = (int) Math.round(bounds.getWidth() / 1000f);
+            this.pageHeight = (int) Math.round(bounds.getHeight() / 1000f);
 
             log.info(
                     "Rendering Page " + pageViewport.getPageNumberString()
                             + " (pageWidth " + pageWidth + ", pageHeight "
                             + pageHeight + ")");
 
-            double scaleX = scaleFactor 
-                * (25.4 / FopFactoryConfigurator.DEFAULT_TARGET_RESOLUTION) 
+            // set scale factor
+            double scaleX = scaleFactor;
+            double scaleY = scaleFactor;
+            String scale = (String) currentPageViewport.getForeignAttributes().get(
+                    PageScale.EXT_PAGE_SCALE);
+            Point2D scales = PageScale.getScale(scale);
+            if (scales != null) {
+                scaleX *= scales.getX();
+                scaleY *= scales.getY();
+            }
+
+            scaleX = scaleX
+                * (25.4f / FopFactoryConfigurator.DEFAULT_TARGET_RESOLUTION)
                 / userAgent.getTargetPixelUnitToMillimeter();
-            double scaleY = scaleFactor
-                * (25.4 / FopFactoryConfigurator.DEFAULT_TARGET_RESOLUTION)
+            scaleY = scaleY
+                * (25.4f / FopFactoryConfigurator.DEFAULT_TARGET_RESOLUTION)
                 / userAgent.getTargetPixelUnitToMillimeter();
             int bitmapWidth = (int) ((pageWidth * scaleX) + 0.5);
             int bitmapHeight = (int) ((pageHeight * scaleY) + 0.5);
-                    
-            
+
+
             BufferedImage currentPageImage = getBufferedImage(bitmapWidth, bitmapHeight);
-            
+
             Graphics2D graphics = currentPageImage.createGraphics();
             graphics.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS,
                     RenderingHints.VALUE_FRACTIONALMETRICS_ON);
@@ -318,19 +347,26 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
             // transform page based on scale factor supplied
             AffineTransform at = graphics.getTransform();
             at.scale(scaleX, scaleY);
+            at.translate(bounds.getMinX() / -1000f, bounds.getMinY() / -1000f);
             graphics.setTransform(at);
 
             // draw page frame
             if (!transparentPageBackground) {
                 graphics.setColor(Color.white);
-                graphics.fillRect(0, 0, pageWidth, pageHeight);
+                graphics.fillRect(
+                        (int)Math.round(bleedBox.getMinX() / 1000f),
+                        (int)Math.round(bleedBox.getMinY() / 1000f),
+                        (int)Math.round(bleedBox.getWidth() / 1000f),
+                        (int)Math.round(bleedBox.getHeight() / 1000f));
             }
+            /* why did we have this???
             graphics.setColor(Color.black);
             graphics.drawRect(-1, -1, pageWidth + 2, pageHeight + 2);
             graphics.drawLine(pageWidth + 2, 0, pageWidth + 2, pageHeight + 2);
             graphics.drawLine(pageWidth + 3, 1, pageWidth + 3, pageHeight + 3);
             graphics.drawLine(0, pageHeight + 2, pageWidth + 2, pageHeight + 2);
             graphics.drawLine(1, pageHeight + 3, pageWidth + 3, pageHeight + 3);
+            */
 
             state = new Java2DGraphicsState(graphics, this.fontInfo, at);
             try {
@@ -361,20 +397,20 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
        return new BufferedImage(
                 bitmapWidth, bitmapHeight, BufferedImage.TYPE_INT_ARGB);
     }
-    
+
     /**
      * Returns a page viewport.
-     * @param pageNum the page number
+     * @param pageIndex the page index (zero-based)
      * @return the requested PageViewport instance
      * @exception FOPException If the page is out of range.
      */
-    public PageViewport getPageViewport(int pageNum) throws FOPException {
-        if (pageNum < 0 || pageNum >= pageViewportList.size()) {
-            throw new FOPException("Requested page number is out of range: " + pageNum
+    public PageViewport getPageViewport(int pageIndex) throws FOPException {
+        if (pageIndex < 0 || pageIndex >= pageViewportList.size()) {
+            throw new FOPException("Requested page number is out of range: " + pageIndex
                      + "; only " + pageViewportList.size()
                      + " page(s) available.");
         }
-        return (PageViewport) pageViewportList.get(pageNum);
+        return (PageViewport) pageViewportList.get(pageIndex);
     }
 
     /**
@@ -401,21 +437,21 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         state.dispose();
         state = (Java2DGraphicsState)stateStack.pop();
     }
-    
+
     /** {@inheritDoc} */
     protected void concatenateTransformationMatrix(AffineTransform at) {
         state.transform(at);
     }
-    
+
     /** {@inheritDoc} */
     protected void startVParea(CTM ctm, Rectangle2D clippingRect) {
 
         saveGraphicsState();
 
         if (clippingRect != null) {
-            clipRect((float)clippingRect.getX() / 1000f, 
-                    (float)clippingRect.getY() / 1000f, 
-                    (float)clippingRect.getWidth() / 1000f, 
+            clipRect((float)clippingRect.getX() / 1000f,
+                    (float)clippingRect.getY() / 1000f,
+                    (float)clippingRect.getWidth() / 1000f,
                     (float)clippingRect.getHeight() / 1000f);
         }
 
@@ -424,16 +460,12 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         state.transform(new AffineTransform(CTMHelper.toPDFArray(ctm)));
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected void endVParea() {
         restoreGraphicsState();
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected List breakOutOfStateStack() {
         log.debug("Block.FIXED --> break out");
         List breakOutList;
@@ -446,31 +478,24 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         return breakOutList;
     }
 
-    /**
-     * {@inheritDoc}
-     *          java.util.List)
-     */
+    /** {@inheritDoc} */
     protected void restoreStateStackAfterBreakOut(List breakOutList) {
         log.debug("Block.FIXED --> restoring context after break-out");
-        
-        Iterator i = breakOutList.iterator();
-        while (i.hasNext()) {
-            Java2DGraphicsState s = (Java2DGraphicsState)i.next();
+
+        Iterator it = breakOutList.iterator();
+        while (it.hasNext()) {
+            Java2DGraphicsState s = (Java2DGraphicsState)it.next();
             stateStack.push(state);
-            state = s;
+            this.state = s;
         }
     }
 
-    /**
-     * {@inheritDoc} 
-     */
+    /** {@inheritDoc} */
     protected void updateColor(Color col, boolean fill) {
         state.updateColor(col);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected void clip() {
         if (currentPath == null) {
             throw new IllegalStateException("No current path available!");
@@ -479,16 +504,12 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         currentPath = null;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected void closePath() {
         currentPath.closePath();
     }
 
-    /**
-     * {@inheritDoc} 
-     */
+    /** {@inheritDoc} */
     protected void lineTo(float x, float y) {
         if (currentPath == null) {
             currentPath = new GeneralPath();
@@ -496,9 +517,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         currentPath.lineTo(x, y);
     }
 
-    /**
-     * {@inheritDoc} 
-     */
+    /** {@inheritDoc} */
     protected void moveTo(float x, float y) {
         if (currentPath == null) {
             currentPath = new GeneralPath();
@@ -506,27 +525,23 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         currentPath.moveTo(x, y);
     }
 
-    /**
-     * {@inheritDoc} 
-     */
+    /** {@inheritDoc} */
     protected void clipRect(float x, float y, float width, float height) {
         state.updateClip(new Rectangle2D.Float(x, y, width, height));
     }
 
-    /**
-     * {@inheritDoc} 
-     */
+    /** {@inheritDoc} */
     protected void fillRect(float x, float y, float width, float height) {
         state.getGraph().fill(new Rectangle2D.Float(x, y, width, height));
     }
-    
-    /**
-     * {@inheritDoc} 
-     */
-    protected void drawBorderLine(float x1, float y1, float x2, float y2, 
+
+    /** {@inheritDoc} */
+    protected void drawBorderLine(float x1, float y1, float x2, float y2,
             boolean horz, boolean startOrBefore, int style, Color col) {
         Graphics2D g2d = state.getGraph();
-        drawBorderLine(new Rectangle2D.Float(x1, y1, x2 - x1, y2 - y1), 
+        float width = x2 - x1;
+        float height = y2 - y1;
+        drawBorderLine(new Rectangle2D.Float(x1, y1, width, height),
                 horz, startOrBefore, style, col, g2d);
     }
 
@@ -534,13 +549,13 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
      * Draw a border segment of an XSL-FO style border.
      * @param lineRect the line defined by its bounding rectangle
      * @param horz true for horizontal border segments, false for vertical border segments
-     * @param startOrBefore true for border segments on the start or before edge, 
+     * @param startOrBefore true for border segments on the start or before edge,
      *                      false for end or after.
      * @param style the border style (one of Constants.EN_DASHED etc.)
      * @param col the color for the border segment
      * @param g2d the Graphics2D instance to paint to
      */
-    public static void drawBorderLine(Rectangle2D.Float lineRect, 
+    public static void drawBorderLine(Rectangle2D.Float lineRect,
             boolean horz, boolean startOrBefore, int style, Color col, Graphics2D g2d) {
         float x1 = lineRect.x;
         float y1 = lineRect.y;
@@ -553,7 +568,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
             return;
         }
         switch (style) {
-            case Constants.EN_DASHED: 
+            case Constants.EN_DASHED:
                 g2d.setColor(col);
                 if (horz) {
                     float unit = Math.abs(2 * h);
@@ -563,7 +578,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
                     }
                     unit = w / rep;
                     float ym = y1 + (h / 2);
-                    BasicStroke s = new BasicStroke(h, BasicStroke.CAP_BUTT, 
+                    BasicStroke s = new BasicStroke(h, BasicStroke.CAP_BUTT,
                             BasicStroke.JOIN_MITER, 10.0f, new float[] {unit}, 0);
                     g2d.setStroke(s);
                     g2d.draw(new Line2D.Float(x1, ym, x2, ym));
@@ -575,7 +590,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
                     }
                     unit = h / rep;
                     float xm = x1 + (w / 2);
-                    BasicStroke s = new BasicStroke(w, BasicStroke.CAP_BUTT, 
+                    BasicStroke s = new BasicStroke(w, BasicStroke.CAP_BUTT,
                             BasicStroke.JOIN_MITER, 10.0f, new float[] {unit}, 0);
                     g2d.setStroke(s);
                     g2d.draw(new Line2D.Float(xm, y1, xm, y2));
@@ -591,7 +606,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
                     }
                     unit = w / rep;
                     float ym = y1 + (h / 2);
-                    BasicStroke s = new BasicStroke(h, BasicStroke.CAP_ROUND, 
+                    BasicStroke s = new BasicStroke(h, BasicStroke.CAP_ROUND,
                             BasicStroke.JOIN_MITER, 10.0f, new float[] {0, unit}, 0);
                     g2d.setStroke(s);
                     g2d.draw(new Line2D.Float(x1, ym, x2, ym));
@@ -603,7 +618,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
                     }
                     unit = h / rep;
                     float xm = x1 + (w / 2);
-                    BasicStroke s = new BasicStroke(w, BasicStroke.CAP_ROUND, 
+                    BasicStroke s = new BasicStroke(w, BasicStroke.CAP_ROUND,
                             BasicStroke.JOIN_MITER, 10.0f, new float[] {0, unit}, 0);
                     g2d.setStroke(s);
                     g2d.draw(new Line2D.Float(xm, y1, xm, y2));
@@ -633,8 +648,8 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
             case Constants.EN_RIDGE:
                 float colFactor = (style == EN_GROOVE ? 0.4f : -0.4f);
                 if (horz) {
-                    Color uppercol = lightenColor(col, -colFactor);
-                    Color lowercol = lightenColor(col, colFactor);
+                    Color uppercol = ColorUtil.lightenColor(col, -colFactor);
+                    Color lowercol = ColorUtil.lightenColor(col, colFactor);
                     float h3 = h / 3;
                     float ym1 = y1 + (h3 / 2);
                     g2d.setStroke(new BasicStroke(h3));
@@ -645,8 +660,8 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
                     g2d.setColor(lowercol);
                     g2d.draw(new Line2D.Float(x1, ym1 + h3 + h3, x2, ym1 + h3 + h3));
                 } else {
-                    Color leftcol = lightenColor(col, -colFactor);
-                    Color rightcol = lightenColor(col, colFactor);
+                    Color leftcol = ColorUtil.lightenColor(col, -colFactor);
+                    Color rightcol = ColorUtil.lightenColor(col, colFactor);
                     float w3 = w / 3;
                     float xm1 = x1 + (w3 / 2);
                     g2d.setStroke(new BasicStroke(w3));
@@ -662,13 +677,13 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
             case Constants.EN_OUTSET:
                 colFactor = (style == EN_OUTSET ? 0.4f : -0.4f);
                 if (horz) {
-                    col = lightenColor(col, (startOrBefore ? 1 : -1) * colFactor);
+                    col = ColorUtil.lightenColor(col, (startOrBefore ? 1 : -1) * colFactor);
                     g2d.setStroke(new BasicStroke(h));
                     float ym1 = y1 + (h / 2);
                     g2d.setColor(col);
                     g2d.draw(new Line2D.Float(x1, ym1, x2, ym1));
                 } else {
-                    col = lightenColor(col, (startOrBefore ? 1 : -1) * colFactor);
+                    col = ColorUtil.lightenColor(col, (startOrBefore ? 1 : -1) * colFactor);
                     float xm1 = x1 + (w / 2);
                     g2d.setStroke(new BasicStroke(w));
                     g2d.setColor(col);
@@ -691,9 +706,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void renderText(TextArea text) {
         renderInlineAreaBackAndBorders(text);
 
@@ -709,7 +722,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         state.transform(at);
         renderText(text, state.getGraph(), font);
         restoreGraphicsState();
-        
+
         currentIPPosition = saveIP + text.getAllocIPD();
         //super.renderText(text);
 
@@ -742,8 +755,8 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
                 int[] letterAdjust = word.getLetterAdjustArray();
                 GlyphVector gv = g2d.getFont().createGlyphVector(g2d.getFontRenderContext(), s);
                 double additionalWidth = 0.0;
-                if (letterAdjust == null 
-                        && text.getTextLetterSpaceAdjust() == 0 
+                if (letterAdjust == null
+                        && text.getTextLetterSpaceAdjust() == 0
                         && text.getTextWordSpaceAdjust() == 0) {
                     //nop
                 } else {
@@ -763,8 +776,8 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
                 SpaceArea space = (SpaceArea)child;
                 String s = space.getSpace();
                 char sp = s.charAt(0);
-                int tws = (space.isAdjustable() 
-                        ? text.getTextWordSpaceAdjust() 
+                int tws = (space.isAdjustable()
+                        ? text.getTextWordSpaceAdjust()
                                 + 2 * text.getTextLetterSpaceAdjust()
                         : 0);
 
@@ -774,8 +787,8 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
             }
         }
     }
-    
-    private static int[] getGlyphOffsets(String s, Font font, TextArea text, 
+
+    private static int[] getGlyphOffsets(String s, Font font, TextArea text,
             int[] letterAdjust) {
         int textLen = s.length();
         int[] offsets = new int[textLen];
@@ -791,11 +804,11 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
             }
             int cw = font.getWidth(mapped);
             int ladj = (letterAdjust != null && i < textLen - 1 ? letterAdjust[i + 1] : 0);
-            int tls = (i < textLen - 1 ? text.getTextLetterSpaceAdjust() : 0); 
+            int tls = (i < textLen - 1 ? text.getTextLetterSpaceAdjust() : 0);
             offsets[i] = cw + ladj + tls + wordSpace;
         }
         return offsets;
-    }    
+    }
 
     /**
      * Render leader area. This renders a leader area which is an area with a
@@ -811,7 +824,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
 
         float startx = (currentIPPosition + area.getBorderAndPaddingWidthStart()) / 1000f;
         float starty = ((currentBPPosition + area.getOffset()) / 1000f);
-        float endx = (currentIPPosition + area.getBorderAndPaddingWidthStart() 
+        float endx = (currentIPPosition + area.getBorderAndPaddingWidthStart()
                 + area.getIPD()) / 1000f;
 
         Color col = (Color) area.getTrait(Trait.COLOR);
@@ -826,7 +839,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         case EN_SOLID:
         case EN_DASHED:
         case EN_DOUBLE:
-            drawBorderLine(startx, starty, endx, starty + ruleThickness, 
+            drawBorderLine(startx, starty, endx, starty + ruleThickness,
                     true, true, style, col);
             break;
         case EN_DOTTED:
@@ -840,7 +853,7 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         case EN_RIDGE:
             float half = area.getRuleThickness() / 2000f;
 
-            state.updateColor(lightenColor(col, 0.6f));
+            state.updateColor(ColorUtil.lightenColor(col, 0.6f));
             moveTo(startx, starty);
             lineTo(endx, starty);
             lineTo(endx, starty + 2 * half);
@@ -875,37 +888,34 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
         super.renderLeader(area);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void renderImage(Image image, Rectangle2D pos) {
         // endTextObject();
         String url = image.getURL();
         drawImage(url, pos);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    private static final ImageFlavor[] FLAVOURS = new ImageFlavor[]
+                                                  {ImageFlavor.GRAPHICS2D,
+                                                      ImageFlavor.BUFFERED_IMAGE,
+                                                      ImageFlavor.RENDERED_IMAGE,
+                                                      ImageFlavor.XML_DOM};
+
+    /** {@inheritDoc} */
     protected void drawImage(String uri, Rectangle2D pos, Map foreignAttributes) {
 
         int x = currentIPPosition + (int)Math.round(pos.getX());
         int y = currentBPPosition + (int)Math.round(pos.getY());
         uri = URISpecification.getURL(uri);
-        
+
         ImageManager manager = getUserAgent().getFactory().getImageManager();
         ImageInfo info = null;
         try {
             ImageSessionContext sessionContext = getUserAgent().getImageSessionContext();
             info = manager.getImageInfo(uri, sessionContext);
-            final ImageFlavor[] flavors = new ImageFlavor[]
-                {ImageFlavor.GRAPHICS2D,
-                    ImageFlavor.BUFFERED_IMAGE, 
-                    ImageFlavor.RENDERED_IMAGE, 
-                    ImageFlavor.XML_DOM};
             Map hints = ImageUtil.getDefaultHints(sessionContext);
             org.apache.xmlgraphics.image.loader.Image img = manager.getImage(
-                    info, flavors, hints, sessionContext);
+                    info, FLAVOURS, hints, sessionContext);
             if (img instanceof ImageGraphics2D) {
                 ImageGraphics2D imageG2D = (ImageGraphics2D)img;
                 int width = (int)pos.getWidth();
@@ -930,18 +940,22 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
                         pos, foreignAttributes);
             }
         } catch (ImageException ie) {
-            log.error("Error while processing image: "
-                    + (info != null ? info.toString() : uri), ie);
-        } catch (FileNotFoundException fnfe) {
-            log.error(fnfe.getMessage());
+            ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
+                    getUserAgent().getEventBroadcaster());
+            eventProducer.imageError(this, (info != null ? info.toString() : uri), ie, null);
+        } catch (FileNotFoundException fe) {
+            ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
+                    getUserAgent().getEventBroadcaster());
+            eventProducer.imageNotFound(this, (info != null ? info.toString() : uri), fe, null);
         } catch (IOException ioe) {
-            log.error("I/O error while processing image: "
-                    + (info != null ? info.toString() : uri), ioe);
+            ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
+                    getUserAgent().getEventBroadcaster());
+            eventProducer.imageIOError(this, (info != null ? info.toString() : uri), ioe, null);
         }
     }
 
     /** {@inheritDoc} */
-    protected RendererContext createRendererContext(int x, int y, int width, int height, 
+    protected RendererContext createRendererContext(int x, int y, int width, int height,
             Map foreignAttributes) {
         RendererContext context = super.createRendererContext(
                 x, y, width, height, foreignAttributes);
@@ -996,5 +1010,4 @@ public abstract class Java2DRenderer extends AbstractPathOrientedRenderer implem
     public void setTransparentPageBackground(boolean transparentPageBackground) {
         this.transparentPageBackground = transparentPageBackground;
     }
-
 }
