@@ -15,23 +15,23 @@
  * limitations under the License.
  */
 
-/* $Id: PCLGenerator.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: PCLGenerator.java 798417 2009-07-28 06:41:49Z jeremias $ */
 
 package org.apache.fop.render.pcl;
 
 import java.awt.Color;
 import java.awt.Dimension;
 import java.awt.Graphics2D;
-import java.awt.color.ColorSpace;
 import java.awt.geom.AffineTransform;
 import java.awt.image.BufferedImage;
 import java.awt.image.BufferedImageOp;
 import java.awt.image.ByteLookupTable;
-import java.awt.image.ColorConvertOp;
 import java.awt.image.ColorModel;
 import java.awt.image.DataBuffer;
+import java.awt.image.DataBufferByte;
 import java.awt.image.IndexColorModel;
 import java.awt.image.LookupOp;
+import java.awt.image.MultiPixelPackedSampleModel;
 import java.awt.image.Raster;
 import java.awt.image.RenderedImage;
 import java.awt.image.WritableRaster;
@@ -43,42 +43,46 @@ import java.text.DecimalFormatSymbols;
 import java.util.Locale;
 
 import org.apache.commons.io.output.ByteArrayOutputStream;
-import org.apache.fop.util.UnitConv;
+
 import org.apache.xmlgraphics.image.GraphicsUtil;
+import org.apache.xmlgraphics.util.UnitConv;
+
+import org.apache.fop.util.bitmap.BitmapImageUtil;
+import org.apache.fop.util.bitmap.DitherUtil;
+import org.apache.fop.util.bitmap.MonochromeBitmapConverter;
 
 /**
  * This class provides methods for generating PCL print files.
  */
 public class PCLGenerator {
 
+    private static final String US_ASCII = "US-ASCII";
+
+    private static final String ISO_8859_1 = "ISO-8859-1";
+
     /** The ESC (escape) character */
     public static final char ESC = '\033';
-    
+
     /** A list of all supported resolutions in PCL (values in dpi) */
     public static final int[] PCL_RESOLUTIONS = new int[] {75, 100, 150, 200, 300, 600};
-    
-    /** Selects a 4x4 Bayer dither matrix (17 grayscales) */
-    public static final int DITHER_MATRIX_4X4 = 4;
-    /** Selects a 8x8 Bayer dither matrix (65 grayscales) */
-    public static final int DITHER_MATRIX_8X8 = 8;
-    
-    private final DecimalFormatSymbols symbols = new DecimalFormatSymbols(Locale.US); 
+
+    private final DecimalFormatSymbols symbols = new DecimalFormatSymbols(Locale.US);
     private final DecimalFormat df2 = new DecimalFormat("0.##", symbols);
     private final DecimalFormat df4 = new DecimalFormat("0.####", symbols);
-    
-    private OutputStream out;
-    
+
+    private final OutputStream out;
+
     private boolean currentSourceTransparency = true;
     private boolean currentPatternTransparency = true;
-    
+
     private int maxBitmapResolution = PCL_RESOLUTIONS[PCL_RESOLUTIONS.length - 1];
 
     /**
      * true: Standard PCL shades are used (poor quality). false: user-defined pattern are used
      * to create custom dither patterns for better grayscale quality.
      */
-    private boolean usePCLShades = false;
-    
+    private final boolean usePCLShades = false;
+
     /**
      * Main constructor.
      * @param out the OutputStream to write the PCL stream to
@@ -86,7 +90,7 @@ public class PCLGenerator {
     public PCLGenerator(OutputStream out) {
         this.out = out;
     }
-    
+
     /**
      * Main constructor.
      * @param out the OutputStream to write the PCL stream to
@@ -106,17 +110,25 @@ public class PCLGenerator {
         }
         this.maxBitmapResolution = maxResolution;
     }
-    
+
     /** @return the OutputStream that this generator writes to */
     public OutputStream getOutputStream() {
         return this.out;
     }
-    
+
+    /**
+     * Returns the currently active text encoding.
+     * @return the text encoding
+     */
+    public String getTextEncoding() {
+        return ISO_8859_1;
+    }
+
     /** @return the maximum resolution to encode bitmap images at */
     public int getMaximumBitmapResolution() {
         return this.maxBitmapResolution;
     }
-    
+
     /**
      * Writes a PCL escape command to the output stream.
      * @param cmd the command (without the ESCAPE character)
@@ -124,21 +136,21 @@ public class PCLGenerator {
      */
     public void writeCommand(String cmd) throws IOException {
         out.write(27); //ESC
-        out.write(cmd.getBytes("US-ASCII"));
+        out.write(cmd.getBytes(US_ASCII));
     }
-    
+
     /**
      * Writes raw text (in ISO-8859-1 encoding) to the output stream.
      * @param s the text
      * @throws IOException In case of an I/O error
      */
     public void writeText(String s) throws IOException {
-        out.write(s.getBytes("ISO-8859-1"));
+        out.write(s.getBytes(ISO_8859_1));
     }
 
     /**
      * Formats a double value with two decimal positions for PCL output.
-     * 
+     *
      * @param value value to format
      * @return the formatted value
      */
@@ -148,7 +160,7 @@ public class PCLGenerator {
 
     /**
      * Formats a double value with four decimal positions for PCL output.
-     * 
+     *
      * @param value value to format
      * @return the formatted value
      */
@@ -163,7 +175,7 @@ public class PCLGenerator {
     public void universalEndOfLanguage() throws IOException {
         writeCommand("%-12345X");
     }
-    
+
     /**
      * Resets the printer and restores the user default environment.
      * @throws IOException In case of an I/O error
@@ -171,7 +183,7 @@ public class PCLGenerator {
     public void resetPrinter() throws IOException {
         writeCommand("E");
     }
-    
+
     /**
      * Sends the job separation command.
      * @throws IOException In case of an I/O error
@@ -179,7 +191,7 @@ public class PCLGenerator {
     public void separateJobs() throws IOException {
         writeCommand("&l1T");
     }
-    
+
     /**
      * Sends the form feed character.
      * @throws IOException In case of an I/O error
@@ -196,7 +208,7 @@ public class PCLGenerator {
     public void setUnitOfMeasure(int value) throws IOException {
         writeCommand("&u" + value + "D");
     }
-    
+
     /**
      * Sets the raster graphics resolution
      * @param value the resolution value (units per inch)
@@ -205,7 +217,7 @@ public class PCLGenerator {
     public void setRasterGraphicsResolution(int value) throws IOException {
         writeCommand("*t" + value + "R");
     }
-    
+
     /**
      * Selects the page size.
      * @param selector the integer representing the page size
@@ -216,7 +228,7 @@ public class PCLGenerator {
     }
 
     /**
-     * Selects the paper source. The parameter is usually printer-specific. Usually, "1" is the 
+     * Selects the paper source. The parameter is usually printer-specific. Usually, "1" is the
      * default tray, "2" is the manual paper feed, "3" is the manual envelope feed, "4" is the
      * "lower" tray and "7" is "auto-select". Consult the technical reference for your printer
      * for all available values.
@@ -227,6 +239,31 @@ public class PCLGenerator {
         writeCommand("&l" + selector + "H");
     }
 
+    /**
+     * Selects the output bin. The parameter is usually printer-specific. Usually, "1" is the
+     * default output bin (upper bin) and "2" is the lower (rear) output bin. Some printers
+     * may support additional output bins. Consult the technical reference for your printer
+     * for all available values.
+     * @param selector the integer representing the output bin
+     * @throws IOException In case of an I/O error
+     */
+    public void selectOutputBin(int selector) throws IOException {
+        writeCommand("&l" + selector + "G");
+    }
+
+    /**
+     * Selects the duplexing mode for the page.
+     * The parameter is usually printer-specific.
+     * "0" means Simplex,
+     * "1" means Duplex, Long-Edge Binding,
+     * "2" means Duplex, Short-Edge Binding.
+     * @param selector the integer representing the duplexing mode of the page
+     * @throws IOException In case of an I/O error
+     */
+    public void selectDuplexMode(int selector) throws IOException {
+        writeCommand("&l" + selector + "S");
+    }
+
     /**
      * Clears the horizontal margins.
      * @throws IOException In case of an I/O error
@@ -234,7 +271,7 @@ public class PCLGenerator {
     public void clearHorizontalMargins() throws IOException {
         writeCommand("9");
     }
-    
+
     /**
      * The Top Margin command designates the number of lines between
      * the top of the logical page and the top of the text area.
@@ -288,7 +325,7 @@ public class PCLGenerator {
     public void pushCursorPos() throws IOException {
         writeCommand("&f0S");
     }
-    
+
     /**
      * Pops the current cursor position from the stack.
      * @throws IOException In case of an I/O error
@@ -296,7 +333,7 @@ public class PCLGenerator {
     public void popCursorPos() throws IOException {
         writeCommand("&f1S");
     }
-    
+
     /**
      * Changes the current print direction while maintaining the current cursor position.
      * @param rotate the rotation angle (counterclockwise), one of 0, 90, 180 and 270.
@@ -305,10 +342,10 @@ public class PCLGenerator {
     public void changePrintDirection(int rotate) throws IOException {
         writeCommand("&a" + rotate + "P");
     }
-    
+
     /**
      * Enters the HP GL/2 mode.
-     * @param restorePreviousHPGL2Cursor true if the previous HP GL/2 pen position should be 
+     * @param restorePreviousHPGL2Cursor true if the previous HP GL/2 pen position should be
      *                                   restored, false if the current position is maintained
      * @throws IOException In case of an I/O error
      */
@@ -319,7 +356,7 @@ public class PCLGenerator {
             writeCommand("%1B");
         }
     }
-    
+
     /**
      * Enters the PCL mode.
      * @param restorePreviousPCLCursor true if the previous PCL cursor position should be restored,
@@ -333,7 +370,7 @@ public class PCLGenerator {
             writeCommand("%1A");
         }
     }
-    
+
     /**
      * Generate a filled rectangle at the current cursor position.
      *
@@ -352,19 +389,19 @@ public class PCLGenerator {
             //y += h;
         }
         setPatternTransparencyMode(false);
-        if (usePCLShades 
+        if (usePCLShades
                 || Color.black.equals(col)
                 || Color.white.equals(col)) {
-            writeCommand("*c" + formatDouble4(w / 100) + "h" 
-                              + formatDouble4(h / 100) + "V");
+            writeCommand("*c" + formatDouble4(w / 100.0) + "h"
+                              + formatDouble4(h / 100.0) + "V");
             int lineshade = convertToPCLShade(col);
             writeCommand("*c" + lineshade + "G");
             writeCommand("*c2P"); //Shaded fill
         } else {
-            defineGrayscalePattern(col, 32, DITHER_MATRIX_4X4);
+            defineGrayscalePattern(col, 32, DitherUtil.DITHER_MATRIX_4X4);
 
-            writeCommand("*c" + formatDouble4(w / 100) + "h" 
-                              + formatDouble4(h / 100) + "V");
+            writeCommand("*c" + formatDouble4(w / 100.0) + "h"
+                              + formatDouble4(h / 100.0) + "V");
             writeCommand("*c32G");
             writeCommand("*c4P"); //User-defined pattern
         }
@@ -372,34 +409,6 @@ public class PCLGenerator {
         setPatternTransparencyMode(true);
     }
 
-    //Bayer dither matrices (4x4 and 8x8 are derived from the 2x2 matrix)
-    private static final int[] BAYER_D2 = new int[] {0, 2, 3, 1};
-    private static final int[] BAYER_D4;
-    private static final int[] BAYER_D8;
-    
-    static {
-        BAYER_D4 = deriveBayerMatrix(BAYER_D2);
-        BAYER_D8 = deriveBayerMatrix(BAYER_D4);
-    }
-    
-    private static void setValueInMatrix(int[] dn, int half, int part, int idx, int value) {
-        int xoff = (part & 1) * half;
-        int yoff = (part & 2) * half * half;
-        int matrixIndex = yoff + ((int)(idx / half) * half * 2) + (idx % half) + xoff;
-        dn[matrixIndex] = value;
-    }
-    
-    private static int[] deriveBayerMatrix(int[] d) {
-        int[] dn = new int[d.length * 4];
-        int half = (int)Math.sqrt(d.length);
-        for (int part = 0; part < 4; part++) {
-            for (int i = 0, c = d.length; i < c; i++) {
-                setValueInMatrix(dn, half, part, i, d[i] * 4 + BAYER_D2[part]);
-            }
-        }
-        return dn;
-    }
-    
     /**
      * Generates a user-defined pattern for a dithering pattern matching the grayscale value
      * of the color given.
@@ -408,7 +417,7 @@ public class PCLGenerator {
      * @param ditherMatrixSize the size of the Bayer dither matrix to use (4 or 8 supported)
      * @throws IOException In case of an I/O error
      */
-    public void defineGrayscalePattern(Color col, int patternID, int ditherMatrixSize) 
+    public void defineGrayscalePattern(Color col, int patternID, int ditherMatrixSize)
             throws IOException {
         ByteArrayOutputStream baout = new ByteArrayOutputStream();
         DataOutputStream data = new DataOutputStream(baout);
@@ -421,38 +430,15 @@ public class PCLGenerator {
         //data.writeShort(600); //X Resolution (didn't manage to get that to work)
         //data.writeShort(600); //Y Resolution
         int gray255 = convertToGray(col.getRed(), col.getGreen(), col.getBlue());
-        
+
         byte[] pattern;
         if (ditherMatrixSize == 8) {
-            int gray65 = gray255 * 65 / 255;
-            
-            pattern = new byte[BAYER_D8.length / 8];
-            
-            for (int i = 0, c = BAYER_D8.length; i < c; i++) {
-                boolean dot = !(BAYER_D8[i] < gray65 - 1);
-                if (dot) {
-                    int byteIdx = i / 8;
-                    pattern[byteIdx] |= 1 << (i % 8); 
-                }
-            }
+            pattern = DitherUtil.getBayerDither(DitherUtil.DITHER_MATRIX_8X8, gray255, false);
         } else {
-            int gray17 = gray255 * 17 / 255;
-            
-            //Since a 4x4 pattern did not work, the 4x4 pattern is applied 4 times to an 
-            //8x8 pattern. Maybe this could be changed to use an 8x8 bayer dither pattern 
+            //Since a 4x4 pattern did not work, the 4x4 pattern is applied 4 times to an
+            //8x8 pattern. Maybe this could be changed to use an 8x8 bayer dither pattern
             //instead of the 4x4 one.
-            pattern = new byte[BAYER_D4.length / 8 * 4];
-            
-            for (int i = 0, c = BAYER_D4.length; i < c; i++) {
-                boolean dot = !(BAYER_D4[i] < gray17 - 1);
-                if (dot) {
-                    int byteIdx = i / 4;
-                    pattern[byteIdx] |= 1 << (i % 4); 
-                    pattern[byteIdx] |= 1 << ((i % 4) + 4); 
-                    pattern[byteIdx + 4] |= 1 << (i % 4); 
-                    pattern[byteIdx + 4] |= 1 << ((i % 4) + 4); 
-                }
-            }
+            pattern = DitherUtil.getBayerDither(DitherUtil.DITHER_MATRIX_4X4, gray255, true);
         }
         data.write(pattern);
         if ((baout.size() % 2) > 0) {
@@ -508,9 +494,9 @@ public class PCLGenerator {
      * @return the gray value
      */
     public final int convertToGray(int r, int g, int b) {
-        return (r * 30 + g * 59 + b * 11) / 100;
+        return BitmapImageUtil.convertToGray(r, g, b);
     }
-    
+
     /**
      * Convert a Color value to a PCL shade value (0-100).
      * @param col the color
@@ -520,7 +506,7 @@ public class PCLGenerator {
         float gray = convertToGray(col.getRed(), col.getGreen(), col.getBlue()) / 255f;
         return (int)(100 - (gray * 100f));
     }
-    
+
     /**
      * Selects the current grayscale color (the given color is converted to grayscales).
      * @param col the color
@@ -535,12 +521,12 @@ public class PCLGenerator {
             if (usePCLShades) {
                 selectCurrentPattern(convertToPCLShade(col), 2);
             } else {
-                defineGrayscalePattern(col, 32, DITHER_MATRIX_4X4);
+                defineGrayscalePattern(col, 32, DitherUtil.DITHER_MATRIX_4X4);
                 selectCurrentPattern(32, 4);
             }
         }
     }
-    
+
     /**
      * Select the current pattern
      * @param patternID the pattern ID (<ESC>*c#G command)
@@ -560,53 +546,42 @@ public class PCLGenerator {
      * @return true if it's a monochrome image
      */
     public static boolean isMonochromeImage(RenderedImage img) {
-        ColorModel cm = img.getColorModel();
-        if (cm instanceof IndexColorModel) {
-            IndexColorModel icm = (IndexColorModel)cm;
-            return icm.getMapSize() == 2;
-        } else {
-            return false;
+        return BitmapImageUtil.isMonochromeImage(img);
         }
-    }
-    
+
     /**
      * Indicates whether an image is a grayscale image.
      * @param img the image
      * @return true if it's a grayscale image
      */
     public static boolean isGrayscaleImage(RenderedImage img) {
-        return (img.getColorModel().getColorSpace().getNumComponents() == 1);
+        return BitmapImageUtil.isGrayscaleImage(img);
     }
-    
-    private MonochromeBitmapConverter createMonochromeBitmapConverter() {
-        MonochromeBitmapConverter converter = null;
-        try {
-            String clName = "org.apache.fop.render.pcl.JAIMonochromeBitmapConverter";
-            Class clazz = Class.forName(clName);
-            converter = (MonochromeBitmapConverter)clazz.newInstance();
-        } catch (ClassNotFoundException cnfe) {
-            // Class was not compiled so is not available. Simply ignore.
-        } catch (LinkageError le) {
-            // This can happen if fop was build with support for a
-            // particular provider (e.g. a binary fop distribution)
-            // but the required support files (i.e. JAI) are not
-            // available in the current runtime environment.
-            // Simply continue with the backup implementation.
-        } catch (InstantiationException e) {
-            // Problem instantiating the class, simply continue with the backup implementation
-        } catch (IllegalAccessException e) {
-            // Problem instantiating the class, simply continue with the backup implementation
+
+    private static int jaiAvailable = -1; //no synchronization necessary, not critical
+
+    /**
+     * Indicates whether JAI is available. JAI has shown to be reliable when dithering a
+     * grayscale or color image to monochrome bitmaps (1-bit).
+     * @return true if JAI is available
+     */
+    public static boolean isJAIAvailable() {
+        if (jaiAvailable < 0) {
+            try {
+                String clName = "javax.media.jai.JAI";
+                Class.forName(clName);
+                jaiAvailable = 1;
+            } catch (ClassNotFoundException cnfe) {
+                jaiAvailable = 0;
+            }
         }
-        if (converter == null) {
-            converter = new DefaultMonochromeBitmapConverter();
-        }
-        return converter;
+        return (jaiAvailable > 0);
     }
 
     private int calculatePCLResolution(int resolution) {
         return calculatePCLResolution(resolution, false);
     }
-    
+
     /**
      * Calculates the ideal PCL resolution for a given resolution.
      * @param resolution the input resolution
@@ -638,35 +613,35 @@ public class PCLGenerator {
         }
         return PCL_RESOLUTIONS[choice];
     }
-    
+
     private boolean isValidPCLResolution(int resolution) {
         return resolution == calculatePCLResolution(resolution);
     }
-    
-    private Dimension getAdjustedDimension(Dimension orgDim, double orgResolution, 
+
+    private Dimension getAdjustedDimension(Dimension orgDim, double orgResolution,
             int pclResolution) {
         if (orgResolution == pclResolution) {
             return orgDim;
         } else {
             Dimension result = new Dimension();
-            result.width = (int)Math.round((double)orgDim.width * pclResolution / orgResolution); 
-            result.height = (int)Math.round((double)orgDim.height * pclResolution / orgResolution); 
+            result.width = (int)Math.round((double)orgDim.width * pclResolution / orgResolution);
+            result.height = (int)Math.round((double)orgDim.height * pclResolution / orgResolution);
             return result;
         }
     }
-    
+
     //Threshold table to convert an alpha channel (8-bit) into a clip mask (1-bit)
     private static final byte[] THRESHOLD_TABLE = new byte[256];
     static { // Initialize the arrays
         for (int i = 0; i < 256; i++) {
             THRESHOLD_TABLE[i] = (byte) ((i < 240) ? 255 : 0);
         }
-    }    
-    
+    }
+
     private RenderedImage getMask(RenderedImage img, Dimension targetDim) {
-        ColorModel cm = img.getColorModel(); 
+        ColorModel cm = img.getColorModel();
         if (cm.hasAlpha()) {
-            BufferedImage alpha = new BufferedImage(img.getWidth(), img.getHeight(), 
+            BufferedImage alpha = new BufferedImage(img.getWidth(), img.getHeight(),
                     BufferedImage.TYPE_BYTE_GRAY);
             Raster raster = img.getData();
             GraphicsUtil.copyBand(raster, cm.getNumColorComponents(), alpha.getRaster(), 0);
@@ -685,7 +660,7 @@ public class PCLGenerator {
                                                    targetDim.width, targetDim.height, 1, 1, null);
                 mask = new BufferedImage(colorModel, wraster, false, null);
             }
-            
+
             Graphics2D g2d = mask.createGraphics();
             try {
                 AffineTransform at = new AffineTransform();
@@ -717,20 +692,30 @@ public class PCLGenerator {
      * @param sourceTransparency true if the background should not be erased
      * @throws IOException In case of an I/O error
      */
-    public void paintBitmap(RenderedImage img, Dimension targetDim, boolean sourceTransparency) 
+    public void paintBitmap(RenderedImage img, Dimension targetDim, boolean sourceTransparency)
                 throws IOException {
-        double targetResolution = img.getWidth() / UnitConv.mpt2in(targetDim.width);
+        double targetHResolution = img.getWidth() / UnitConv.mpt2in(targetDim.width);
+        double targetVResolution = img.getHeight() / UnitConv.mpt2in(targetDim.height);
+        double targetResolution = Math.max(targetHResolution, targetVResolution);
         int resolution = (int)Math.round(targetResolution);
         int effResolution = calculatePCLResolution(resolution, true);
         Dimension orgDim = new Dimension(img.getWidth(), img.getHeight());
-        Dimension effDim = getAdjustedDimension(orgDim, targetResolution, effResolution);
+        Dimension effDim;
+        if (targetResolution == effResolution) {
+            effDim = orgDim; //avoid scaling side-effects
+        } else {
+            effDim = new Dimension(
+                    (int)Math.ceil(UnitConv.mpt2px(targetDim.width, effResolution)),
+                    (int)Math.ceil(UnitConv.mpt2px(targetDim.height, effResolution)));
+        }
         boolean scaled = !orgDim.equals(effDim);
+        //ImageWriterUtil.saveAsPNG(img, new java.io.File("D:/text-0-org.png"));
 
         boolean monochrome = isMonochromeImage(img);
         if (!monochrome) {
             //Transparency mask disabled. Doesn't work reliably
             final boolean transparencyDisabled = true;
-            RenderedImage mask = (transparencyDisabled ? null : getMask(img, effDim)); 
+            RenderedImage mask = (transparencyDisabled ? null : getMask(img, effDim));
             if (mask != null) {
                 pushCursorPos();
                 selectCurrentPattern(0, 1); //Solid white
@@ -738,66 +723,65 @@ public class PCLGenerator {
                 paintMonochromeBitmap(mask, effResolution);
                 popCursorPos();
             }
-            
+
             BufferedImage src = null;
             if (img instanceof BufferedImage && !scaled) {
                 if (!isGrayscaleImage(img) || img.getColorModel().hasAlpha()) {
-                    src = new BufferedImage(effDim.width, effDim.height, 
+                    /* Disabled as this doesn't work reliably, use the fallback below
+                    src = new BufferedImage(effDim.width, effDim.height,
                             BufferedImage.TYPE_BYTE_GRAY);
+                    Graphics2D g2d = src.createGraphics();
+                    try {
+                        clearBackground(g2d, effDim);
+                    } finally {
+                        g2d.dispose();
+                    }
                     ColorConvertOp op = new ColorConvertOp(
                             ColorSpace.getInstance(ColorSpace.CS_GRAY), null);
                     op.filter((BufferedImage)img, src);
+                    */
                 } else {
                     src = (BufferedImage)img;
                 }
             }
             if (src == null) {
-                src = new BufferedImage(effDim.width, effDim.height, 
-                        BufferedImage.TYPE_BYTE_GRAY);
-                Graphics2D g2d = src.createGraphics();
-                try {
-                    AffineTransform at = new AffineTransform();
-                    double sx = effDim.getWidth() / orgDim.getWidth();
-                    double sy = effDim.getHeight() / orgDim.getHeight();
-                    at.scale(sx, sy);
-                    g2d.drawRenderedImage(img, at);
-                } finally {
-                    g2d.dispose();
+                src = BitmapImageUtil.convertToGrayscale(img, effDim);
                 }
-            }
-            MonochromeBitmapConverter converter = createMonochromeBitmapConverter();
+            MonochromeBitmapConverter converter
+                = BitmapImageUtil.createDefaultMonochromeBitmapConverter();
             converter.setHint("quality", "false");
 
-            BufferedImage buf = (BufferedImage)converter.convertToMonochrome(src);
-            
-            RenderedImage red = buf;
+            RenderedImage red = converter.convertToMonochrome(src);
             selectCurrentPattern(0, 0); //Solid black
             setTransparencyMode(sourceTransparency || mask != null, true);
             paintMonochromeBitmap(red, effResolution);
         } else {
-            //TODO untested!
             RenderedImage effImg = img;
             if (scaled) {
-                BufferedImage buf = new BufferedImage(effDim.width, effDim.height, 
-                        BufferedImage.TYPE_BYTE_BINARY);
-                Graphics2D g2d = buf.createGraphics();
-                try {
-                    AffineTransform at = new AffineTransform();
-                    double sx = effDim.getWidth() / orgDim.getWidth();
-                    double sy = effDim.getHeight() / orgDim.getHeight();
-                    at.scale(sx, sy);
-                    g2d.drawRenderedImage(img, at);
-                } finally {
-                    g2d.dispose();
+                effImg = BitmapImageUtil.convertToMonochrome(img, effDim);
                 }
-                effImg = buf;
-            }
             setSourceTransparencyMode(sourceTransparency);
             selectCurrentPattern(0, 0); //Solid black
             paintMonochromeBitmap(effImg, effResolution);
         }
     }
 
+    private void clearBackground(Graphics2D g2d, Dimension effDim) {
+        //white background
+        g2d.setBackground(Color.WHITE);
+        g2d.clearRect(0, 0, effDim.width, effDim.height);
+    }
+
+    private int toGray(int rgb) {
+        // see http://www.jguru.com/faq/view.jsp?EID=221919
+        double greyVal = 0.072169d * (rgb & 0xff);
+        rgb >>= 8;
+        greyVal += 0.715160d * (rgb & 0xff);
+        rgb >>= 8;
+        greyVal += 0.212671d * (rgb & 0xff);
+        return (int)greyVal;
+    }
+
     /**
      * Paint a bitmap at the current cursor position. The bitmap must be a monochrome
      * (1-bit) bitmap image.
@@ -809,79 +793,159 @@ public class PCLGenerator {
         if (!isValidPCLResolution(resolution)) {
             throw new IllegalArgumentException("Invalid PCL resolution: " + resolution);
         }
-        setRasterGraphicsResolution(resolution);
-        writeCommand("*r0f" + img.getHeight() + "t" + img.getWidth() + "s1A");
-        Raster raster = img.getData();
         boolean monochrome = isMonochromeImage(img);
         if (!monochrome) {
             throw new IllegalArgumentException("img must be a monochrome image");
         }
-        
-        int x = 0;
-        int y = 0;
-        int imgw = img.getWidth();
-        int imgh = img.getHeight();
-        int bytewidth = (imgw / 8);
-        if ((imgw % 8) != 0) {
-            bytewidth++;
-        }
-        byte ib;
-        byte[] rle = new byte[bytewidth * 2]; //compressed (RLE)
-        byte[] uncompressed = new byte[bytewidth]; //uncompressed
-        int lastcount = -1;
-        byte lastbyte = 0;
-        int rlewidth = 0;
 
+        setRasterGraphicsResolution(resolution);
+        writeCommand("*r0f" + img.getHeight() + "t" + img.getWidth() + "s1A");
+        Raster raster = img.getData();
+
+        Encoder encoder = new Encoder(img);
         // Transfer graphics data
-        for (y = 0; y < imgh; y++) {
-            ib = 0;
-            for (x = 0; x < imgw; x++) {
-                int sample = raster.getSample(x, y, 0);
-                //Set image bit for black
-                if ((sample == 0)) {
-                    ib |= (1 << (7 - (x % 8)));
-                }
-                    
-                //RLE encoding
-                if ((x % 8) == 7 || ((x + 1) == imgw)) {
-                    if (rlewidth < bytewidth) {
-                        if (lastcount >= 0) {
-                            if (ib == lastbyte) {
-                                lastcount++;
-                            } else {
-                                rle[rlewidth++] = (byte)(lastcount & 0xFF);
-                                rle[rlewidth++] = lastbyte;
-                                lastbyte = ib;
-                                lastcount = 0;
-                            }
+        int imgw = img.getWidth();
+        IndexColorModel cm = (IndexColorModel)img.getColorModel();
+        if (cm.getTransferType() == DataBuffer.TYPE_BYTE) {
+            DataBufferByte dataBuffer = (DataBufferByte)raster.getDataBuffer();
+            MultiPixelPackedSampleModel packedSampleModel = new MultiPixelPackedSampleModel(
+                    DataBuffer.TYPE_BYTE, img.getWidth(), img.getHeight(), 1);
+            if (img.getSampleModel().equals(packedSampleModel)
+                    && dataBuffer.getNumBanks() == 1) {
+                //Optimized packed encoding
+                byte[] buf = dataBuffer.getData();
+                int scanlineStride = packedSampleModel.getScanlineStride();
+                int idx = 0;
+                int c0 = toGray(cm.getRGB(0));
+                int c1 = toGray(cm.getRGB(1));
+                boolean zeroIsWhite = c0 > c1;
+                for (int y = 0, maxy = img.getHeight(); y < maxy; y++) {
+                    for (int x = 0, maxx = scanlineStride; x < maxx; x++) {
+                        if (zeroIsWhite) {
+                            encoder.add8Bits(buf[idx]);
                         } else {
-                            lastbyte = ib;
-                            lastcount = 0;
-                        }
-                        if (lastcount == 255 || ((x + 1) == imgw)) {
-                            rle[rlewidth++] = (byte)(lastcount & 0xFF);
-                            rle[rlewidth++] = lastbyte;
-                            lastbyte = 0;
-                            lastcount = -1;
+                            encoder.add8Bits((byte)~buf[idx]);
                         }
+                        idx++;
                     }
-                    uncompressed[x / 8] = ib;
-                    ib = 0;
+                    encoder.endLine();
+                }
+            } else {
+                //Optimized non-packed encoding
+                for (int y = 0, maxy = img.getHeight(); y < maxy; y++) {
+                    byte[] line = (byte[])raster.getDataElements(0, y, imgw, 1, null);
+                    for (int x = 0, maxx = imgw; x < maxx; x++) {
+                        encoder.addBit(line[x] == 0);
+                    }
+                    encoder.endLine();
                 }
             }
-            if (rlewidth < bytewidth) {
-                writeCommand("*b1m" + rlewidth + "W");
-                this.out.write(rle, 0, rlewidth);
-            } else {
-                writeCommand("*b0m" + bytewidth + "W");
-                this.out.write(uncompressed);
+        } else {
+            //Safe but slow fallback
+            for (int y = 0, maxy = img.getHeight(); y < maxy; y++) {
+                for (int x = 0, maxx = imgw; x < maxx; x++) {
+                    int sample = raster.getSample(x, y, 0);
+                    encoder.addBit(sample == 0);
+                }
+                encoder.endLine();
             }
-            lastcount = -1;
-            rlewidth = 0;
         }
 
         // End raster graphics
         writeCommand("*rB");
     }
 
+    private class Encoder {
+
+        private int imgw;
+        private int bytewidth;
+        private byte[] rle; //compressed (RLE)
+        private byte[] uncompressed; //uncompressed
+        private int lastcount = -1;
+        private byte lastbyte = 0;
+        private int rlewidth = 0;
+        private byte ib = 0; //current image bits
+        private int x = 0;
+        private boolean zeroRow = true;
+
+        public Encoder(RenderedImage img) {
+            imgw = img.getWidth();
+            bytewidth = (imgw / 8);
+            if ((imgw % 8) != 0) {
+                bytewidth++;
+            }
+            rle = new byte[bytewidth * 2];
+            uncompressed = new byte[bytewidth];
+        }
+
+        public void addBit(boolean bit) {
+            //Set image bit for black
+            if (bit) {
+                ib |= 1;
+            }
+
+            //RLE encoding
+            if ((x % 8) == 7 || ((x + 1) == imgw)) {
+                finishedByte();
+            } else {
+                ib <<= 1;
+            }
+            x++;
+        }
+
+        public void add8Bits(byte b) {
+            ib = b;
+            finishedByte();
+            x += 8;
+        }
+
+        private void finishedByte() {
+            if (rlewidth < bytewidth) {
+                if (lastcount >= 0) {
+                    if (ib == lastbyte) {
+                        lastcount++;
+                    } else {
+                        rle[rlewidth++] = (byte)(lastcount & 0xFF);
+                        rle[rlewidth++] = lastbyte;
+                        lastbyte = ib;
+                        lastcount = 0;
+                    }
+                } else {
+                    lastbyte = ib;
+                    lastcount = 0;
+                }
+                if (lastcount == 255 || ((x + 1) == imgw)) {
+                    rle[rlewidth++] = (byte)(lastcount & 0xFF);
+                    rle[rlewidth++] = lastbyte;
+                    lastbyte = 0;
+                    lastcount = -1;
+                }
+            }
+            uncompressed[x / 8] = ib;
+            if (ib != 0) {
+                zeroRow = false;
+            }
+            ib = 0;
+        }
+
+        public void endLine() throws IOException {
+            if (zeroRow && PCLGenerator.this.currentSourceTransparency) {
+                writeCommand("*b1Y");
+            } else if (rlewidth < bytewidth) {
+                writeCommand("*b1m" + rlewidth + "W");
+                out.write(rle, 0, rlewidth);
+            } else {
+                writeCommand("*b0m" + bytewidth + "W");
+                out.write(uncompressed);
+            }
+            lastcount = -1;
+            rlewidth = 0;
+            ib = 0;
+            x = 0;
+            zeroRow = true;
+        }
+
+
+    }
+
 }
