@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractPSTranscoder.java 627367 2008-02-13 12:03:30Z maxberger $ */
- 
+/* $Id: AbstractPSTranscoder.java 820689 2009-10-01 15:36:10Z jeremias $ */
+
 package org.apache.fop.render.ps;
 
 
@@ -28,60 +28,75 @@ import java.io.OutputStream;
 import org.w3c.dom.Document;
 import org.w3c.dom.svg.SVGLength;
 
-import org.apache.avalon.framework.configuration.Configuration;
 import org.apache.batik.bridge.BridgeContext;
 import org.apache.batik.bridge.UnitProcessor;
 import org.apache.batik.transcoder.TranscoderException;
 import org.apache.batik.transcoder.TranscoderOutput;
 import org.apache.batik.transcoder.image.ImageTranscoder;
-import org.apache.xmlgraphics.java2d.ps.AbstractPSDocumentGraphics2D;
-import org.apache.xmlgraphics.java2d.TextHandler;
 
+import org.apache.xmlgraphics.java2d.ps.AbstractPSDocumentGraphics2D;
+
+import org.apache.fop.apps.FOPException;
 import org.apache.fop.fonts.FontInfo;
-import org.apache.fop.fonts.FontSetup;
 import org.apache.fop.svg.AbstractFOPTranscoder;
+import org.apache.fop.svg.PDFDocumentGraphics2DConfigurator;
 
 /**
  * This class enables to transcode an input to a PostScript document.
  *
- * <p>Two transcoding hints (<tt>KEY_WIDTH</tt> and
- * <tt>KEY_HEIGHT</tt>) can be used to respectively specify the image
+ * <p>Two transcoding hints (<code>KEY_WIDTH</code> and
+ * <code>KEY_HEIGHT</code>) can be used to respectively specify the image
  * width and the image height. If only one of these keys is specified,
  * the transcoder preserves the aspect ratio of the original image.
  *
- * <p>The <tt>KEY_BACKGROUND_COLOR</tt> defines the background color
+ * <p>The <code>KEY_BACKGROUND_COLOR</code> defines the background color
  * to use for opaque image formats, or the background color that may
  * be used for image formats that support alpha channel.
  *
- * <p>The <tt>KEY_AOI</tt> represents the area of interest to paint
+ * <p>The <code>KEY_AOI</code> represents the area of interest to paint
  * in device space.
  *
  * <p>Three additional transcoding hints that act on the SVG
  * processor can be specified:
  *
- * <p><tt>KEY_LANGUAGE</tt> to set the default language to use (may be
+ * <p><code>KEY_LANGUAGE</code> to set the default language to use (may be
  * used by a &lt;switch> SVG element for example),
- * <tt>KEY_USER_STYLESHEET_URI</tt> to fix the URI of a user
- * stylesheet, and <tt>KEY_PIXEL_TO_MM</tt> to specify the pixel to
+ * <code>KEY_USER_STYLESHEET_URI</code> to fix the URI of a user
+ * stylesheet, and <code>KEY_PIXEL_TO_MM</code> to specify the pixel to
  * millimeter conversion factor.
  *
  * @author <a href="mailto:keiron@aftexsw.com">Keiron Liddle</a>
- * @version $Id: AbstractPSTranscoder.java 627367 2008-02-13 12:03:30Z maxberger $
+ * @version $Id: AbstractPSTranscoder.java 820689 2009-10-01 15:36:10Z jeremias $
  */
 public abstract class AbstractPSTranscoder extends AbstractFOPTranscoder {
 
-    private   Configuration                cfg      = null;
+    /** the root Graphics2D instance for generating PostScript */
     protected AbstractPSDocumentGraphics2D graphics = null;
 
+    private FontInfo fontInfo;
+
     /**
-     * Constructs a new <tt>AbstractPSTranscoder</tt>.
+     * Constructs a new {@link AbstractPSTranscoder}.
      */
     public AbstractPSTranscoder() {
         super();
     }
 
+    /**
+     * Creates the root Graphics2D instance for generating PostScript.
+     * @return the root Graphics2D
+     */
     protected abstract AbstractPSDocumentGraphics2D createDocumentGraphics2D();
 
+    /** {@inheritDoc} */
+    protected boolean getAutoFontsDefault() {
+        //Currently set to false because auto-fonts requires a lot of memory in the PostScript
+        //case: All fonts (even the unsupported TTF fonts) need to be loaded and TrueType loading
+        //is currently very memory-intensive. At default JVM memory settings, this would result
+        //in OutOfMemoryErrors otherwise.
+        return false;
+    }
+
     /**
      * Transcodes the specified Document as an image in the specified output.
      *
@@ -91,28 +106,31 @@ public abstract class AbstractPSTranscoder extends AbstractFOPTranscoder {
      * @exception TranscoderException if an error occured while transcoding
      */
     protected void transcode(Document document, String uri,
-                             TranscoderOutput output) 
+                             TranscoderOutput output)
         throws TranscoderException {
 
         graphics = createDocumentGraphics2D();
         if (!isTextStroked()) {
-            FontInfo fontInfo = new FontInfo();   
-            //TODO Do custom font configuration here somewhere/somehow 
-            FontSetup.setup(fontInfo, null, null);   
-            graphics.setCustomTextHandler(new NativeTextHandler(graphics, fontInfo));
+            try {
+                this.fontInfo = PDFDocumentGraphics2DConfigurator.createFontInfo(
+                        getEffectiveConfiguration());
+                graphics.setCustomTextHandler(new NativeTextHandler(graphics, fontInfo));
+            } catch (FOPException fe) {
+                throw new TranscoderException(fe);
+            }
         }
 
         super.transcode(document, uri, output);
 
         getLogger().trace("document size: " + width + " x " + height);
-        
+
         // prepare the image to be painted
-        UnitProcessor.Context uctx = UnitProcessor.createContext(ctx, 
+        UnitProcessor.Context uctx = UnitProcessor.createContext(ctx,
                     document.getDocumentElement());
-        float widthInPt = UnitProcessor.userSpaceToSVG(width, SVGLength.SVG_LENGTHTYPE_PT, 
+        float widthInPt = UnitProcessor.userSpaceToSVG(width, SVGLength.SVG_LENGTHTYPE_PT,
                     UnitProcessor.HORIZONTAL_LENGTH, uctx);
         int w = (int)(widthInPt + 0.5);
-        float heightInPt = UnitProcessor.userSpaceToSVG(height, SVGLength.SVG_LENGTHTYPE_PT, 
+        float heightInPt = UnitProcessor.userSpaceToSVG(height, SVGLength.SVG_LENGTHTYPE_PT,
                 UnitProcessor.HORIZONTAL_LENGTH, uctx);
         int h = (int)(heightInPt + 0.5);
         getLogger().trace("document size: " + w + "pt x " + h + "pt");
@@ -140,24 +158,18 @@ public abstract class AbstractPSTranscoder extends AbstractFOPTranscoder {
             throw new TranscoderException(ex);
         }
     }
-    
+
     /** {@inheritDoc} */
     protected BridgeContext createBridgeContext() {
+        //For compatibility with Batik 1.6
+        return createBridgeContext("1.x");
+    }
 
-        BridgeContext ctx = new BridgeContext(userAgent);
-        if (!isTextStroked()) {
-            TextHandler handler = graphics.getCustomTextHandler();
-            if (handler instanceof NativeTextHandler) {
-                NativeTextHandler nativeTextHandler = (NativeTextHandler)handler; 
-                PSTextPainter textPainter = new PSTextPainter(nativeTextHandler);
-                ctx.setTextPainter(textPainter);
-                ctx.putBridge(new PSTextElementBridge(textPainter));
-            }
-        }
-
-        //ctx.putBridge(new PSImageElementBridge());
+    /** {@inheritDoc} */
+    public BridgeContext createBridgeContext(String version) {
+        BridgeContext ctx = new PSBridgeContext(userAgent, (isTextStroked() ? null : fontInfo),
+                getImageManager(), getImageSessionContext());
         return ctx;
     }
 
-
 }
