@@ -15,21 +15,23 @@
  * limitations under the License.
  */
 
-/* $Id: NativeTextHandler.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: NativeTextHandler.java 719629 2008-11-21 16:33:33Z jeremias $ */
 
 package org.apache.fop.render.ps;
 
+import java.awt.Graphics2D;
 import java.awt.Shape;
 import java.awt.geom.AffineTransform;
 import java.io.IOException;
 
+import org.apache.xmlgraphics.java2d.ps.PSGraphics2D;
+import org.apache.xmlgraphics.java2d.ps.PSTextHandler;
+import org.apache.xmlgraphics.ps.PSGenerator;
+
 import org.apache.fop.fonts.Font;
 import org.apache.fop.fonts.FontInfo;
 import org.apache.fop.fonts.FontSetup;
 import org.apache.fop.fonts.FontTriplet;
-import org.apache.xmlgraphics.java2d.ps.PSGraphics2D;
-import org.apache.xmlgraphics.java2d.ps.PSTextHandler;
-import org.apache.xmlgraphics.ps.PSGenerator;
 
 /**
  * Specialized TextHandler implementation that the PSGraphics2D class delegates to to paint text
@@ -37,17 +39,17 @@ import org.apache.xmlgraphics.ps.PSGenerator;
  */
 public class NativeTextHandler implements PSTextHandler {
 
-    private PSGraphics2D g2d;
-    
+    private PSGraphics2D rootG2D;
+
     /** FontInfo containing all available fonts */
     protected FontInfo fontInfo;
 
     /** Currently valid Font */
     protected Font font;
-    
+
     /** Overriding FontState */
     protected Font overrideFont = null;
-    
+
     /** the current (internal) font name */
     protected String currentFontName;
 
@@ -60,20 +62,20 @@ public class NativeTextHandler implements PSTextHandler {
      * @param fontInfo the FontInfo object with all available fonts
      */
     public NativeTextHandler(PSGraphics2D g2d, FontInfo fontInfo) {
-        this.g2d = g2d;
+        this.rootG2D = g2d;
         if (fontInfo != null) {
             this.fontInfo = fontInfo;
         } else {
             setupFontInfo();
         }
     }
-    
+
     private void setupFontInfo() {
         //Sets up a FontInfo with default fonts
         fontInfo = new FontInfo();
-        FontSetup.setup(fontInfo, null, null);
+        FontSetup.setup(fontInfo);
     }
-    
+
     /**
      * Return the font information associated with this object
      * @return the FontInfo object
@@ -83,9 +85,9 @@ public class NativeTextHandler implements PSTextHandler {
     }
 
     private PSGenerator getPSGenerator() {
-        return this.g2d.getPSGenerator();
+        return this.rootG2D.getPSGenerator();
     }
-    
+
     /** {@inheritDoc} */
     public void writeSetup() throws IOException {
         if (fontInfo != null) {
@@ -98,12 +100,19 @@ public class NativeTextHandler implements PSTextHandler {
         //nop
     }
 
+    /** {@inheritDoc} */
+    public void drawString(String text, float x, float y) throws IOException {
+        // TODO Remove me after removing the deprecated method in TextHandler.
+        throw new UnsupportedOperationException("Deprecated method!");
+    }
+
     /**
-     * Draw a string to the PostScript document. The text is painted using 
+     * Draw a string to the PostScript document. The text is painted using
      * text operations.
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
-    public void drawString(String s, float x, float y) throws IOException {
+    public void drawString(Graphics2D g, String s, float x, float y) throws IOException {
+        PSGraphics2D g2d = (PSGraphics2D)g;
         g2d.preparePainting();
         if (this.overrideFont == null) {
             java.awt.Font awtFont = g2d.getFont();
@@ -112,7 +121,7 @@ public class NativeTextHandler implements PSTextHandler {
             this.font = this.overrideFont;
             this.overrideFont = null;
         }
-        
+
         //Color and Font state
         g2d.establishColor(g2d.getColor());
         establishCurrentFont();
@@ -130,14 +139,14 @@ public class NativeTextHandler implements PSTextHandler {
         gen.writeln(gen.formatDouble(x) + " "
                   + gen.formatDouble(y) + " moveto ");
         gen.writeln("1 -1 scale");
-  
+
         StringBuffer sb = new StringBuffer("(");
         escapeText(s, sb);
         sb.append(") t ");
 
         gen.writeln(sb.toString());
-        
-        gen.restoreGraphicsState();        
+
+        gen.restoreGraphicsState();
     }
 
     private void escapeText(final String text, StringBuffer target) {
@@ -157,7 +166,7 @@ public class NativeTextHandler implements PSTextHandler {
         int fontSize = 1000 * f.getSize();
         String style = f.isItalic() ? "italic" : "normal";
         int weight = f.isBold() ? Font.WEIGHT_BOLD : Font.WEIGHT_NORMAL;
-                
+
         FontTriplet triplet = fontInfo.findAdjustWeight(fontFamily, style, weight);
         if (triplet == null) {
             triplet = fontInfo.findAdjustWeight("sans-serif", style, weight);
@@ -166,10 +175,10 @@ public class NativeTextHandler implements PSTextHandler {
     }
 
     private void establishCurrentFont() throws IOException {
-        if ((currentFontName != this.font.getFontName()) 
+        if ((currentFontName != this.font.getFontName())
                 || (currentFontSize != this.font.getFontSize())) {
             PSGenerator gen = getPSGenerator();
-            gen.writeln(this.font.getFontName() + " " 
+            gen.writeln(this.font.getFontName() + " "
                     + gen.formatDouble(font.getFontSize() / 1000f) + " F");
             currentFontName = this.font.getFontName();
             currentFontSize = this.font.getFontSize();
@@ -183,6 +192,6 @@ public class NativeTextHandler implements PSTextHandler {
     public void setOverrideFont(Font override) {
         this.overrideFont = override;
     }
-    
+
 
 }
