@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,21 +15,27 @@
  * limitations under the License.
  */
 
-/* $Id: PSGraphics2DAdapter.java 656286 2008-05-14 14:16:49Z jeremias $ */
- 
+/* $Id: PSGraphics2DAdapter.java 746664 2009-02-22 12:40:44Z jeremias $ */
+
 package org.apache.fop.render.ps;
 
 import java.awt.Dimension;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Rectangle2D;
+import java.awt.image.BufferedImage;
 import java.io.IOException;
+import java.util.Map;
 
 import org.apache.xmlgraphics.java2d.Graphics2DImagePainter;
 import org.apache.xmlgraphics.java2d.ps.PSGraphics2D;
 import org.apache.xmlgraphics.ps.PSGenerator;
 
 import org.apache.fop.render.AbstractGraphics2DAdapter;
+import org.apache.fop.render.ImageHandlerUtil;
 import org.apache.fop.render.RendererContext;
+import org.apache.fop.render.RendererContextConstants;
+import org.apache.fop.render.RendererContext.RendererContextWrapper;
+import org.apache.fop.render.pdf.PDFRenderer;
 
 /**
  * Graphics2DAdapter implementation for PostScript.
@@ -46,7 +52,7 @@ public class PSGraphics2DAdapter extends AbstractGraphics2DAdapter {
     public PSGraphics2DAdapter(PSRenderer renderer) {
         this(renderer.gen, true);
     }
-    
+
     /**
      * Constructor for use without a PSRenderer instance.
      * @param gen the PostScript generator
@@ -56,23 +62,30 @@ public class PSGraphics2DAdapter extends AbstractGraphics2DAdapter {
         this.gen = gen;
         this.clip = clip;
     }
-    
+
     /** {@inheritDoc} */
-    public void paintImage(Graphics2DImagePainter painter, 
+    public void paintImage(Graphics2DImagePainter painter,
             RendererContext context,
             int x, int y, int width, int height) throws IOException {
         float fwidth = width / 1000f;
         float fheight = height / 1000f;
         float fx = x / 1000f;
         float fy = y / 1000f;
-        
+
         // get the 'width' and 'height' attributes of the SVG document
         Dimension dim = painter.getImageSize();
         float imw = (float)dim.getWidth() / 1000f;
         float imh = (float)dim.getHeight() / 1000f;
 
-        float sx = fwidth / (float)imw;
-        float sy = fheight / (float)imh;
+        boolean paintAsBitmap = false;
+        if (context != null) {
+            Map foreign = (Map)context.getProperty(RendererContextConstants.FOREIGN_ATTRIBUTES);
+            paintAsBitmap = (foreign != null
+                   && ImageHandlerUtil.isConversionModeBitmap(foreign));
+        }
+
+        float sx = paintAsBitmap ? 1.0f : (fwidth / (float)imw);
+        float sy = paintAsBitmap ? 1.0f : (fheight / (float)imh);
 
         gen.commentln("%FOPBeginGraphics2D");
         gen.saveGraphicsState();
@@ -82,7 +95,7 @@ public class PSGraphics2DAdapter extends AbstractGraphics2DAdapter {
             gen.defineRect(fx, fy, fwidth, fheight);
             gen.writeln("clip");
         }
-        
+
         // transform so that the coordinates (0,0) is from the top left
         // and positive is down and to the right. (0,0) is where the
         // viewBox puts it.
@@ -95,8 +108,19 @@ public class PSGraphics2DAdapter extends AbstractGraphics2DAdapter {
         // scale to viewbox
         transform.translate(fx, fy);
         gen.getCurrentState().concatMatrix(transform);
-        Rectangle2D area = new Rectangle2D.Double(0.0, 0.0, imw, imh);
-        painter.paint(graphics, area);
+        if (paintAsBitmap) {
+            //Fallback solution: Paint to a BufferedImage
+            int resolution = (int)Math.round(context.getUserAgent().getTargetResolution());
+            RendererContextWrapper ctx = RendererContext.wrapRendererContext(context);
+            BufferedImage bi = paintToBufferedImage(painter, ctx, resolution, false, false);
+
+            float scale = PDFRenderer.NORMAL_PDF_RESOLUTION
+                            / context.getUserAgent().getTargetResolution();
+            graphics.drawImage(bi, new AffineTransform(scale, 0, 0, scale, 0, 0), null);
+        } else {
+            Rectangle2D area = new Rectangle2D.Double(0.0, 0.0, imw, imh);
+            painter.paint(graphics, area);
+        }
 
         gen.restoreGraphicsState();
         gen.commentln("%FOPEndGraphics2D");
