@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfFontManager.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: RtfFontManager.java 684577 2008-08-10 19:35:27Z jeremias $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -144,21 +144,22 @@ public final class RtfFontManager {
         }
 
         header.newLine();
-        header.writeGroupMark (true);
-        header.writeControlWord ("fonttbl;");
+        header.writeGroupMark(true);
+        header.writeControlWord("fonttbl");
 
         int len = fontTable.size ();
 
         for (int i = 0; i < len; i++) {
-            header.writeGroupMark (true);
+            header.writeGroupMark(true);
             header.newLine();
-            header.write ("\\f" + i);
-            header.write (" " + (String) fontTable.elementAt (i));
-            header.writeGroupMark (false);
+            header.write("\\f" + i);
+            header.write(" " + (String) fontTable.elementAt (i));
+            header.write(";");
+            header.writeGroupMark(false);
         }
 
         header.newLine();
-        header.writeGroupMark (false);
+        header.writeGroupMark(false);
     }
 
 
@@ -169,7 +170,7 @@ public final class RtfFontManager {
     private String getFontKey(String family) {
         return family.toLowerCase();
     }
-    
+
     /**
      * Adds a font to the table.
      *
