@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfTableCell.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: RtfTableCell.java 693742 2008-09-10 09:00:16Z jeremias $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -26,8 +26,8 @@ package org.apache.fop.render.rtf.rtflib.rtfdoc;
  * the FOP project.
  */
 
-import java.io.Writer;
 import java.io.IOException;
+import java.io.Writer;
 import java.util.Iterator;
 
 /**  A cell in an RTF table, container for paragraphs, lists, etc.
@@ -100,7 +100,7 @@ public class RtfTableCell
         if (attrs == null) {
             attrs = new RtfAttributes();
         }
-        
+
         attrs.set("intbl");
 
         paragraph = new RtfParagraph(this, writer, attrs);
@@ -192,13 +192,13 @@ public class RtfTableCell
          * table (Word2000 seems to do the same).
          * Cause of this, dont't write horizontally merged cells.
          * They just exist as placeholders in TableContext class,
-         * and are never written to RTF file.    
+         * and are never written to RTF file.
          */
         // horizontal cell merge codes
         if (hMerge == MERGE_WITH_PREVIOUS) {
             return offset;
         }
-        
+
         newLine();
         this.widthOffset = offset;
 
@@ -225,24 +225,24 @@ public class RtfTableCell
         if (attrib.getValue("number-columns-spanned") != null) {
             // Get the number of columns spanned
             int nbMergedCells = ((Integer)attrib.getValue("number-columns-spanned")).intValue();
-            
+
             RtfTable tab = getRow().getTable();
-            
+
             // Get the context of the current table in order to get the width of each column
             ITableColumnsInfo tableColumnsInfo
                 = tab.getITableColumnsInfo();
-            
+
             tableColumnsInfo.selectFirstColumn();
 
             // Reach the column index in table context corresponding to the current column cell
             // id is the index of the current cell (it begins at 1)
             // getColumnIndex() is the index of the current column in table context (it begins at 0)
-            //  => so we must widthdraw 1 when comparing these two variables.
+            //  => so we must withdraw 1 when comparing these two variables.
             while ((this.id - 1) != tableColumnsInfo.getColumnIndex()) {
                tableColumnsInfo.selectNextColumn();
             }
 
-            // We widthdraw one cell because the first cell is already created
+            // We withdraw one cell because the first cell is already created
             // (it's the current cell) !
             int i = nbMergedCells - 1;
             while (i > 0) {
@@ -255,25 +255,22 @@ public class RtfTableCell
         final int xPos = offset + iCurrentWidth;
 
         //these lines added by Chris Scott, Westinghouse
-        //some attributes need to be writting before opening block
+        //some attributes need to be written before opening block
         if (setCenter) {
-            writeControlWord("qc");
+            writeControlWord("trqc");
         } else if (setRight) {
-            writeControlWord("qr");
+            writeControlWord("trqr");
         } else {
-            writeControlWord("ql");
+            writeControlWord("trql");
         }
         writeAttributes (attrib, ITableAttributes.CELL_VERT_ALIGN);
 
         writeControlWord("cellx" + xPos);
 
-        //TODO Why is this here, right after an alignment command is written (see above)?
-        writeControlWord("ql");
-
         return xPos;
 
     }
-    
+
     /**
      * Overriden to avoid writing any it's a merged cell.
      * @throws IOException for I/O problems
@@ -283,7 +280,7 @@ public class RtfTableCell
        if (hMerge == MERGE_WITH_PREVIOUS) {
            return;
        }
-       
+
        super.writeRtfContent();
     }
 
@@ -297,7 +294,7 @@ public class RtfTableCell
         if (hMerge == MERGE_WITH_PREVIOUS) {
             return;
         }
-        
+
         super.writeRtfPrefix();
     }
 
@@ -310,7 +307,7 @@ public class RtfTableCell
         if (hMerge == MERGE_WITH_PREVIOUS) {
             return;
         }
-        
+
         if (getRow().getTable().isNestedTable()) {
             //nested table
             writeControlWordNS("nestcell");
@@ -330,19 +327,19 @@ public class RtfTableCell
                 writeControlWord("qr");
             } else {
                 RtfElement lastChild = null;
-                
+
                 if (getChildren().size() > 0) {
                     lastChild = (RtfElement) getChildren().get(getChildren().size() - 1);
                 }
-                    
-                
+
+
                 if (lastChild != null
                         && lastChild instanceof RtfTextrun) {
-                    //Don't write \ql in order to allow for example a right aligned paragraph 
+                    //Don't write \ql in order to allow for example a right aligned paragraph
                     //in a not right aligned table-cell to write its \qr.
                 } else {
                     writeControlWord("ql");
-                }                
+                }
             }
 
             if (!containsText()) {
@@ -497,7 +494,7 @@ public class RtfTableCell
         }
         return result;
     }
-    
+
     /**
      * Returns the current RtfTextrun object.
      * Opens a new one if necessary.
@@ -506,20 +503,20 @@ public class RtfTableCell
      */
     public RtfTextrun getTextrun() throws IOException {
         RtfAttributes attrs = new RtfAttributes();
-        
+
         if (!getRow().getTable().isNestedTable()) {
             attrs.set("intbl");
         }
-        
+
         RtfTextrun textrun = RtfTextrun.getTextrun(this, writer, attrs);
 
         //Suppress the very last \par, because the closing \cell applies the
-        //paragraph attributes. 
-        textrun.setSuppressLastPar(true);  
-        
+        //paragraph attributes.
+        textrun.setSuppressLastPar(true);
+
         return textrun;
     }
-    
+
     /**
      * Get the parent row.
      * @return The parent row.
@@ -534,6 +531,6 @@ public class RtfTableCell
             e = e.parent;
         }
 
-        return null;  
+        return null;
     }
 }
