@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: XMLRenderer.java 651302 2008-04-24 16:08:53Z jeremias $ */
+/* $Id: XMLRenderer.java 830563 2009-10-28 14:42:11Z jeremias $ */
 
 package org.apache.fop.render.xml;
 
@@ -34,13 +34,12 @@ import javax.xml.transform.sax.TransformerHandler;
 import javax.xml.transform.stream.StreamResult;
 
 import org.w3c.dom.Document;
+import org.w3c.dom.Node;
+import org.w3c.dom.NodeList;
 
-import org.xml.sax.Attributes;
-import org.xml.sax.ContentHandler;
 import org.xml.sax.SAXException;
-import org.xml.sax.ext.LexicalHandler;
-import org.xml.sax.helpers.AttributesImpl;
 
+import org.apache.xmlgraphics.util.QName;
 import org.apache.xmlgraphics.util.XMLizable;
 
 import org.apache.fop.apps.FOPException;
@@ -83,54 +82,35 @@ import org.apache.fop.area.inline.Viewport;
 import org.apache.fop.area.inline.WordArea;
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.extensions.ExtensionAttachment;
+import org.apache.fop.fo.extensions.ExtensionElementMapping;
+import org.apache.fop.fo.extensions.InternalElementMapping;
 import org.apache.fop.fonts.FontInfo;
 import org.apache.fop.fonts.FontTriplet;
-import org.apache.fop.render.PrintRenderer;
 import org.apache.fop.render.Renderer;
 import org.apache.fop.render.RendererContext;
 import org.apache.fop.render.XMLHandler;
 import org.apache.fop.util.ColorUtil;
-import org.apache.fop.util.QName;
+import org.apache.fop.util.DOM2SAX;
 
 /**
  * Renderer that renders areas to XML for debugging purposes.
  * This creates an xml that contains the information of the area
  * tree. It does not output any state or derived information.
- * The output can be used to build a new area tree which can be 
+ * The output can be used to build a new area tree which can be
  * rendered to any renderer.
  */
-public class XMLRenderer extends PrintRenderer {
+public class XMLRenderer extends AbstractXMLRenderer {
 
     /** XML MIME type */
     public static final String XML_MIME_TYPE = MimeConstants.MIME_FOP_AREA_TREE;
 
-    /** Main namespace in use. */
-    public static final String NS = "";
-
-    /** CDATA type */
-    public static final String CDATA = "CDATA";
-
-    /** An empty Attributes object used when no attributes are needed. */
-    public static final Attributes EMPTY_ATTS = new AttributesImpl();
-
     private boolean startedSequence = false;
-    private RendererContext context;
     private boolean compactFormat = false;
 
     /** If not null, the XMLRenderer will mimic another renderer by using its font setup. */
     protected Renderer mimic;
 
-    /** ContentHandler that the generated XML is written to */
-    protected ContentHandler handler;
-
-    /** AttributesImpl instance that can be used during XML generation. */
-    protected AttributesImpl atts = new AttributesImpl();
-
-    /** The OutputStream to write the generated XML to. */
-    protected OutputStream out;
-
-    /** A list of ExtensionAttachements received through processOffDocumentItem() */
-    protected List extensionAttachments;
+    private int pageSequenceIndex;
 
     /**
      * Creates a new XML renderer.
@@ -163,7 +143,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /** {@inheritDoc} */
-    public void setupFontInfo(FontInfo inFontInfo) {
+    public void setupFontInfo(FontInfo inFontInfo) throws FOPException {
         if (mimic != null) {
             mimic.setupFontInfo(inFontInfo);
         } else {
@@ -172,127 +152,17 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * Sets an outside TransformerHandler to use instead of the default one
-     * create in this class in startRenderer().
-     * @param handler Overriding TransformerHandler
+     * Controls whether to create a more compact format which omit certain attributes.
+     * @param compact true to activate the compact format
      */
-    public void setContentHandler(ContentHandler handler) {
-        this.handler = handler;
-    }
-
     public void setCompactFormat(boolean compact) {
         this.compactFormat = compact;
     }
-    
+
     private boolean isDetailedFormat() {
         return !this.compactFormat;
     }
 
-    /**
-     * Handles SAXExceptions.
-     * @param saxe the SAXException to handle
-     */
-    protected void handleSAXException(SAXException saxe) {
-        throw new RuntimeException(saxe.getMessage());
-    }
-
-    /**
-     * Writes a comment to the generated XML.
-     * @param comment the comment
-     */
-    protected void comment(String comment) {
-        if (handler instanceof LexicalHandler) { 
-            try {
-                ((LexicalHandler) handler).comment(comment.toCharArray(), 0, comment.length());
-            } catch (SAXException saxe) {
-                handleSAXException(saxe);
-            }
-        }
-    }
-    
-    /**
-     * Starts a new element (without attributes).
-     * @param tagName tag name of the element
-     */
-    protected void startElement(String tagName) {
-        startElement(tagName, EMPTY_ATTS);
-    }
-
-    /**
-     * Starts a new element.
-     * @param tagName tag name of the element
-     * @param atts attributes to add
-     */
-    protected void startElement(String tagName, Attributes atts) {
-        try {
-            handler.startElement(NS, tagName, tagName, atts);
-        } catch (SAXException saxe) {
-            handleSAXException(saxe);
-        }
-    }
-
-    /**
-     * Ends an element.
-     * @param tagName tag name of the element
-     */
-    protected void endElement(String tagName) {
-        try {
-            handler.endElement(NS, tagName, tagName);
-        } catch (SAXException saxe) {
-            handleSAXException(saxe);
-        }
-    }
-
-    /**
-     * Sends plain text to the XML
-     * @param text the text
-     */
-    protected void characters(String text) {
-        try {
-            char[] ca = text.toCharArray();
-            handler.characters(ca, 0, ca.length);
-        } catch (SAXException saxe) {
-            handleSAXException(saxe);
-        }
-    }
-
-    /**
-     * Adds a new attribute to the protected member variable "atts".
-     * @param name name of the attribute
-     * @param value value of the attribute
-     */
-    protected void addAttribute(String name, String value) {
-        atts.addAttribute(NS, name, name, CDATA, value);
-    }
-
-    /**
-     * Adds a new attribute to the protected member variable "atts".
-     * @param name name of the attribute
-     * @param value value of the attribute
-     */
-    protected void addAttribute(QName name, String value) {
-        atts.addAttribute(name.getNamespaceURI(), name.getLocalName(), name.getQName(), 
-                CDATA, value);
-    }
-
-    /**
-     * Adds a new attribute to the protected member variable "atts".
-     * @param name name of the attribute
-     * @param value value of the attribute
-     */
-    protected void addAttribute(String name, int value) {
-        addAttribute(name, Integer.toString(value));
-    }
-
-    /**
-     * Adds a new attribute to the protected member variable "atts".
-     * @param name name of the attribute
-     * @param rect a Rectangle2D to format and use as attribute value
-     */
-    protected void addAttribute(String name, Rectangle2D rect) {
-        addAttribute(name, createString(rect));
-    }
-
     /**
      * Adds the general Area attributes.
      * @param area Area to extract attributes from
@@ -383,7 +253,7 @@ public class XMLRenderer extends PrintRenderer {
                 }
             }
         }
-        
+
         transferForeignObjects(area);
     }
 
@@ -401,23 +271,11 @@ public class XMLRenderer extends PrintRenderer {
         while (iter.hasNext()) {
             Map.Entry entry = (Map.Entry)iter.next();
             String qn = "xmlns:" + (String)entry.getKey();
-            atts.addAttribute("", (String)entry.getKey(), qn, 
+            atts.addAttribute("", (String)entry.getKey(), qn,
                     CDATA, (String)entry.getValue());
         }
     }
 
-    private String createString(Rectangle2D rect) {
-        return "" + (int) rect.getX() + " " + (int) rect.getY() + " "
-                  + (int) rect.getWidth() + " " + (int) rect.getHeight();
-    }
-
-    private void handleDocumentExtensionAttachments() {
-        if (extensionAttachments != null && extensionAttachments.size() > 0) {
-            handleExtensionAttachments(extensionAttachments);
-            extensionAttachments.clear();
-        }
-    }
-
     /** {@inheritDoc} */
     public void processOffDocumentItem(OffDocumentItem oDI) {
         if (oDI instanceof BookmarkData) {
@@ -518,9 +376,7 @@ public class XMLRenderer extends PrintRenderer {
         startElement("areaTree");
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void stopRenderer() throws IOException {
         endPageSequence();
         endElement("areaTree");
@@ -535,9 +391,7 @@ public class XMLRenderer extends PrintRenderer {
         log.debug("Written out Area Tree XML");
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void renderPage(PageViewport page) throws IOException, FOPException {
         atts.clear();
         addAttribute("bounds", page.getViewArea());
@@ -561,7 +415,8 @@ public class XMLRenderer extends PrintRenderer {
         endElement("pageViewport");
     }
 
-    private void handleExtensionAttachments(List attachments) {
+    /** {@inheritDoc} */
+    protected void handleExtensionAttachments(List attachments) {
         if (attachments != null && attachments.size() > 0) {
             startElement("extension-attachments");
             Iterator i = attachments.iterator();
@@ -582,10 +437,6 @@ public class XMLRenderer extends PrintRenderer {
         }
     }
 
-    private void handlePageExtensionAttachments(PageViewport page) {
-        handleExtensionAttachments(page.getExtensionAttachments());
-    }
-
     /** {@inheritDoc} */
     public void startPageSequence(PageSequence pageSequence) {
         handleDocumentExtensionAttachments();
@@ -598,7 +449,32 @@ public class XMLRenderer extends PrintRenderer {
         if (pageSequence.getCountry() != null) {
             addAttribute("country", pageSequence.getCountry());
         }
+        transferForeignObjects(pageSequence);
         startElement("pageSequence", atts);
+        if (this.getUserAgent().isAccessibilityEnabled()) {
+            String structureTreeElement = "structureTree";
+            startElement(structureTreeElement);
+            try {
+                this.handler.startPrefixMapping("foi", InternalElementMapping.URI);
+                this.handler.startPrefixMapping("fox", ExtensionElementMapping.URI);
+                NodeList nodes = getUserAgent().getStructureTree().getPageSequence(
+                        pageSequenceIndex++);
+                for (int i = 0, n = nodes.getLength(); i < n; i++) {
+                    Node node = nodes.item(i);
+                    try {
+                        new DOM2SAX(handler).writeFragment(node);
+                    } catch (SAXException e) {
+                        handleSAXException(e);
+                    }
+                }
+                this.handler.endPrefixMapping("fox");
+                this.handler.endPrefixMapping("foi");
+            } catch (SAXException se) {
+                handleSAXException(se);
+            }
+            endElement(structureTreeElement);
+        }
+        handleExtensionAttachments(pageSequence.getExtensionAttachments());
         LineArea seqTitle = pageSequence.getTitle();
         if (seqTitle != null) {
             startElement("title");
@@ -703,7 +579,9 @@ public class XMLRenderer extends PrintRenderer {
      * {@inheritDoc}
      */
     protected void renderFootnote(Footnote footnote) {
-        startElement("footnote");
+        atts.clear();
+        addAttribute("top-offset", footnote.getTop());
+        startElement("footnote", atts);
         super.renderFootnote(footnote);
         endElement("footnote");
     }
@@ -763,7 +641,7 @@ public class XMLRenderer extends PrintRenderer {
             renderBlocks(block, children);
         }
     }
-    
+
     /** {@inheritDoc} */
     protected void renderBlock(Block block) {
         atts.clear();
@@ -936,7 +814,7 @@ public class XMLRenderer extends PrintRenderer {
     protected void renderWord(WordArea word) {
         atts.clear();
         addAttribute("offset", word.getOffset());
-        int[] letterAdjust = word.getLetterAdjustArray(); 
+        int[] letterAdjust = word.getLetterAdjustArray();
         if (letterAdjust != null) {
             StringBuffer sb = new StringBuffer(64);
             boolean nonZeroFound = false;
