@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,10 +15,23 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractFOPTranscoder.java 591587 2007-11-03 10:59:09Z jeremias $ */
- 
+/* $Id: AbstractFOPTranscoder.java 820689 2009-10-01 15:36:10Z jeremias $ */
+
 package org.apache.fop.svg;
 
+import java.io.IOException;
+import java.io.InputStream;
+
+import javax.xml.transform.Source;
+import javax.xml.transform.stream.StreamSource;
+
+import org.w3c.dom.DOMImplementation;
+
+import org.xml.sax.EntityResolver;
+
+import org.apache.avalon.framework.configuration.Configuration;
+import org.apache.avalon.framework.configuration.ConfigurationException;
+import org.apache.avalon.framework.configuration.DefaultConfiguration;
 import org.apache.batik.bridge.UserAgent;
 import org.apache.batik.dom.svg.SVGDOMImplementation;
 import org.apache.batik.dom.util.DocumentFactory;
@@ -28,11 +41,16 @@ import org.apache.batik.transcoder.TranscoderException;
 import org.apache.batik.transcoder.TranscodingHints;
 import org.apache.batik.transcoder.image.ImageTranscoder;
 import org.apache.batik.transcoder.keys.BooleanKey;
+import org.apache.batik.transcoder.keys.FloatKey;
+import org.apache.batik.util.ParsedURL;
 import org.apache.batik.util.SVGConstants;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.impl.SimpleLog;
-import org.w3c.dom.DOMImplementation;
-import org.xml.sax.EntityResolver;
+
+import org.apache.xmlgraphics.image.loader.ImageContext;
+import org.apache.xmlgraphics.image.loader.ImageManager;
+import org.apache.xmlgraphics.image.loader.ImageSessionContext;
+import org.apache.xmlgraphics.image.loader.impl.AbstractImageSessionContext;
 
 /**
  * This is the common base class of all of FOP's transcoders.
@@ -40,11 +58,24 @@ import org.xml.sax.EntityResolver;
 public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
 
     /**
-     * The key to specify whether to stroke text instead of using text 
+     * The key is used to specify the resolution for on-the-fly images generated
+     * due to complex effects like gradients and filters.
+     */
+    public static final TranscodingHints.Key KEY_DEVICE_RESOLUTION = new FloatKey();
+
+    /**
+     * The key to specify whether to stroke text instead of using text
      * operations.
      */
     public static final TranscodingHints.Key KEY_STROKE_TEXT = new BooleanKey();
 
+    /**
+     * The key is used to specify whether the available fonts should be automatically
+     * detected. The alternative is to configure the transcoder manually using a configuration
+     * file.
+     */
+    public static final TranscodingHints.Key KEY_AUTO_FONTS = new BooleanKey();
+
     /** The value to turn on text stroking. */
     public static final Boolean VALUE_FORMAT_ON = Boolean.TRUE;
 
@@ -58,6 +89,9 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
 
     private Log logger;
     private EntityResolver resolver;
+    private Configuration cfg = null;
+    private ImageManager imageManager;
+    private ImageSessionContext imageSessionContext;
 
     /**
      * Constructs a new FOP-style transcoder.
@@ -69,7 +103,7 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
         hints.put(KEY_DOM_IMPLEMENTATION,
                   SVGDOMImplementation.getDOMImplementation());
     }
-    
+
     /**
      * Creates and returns the default user agent for this transcoder. Override
      * this method if you need non-default behaviour.
@@ -78,9 +112,10 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
     protected UserAgent createUserAgent() {
         return new FOPTranscoderUserAgent();
     }
-    
+
     /**
-     * @param logger
+     * Sets the logger.
+     * @param logger the logger
      */
     public void setLogger(Log logger) {
         this.logger = logger;
@@ -93,9 +128,46 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
     public void setEntityResolver(EntityResolver resolver) {
         this.resolver = resolver;
     }
-    
+
+    /** {@inheritDoc} */
+    public void configure(Configuration cfg) throws ConfigurationException {
+        this.cfg = cfg;
+    }
+
     /**
-     * Returns the logger associated with this transcoder. It returns a 
+     * Returns the default value for the KEY_AUTO_FONTS value.
+     * @return the default value
+     */
+    protected boolean getAutoFontsDefault() {
+        return true;
+    }
+
+    /**
+     * Returns the effective configuration for the transcoder.
+     * @return the effective configuration
+     */
+    protected Configuration getEffectiveConfiguration() {
+        Configuration effCfg = this.cfg;
+        if (effCfg == null) {
+            //By default, enable font auto-detection if no cfg is given
+            boolean autoFonts = getAutoFontsDefault();
+            if (hints.containsKey(KEY_AUTO_FONTS)) {
+                autoFonts = ((Boolean)hints.get(KEY_AUTO_FONTS)).booleanValue();
+            }
+            if (autoFonts) {
+                DefaultConfiguration c = new DefaultConfiguration("cfg");
+                DefaultConfiguration fonts = new DefaultConfiguration("fonts");
+                c.addChild(fonts);
+                DefaultConfiguration autodetect = new DefaultConfiguration("auto-detect");
+                fonts.addChild(autodetect);
+                effCfg = c;
+            }
+        }
+        return effCfg;
+    }
+
+    /**
+     * Returns the logger associated with this transcoder. It returns a
      * SimpleLog if no logger has been explicitly set.
      * @return Logger the logger for the transcoder.
      */
@@ -106,9 +178,9 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
         }
         return this.logger;
     }
-    
+
     /**
-     * Creates a <tt>DocumentFactory</tt> that is used to create an SVG DOM
+     * Creates a {@link DocumentFactory} that is used to create an SVG DOM
      * tree. The specified DOM Implementation is ignored and the Batik
      * SVG DOM Implementation is automatically used.
      *
@@ -118,7 +190,7 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
      */
     protected DocumentFactory createDocumentFactory(DOMImplementation domImpl,
             String parserClassname) {
-        final FOPSAXSVGDocumentFactory factory 
+        final FOPSAXSVGDocumentFactory factory
                 = new FOPSAXSVGDocumentFactory(parserClassname);
         if (this.resolver != null) {
             factory.setAdditionalEntityResolver(this.resolver);
@@ -141,18 +213,83 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
         }
         return stroke;
     }
-    
+
+    /**
+     * Returns the device resolution that has been set up.
+     * @return the device resolution (in dpi)
+     */
+    protected float getDeviceResolution() {
+        if (hints.containsKey(KEY_DEVICE_RESOLUTION)) {
+            return ((Float)hints.get(KEY_DEVICE_RESOLUTION)).floatValue();
+        } else {
+            return 72;
+        }
+    }
+
+    /**
+     * Returns the ImageManager to be used by the transcoder.
+     * @return the image manager
+     */
+    protected ImageManager getImageManager() {
+        return this.imageManager;
+    }
+
+    /**
+     * Returns the ImageSessionContext to be used by the transcoder.
+     * @return the image session context
+     */
+    protected ImageSessionContext getImageSessionContext() {
+        return this.imageSessionContext;
+    }
+
+    /**
+     * Sets up the image infrastructure (the image loading framework).
+     * @param baseURI the base URI of the current document
+     */
+    protected void setupImageInfrastructure(final String baseURI) {
+        final ImageContext imageContext = new ImageContext() {
+            public float getSourceResolution() {
+                return 25.4f / userAgent.getPixelUnitToMillimeter();
+            }
+        };
+        this.imageManager = new ImageManager(imageContext);
+        this.imageSessionContext = new AbstractImageSessionContext() {
+
+            public ImageContext getParentContext() {
+                return imageContext;
+            }
+
+            public float getTargetResolution() {
+                return getDeviceResolution();
+            }
+
+            public Source resolveURI(String uri) {
+                System.out.println("resolve " + uri);
+                try {
+                    ParsedURL url = new ParsedURL(baseURI, uri);
+                    InputStream in = url.openStream();
+                    StreamSource source = new StreamSource(in, url.toString());
+                    return source;
+                } catch (IOException ioe) {
+                    userAgent.displayError(ioe);
+                    return null;
+                }
+            }
+
+        };
+    }
+
     // --------------------------------------------------------------------
     // FOP's default error handler (for transcoders)
     // --------------------------------------------------------------------
 
     /**
      * This is the default transcoder error handler for FOP. It logs error
-     * to an Commons Logger instead of to System.out. The remaining behaviour 
+     * to an Commons Logger instead of to System.out. The remaining behaviour
      * is the same as Batik's DefaultErrorHandler.
-     */    
+     */
     protected class FOPErrorHandler implements ErrorHandler {
-        
+
         /**
          * {@inheritDoc}
          */
@@ -189,7 +326,7 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
     protected class FOPTranscoderUserAgent extends SVGAbstractTranscoderUserAgent {
 
         /**
-         * Displays the specified error message using the <tt>ErrorHandler</tt>.
+         * Displays the specified error message using the {@link ErrorHandler}.
          * @param message the message to display
          */
         public void displayError(String message) {
@@ -201,7 +338,7 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
         }
 
         /**
-         * Displays the specified error using the <tt>ErrorHandler</tt>.
+         * Displays the specified error using the {@link ErrorHandler}.
          * @param e the exception to display
          */
         public void displayError(Exception e) {
@@ -213,7 +350,7 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
         }
 
         /**
-         * Displays the specified message using the <tt>ErrorHandler</tt>.
+         * Displays the specified message using the {@link ErrorHandler}.
          * @param message the message to display
          */
         public void displayMessage(String message) {
@@ -222,7 +359,7 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
 
         /**
          * Returns the pixel to millimeter conversion factor specified in the
-         * <tt>TranscodingHints</tt> or 0.3528 if any.
+         * {@link TranscodingHints} or 0.3528 if any.
          * @return the pixel unit to millimeter factor
          */
         public float getPixelUnitToMillimeter() {
@@ -244,5 +381,5 @@ public abstract class AbstractFOPTranscoder extends SVGAbstractTranscoder {
         }
 
     }
-    
+
 }
