@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFDocumentGraphics2DConfigurator.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: PDFDocumentGraphics2DConfigurator.java 766594 2009-04-20 06:50:59Z jeremias $ */
 
 package org.apache.fop.svg;
 
@@ -23,13 +23,15 @@ import java.util.List;
 
 import org.apache.avalon.framework.configuration.Configuration;
 import org.apache.avalon.framework.configuration.ConfigurationException;
+
 import org.apache.fop.apps.FOPException;
-import org.apache.fop.fonts.FontCache;
+import org.apache.fop.fonts.FontEventListener;
 import org.apache.fop.fonts.FontInfo;
+import org.apache.fop.fonts.FontInfoConfigurator;
+import org.apache.fop.fonts.FontManager;
 import org.apache.fop.fonts.FontResolver;
 import org.apache.fop.fonts.FontSetup;
 import org.apache.fop.pdf.PDFDocument;
-import org.apache.fop.render.PrintRendererConfigurator;
 import org.apache.fop.render.pdf.PDFRendererConfigurator;
 
 /**
@@ -43,31 +45,55 @@ public class PDFDocumentGraphics2DConfigurator {
      * @param cfg the configuration
      * @throws ConfigurationException if an error occurs while configuring the object
      */
-    public void configure(PDFDocumentGraphics2D graphics, Configuration cfg) 
+    public void configure(PDFDocumentGraphics2D graphics, Configuration cfg)
             throws ConfigurationException {
         PDFDocument pdfDoc = graphics.getPDFDocument();
-        
+
         //Filter map
         pdfDoc.setFilterMap(
                 PDFRendererConfigurator.buildFilterMapFromConfiguration(cfg));
-        
+
         //Fonts
         try {
-            FontResolver fontResolver = FontSetup.createMinimalFontResolver();
-            //TODO The following could be optimized by retaining the FontCache somewhere
-            FontCache fontCache = FontCache.load();
-            if (fontCache == null) {
-                fontCache = new FontCache();
-            }
-            List fontList = PrintRendererConfigurator.buildFontListFromConfiguration(
-                    cfg, null, fontResolver, false, fontCache);
-            fontCache.save();
-            FontInfo fontInfo = new FontInfo();
-            FontSetup.setup(fontInfo, fontList, fontResolver);
+            FontInfo fontInfo = createFontInfo(cfg);
             graphics.setFontInfo(fontInfo);
         } catch (FOPException e) {
             throw new ConfigurationException("Error while setting up fonts", e);
         }
     }
-    
+
+    /**
+     * Creates the {@link FontInfo} instance for the given configuration.
+     * @param cfg the configuration
+     * @return the font collection
+     * @throws FOPException if an error occurs while setting up the fonts
+     */
+    public static FontInfo createFontInfo(Configuration cfg) throws FOPException {
+        FontInfo fontInfo = new FontInfo();
+        if (cfg != null) {
+            FontResolver fontResolver = FontManager.createMinimalFontResolver();
+            //TODO The following could be optimized by retaining the FontManager somewhere
+            FontManager fontManager = new FontManager();
+
+            //TODO Make use of fontBaseURL, font substitution and referencing configuration
+            //Requires a change to the expected configuration layout
+
+            //TODO Wire in the FontEventListener
+            final FontEventListener listener = null;
+            final boolean strict = false;
+            FontInfoConfigurator fontInfoConfigurator
+                = new FontInfoConfigurator(cfg, fontManager, fontResolver, listener, strict);
+            List/*<EmbedFontInfo>*/ fontInfoList = new java.util.ArrayList/*<EmbedFontInfo>*/();
+            fontInfoConfigurator.configure(fontInfoList);
+
+            if (fontManager.useCache()) {
+                fontManager.getFontCache().save();
+            }
+            FontSetup.setup(fontInfo, fontInfoList, fontResolver);
+        } else {
+            FontSetup.setup(fontInfo);
+        }
+        return fontInfo;
+    }
+
 }
