@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFTranscoder.java 611278 2008-01-11 19:50:53Z jeremias $ */
+/* $Id: PDFTranscoder.java 820689 2009-10-01 15:36:10Z jeremias $ */
 
 package org.apache.fop.svg;
 
@@ -23,34 +23,19 @@ import java.awt.Color;
 import java.io.BufferedOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
-import java.io.InputStream;
-
-import javax.xml.transform.Source;
-import javax.xml.transform.stream.StreamSource;
 
 import org.w3c.dom.Document;
 import org.w3c.dom.svg.SVGLength;
 
 import org.apache.avalon.framework.configuration.Configurable;
 import org.apache.avalon.framework.configuration.Configuration;
-import org.apache.avalon.framework.configuration.ConfigurationException;
-import org.apache.avalon.framework.configuration.DefaultConfiguration;
 import org.apache.batik.bridge.BridgeContext;
 import org.apache.batik.bridge.UnitProcessor;
 import org.apache.batik.bridge.UserAgent;
 import org.apache.batik.ext.awt.RenderingHintsKeyExt;
 import org.apache.batik.transcoder.TranscoderException;
 import org.apache.batik.transcoder.TranscoderOutput;
-import org.apache.batik.transcoder.TranscodingHints;
 import org.apache.batik.transcoder.image.ImageTranscoder;
-import org.apache.batik.transcoder.keys.BooleanKey;
-import org.apache.batik.transcoder.keys.FloatKey;
-import org.apache.batik.util.ParsedURL;
-
-import org.apache.xmlgraphics.image.loader.ImageContext;
-import org.apache.xmlgraphics.image.loader.ImageManager;
-import org.apache.xmlgraphics.image.loader.ImageSessionContext;
-import org.apache.xmlgraphics.image.loader.impl.AbstractImageSessionContext;
 
 import org.apache.fop.Version;
 import org.apache.fop.fonts.FontInfo;
@@ -58,62 +43,44 @@ import org.apache.fop.fonts.FontInfo;
 /**
  * This class enables to transcode an input to a pdf document.
  *
- * <p>Two transcoding hints (<tt>KEY_WIDTH</tt> and
- * <tt>KEY_HEIGHT</tt>) can be used to respectively specify the image
+ * <p>Two transcoding hints (<code>KEY_WIDTH</code> and
+ * <code>KEY_HEIGHT</code>) can be used to respectively specify the image
  * width and the image height. If only one of these keys is specified,
  * the transcoder preserves the aspect ratio of the original image.
  *
- * <p>The <tt>KEY_BACKGROUND_COLOR</tt> defines the background color
+ * <p>The <code>KEY_BACKGROUND_COLOR</code> defines the background color
  * to use for opaque image formats, or the background color that may
  * be used for image formats that support alpha channel.
  *
- * <p>The <tt>KEY_AOI</tt> represents the area of interest to paint
+ * <p>The <code>KEY_AOI</code> represents the area of interest to paint
  * in device space.
  *
  * <p>Three additional transcoding hints that act on the SVG
  * processor can be specified:
  *
- * <p><tt>KEY_LANGUAGE</tt> to set the default language to use (may be
+ * <p><code>KEY_LANGUAGE</code> to set the default language to use (may be
  * used by a &lt;switch> SVG element for example),
- * <tt>KEY_USER_STYLESHEET_URI</tt> to fix the URI of a user
- * stylesheet, and <tt>KEY_PIXEL_TO_MM</tt> to specify the pixel to
+ * <code>KEY_USER_STYLESHEET_URI</code> to fix the URI of a user
+ * stylesheet, and <code>KEY_PIXEL_TO_MM</code> to specify the pixel to
  * millimeter conversion factor.
- * 
- * <p><tt>KEY_AUTO_FONTS</tt> to disable the auto-detection of fonts installed in the system.
+ *
+ * <p><code>KEY_AUTO_FONTS</code> to disable the auto-detection of fonts installed in the system.
  * The PDF Transcoder cannot use AWT's font subsystem and that's why the fonts have to be
  * configured differently. By default, font auto-detection is enabled to match the behaviour
  * of the other transcoders, but this may be associated with a price in the form of a small
  * performance penalty. If font auto-detection is not desired, it can be disable using this key.
  *
  * @author <a href="mailto:keiron@aftexsw.com">Keiron Liddle</a>
- * @version $Id: PDFTranscoder.java 611278 2008-01-11 19:50:53Z jeremias $
+ * @version $Id: PDFTranscoder.java 820689 2009-10-01 15:36:10Z jeremias $
  */
 public class PDFTranscoder extends AbstractFOPTranscoder
         implements Configurable {
 
-    /**
-     * The key is used to specify the resolution for on-the-fly images generated
-     * due to complex effects like gradients and filters. 
-     */
-    public static final TranscodingHints.Key KEY_DEVICE_RESOLUTION = new FloatKey();
-
-    /**
-     * The key is used to specify whether the available fonts should be automatically
-     * detected. The alternative is to configure the transcoder manually using a configuration
-     * file. 
-     */
-    public static final TranscodingHints.Key KEY_AUTO_FONTS = new BooleanKey();
-
-    private Configuration cfg = null;
-    
     /** Graphics2D instance that is used to paint to */
     protected PDFDocumentGraphics2D graphics = null;
 
-    private ImageManager imageManager;
-    private ImageSessionContext imageSessionContext;
-    
     /**
-     * Constructs a new <tt>PDFTranscoder</tt>.
+     * Constructs a new {@link PDFTranscoder}.
      */
     public PDFTranscoder() {
         super();
@@ -132,11 +99,6 @@ public class PDFTranscoder extends AbstractFOPTranscoder
             }
         };
     }
-    
-    /** {@inheritDoc} */
-    public void configure(Configuration cfg) throws ConfigurationException {
-        this.cfg = cfg;
-    }
 
     /**
      * Transcodes the specified Document as an image in the specified output.
@@ -147,37 +109,22 @@ public class PDFTranscoder extends AbstractFOPTranscoder
      * @exception TranscoderException if an error occured while transcoding
      */
     protected void transcode(Document document, String uri,
-                             TranscoderOutput output) 
+                             TranscoderOutput output)
         throws TranscoderException {
 
         graphics = new PDFDocumentGraphics2D(isTextStroked());
-        graphics.getPDFDocument().getInfo().setProducer("Apache FOP Version " 
-                + Version.getVersion() 
+        graphics.getPDFDocument().getInfo().setProducer("Apache FOP Version "
+                + Version.getVersion()
                 + ": PDF Transcoder for Batik");
         if (hints.containsKey(KEY_DEVICE_RESOLUTION)) {
-            graphics.setDeviceDPI(((Float)hints.get(KEY_DEVICE_RESOLUTION)).floatValue());
+            graphics.setDeviceDPI(getDeviceResolution());
         }
-        
+
         setupImageInfrastructure(uri);
-        
+
         try {
-            Configuration effCfg = this.cfg; 
-            if (effCfg == null) {
-                //By default, enable font auto-detection if no cfg is given
-                boolean autoFonts = true;
-                if (hints.containsKey(KEY_AUTO_FONTS)) {
-                    autoFonts = ((Boolean)hints.get(KEY_AUTO_FONTS)).booleanValue();
-                }
-                if (autoFonts) {
-                    DefaultConfiguration c = new DefaultConfiguration("pdf-transcoder");
-                    DefaultConfiguration fonts = new DefaultConfiguration("fonts");
-                    c.addChild(fonts);
-                    DefaultConfiguration autodetect = new DefaultConfiguration("auto-detect");
-                    fonts.addChild(autodetect);
-                    effCfg = c;
-                }
-            }
-            
+            Configuration effCfg = getEffectiveConfiguration();
+
             if (effCfg != null) {
                 PDFDocumentGraphics2DConfigurator configurator
                         = new PDFDocumentGraphics2DConfigurator();
@@ -195,14 +142,14 @@ public class PDFTranscoder extends AbstractFOPTranscoder
         if (getLogger().isTraceEnabled()) {
             getLogger().trace("document size: " + width + " x " + height);
         }
-        
+
         // prepare the image to be painted
-        UnitProcessor.Context uctx = UnitProcessor.createContext(ctx, 
+        UnitProcessor.Context uctx = UnitProcessor.createContext(ctx,
                     document.getDocumentElement());
-        float widthInPt = UnitProcessor.userSpaceToSVG(width, SVGLength.SVG_LENGTHTYPE_PT, 
+        float widthInPt = UnitProcessor.userSpaceToSVG(width, SVGLength.SVG_LENGTHTYPE_PT,
                     UnitProcessor.HORIZONTAL_LENGTH, uctx);
         int w = (int)(widthInPt + 0.5);
-        float heightInPt = UnitProcessor.userSpaceToSVG(height, SVGLength.SVG_LENGTHTYPE_PT, 
+        float heightInPt = UnitProcessor.userSpaceToSVG(height, SVGLength.SVG_LENGTHTYPE_PT,
                 UnitProcessor.HORIZONTAL_LENGTH, uctx);
         int h = (int)(heightInPt + 0.5);
         if (getLogger().isTraceEnabled()) {
@@ -242,39 +189,6 @@ public class PDFTranscoder extends AbstractFOPTranscoder
         }
     }
 
-    private void setupImageInfrastructure(final String baseURI) {
-        final ImageContext imageContext = new ImageContext() {
-            public float getSourceResolution() {
-                return 25.4f / userAgent.getPixelUnitToMillimeter();
-            }
-        };
-        this.imageManager = new ImageManager(imageContext);
-        this.imageSessionContext = new AbstractImageSessionContext() {
-
-            public ImageContext getParentContext() {
-                return imageContext;
-            }
-
-            public float getTargetResolution() {
-                return graphics.getDeviceDPI();
-            }
-
-            public Source resolveURI(String uri) {
-                System.out.println("resolve " + uri);
-                try {
-                    ParsedURL url = new ParsedURL(baseURI, uri);
-                    InputStream in = url.openStream();
-                    StreamSource source = new StreamSource(in, url.toString());
-                    return source;
-                } catch (IOException ioe) {
-                    userAgent.displayError(ioe);
-                    return null;
-                }
-            }
-            
-        };
-    }
-
     /** {@inheritDoc} */
     protected BridgeContext createBridgeContext() {
         //For compatibility with Batik 1.6
@@ -288,8 +202,8 @@ public class PDFTranscoder extends AbstractFOPTranscoder
             fontInfo = null;
         }
         BridgeContext ctx = new PDFBridgeContext(userAgent, fontInfo,
-                this.imageManager, this.imageSessionContext);
+                getImageManager(), getImageSessionContext());
         return ctx;
     }
-    
+
 }
