@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Fop.java 557337 2007-07-18 17:37:14Z adelmelle $ */
+/* $Id: Fop.java 828999 2009-10-23 11:22:53Z acumiskey $ */
 
 package org.apache.fop.tools.anttasks;
 
@@ -34,6 +34,7 @@ import java.io.IOException;
 import java.io.OutputStream;
 import java.net.MalformedURLException;
 import java.util.List;
+import java.util.Vector;
 
 // FOP
 import org.apache.fop.apps.FOPException;
@@ -54,7 +55,7 @@ import org.xml.sax.SAXException;
  * <li>format -> MIME type of the format to generate ex. "application/pdf"</li>
  * <li>outfile -> output filename</li>
  * <li>baseDir -> directory to work from</li>
- * <li>relativebase -> (true | false) control whether to use each FO's 
+ * <li>relativebase -> (true | false) control whether to use each FO's
  *      directory as base directory. false uses the baseDir parameter.</li>
  * <li>userconfig -> file with user configuration (same as the "-c" command
  *      line option)</li>
@@ -67,7 +68,10 @@ import org.xml.sax.SAXException;
 public class Fop extends Task {
 
     private File foFile;
-    private List filesets = new java.util.ArrayList();
+    private File xmlFile;
+    private File xsltFile;
+    private String xsltParams;
+    private List/*<FileSet>*/ filesets = new java.util.ArrayList/*<FileSet>*/();
     private File outFile;
     private File outDir;
     private String format; //MIME type
@@ -77,6 +81,7 @@ public class Fop extends Task {
     private boolean logFiles = true;
     private boolean force = false;
     private boolean relativebase = false;
+    private boolean throwExceptions = true;
 
     /**
      * Sets the filename for the userconfig.xml.
@@ -110,6 +115,54 @@ public class Fop extends Task {
         return foFile;
     }
 
+    /**
+     * Gets the input XML file.
+     * @return the input XML file.
+     */
+    public File getXmlFile() {
+        return xmlFile;
+    }
+
+    /**
+     * Sets the input XML file.
+     * @param xmlFile the input XML file.
+     */
+    public void setXmlFile(File xmlFile) {
+        this.xmlFile = xmlFile;
+    }
+
+    /**
+     * Gets the input XSLT file.
+     * @return the input XSLT file.
+     */
+    public File getXsltFile() {
+        return xsltFile;
+    }
+
+    /**
+     * Sets the input XSLT file.
+     * @param xsltFile the input XSLT file.
+     */
+    public void setXsltFile(File xsltFile) {
+        this.xsltFile = xsltFile;
+    }
+
+    /**
+     * Gets the XSLT parameters
+     * @return the XSLT parameters
+     */
+    public String getXsltParams() {
+        return xsltParams;
+    }
+
+    /**
+     * Sets the XSLT parameters 
+     * @param xsltParams the XSLT parameters
+     */
+    public void setXsltParams(String xsltParams) {
+        this.xsltParams = xsltParams;
+    }
+
     /**
      * Adds a set of XSL-FO files (nested fileset attribute).
      * @param set a fileset
@@ -136,7 +189,7 @@ public class Fop extends Task {
     public void setRelativebase(boolean relbase) {
         this.relativebase = relbase;
     }
-    
+
     /**
      * Gets the relative base attribute
      * @return the relative base attribute
@@ -211,6 +264,24 @@ public class Fop extends Task {
         return this.format;
     }
 
+    /**
+     * Set whether exceptions are thrown.
+     * default is false.
+     *
+     * @param force true if always generate.
+     */
+    public void setThrowexceptions(boolean throwExceptions) {
+        this.throwExceptions = throwExceptions;
+    }
+
+    /**
+     * Gets the throw exceptions attribute
+     * @return the throw exceptions attribute
+     */
+    public boolean getThrowexceptions() {
+        return this.throwExceptions;
+    }
+
     /**
      * Sets the message level to be used while processing.
      * @param messageLevel (error | warn| info | verbose | debug)
@@ -275,7 +346,7 @@ public class Fop extends Task {
     public boolean getLogFiles() {
         return this.logFiles;
     }
-    
+
     /**
      * {@inheritDoc}
      */
@@ -311,7 +382,7 @@ class FOPTaskStarter {
 
     // configure fopFactory as desired
     private FopFactory fopFactory = FopFactory.newInstance();
-    
+
     private Fop task;
     private String baseURL = null;
 
@@ -392,13 +463,13 @@ class FOPTaskStarter {
         {MimeConstants.MIME_AFP_ALT,         ".afp"},
         {MimeConstants.MIME_XSL_FO,          ".fo"}
     };
-    
+
     private String determineExtension(String outputFormat) {
         for (int i = 0; i < EXTENSIONS.length; i++) {
             if (EXTENSIONS[i][0].equals(outputFormat)) {
                 String ext = EXTENSIONS[i][1];
                 if (ext == null) {
-                    throw new RuntimeException("Output format '" 
+                    throw new RuntimeException("Output format '"
                             + outputFormat + "' does not produce a file.");
                 } else {
                     return ext;
@@ -425,14 +496,14 @@ class FOPTaskStarter {
         //Set base directory
         if (task.getBasedir() != null) {
             try {
-                this.baseURL = task.getBasedir().toURL().toExternalForm();
+                this.baseURL = task.getBasedir().toURI().toURL().toExternalForm();
             } catch (MalformedURLException mfue) {
                 logger.error("Error creating base URL from base directory", mfue);
             }
         } else {
             try {
                 if (task.getFofile() != null) {
-                    this.baseURL =  task.getFofile().getParentFile().toURL().
+                    this.baseURL =  task.getFofile().getParentFile().toURI().toURL().
                                       toExternalForm();
                 }
             } catch (MalformedURLException mfue) {
@@ -448,7 +519,7 @@ class FOPTaskStarter {
         // actioncount = # of fofiles actually processed through FOP
         int actioncount = 0;
         // skippedcount = # of fofiles which haven't changed (force = "false")
-        int skippedcount = 0; 
+        int skippedcount = 0;
 
         // deal with single source file
         if (task.getFofile() != null) {
@@ -460,22 +531,51 @@ class FOPTaskStarter {
                 if (task.getOutdir() != null) {
                     outf = new File(task.getOutdir(), outf.getName());
                 }
-                // Render if "force" flag is set OR 
+                // Render if "force" flag is set OR
                 // OR output file doesn't exist OR
                 // output file is older than input file
-                if (task.getForce() || !outf.exists() 
+                if (task.getForce() || !outf.exists()
                     || (task.getFofile().lastModified() > outf.lastModified() )) {
                     render(task.getFofile(), outf, outputFormat);
                     actioncount++;
-                } else if (outf.exists() 
+                } else if (outf.exists()
                         && (task.getFofile().lastModified() <= outf.lastModified() )) {
                     skippedcount++;
                 }
             }
+        } else if (task.getXmlFile() != null && task.getXsltFile() != null) {
+            if (task.getXmlFile().exists() && task.getXsltFile().exists()) {
+                File outf = task.getOutfile();
+                if (outf == null) {
+                    throw new BuildException("outfile is required when fofile is used");
+                }
+                if (task.getOutdir() != null) {
+                    outf = new File(task.getOutdir(), outf.getName());
+                }
+                // Render if "force" flag is set OR
+                // OR output file doesn't exist OR
+                // output file is older than input file
+                if (task.getForce() || !outf.exists()
+                    || (task.getXmlFile().lastModified() > outf.lastModified() ||
+                            task.getXsltFile().lastModified() > outf.lastModified())) {
+                    render(task.getXmlFile(), task.getXsltFile(), outf, outputFormat);
+                    actioncount++;
+                } else if (outf.exists()
+                        && (task.getXmlFile().lastModified() <= outf.lastModified() ||
+                                task.getXsltFile().lastModified() <= outf.lastModified())) {
+                    skippedcount++;
+                }
+            }
         }
 
         GlobPatternMapper mapper = new GlobPatternMapper();
-        mapper.setFrom("*.fo");
+
+        String inputExtension = ".fo";
+        File xsltFile = task.getXsltFile();
+        if (xsltFile != null) {
+            inputExtension = ".xml";
+        }
+        mapper.setFrom("*" + inputExtension);
         mapper.setTo("*" + newExtension);
 
         // deal with the filesets
@@ -488,23 +588,26 @@ class FOPTaskStarter {
                 File f = new File(fs.getDir(task.getProject()), files[j]);
 
                 File outf = null;
-                if (task.getOutdir() != null && files[j].endsWith(".fo")) {
+                if (task.getOutdir() != null && files[j].endsWith(inputExtension)) {
                   String[] sa = mapper.mapFileName(files[j]);
                   outf = new File(task.getOutdir(), sa[0]);
                 } else {
-                  outf = replaceExtension(f, ".fo", newExtension);
+                  outf = replaceExtension(f, inputExtension, newExtension);
                   if (task.getOutdir() != null) {
                       outf = new File(task.getOutdir(), outf.getName());
                   }
                 }
-
+                File dir = outf.getParentFile();
+                if (!dir.exists()) {
+                    dir.mkdirs();
+                }
                 try {
                     if (task.getRelativebase()) {
-                        this.baseURL = f.getParentFile().toURL().
+                        this.baseURL = f.getParentFile().toURI().toURL().
                                          toExternalForm();
                     }
                     if (this.baseURL == null) {
-                        this.baseURL = fs.getDir(task.getProject()).toURL().
+                        this.baseURL = fs.getDir(task.getProject()).toURI().toURL().
                                           toExternalForm();
                     }
 
@@ -512,33 +615,34 @@ class FOPTaskStarter {
                     task.log("Error setting base URL", Project.MSG_DEBUG);
                 }
 
-                // Render if "force" flag is set OR 
+                // Render if "force" flag is set OR
                 // OR output file doesn't exist OR
                 // output file is older than input file
-                if (task.getForce() || !outf.exists() 
+                if (task.getForce() || !outf.exists()
                     || (f.lastModified() > outf.lastModified() )) {
-                    render(f, outf, outputFormat);
+                    if (xsltFile != null) {
+                        render(f, xsltFile, outf, outputFormat);
+                    } else {
+                        render(f, outf, outputFormat);
+                    }
                     actioncount++;
                 } else if (outf.exists() && (f.lastModified() <= outf.lastModified() )) {
                     skippedcount++;
                 }
             }
         }
-        
+
         if (actioncount + skippedcount == 0) {
             task.log("No files processed. No files were selected by the filesets "
                 + "and no fofile was set." , Project.MSG_WARN);
-        } else if (skippedcount > 0) {  
+        } else if (skippedcount > 0) {
             task.log(skippedcount + " xslfo file(s) skipped (no change found"
                 + " since last generation; set force=\"true\" to override)."
                 , Project.MSG_INFO);
         }
     }
 
-    private void render(File foFile, File outFile,
-                        String outputFormat) throws FOPException {
-        InputHandler inputHandler = new InputHandler(foFile);
-
+    private void renderInputHandler(InputHandler inputHandler, File outFile, String outputFormat) throws Exception {
         OutputStream out = null;
         try {
             out = new java.io.FileOutputStream(outFile);
@@ -546,11 +650,6 @@ class FOPTaskStarter {
         } catch (Exception ex) {
             throw new BuildException("Failed to open " + outFile, ex);
         }
-
-        if (task.getLogFiles()) {
-            task.log(foFile + " -> " + outFile, Project.MSG_INFO);
-        }
-
         boolean success = false;
         try {
             FOUserAgent userAgent = fopFactory.newFOUserAgent();
@@ -558,7 +657,10 @@ class FOPTaskStarter {
             inputHandler.renderTo(userAgent, outputFormat, out);
             success = true;
         } catch (Exception ex) {
-            throw new BuildException(ex);
+            if (task.getThrowexceptions()) {
+                throw new BuildException(ex);
+            }
+            throw ex;
         } finally {
             try {
                 out.close();
@@ -571,5 +673,31 @@ class FOPTaskStarter {
         }
     }
 
+    private void render(File foFile, File outFile,
+                        String outputFormat) throws FOPException {
+        InputHandler inputHandler = new InputHandler(foFile);
+        try {
+            renderInputHandler(inputHandler, outFile, outputFormat);
+        } catch (Exception ex) {
+            logger.error("Error rendering fo file: " + foFile, ex);
+        }
+        if (task.getLogFiles()) {
+            task.log(foFile + " -> " + outFile, Project.MSG_INFO);
+        }
+    }
+
+    private void render(File xmlFile, File xsltFile, File outFile, String outputFormat) {
+        //TODO: implement support for XSLT params
+        final Vector xsltParams = null;
+        InputHandler inputHandler = new InputHandler(xmlFile, xsltFile, xsltParams);
+        try {
+            renderInputHandler(inputHandler, outFile, outputFormat);
+        } catch (Exception ex) {
+            logger.error("Error rendering xml/xslt files: " + xmlFile + ", " + xsltFile, ex);
+        }
+        if (task.getLogFiles()) {
+            task.log("xml: " + xmlFile + ", xslt: " + xsltFile + " -> " + outFile, Project.MSG_INFO);
+        }
+    }
 }
 
