@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: RunTest.java 557337 2007-07-18 17:37:14Z adelmelle $ */
- 
+/* $Id: RunTest.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+
 package org.apache.fop.tools.anttasks;
 
 // Ant
@@ -42,7 +42,7 @@ import java.util.Map;
  * to run the tests and check the results.
  */
 public class RunTest extends Task {
-    
+
     private String basedir;
     private String testsuite = "";
     private String referenceJar = "";
@@ -218,17 +218,17 @@ public class RunTest extends Task {
     }
 
     /**
-     * Return a list of URL's with the specified URL first and followed 
+     * Return a list of URL's with the specified URL first and followed
      * by all the jar files from lib/.
      * @return a list of urls to the runtime jar files.
      */
     private URL[] createUrls(String mainJar) throws MalformedURLException {
         ArrayList urls = new ArrayList();
-        urls.add(new File(mainJar).toURL());
+        urls.add(new File(mainJar).toURI().toURL());
         File[] libFiles = new File("lib").listFiles();
         for (int i = 0; i < libFiles.length; i++) {
             if (libFiles[i].getPath().endsWith(".jar")) {
-                urls.add(libFiles[i].toURL());
+                urls.add(libFiles[i].toURI().toURL());
             }
         }
         return (URL[]) urls.toArray(new URL[urls.size()]);
