@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,32 +15,32 @@
  * limitations under the License.
  */
 
-/* $Id: BorderProps.java 557337 2007-07-18 17:37:14Z adelmelle $ */
- 
+/* $Id: BorderProps.java 746664 2009-02-22 12:40:44Z jeremias $ */
+
 package org.apache.fop.traits;
 
 import java.awt.Color;
 import java.io.Serializable;
-import java.util.StringTokenizer;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 import org.apache.fop.apps.FOUserAgent;
-import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.expr.PropertyException;
 import org.apache.fop.util.ColorUtil;
 
 /**
  * Border properties.
- * Class to store border trait propties for the area tree.
+ * Class to store border trait properties for the area tree.
  */
 public class BorderProps implements Serializable {
-    
+
     /** Separate border model */
     public static final int SEPARATE = 0;
     /** Collapsing border model, for borders inside a table */
     public static final int COLLAPSE_INNER = 1;
     /** Collapsing border model, for borders at the table's outer border */
     public static final int COLLAPSE_OUTER = 2;
-    
+
     /** Border style (one of EN_*) */
     public int style; // Enum for border style
     /** Border color */
@@ -86,49 +86,15 @@ public class BorderProps implements Serializable {
             return 0;
         }
     }
-    
+
     private String getStyleString() {
-        switch (style) {
-        case Constants.EN_NONE: return "none";
-        case Constants.EN_HIDDEN: return "hidden";
-        case Constants.EN_DOTTED: return "dotted";
-        case Constants.EN_DASHED: return "dashed";
-        case Constants.EN_SOLID: return "solid";
-        case Constants.EN_DOUBLE: return "double";
-        case Constants.EN_GROOVE: return "groove";
-        case Constants.EN_RIDGE: return "ridge";
-        case Constants.EN_INSET: return "inset";
-        case Constants.EN_OUTSET: return "outset";
-        default: throw new IllegalStateException("Illegal border style: " + style);
-        }
+        return BorderStyle.valueOf(style).getName();
     }
-    
+
     private static int getConstantForStyle(String style) {
-        if ("none".equalsIgnoreCase(style)) {
-            return Constants.EN_NONE;
-        } else if ("hidden".equalsIgnoreCase(style)) {
-            return Constants.EN_HIDDEN;
-        } else if ("dotted".equalsIgnoreCase(style)) {
-            return Constants.EN_DOTTED;
-        } else if ("dashed".equalsIgnoreCase(style)) {
-            return Constants.EN_DASHED;
-        } else if ("solid".equalsIgnoreCase(style)) {
-            return Constants.EN_SOLID;
-        } else if ("double".equalsIgnoreCase(style)) {
-            return Constants.EN_DOUBLE;
-        } else if ("groove".equalsIgnoreCase(style)) {
-            return Constants.EN_GROOVE;
-        } else if ("ridge".equalsIgnoreCase(style)) {
-            return Constants.EN_RIDGE;
-        } else if ("inset".equalsIgnoreCase(style)) {
-            return Constants.EN_INSET;
-        } else if ("outset".equalsIgnoreCase(style)) {
-            return Constants.EN_OUTSET;
-        } else {
-            throw new IllegalStateException("Illegal border style: " + style);
-        }
+        return BorderStyle.valueOf(style).getEnumValue();
     }
-    
+
     /** {@inheritDoc} */
     public int hashCode() {
         return toString().hashCode();
@@ -144,7 +110,7 @@ public class BorderProps implements Serializable {
             if (obj instanceof BorderProps) {
                 BorderProps other = (BorderProps)obj;
                 return (style == other.style)
-                        && color.equals(other.color) 
+                        && color.equals(other.color)
                         && width == other.width
                         && mode == other.mode;
             }
@@ -153,7 +119,7 @@ public class BorderProps implements Serializable {
     }
 
     /**
-     * Returns a BorderProps represtation of a string of the format as written by 
+     * Returns a BorderProps represtation of a string of the format as written by
      * BorderProps.toString().
      * @param foUserAgent FOP user agent caching ICC profiles
      * @param s the string
@@ -162,13 +128,19 @@ public class BorderProps implements Serializable {
     public static BorderProps valueOf(FOUserAgent foUserAgent, String s) {
         if (s.startsWith("(") && s.endsWith(")")) {
             s = s.substring(1, s.length() - 1);
-            StringTokenizer st = new StringTokenizer(s, ",");
-            String style = st.nextToken();
-            String color = st.nextToken();
-            int width = Integer.parseInt(st.nextToken());
+            Pattern pattern = Pattern.compile("([^,\\(]+(?:\\(.*\\))?)");
+            Matcher m = pattern.matcher(s);
+            boolean found;
+            found = m.find();
+            String style = m.group();
+            found = m.find();
+            String color = m.group();
+            found = m.find();
+            int width = Integer.parseInt(m.group());
             int mode = SEPARATE;
-            if (st.hasMoreTokens()) {
-                String ms = st.nextToken();
+            found = m.find();
+            if (found) {
+                String ms = m.group();
                 if ("collapse-inner".equalsIgnoreCase(ms)) {
                     mode = COLLAPSE_INNER;
                 } else if ("collapse-outer".equalsIgnoreCase(ms)) {
@@ -180,8 +152,8 @@ public class BorderProps implements Serializable {
                 c = ColorUtil.parseColorString(foUserAgent, color);
             } catch (PropertyException e) {
                 throw new IllegalArgumentException(e.getMessage());
-            } 
-            
+            }
+
             return new BorderProps(style, width, c, mode);
         } else {
             throw new IllegalArgumentException("BorderProps must be surrounded by parentheses");
