@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: CharUtilities.java 627679 2008-02-14 08:12:34Z jeremias $ */
+/* $Id: CharUtilities.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.util;
 
@@ -54,10 +54,20 @@ public class CharUtilities {
     public static final int XMLWHITESPACE = 4;
 
 
+    /** null char */
+    public static final char NULL_CHAR = '\u0000';
+    /** linefeed character */
+    public static final char LINEFEED_CHAR = '\n';
+    /** carriage return */
+    public static final char CARRIAGE_RETURN = '\r';
+    /** normal tab */
+    public static final char TAB = '\t';
     /** normal space */
     public static final char SPACE = '\u0020';
     /** non-breaking space */
     public static final char NBSPACE = '\u00A0';
+    /** next line control character */
+    public static final char NEXT_LINE = '\u0085';
     /** zero-width space */
     public static final char ZERO_WIDTH_SPACE = '\u200B';
     /** word joiner */
@@ -68,11 +78,18 @@ public class CharUtilities {
     public static final char ZERO_WIDTH_NOBREAK_SPACE = '\uFEFF';
     /** soft hyphen */
     public static final char SOFT_HYPHEN = '\u00AD';
+    /** line-separator */
+    public static final char LINE_SEPARATOR = '\u2028';
+    /** paragraph-separator */
+    public static final char PARAGRAPH_SEPARATOR = '\u2029';
     /** missing ideograph */
     public static final char MISSING_IDEOGRAPH = '\u25A1';
+    /** Ideogreaphic space */
+    public static final char IDEOGRAPHIC_SPACE = '\u3000';
     /** Unicode value indicating the the character is "not a character". */
     public static final char NOT_A_CHARACTER = '\uFFFF';
 
+
     /**
      * Utility class: Constructor prevents instantiating when subclassed.
      */
@@ -87,11 +104,18 @@ public class CharUtilities {
      * @return the determined character class
      */
     public static int classOf(char c) {
-        if (c == CODE_EOT) { return EOT; }
-        if (c == '\n') { return LINEFEED; }
-        if (c == ' ' || c == '\r' || c == '\t') { return XMLWHITESPACE; }
-        if (isAnySpace(c)) { return UCWHITESPACE; }
-        return NONWHITESPACE;
+        switch (c) {
+            case CODE_EOT:
+                return EOT;
+            case LINEFEED_CHAR:
+                return LINEFEED;
+            case SPACE:
+            case CARRIAGE_RETURN:
+            case TAB:
+                return XMLWHITESPACE;
+            default:
+                return isAnySpace(c) ? UCWHITESPACE : NONWHITESPACE;
+        }
     }
 
 
@@ -123,7 +147,7 @@ public class CharUtilities {
      * @return true if the character has a fixed-width
      */
     public static boolean isFixedWidthSpace(char c) {
-        return (c >= '\u2000' && c <= '\u200B') 
+        return (c >= '\u2000' && c <= '\u200B')
                 || c == '\u3000';
 //      c == '\u2000'                   // en quad
 //      c == '\u2001'                   // em quad
@@ -167,17 +191,16 @@ public class CharUtilities {
             (c == '\u0020'    // normal space
             || c == NBSPACE); // no-break space
     }
-    
+
     /**
      * Determines if the character represents any kind of space.
      * @param c character to check
      * @return True if the character represents any kind of space
      */
     public static boolean isAnySpace(char c) {
-        boolean ret = (isBreakableSpace(c) || isNonBreakableSpace(c));
-        return ret;
+        return (isBreakableSpace(c) || isNonBreakableSpace(c));
     }
-    
+
     /**
      * Indicates whether a character is classified as "Alphabetic" by the Unicode standard.
      * @param ch the character
@@ -188,19 +211,31 @@ public class CharUtilities {
         //Generated from: Other_Alphabetic + Lu + Ll + Lt + Lm + Lo + Nl
         int generalCategory = Character.getType(ch);
         switch (generalCategory) {
-        case Character.UPPERCASE_LETTER: //Lu
-        case Character.LOWERCASE_LETTER: //Ll
-        case Character.TITLECASE_LETTER: //Lt
-        case Character.MODIFIER_LETTER: //Lm
-        case Character.OTHER_LETTER: //Lo
-        case Character.LETTER_NUMBER: //Nl
-            return true;
-        default:
-            //TODO if (ch in Other_Alphabetic) return true; (Probably need ICU4J for that)
-            //Other_Alphabetic contains mostly more exotic characters
-            return false;
+            case Character.UPPERCASE_LETTER: //Lu
+            case Character.LOWERCASE_LETTER: //Ll
+            case Character.TITLECASE_LETTER: //Lt
+            case Character.MODIFIER_LETTER: //Lm
+            case Character.OTHER_LETTER: //Lo
+            case Character.LETTER_NUMBER: //Nl
+                return true;
+            default:
+                //TODO if (ch in Other_Alphabetic) return true; (Probably need ICU4J for that)
+                //Other_Alphabetic contains mostly more exotic characters
+                return false;
         }
     }
-    
+
+    /**
+     * Indicates whether the given character is an explicit break-character
+     * @param ch    the character to check
+     * @return  true if the character represents an explicit break
+     */
+    public static boolean isExplicitBreak(char ch) {
+        return (ch == LINEFEED_CHAR
+            || ch == CARRIAGE_RETURN
+            || ch == NEXT_LINE
+            || ch == LINE_SEPARATOR
+            || ch == PARAGRAPH_SEPARATOR);
+    }
 }
 
