@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ColorUtil.java 628775 2008-02-18 15:02:39Z jeremias $ */
+/* $Id: ColorUtil.java 959945 2010-07-02 10:44:18Z jeremias $ */
 
 package org.apache.fop.util;
 
@@ -27,6 +27,8 @@ import java.util.Map;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.xmlgraphics.java2d.color.DeviceCMYKColorSpace;
+
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.fo.expr.PropertyException;
 
@@ -38,22 +40,25 @@ import org.apache.fop.fo.expr.PropertyException;
  */
 public final class ColorUtil {
 
+    /** The name for the uncalibrated CMYK pseudo-profile */
+    public static final String CMYK_PSEUDO_PROFILE = "#CMYK";
+
     /**
-     * 
+     *
      * keeps all the predefined and parsed colors.
      * <p>
      * This map is used to predefine given colors, as well as speeding up
      * parsing of already parsed colors.
      */
     private static Map colorMap = null;
-    
+
     /** Logger instance */
     protected static Log log = LogFactory.getLog(ColorUtil.class);
-    
+
     static {
         initializeColorMap();
     }
-    
+
     /**
      * Private constructor since this is an utility class.
      */
@@ -77,8 +82,8 @@ public final class ColorUtil {
      * <li>fop-rgb-icc(r,g,b,cs,cs-src,[num]+) (r/g/b: 0..1, num: 0..1)</li>
      * <li>cmyk(c,m,y,k) (0..1)</li>
      * </ul>
-     * 
-     * @param foUserAgent FOUserAgent object  
+     *
+     * @param foUserAgent FOUserAgent object
      * @param value
      *            the string to parse.
      * @return a Color representing the string if possible
@@ -86,7 +91,7 @@ public final class ColorUtil {
      *             if the string is not parsable or does not follow any of the
      *             given formats.
      */
-    public static Color parseColorString(FOUserAgent foUserAgent, String value) 
+    public static Color parseColorString(FOUserAgent foUserAgent, String value)
             throws PropertyException {
         if (value == null) {
             return null;
@@ -111,11 +116,11 @@ public final class ColorUtil {
             } else if (value.startsWith("cmyk")) {
                 parsedColor = parseAsCMYK(value);
             }
-            
+
             if (parsedColor == null) {
                 throw new PropertyException("Unknown Color: " + value);
             }
-            
+
             colorMap.put(value, parsedColor);
         }
 
@@ -127,7 +132,7 @@ public final class ColorUtil {
 
     /**
      * Tries to parse a color given with the system-color() function.
-     * 
+     *
      * @param value
      *            the complete line
      * @return a color if possible
@@ -149,7 +154,7 @@ public final class ColorUtil {
 
     /**
      * Tries to parse the standard java.awt.Color toString output.
-     * 
+     *
      * @param value
      *            the complete line
      * @return a color if possible
@@ -170,11 +175,11 @@ public final class ColorUtil {
                     throw new PropertyException(
                             "Invalid number of arguments for a java.awt.Color: " + value);
                 }
-                
+
                 red = Float.parseFloat(args[0].trim().substring(2)) / 255f;
                 green = Float.parseFloat(args[1].trim().substring(2)) / 255f;
                 blue = Float.parseFloat(args[2].trim().substring(2)) / 255f;
-                if ((red < 0.0 || red > 1.0) 
+                if ((red < 0.0 || red > 1.0)
                         || (green < 0.0 || green > 1.0)
                         || (blue < 0.0 || blue > 1.0)) {
                     throw new PropertyException("Color values out of range");
@@ -193,7 +198,7 @@ public final class ColorUtil {
 
     /**
      * Parse a color given with the rgb() function.
-     * 
+     *
      * @param value
      *            the complete line
      * @return a color if possible
@@ -234,7 +239,7 @@ public final class ColorUtil {
                 } else {
                     blue = Float.parseFloat(str) / 255f;
                 }
-                if ((red < 0.0 || red > 1.0) 
+                if ((red < 0.0 || red > 1.0)
                         || (green < 0.0 || green > 1.0)
                         || (blue < 0.0 || blue > 1.0)) {
                     throw new PropertyException("Color values out of range");
@@ -256,7 +261,7 @@ public final class ColorUtil {
 
     /**
      * parse a color given in the #.... format.
-     * 
+     *
      * @param value
      *            the complete line
      * @return a color if possible
@@ -297,19 +302,19 @@ public final class ColorUtil {
 
     /**
      * Parse a color specified using the fop-rgb-icc() function.
-     * 
+     *
      * @param value the function call
      * @return a color if possible
      * @throws PropertyException if the format is wrong.
      */
-    private static Color parseAsFopRgbIcc(FOUserAgent foUserAgent, String value) 
+    private static Color parseAsFopRgbIcc(FOUserAgent foUserAgent, String value)
             throws PropertyException {
         Color parsedColor;
         int poss = value.indexOf("(");
         int pose = value.indexOf(")");
         if (poss != -1 && pose != -1) {
             String[] args = value.substring(poss + 1, pose).split(",");
-            
+
             try {
                 if (args.length < 5) {
                     throw new PropertyException("Too few arguments for rgb-icc() function");
@@ -319,47 +324,58 @@ public final class ColorUtil {
                 if (iccProfileName == null || "".equals(iccProfileName)) {
                     throw new PropertyException("ICC profile name missing");
                 }
-                /* Get and verify ICC profile source */
-                String iccProfileSrc = args[4].trim();
-                if (iccProfileSrc == null || "".equals(iccProfileSrc)) {
-                    throw new PropertyException("ICC profile source missing");
-                }
-                if (iccProfileSrc.startsWith("\"") || iccProfileSrc.startsWith("'")) {
-                    iccProfileSrc = iccProfileSrc.substring(1);
-                }
-                if (iccProfileSrc.endsWith("\"") || iccProfileSrc.endsWith("'")) {
-                    iccProfileSrc = iccProfileSrc.substring(0, iccProfileSrc.length() - 1);
+                ColorSpace colorSpace = null;
+                String iccProfileSrc = null;
+                if (isPseudoProfile(iccProfileName)) {
+                    if (CMYK_PSEUDO_PROFILE.equalsIgnoreCase(iccProfileName)) {
+                        colorSpace = DeviceCMYKColorSpace.getInstance();
+                    } else {
+                        assert false : "Incomplete implementation";
+                    }
+                } else {
+                    /* Get and verify ICC profile source */
+                    iccProfileSrc = args[4].trim();
+                    if (iccProfileSrc == null || "".equals(iccProfileSrc)) {
+                        throw new PropertyException("ICC profile source missing");
+                    }
+                    if (iccProfileSrc.startsWith("\"") || iccProfileSrc.startsWith("'")) {
+                        iccProfileSrc = iccProfileSrc.substring(1);
+                    }
+                    if (iccProfileSrc.endsWith("\"") || iccProfileSrc.endsWith("'")) {
+                        iccProfileSrc = iccProfileSrc.substring(0, iccProfileSrc.length() - 1);
+                    }
                 }
                 /* ICC profile arguments */
                 float[] iccComponents = new float[args.length - 5];
                 for (int ix = 4; ++ix < args.length;) {
                     iccComponents[ix - 5] = Float.parseFloat(args[ix].trim());
                 }
-                /* Ask FOP factory to get ColorSpace for the specified ICC profile source */
-                ColorSpace colorSpace = (foUserAgent != null
-                        ? foUserAgent.getFactory().getColorSpace(
-                                foUserAgent.getBaseURL(), iccProfileSrc) : null);
-                
+
                 float red = 0, green = 0, blue = 0;
                 red = Float.parseFloat(args[0].trim());
                 green = Float.parseFloat(args[1].trim());
                 blue = Float.parseFloat(args[2].trim());
                 /* Verify rgb replacement arguments */
-                if ((red < 0 || red > 1) 
-                        || (green < 0 || green > 1) 
+                if ((red < 0 || red > 1)
+                        || (green < 0 || green > 1)
                         || (blue < 0 || blue > 1)) {
                     throw new PropertyException("Color values out of range. "
                             + "Fallback RGB arguments to fop-rgb-icc() must be [0..1]");
                 }
 
+                /* Ask FOP factory to get ColorSpace for the specified ICC profile source */
+                if (foUserAgent != null && iccProfileSrc != null) {
+                    colorSpace = foUserAgent.getFactory().getColorSpace(
+                            foUserAgent.getBaseURL(), iccProfileSrc);
+                }
                 if (colorSpace != null) {
-                    // ColorSpace available - create ColorExt (keeps track of replacement rgb 
+                    // ColorSpace available - create ColorExt (keeps track of replacement rgb
                     // values for possible later colorTOsRGBString call
-                    parsedColor = ColorExt.createFromFoRgbIcc(red, green, blue, 
+                    parsedColor = ColorExt.createFromFoRgbIcc(red, green, blue,
                             iccProfileName, iccProfileSrc, colorSpace, iccComponents);
                 } else {
                     // ICC profile could not be loaded - use rgb replacement values */
-                    log.warn("Color profile '" + iccProfileSrc 
+                    log.warn("Color profile '" + iccProfileSrc
                             + "' not found. Using rgb replacement values.");
                     parsedColor = new Color(Math.round(red * 255),
                             Math.round(green * 255), Math.round(blue * 255));
@@ -380,7 +396,7 @@ public final class ColorUtil {
 
     /**
      * Parse a color given with the cmyk() function.
-     * 
+     *
      * @param value
      *            the complete line
      * @return a color if possible
@@ -428,8 +444,8 @@ public final class ColorUtil {
                 } else {
                   black = Float.parseFloat(str);
                 }
-                
-                if ((cyan < 0.0 || cyan > 1.0) 
+
+                if ((cyan < 0.0 || cyan > 1.0)
                         || (magenta < 0.0 || magenta > 1.0)
                         || (yellow < 0.0 || yellow > 1.0)
                         || (black < 0.0 || black > 1.0)) {
@@ -437,10 +453,10 @@ public final class ColorUtil {
                             + "Arguments to cmyk() must be in the range [0%-100%] or [0.0-1.0]");
                 }
                 float[] cmyk = new float[] {cyan, magenta, yellow, black};
-                CMYKColorSpace cmykCs = CMYKColorSpace.getInstance();
+                DeviceCMYKColorSpace cmykCs = DeviceCMYKColorSpace.getInstance();
                 float[] rgb = cmykCs.toRGB(cmyk);
-                parsedColor = ColorExt.createFromFoRgbIcc(rgb[0], rgb[1], rgb[2], 
-                        null, "#CMYK", cmykCs, cmyk);
+                parsedColor = ColorExt.createFromFoRgbIcc(rgb[0], rgb[1], rgb[2],
+                        CMYK_PSEUDO_PROFILE, null, cmykCs, cmyk);
             } catch (PropertyException pe) {
                 throw pe;
             } catch (Exception e) {
@@ -452,26 +468,26 @@ public final class ColorUtil {
         }
         return parsedColor;
     }
-    
+
     /**
      * Creates a re-parsable string representation of the given color.
      * <p>
      * First, the color will be converted into the sRGB colorspace. It will then
      * be printed as #rrggbb, or as #rrrggbbaa if an alpha value is present.
-     * 
+     *
      * @param color
      *            the color to represent.
      * @return a re-parsable string representadion.
      */
     public static String colorToString(Color color) {
         ColorSpace cs = color.getColorSpace();
-        if (cs != null && cs.getType() == ColorSpace.TYPE_CMYK) {
+        if (color instanceof ColorExt) {
+            return ((ColorExt)color).toFunctionCall();
+        } else if (cs != null && cs.getType() == ColorSpace.TYPE_CMYK) {
             StringBuffer sbuf = new StringBuffer(24);
             float[] cmyk = color.getColorComponents(null);
             sbuf.append("cmyk(" + cmyk[0] + "," + cmyk[1] + "," + cmyk[2] + "," +  cmyk[3] + ")");
             return sbuf.toString();
-        } else if (color instanceof ColorExt) {
-            return ((ColorExt)color).toFunctionCall();
         } else {
             StringBuffer sbuf = new StringBuffer();
             sbuf.append('#');
@@ -658,4 +674,46 @@ public final class ColorUtil {
         colorMap.put("transparent", new Color(0, 0, 0, 0));
     }
 
+    /**
+     * Lightens up a color for groove, ridge, inset and outset border effects.
+     * @param col the color to lighten up
+     * @param factor factor by which to lighten up (negative values darken the color)
+     * @return the modified color
+     */
+    public static Color lightenColor(Color col, float factor) {
+        return org.apache.xmlgraphics.java2d.color.ColorUtil.lightenColor(col, factor);
+    }
+
+    /**
+     * Indicates whether the given color profile name is one of the pseudo-profiles supported
+     * by FOP (ex. #CMYK).
+     * @param colorProfileName the color profile name to check
+     * @return true if the color profile name is of a built-in pseudo-profile
+     */
+    public static boolean isPseudoProfile(String colorProfileName) {
+        return CMYK_PSEUDO_PROFILE.equalsIgnoreCase(colorProfileName);
+    }
+
+    /**
+     * Indicates whether the color is a gray value.
+     * @param col the color
+     * @return true if it is a gray value
+     */
+    public static boolean isGray(Color col) {
+        return org.apache.xmlgraphics.java2d.color.ColorUtil.isGray(col);
+    }
+
+    /**
+     * Creates an uncalibrated CMYK color with the given gray value.
+     * @param black the gray component (0 - 1)
+     * @return the CMYK color
+     */
+    public static Color toCMYKGrayColor(float black) {
+        float[] cmyk = new float[] {0f, 0f, 0f, 1.0f - black};
+        DeviceCMYKColorSpace cmykCs = DeviceCMYKColorSpace.getInstance();
+        float[] rgb = cmykCs.toRGB(cmyk);
+        return ColorExt.createFromFoRgbIcc(rgb[0], rgb[1], rgb[2],
+                CMYK_PSEUDO_PROFILE, null, cmykCs, cmyk);
+    }
+
 }
