@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,27 +15,20 @@
  * limitations under the License.
  */
 
-/* $Id: FontBaseBadTestCase.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: FontBaseBadTestCase.java 752153 2009-03-10 16:16:31Z jeremias $ */
 
 package org.apache.fop.config;
 
 /*
- * this font base does not exist and a relative font path is used
- */ 
+ * This font base does not exist and a relative font path is used.
+ */
 public class FontBaseBadTestCase extends BaseDestructiveUserConfigTestCase {
 
     public FontBaseBadTestCase(String name) {
         super(name);
     }
 
-    public void testUserConfig() throws Exception {
-        // Override this method from the super-class and do nothing as this test doesn't pass ATM
-        // TODO re-enable later
-    }
-
-    /**
-     * @see org.apache.fop.config.BaseUserConfigTestCase#getUserConfigFilename()
-     */
+    /** {@inheritDoc} */
     public String getUserConfigFilename() {
         return "test_fontbase_bad.xconf";
     }
