@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: UserConfigTestSuite.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: UserConfigTestSuite.java 772672 2009-05-07 15:08:30Z cbowditch $ */
 
 package org.apache.fop.config;
 
@@ -44,6 +44,8 @@ public class UserConfigTestSuite {
         suite.addTest(new TestSuite(FontEmbedUrlMalformedTestCase.class));
         suite.addTest(new TestSuite(FontsDirectoryRecursiveTestCase.class));
         suite.addTest(new TestSuite(FontsAutoDetectTestCase.class));
+        suite.addTest(new TestSuite(FontsSubstitutionTestCase.class));
+        suite.addTest(new TestSuite(FOURIResolverTestCase.class));
         //$JUnit-END$
         return suite;
     }
