@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FOTreeTestSuite.java 614566 2008-01-23 15:22:05Z vhennebert $ */
+/* $Id: FOTreeTestSuite.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fotreetest;
 
@@ -35,6 +35,7 @@ import org.apache.commons.io.filefilter.NameFileFilter;
 import org.apache.commons.io.filefilter.PrefixFileFilter;
 import org.apache.commons.io.filefilter.SuffixFileFilter;
 import org.apache.commons.io.filefilter.TrueFileFilter;
+
 import org.apache.fop.DebugHelper;
 import org.apache.fop.fo.flow.table.CollapsedConditionalBorderTestCase;
 import org.apache.fop.fo.flow.table.IllegalRowSpanTestCase;
@@ -51,11 +52,11 @@ public final class FOTreeTestSuite {
     static {
         DebugHelper.registerStandardElementListObservers();
     }
-    
+
     private FOTreeTestSuite() {
         //don't instantiate!
     }
-    
+
     /**
      * @return the test suite with all the tests (one for each XML file)
      * @throws IOException in case of an I/O problem
@@ -71,7 +72,7 @@ public final class FOTreeTestSuite {
         File mainDir = new File("test/fotree");
 
         final FOTreeTester tester = new FOTreeTester();
-        
+
         IOFileFilter filter;
         String single = System.getProperty("fop.fotree.single");
         String startsWith = System.getProperty("fop.fotree.starts-with");
@@ -84,12 +85,12 @@ public final class FOTreeTestSuite {
             filter = new SuffixFileFilter(".fo");
             filter = LayoutEngineTestSuite.decorateWithDisabledList(filter);
         }
-        Collection files = FileUtils.listFiles(new File(mainDir, "testcases"), 
+        Collection files = FileUtils.listFiles(new File(mainDir, "testcases"),
                 filter, TrueFileFilter.INSTANCE);
         String privateTests = System.getProperty("fop.fotree.private");
         if ("true".equalsIgnoreCase(privateTests)) {
             Collection privateFiles = FileUtils.listFiles(
-                    new File(mainDir, "private-testcases"), 
+                    new File(mainDir, "private-testcases"),
                     filter, TrueFileFilter.INSTANCE);
             files.addAll(privateFiles);
         }
@@ -99,14 +100,19 @@ public final class FOTreeTestSuite {
             addTestCase(suite, tester, f);
         }
     }
-    
-    private static void addTestCase(TestSuite suite, 
+
+    private static void addTestCase(TestSuite suite,
                 final FOTreeTester tester, final File f) {
         suite.addTest(new FOTreeTestCase(f.getName()) {
             public void runTest() throws Exception {
-                org.apache.commons.logging.LogFactory.getLog(this.getClass()).info("Starting " + f.getName());
-                prepare(tester, f);
-                testMain();
+                try {
+                    prepare(tester, f);
+                    testMain();
+                } catch (Exception e) {
+                    org.apache.commons.logging.LogFactory.getLog(this.getClass()).info(
+                            "Error on " + f.getName());
+                    throw e;
+                }
             }
         });
     }
@@ -120,22 +126,22 @@ public final class FOTreeTestSuite {
     }
 
     private static class FOTreeTestCase extends TestCase {
-        
+
         private FOTreeTester tester;
         private File testFile;
-        
+
         public FOTreeTestCase(String name) {
             super(name);
         }
-        
+
         public void prepare(FOTreeTester tester, File testFile) {
             //super(testFile.getName());
             this.tester = tester;
             this.testFile = testFile;
         }
-        
+
         public void testMain() throws Exception {
             tester.runTest(testFile);
         }
-    }    
+    }
 }
