@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,8 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: AssertElement.java 619674 2008-02-07 22:41:26Z adelmelle $ */
- 
+/* $Id: AssertElement.java 766492 2009-04-19 17:54:06Z adelmelle $ */
+
 package org.apache.fop.fotreetest.ext;
 
 
@@ -41,7 +41,10 @@ import org.xml.sax.Locator;
 public class AssertElement extends TestObj {
 
     /**
-     * @see org.apache.fop.fo.FONode#FONode(FONode)
+     * Creates a new AssertElement instance that is a child
+     * of the given {@link FONode}
+     *
+     * @param   parent  the parent {@link FONode}
      */
     public AssertElement(FONode parent) {
         super(parent);
@@ -50,14 +53,15 @@ public class AssertElement extends TestObj {
     /**
      * @see org.apache.fop.fo.FONode#processNode
      */
-    public void processNode(String elementName, 
-                            Locator locator, 
-                            Attributes attlist, 
+    public void processNode(String elementName,
+                            Locator locator,
+                            Attributes attlist,
                             PropertyList propertyList) throws FOPException {
         //super.processNode(elementName, locator, attlist, propertyList);
 
         ResultCollector collector = ResultCollector.getInstance();
         String propName = attlist.getValue("property");
+        String expected = attlist.getValue("expected");        
         String component = null;
         int dotIndex = propName.indexOf('.');
         if (dotIndex >= 0) {
@@ -66,8 +70,7 @@ public class AssertElement extends TestObj {
         }
         int propID = FOPropertyMapping.getPropertyId(propName);
         if (propID < 0) {
-            collector.notifyException(new IllegalArgumentException(
-                    "Property not found: " + propName));
+            collector.notifyAssertionFailure("Property not found: " + propName);
         } else {
             Property prop;
             prop = propertyList.getParentPropertyList().get(propID);
@@ -94,30 +97,28 @@ public class AssertElement extends TestObj {
             }
             String s;
             if (prop instanceof PercentLength) {
-                s = ((PercentLength)prop).getString();
+                s = prop.getString();
             } else {
                 s = String.valueOf(prop);
             }
-            String expected = attlist.getValue("expected");
             if (!expected.equals(s)) {
-                collector.notifyException(
-                    new IllegalStateException(locator.getSystemId()
-                        + "\nProperty '" + propName 
-                        + "' expected to evaluate to '" + expected 
+                collector.notifyAssertionFailure(
+                    locator.getSystemId()
+                        + "\nProperty '" + propName
+                        + "' expected to evaluate to '" + expected
                         + "' but got '" + s
-                        + "'\n(test:assert in " 
+                        + "'\n(test:assert in "
                         + propertyList.getParentFObj().getName()
-                        + " at line #" + locator.getLineNumber() 
-                        + ", column #" + locator.getColumnNumber() + ")\n"));
+                        + " at line #" + locator.getLineNumber()
+                        + ", column #" + locator.getColumnNumber() + ")\n");
             }
         }
-        
+
     }
 
     /** @see org.apache.fop.fo.FONode#getLocalName() */
     public String getLocalName() {
         return "assert";
     }
-    
-}
 
+}
