@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: IntermediateFormatTestSuite.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: IntermediateFormatTestSuite.java 746664 2009-02-22 12:40:44Z jeremias $ */
 
 package org.apache.fop.intermediate;
 
@@ -46,22 +46,26 @@ public class IntermediateFormatTestSuite {
         Iterator i = files.iterator();
         while (i.hasNext()) {
             File f = (File)i.next();
-            addTestCase(suite, f);
+            addIFTestCase(suite, f);
         }
-        
+
         return suite;
     }
-    
-    private static void addTestCase(TestSuite suite, 
-            final File f) {
-        suite.addTest(new AreaTreeParserTestCase(f) {
+
+    private static void addIFTestCase(TestSuite suite,
+            final File f) throws IOException {
+        suite.addTest(new IFParserTestCase(f) {
             public void runTest() throws Exception {
-                org.apache.commons.logging.LogFactory.getLog(
-                        this.getClass()).info("Starting " + f.getName());
-                testParserToAT();
-                testParserToPDF();
+                try {
+                    testParserToIntermediateFormat();
+                    testParserToPDF();
+                } catch (Exception e) {
+                    org.apache.commons.logging.LogFactory.getLog(
+                            this.getClass()).error("Error on " + f.getName());
+                    throw e;
+                }
             }
         });
     }
-    
+
 }
