@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ElementListCheck.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: ElementListCheck.java 825646 2009-10-15 20:43:13Z acumiskey $ */
 
 package org.apache.fop.layoutengine;
 
@@ -41,7 +41,7 @@ public class ElementListCheck implements LayoutEngineCheck {
     private String id;
     private int index = -1;
     private Element checkElement;
-    
+
     /**
      * Creates a new instance from a DOM node.
      * @param node DOM node that defines this check
@@ -68,7 +68,7 @@ public class ElementListCheck implements LayoutEngineCheck {
         NodeList children = checkElement.getChildNodes();
         int pos = -1;
         for (int i = 0; i < children.getLength(); i++) {
-            Node node = children.item(i); 
+            Node node = children.item(i);
             if (node instanceof Element) {
                 pos++;
                 Element domEl = (Element)node;
@@ -78,15 +78,15 @@ public class ElementListCheck implements LayoutEngineCheck {
                 } else if ("box".equals(domEl.getLocalName())) {
                     if (!(knuthEl instanceof KnuthBox)) {
                         fail("Expected KnuthBox"
-                                + " at position " + pos 
+                                + " at position " + pos
                                 + " but got: " + knuthEl.getClass().getName());
                     }
                     if (domEl.getAttribute("w").length() > 0) {
                         int w = Integer.parseInt(domEl.getAttribute("w"));
-                        if (w != knuthEl.getW()) {
-                            fail("Expected w=" + w 
-                                    + " at position " + pos 
-                                    + " but got: " + knuthEl.getW());
+                        if (w != knuthEl.getWidth()) {
+                            fail("Expected w=" + w
+                                    + " at position " + pos
+                                    + " but got: " + knuthEl.getWidth());
                         }
                     }
                     if ("true".equals(domEl.getAttribute("aux"))) {
@@ -104,30 +104,30 @@ public class ElementListCheck implements LayoutEngineCheck {
                 } else if ("penalty".equals(domEl.getLocalName())) {
                     if (!(knuthEl instanceof KnuthPenalty)) {
                         fail("Expected KnuthPenalty "
-                                + " at position " + pos 
+                                + " at position " + pos
                                 + " but got: " + knuthEl.getClass().getName());
                     }
                     KnuthPenalty pen = (KnuthPenalty)knuthEl;
                     if (domEl.getAttribute("w").length() > 0) {
                         int w = Integer.parseInt(domEl.getAttribute("w"));
-                        if (w != knuthEl.getW()) {
-                            fail("Expected w=" + w 
-                                    + " at position " + pos 
-                                    + " but got: " + knuthEl.getW());
+                        if (w != knuthEl.getWidth()) {
+                            fail("Expected w=" + w
+                                    + " at position " + pos
+                                    + " but got: " + knuthEl.getWidth());
                         }
                     }
                     if (domEl.getAttribute("p").length() > 0) {
                         if ("<0".equals(domEl.getAttribute("p"))) {
-                            if (knuthEl.getP() >= 0) {
-                                fail("Expected p<0" 
-                                        + " at position " + pos 
-                                        + " but got: " + knuthEl.getP());
+                            if (knuthEl.getPenalty() >= 0) {
+                                fail("Expected p<0"
+                                        + " at position " + pos
+                                        + " but got: " + knuthEl.getPenalty());
                             }
                         } else if (">0".equals(domEl.getAttribute("p"))) {
-                            if (knuthEl.getP() <= 0) {
-                                fail("Expected p>0" 
-                                        + " at position " + pos 
-                                        + " but got: " + knuthEl.getP());
+                            if (knuthEl.getPenalty() <= 0) {
+                                fail("Expected p>0"
+                                        + " at position " + pos
+                                        + " but got: " + knuthEl.getPenalty());
                             }
                         } else {
                             int p;
@@ -142,20 +142,20 @@ public class ElementListCheck implements LayoutEngineCheck {
                             } else {
                                 p = Integer.parseInt(domEl.getAttribute("p"));
                             }
-                            if (p != knuthEl.getP()) {
-                                fail("Expected p=" + p 
-                                        + " at position " + pos 
-                                        + " but got: " + knuthEl.getP());
+                            if (p != knuthEl.getPenalty()) {
+                                fail("Expected p=" + p
+                                        + " at position " + pos
+                                        + " but got: " + knuthEl.getPenalty());
                             }
                         }
                     }
                     if ("true".equals(domEl.getAttribute("flagged"))) {
-                        if (!pen.isFlagged()) {
+                        if (!pen.isPenaltyFlagged()) {
                             fail("Expected flagged penalty"
-                                    + " at position " + pos); 
+                                    + " at position " + pos);
                         }
                     } else if ("false".equals(domEl.getAttribute("flagged"))) {
-                        if (pen.isFlagged()) {
+                        if (pen.isPenaltyFlagged()) {
                             fail("Expected non-flagged penalty"
                                     + " at position " + pos);
                         }
@@ -163,57 +163,57 @@ public class ElementListCheck implements LayoutEngineCheck {
                     if ("true".equals(domEl.getAttribute("aux"))) {
                         if (!pen.isAuxiliary()) {
                             fail("Expected auxiliary penalty"
-                                    + " at position " + pos); 
+                                    + " at position " + pos);
                         }
                     } else if ("false".equals(domEl.getAttribute("aux"))) {
                         if (pen.isAuxiliary()) {
                             fail("Expected non-auxiliary penalty"
-                                    + " at position " + pos); 
+                                    + " at position " + pos);
                         }
                     }
                 } else if ("glue".equals(domEl.getLocalName())) {
                     if (!(knuthEl instanceof KnuthGlue)) {
-                        fail("Expected KnuthGlue" 
-                                + " at position " + pos 
+                        fail("Expected KnuthGlue"
+                                + " at position " + pos
                                 + " but got: " + knuthEl.getClass().getName());
                     }
                     KnuthGlue glue = (KnuthGlue)knuthEl;
                     if (domEl.getAttribute("w").length() > 0) {
                         int w = Integer.parseInt(domEl.getAttribute("w"));
-                        if (w != knuthEl.getW()) {
-                            fail("Expected w=" + w 
-                                    + " at position " + pos 
-                                    + " but got: " + knuthEl.getW());
+                        if (w != knuthEl.getWidth()) {
+                            fail("Expected w=" + w
+                                    + " at position " + pos
+                                    + " but got: " + knuthEl.getWidth());
                         }
                     }
                     if (domEl.getAttribute("y").length() > 0) {
                         int stretch = Integer.parseInt(domEl.getAttribute("y"));
-                        if (stretch != knuthEl.getY()) {
-                            fail("Expected y=" + stretch 
-                                    + " (stretch) at position " + pos 
-                                    + " but got: " + knuthEl.getY());
+                        if (stretch != knuthEl.getStretch()) {
+                            fail("Expected y=" + stretch
+                                    + " (stretch) at position " + pos
+                                    + " but got: " + knuthEl.getStretch());
                         }
                     }
                     if (domEl.getAttribute("z").length() > 0) {
                         int shrink = Integer.parseInt(domEl.getAttribute("z"));
-                        if (shrink != knuthEl.getZ()) {
-                            fail("Expected z=" + shrink 
-                                    + " (shrink) at position " + pos 
-                                    + " but got: " + knuthEl.getZ());
+                        if (shrink != knuthEl.getShrink()) {
+                            fail("Expected z=" + shrink
+                                    + " (shrink) at position " + pos
+                                    + " but got: " + knuthEl.getShrink());
                         }
                     }
                 } else {
-                    throw new IllegalArgumentException("Invalid child node for 'element-list': " 
+                    throw new IllegalArgumentException("Invalid child node for 'element-list': "
                             + domEl.getLocalName()
                             + " at position " + pos + " (" + this + ")");
                 }
-                
+
             }
         }
         pos++;
         if (elementList.getElementList().size() > pos) {
-            fail("There are " 
-                    + (elementList.getElementList().size() - pos) 
+            fail("There are "
+                    + (elementList.getElementList().size() - pos)
                     + " unchecked elements at the end of the list");
         }
     }
@@ -221,7 +221,7 @@ public class ElementListCheck implements LayoutEngineCheck {
     private void fail(String msg) {
         throw new RuntimeException(msg + " (" + this + ")");
     }
-    
+
     private boolean haveID() {
         return (this.id != null && this.id.length() > 0);
     }
@@ -262,7 +262,7 @@ public class ElementListCheck implements LayoutEngineCheck {
         }
         return sb.toString();
     }
-    
+
     /** @see java.lang.Object#toString() */
     public String toString() {
         StringBuffer sb = new StringBuffer("element-list");
