@@ -15,28 +15,32 @@
  * limitations under the License.
  */
 
-/* $Id: EvalCheck.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: EvalCheck.java 746664 2009-02-22 12:40:44Z jeremias $ */
 
 package org.apache.fop.layoutengine;
 
 import javax.xml.transform.TransformerException;
 
+import org.w3c.dom.Document;
+import org.w3c.dom.Node;
+
 import org.apache.xml.utils.PrefixResolver;
 import org.apache.xml.utils.PrefixResolverDefault;
 import org.apache.xpath.XPathAPI;
 import org.apache.xpath.objects.XObject;
-import org.w3c.dom.Node;
+
+import org.apache.fop.intermediate.IFCheck;
 
 /**
  * Simple check that requires an XPath expression to evaluate to true.
  */
-public class EvalCheck implements LayoutEngineCheck {
+public class EvalCheck implements LayoutEngineCheck, IFCheck {
 
     private String expected;
     private String xpath;
     private double tolerance;
     private PrefixResolver prefixResolver;
-    
+
     /**
      * Creates a new instance
      * @param expected expected value
@@ -46,7 +50,7 @@ public class EvalCheck implements LayoutEngineCheck {
         this.expected = expected;
         this.xpath = xpath;
     }
-    
+
     /**
      * Creates a new instance from a DOM node.
      * @param node DOM node that defines this check
@@ -60,12 +64,21 @@ public class EvalCheck implements LayoutEngineCheck {
         }
         this.prefixResolver = new PrefixResolverDefault(node);
     }
-    
-    /** @see org.apache.fop.layoutengine.LayoutEngineCheck */
+
+    /** {@inheritDoc} */
     public void check(LayoutResult result) {
+        doCheck(result.getAreaTree());
+    }
+
+    /** {@inheritDoc} */
+    public void check(Document intermediate) {
+        doCheck(intermediate);
+    }
+
+    private void doCheck(Document doc) {
         XObject res;
         try {
-            res = XPathAPI.eval(result.getAreaTree(), xpath, prefixResolver);
+            res = XPathAPI.eval(doc, xpath, prefixResolver);
         } catch (TransformerException e) {
             throw new RuntimeException("XPath evaluation failed: " + e.getMessage());
         }
@@ -75,19 +88,19 @@ public class EvalCheck implements LayoutEngineCheck {
             double v2 = Double.parseDouble(actual);
             if (Math.abs(v1 - v2) > tolerance) {
                 throw new RuntimeException(
-                        "Expected XPath expression to evaluate to '" + expected + "', but got '" 
+                        "Expected XPath expression to evaluate to '" + expected + "', but got '"
                         + actual + "' (" + this + ", outside tolerance)");
             }
         } else {
             if (!expected.equals(actual)) {
                 throw new RuntimeException(
-                        "Expected XPath expression to evaluate to '" + expected + "', but got '" 
+                        "Expected XPath expression to evaluate to '" + expected + "', but got '"
                         + actual + "' (" + this + ")");
             }
         }
     }
 
-    /** @see java.lang.Object#toString() */
+    /** {@inheritDoc} */
     public String toString() {
         return "XPath: " + xpath;
     }
