@@ -15,28 +15,32 @@
  * limitations under the License.
  */
 
-/* $Id: TrueCheck.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: TrueCheck.java 746664 2009-02-22 12:40:44Z jeremias $ */
 
 package org.apache.fop.layoutengine;
 
 import javax.xml.transform.TransformerException;
 
+import org.w3c.dom.Document;
+import org.w3c.dom.Node;
+
 import org.apache.xml.utils.PrefixResolver;
 import org.apache.xml.utils.PrefixResolverDefault;
 import org.apache.xpath.XPathAPI;
 import org.apache.xpath.objects.XBoolean;
 import org.apache.xpath.objects.XObject;
-import org.w3c.dom.Node;
+
+import org.apache.fop.intermediate.IFCheck;
 
 /**
  * Simple check that requires an XPath expression to evaluate to true.
  */
-public class TrueCheck implements LayoutEngineCheck {
+public class TrueCheck implements LayoutEngineCheck, IFCheck {
 
     private String xpath;
     private String failureMessage;
     private PrefixResolver prefixResolver;
-    
+
     /**
      * Creates a new instance
      * @param xpath XPath statement that needs to be evaluated
@@ -44,7 +48,7 @@ public class TrueCheck implements LayoutEngineCheck {
     public TrueCheck(String xpath) {
         this.xpath = xpath;
     }
-    
+
     /**
      * Creates a new instance from a DOM node.
      * @param node DOM node that defines this check
@@ -57,12 +61,21 @@ public class TrueCheck implements LayoutEngineCheck {
         }
         this.prefixResolver = new PrefixResolverDefault(node);
     }
-    
-    /** @see org.apache.fop.layoutengine.LayoutEngineCheck */
+
+    /** {@inheritDoc} */
     public void check(LayoutResult result) {
+        doCheck(result.getAreaTree());
+    }
+
+    /** {@inheritDoc} */
+    public void check(Document intermediate) {
+        doCheck(intermediate);
+    }
+
+    private void doCheck(Document doc) {
         XObject res;
         try {
-            res = XPathAPI.eval(result.getAreaTree(), xpath, prefixResolver);
+            res = XPathAPI.eval(doc, xpath, prefixResolver);
         } catch (TransformerException e) {
             throw new RuntimeException("XPath evaluation failed: " + e.getMessage());
         }
@@ -71,16 +84,16 @@ public class TrueCheck implements LayoutEngineCheck {
                 throw new RuntimeException(failureMessage);
             } else {
                 throw new RuntimeException(
-                        "Expected XPath expression to evaluate to 'true', but got '" 
+                        "Expected XPath expression to evaluate to 'true', but got '"
                         + res + "' (" + this + ")");
             }
         }
 
     }
 
-    /** @see java.lang.Object#toString() */
+    /** {@inheritDoc} */
     public String toString() {
         return "XPath: " + xpath;
     }
-    
+
 }
