@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: MemoryEater.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: MemoryEater.java 688078 2008-08-22 13:02:37Z jeremias $ */
 
 package org.apache.fop.memory;
 
@@ -34,6 +34,7 @@ import javax.xml.transform.sax.SAXResult;
 import javax.xml.transform.sax.SAXTransformerFactory;
 import javax.xml.transform.stream.StreamSource;
 
+import org.apache.commons.io.IOUtils;
 import org.apache.commons.io.output.NullOutputStream;
 
 import org.apache.fop.apps.FOUserAgent;
@@ -50,29 +51,49 @@ public class MemoryEater {
             = (SAXTransformerFactory)SAXTransformerFactory.newInstance();
     private FopFactory fopFactory = FopFactory.newInstance();
     private Templates replicatorTemplates;
-    
+
+    private Stats stats;
+
     public MemoryEater() throws TransformerConfigurationException, MalformedURLException {
         File xsltFile = new File("test/xsl/fo-replicator.xsl");
         Source xslt = new StreamSource(xsltFile);
         replicatorTemplates = tFactory.newTemplates(xslt);
     }
-    
-    private void eatMemory(File foFile, int replicatorRepeats) throws Exception {
+
+    private void eatMemory(File foFile, int runRepeats, int replicatorRepeats) throws Exception {
+        stats = new Stats();
+        for (int i = 0; i < runRepeats; i++) {
+            eatMemory(i, foFile, replicatorRepeats);
+            stats.progress(i, runRepeats);
+        }
+        stats.dumpFinalStats();
+        System.out.println(stats.getGoogleChartURL());
+    }
+
+    private void eatMemory(int callIndex, File foFile, int replicatorRepeats) throws Exception {
         Source src = new StreamSource(foFile);
-        
+
         Transformer transformer = replicatorTemplates.newTransformer();
         transformer.setParameter("repeats", new Integer(replicatorRepeats));
-        
+
         OutputStream out = new NullOutputStream(); //write to /dev/nul
-        FOUserAgent userAgent = fopFactory.newFOUserAgent();
-        userAgent.setBaseURL(foFile.getParentFile().toURL().toExternalForm());
-        Fop fop = fopFactory.newFop(MimeConstants.MIME_PDF, userAgent, out);
-        Result res = new SAXResult(fop.getDefaultHandler());
-        
-        transformer.transform(src, res);
-        
-        System.out.println("Generated " + fop.getResults().getPageCount() + " pages.");
-        
+        try {
+            FOUserAgent userAgent = fopFactory.newFOUserAgent();
+            userAgent.setBaseURL(foFile.getParentFile().toURL().toExternalForm());
+            Fop fop = fopFactory.newFop(MimeConstants.MIME_PDF, userAgent, out);
+            Result res = new SAXResult(fop.getDefaultHandler());
+
+            transformer.transform(src, res);
+
+            stats.notifyPagesProduced(fop.getResults().getPageCount());
+            if (callIndex == 0) {
+                System.out.println(foFile.getName() + " generates "
+                        + fop.getResults().getPageCount() + " pages.");
+            }
+            stats.checkStats();
+        } finally {
+            IOUtils.closeQuietly(out);
+        }
     }
 
     private static void prompt() throws IOException {
@@ -80,7 +101,7 @@ public class MemoryEater {
         System.out.print("Press return to continue...");
         in.readLine();
     }
-    
+
     /**
      * Main method.
      * @param args the command-line arguments
@@ -97,24 +118,22 @@ public class MemoryEater {
                 runRepeats = Integer.parseInt(args[1]);
             }
             File testFile = new File("examples/fo/basic/readme.fo");
-            
-            System.out.println("MemoryEater! About to replicate the test file " 
+
+            System.out.println("MemoryEater! About to replicate the test file "
                     + replicatorRepeats + " times and run it " + runRepeats + " times...");
             if (doPrompt) {
                 prompt();
             }
-            
-            System.out.println("Processing..."); 
+
+            System.out.println("Processing...");
             long start = System.currentTimeMillis();
-            
+
             MemoryEater app = new MemoryEater();
-            for (int i = 0; i < runRepeats; i++) {
-                app.eatMemory(testFile, replicatorRepeats);
-            }
-            
+            app.eatMemory(testFile, runRepeats, replicatorRepeats);
+
             long duration = System.currentTimeMillis() - start;
             System.out.println("Success! Job took " + duration + " ms");
-            
+
             if (doPrompt) {
                 prompt();
             }
