@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,12 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: FOProcessorImpl.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: FOProcessorImpl.java 759147 2009-03-27 14:19:00Z jeremias $ */
 
 package org.apache.fop.threading;
 
-import java.io.InputStream;
 import java.io.OutputStream;
+import java.net.MalformedURLException;
+import java.net.URL;
 
 import javax.xml.transform.Result;
 import javax.xml.transform.Source;
@@ -29,19 +30,23 @@ import javax.xml.transform.Transformer;
 import javax.xml.transform.TransformerException;
 import javax.xml.transform.TransformerFactory;
 import javax.xml.transform.sax.SAXResult;
-import javax.xml.transform.stream.StreamSource;
 
 import org.apache.avalon.framework.activity.Initializable;
 import org.apache.avalon.framework.configuration.Configurable;
 import org.apache.avalon.framework.configuration.Configuration;
 import org.apache.avalon.framework.configuration.ConfigurationException;
 import org.apache.avalon.framework.logger.AbstractLogEnabled;
+import org.apache.commons.io.FilenameUtils;
 
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.apps.Fop;
 import org.apache.fop.apps.FopFactory;
 import org.apache.fop.apps.MimeConstants;
+import org.apache.fop.events.Event;
+import org.apache.fop.events.EventFormatter;
+import org.apache.fop.events.EventListener;
+import org.apache.fop.events.model.EventSeverity;
 
 /**
  * Default implementation of the FOProcessor interface using FOP.
@@ -71,9 +76,17 @@ public class FOProcessorImpl extends AbstractLogEnabled
     }
 
     /** {@inheritDoc} */
-    public void process(InputStream in, Templates templates, OutputStream out) 
+    public void process(Source src, Templates templates, OutputStream out)
                 throws org.apache.fop.apps.FOPException, java.io.IOException {
         FOUserAgent foUserAgent = fopFactory.newFOUserAgent();
+        foUserAgent.setBaseURL(src.getSystemId());
+        try {
+            URL url = new URL(src.getSystemId());
+            String filename = FilenameUtils.getName(url.getPath());
+            foUserAgent.getEventBroadcaster().addEventListener(new AvalonAdapter(filename));
+        } catch (MalformedURLException mfue) {
+            throw new RuntimeException(mfue);
+        }
         Fop fop = fopFactory.newFop(this.mime, foUserAgent, out);
 
         try {
@@ -83,7 +96,6 @@ public class FOProcessorImpl extends AbstractLogEnabled
             } else {
                 transformer = templates.newTransformer();
             }
-            Source src = new StreamSource(in);
             Result res = new SAXResult(fop.getDefaultHandler());
             transformer.transform(src, res);
         } catch (TransformerException e) {
@@ -96,4 +108,29 @@ public class FOProcessorImpl extends AbstractLogEnabled
         return this.fileExtension;
     }
 
+    private class AvalonAdapter implements EventListener {
+
+        private String filename;
+
+        public AvalonAdapter(String filename) {
+            this.filename = filename;
+        }
+
+        public void processEvent(Event event) {
+            String msg = EventFormatter.format(event);
+            EventSeverity severity = event.getSeverity();
+            if (severity == EventSeverity.INFO) {
+                //getLogger().info(filename + ": " + msg);
+            } else if (severity == EventSeverity.WARN) {
+                //getLogger().warn(filename + ": "  + msg);
+            } else if (severity == EventSeverity.ERROR) {
+                getLogger().error(filename + ": "  + msg);
+            } else if (severity == EventSeverity.FATAL) {
+                getLogger().fatalError(filename + ": "  + msg);
+            } else {
+                assert false;
+            }
+        }
+
+    }
 }
\ No newline at end of file
