@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BorderPropsTestCase.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: BorderPropsTestCase.java 959945 2010-07-02 10:44:18Z jeremias $ */
 
 package org.apache.fop.traits;
 
@@ -23,7 +23,10 @@ import java.awt.Color;
 
 import junit.framework.TestCase;
 
+import org.apache.xmlgraphics.java2d.color.DeviceCMYKColorSpace;
+
 import org.apache.fop.fo.Constants;
+import org.apache.fop.util.ColorExt;
 import org.apache.fop.util.ColorUtil;
 
 /**
@@ -39,18 +42,23 @@ public class BorderPropsTestCase extends TestCase {
         Color col = new Color(1.0f, 1.0f, 0.5f, 1.0f);
         //Normalize: Avoid false alarms due to color conversion (rounding)
         col = ColorUtil.parseColorString(null, ColorUtil.colorToString(col));
-        
-        BorderProps b1 = new BorderProps(Constants.EN_DOUBLE, 1250, 
+
+        BorderProps b1 = new BorderProps(Constants.EN_DOUBLE, 1250,
                 col, BorderProps.COLLAPSE_OUTER);
         String ser = b1.toString();
         BorderProps b2 = BorderProps.valueOf(null, ser);
         assertEquals(b1, b2);
 
-        b1 = new BorderProps(Constants.EN_INSET, 9999, 
+        float[] cmyk = new float[] {1.0f, 1.0f, 0.5f, 1.0f};
+        DeviceCMYKColorSpace cmykCs = DeviceCMYKColorSpace.getInstance();
+        float[] rgb = cmykCs.toRGB(cmyk);
+        col = ColorExt.createFromFoRgbIcc(rgb[0], rgb[1], rgb[2],
+                "#CMYK", null, cmykCs, cmyk);
+        b1 = new BorderProps(Constants.EN_INSET, 9999,
                 col, BorderProps.SEPARATE);
         ser = b1.toString();
         b2 = BorderProps.valueOf(null, ser);
         assertEquals(b1, b2);
     }
-    
+
 }
